
/******************************************************************************
* MODULE     : gg_system.cc
* DESCRIPTION: system dependent code
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "gg_system.h"

/******************************************************************************
* system dependent definitions
******************************************************************************/

#ifdef __i386__
string os ("linux");
/*
string obj_optimize ("-O3 -fexpensive-optimizations -malign-loops=2 -malign-jumps=2 -malign-functions=2");
string obj_x_windows ("");
string lib_x_windows ("-L/usr/X11R6/lib -lXext -lX11 -Wl,-rpath,/usr/X11R6/lib");
string lib_static ("");
string lib_shared ("-shared");
string lib_search ("-Wl,-rpath,");
string bin_x_windows ("-L/usr/X11R6/lib -lXext -lX11 -Wl,-rpath,/usr/X11R6/lib");
string bin_static ("-Wl,-Bstatic");
string bin_shared ("-Wl,-Bdynamic");
*/
#endif

#if defined (__sparc) || defined (sparc)
string os ("sun");
/*
string obj_optimize ("-O");
string obj_x_windows ("-I/usr/openwin/include");
string lib_x_windows ("-L/usr/X11/lib -L/usr/openwin/lib -lXext -lX11 -Wl,-R,/usr/X11/lib -Wl,-R,/usr/openwin/lib");
string lib_static ("");
string lib_shared ("-G");
string lib_search ("-Wl,-R,");
string bin_x_windows ("-L/usr/X11/lib -L/usr/openwin/lib -lXext -lX11 -lsocket -Wl,-R,/usr/X11/lib -Wl,-R,/usr/openwin/lib");
string bin_static ("");
string bin_shared ("");
*/
#endif

#ifdef __alpha
string os ("dec");
/*
string obj_optimize ("");
string obj_x_windows ("-I/usr/include/X11");
string lib_x_windows ("-lXext -lX11");
string lib_static ("");
string lib_shared ("-shared");
string lib_search ("-Wl,-rpath,");
string bin_x_windows ("-lXext -lX11");
string bin_static ("");
string bin_shared ("-shared");
*/
#endif

/******************************************************************************
* compilation and linking
******************************************************************************/

string
make_env_variables () {
#ifdef __i386__
  return "
CXX = g++
CXXFLAGS =
CXXOPTIMIZE = -O3 -fexpensive-optimizations -malign-loops=2 -malign-jumps=2 -malign-functions=2
CXXXWINDOWS =
LD = g++
LDFLAGS =
LDXWINDOWS = -L/usr/X11R6/lib -lXext -lX11 -Wl,-rpath,/usr/X11R6/lib
LDSTATIC = -static
LDSHARED = -shared
LDPATH = -Wl,-rpath,
LDB = g++
LDBFLAGS =
LDBXWINDOWS = -L/usr/X11R6/lib -lXext -lX11 -Wl,-rpath,/usr/X11R6/lib
LDBSTATIC = -Wl,-Bstatic
LDBSHARED = -Wl,-Bdynamic
LDBPATH = -Wl,-rpath,
AR = ar
ARFLAGS = -r
RM = rm -f
GENCC = gencc
";
#endif

#if defined (__sparc) || defined (sparc)
  return "
CXX = g++
CXXFLAGS =
CXXOPTIMIZE = -O
CXXXWINDOWS = -I/usr/openwin/include
LD = g++
LDFLAGS =
LDXWINDOWS = -L/usr/X11/lib -L/usr/openwin/lib -lXext -lX11 -Wl,-R,/usr/X11/lib -Wl,-R,/usr/openwin/lib
LDSTATIC =
LDSHARED = -G
LDPATH = -Wl,-R,
LDB = g++
LDBFLAGS =
LDBXWINDOWS = -L/usr/X11/lib -L/usr/openwin/lib -lXext -lX11 -lsocket -Wl,-R,/usr/X11/lib -Wl,-R,/usr/openwin/lib
LDBSTATIC =
LDBSHARED =
LDBPATH = -Wl,-R,
AR = ar
ARFLAGS = -r
RM = rm -f
GENCC = gencc
";
#endif

#ifdef __alpha
  return "
CXX = g++
CXXFLAGS =
CXXOPTIMIZE =
CXXXWINDOWS = -I/usr/include/X11
LD = gcc -lstdc++ -lg++
LDFLAGS =
LDXWINDOWS = -lXext -lX11
LDSTATIC =
LDSHARED = -shared
LDPATH = -Wl,-rpath,
LDB = gcc
LDBFLAGS =
LDBXWINDOWS = -lXext -lX11
LDBSTATIC =
LDBSHARED = -shared
LDBPATH = -Wl,-rpath,
AR = ar
ARFLAGS = -r
RM = rm -f
GENCC = gencc
";
#endif
}

string
make_object (string in, string out, string options) {
  return "$(CXX) $(CXXFLAGS) -c " *
    options * (N(options)==0? "": " ") * in * " -o " * out;
}

string
make_static_library (string in, string out, string options, string lib_path) {
  (void) options;
  (void) lib_path;
  return "$(AR) $(ARFLAGS) " * out * " " * in;
}

string
make_dynamic_library (string in, string out, string options, string lib_path) {
  return
    "$(LD) $(LDFLAGS) $(LDSHARED) -L" * lib_path * " " *
    options * (N(options)==0? "": " ") *
    "$(LDPATH)" * lib_path * " " *
    in * " -o " * out;
}

string
make_static_binary (string in, string out, string options, string lib_path) {
#ifdef __alpha
  return "$(LDB) $(LDBFLAGS) " * in * " " * "$(LDBSTATIC) " *
    "-L" * lib_path * " " *
    options * (N(options)==0? "": " ") *
    "-lstdc++ -o " * out;
#else
  return "$(LDB) $(LDBFLAGS) " * in * " " * "$(LDBSTATIC) " *
    "-L" * lib_path * " " *
    options * (N(options)==0? "": " ") *
    "-o " * out;
#endif
}

string
make_dynamic_binary (string in, string out, string options, string lib_path) {
#ifdef __alpha
  return
    "$(LDB) $(LDBFLAGS) -lstdc++ -lg++ $(LDBSHARED) -L" * lib_path * " " *
    options * (N(options)==0? "": " ") *
    "$(LDBPATH)" * lib_path * " " *
    in * " -o " * out;
#else
  return
    "$(LDB) $(LDBFLAGS) $(LDBSHARED) -L" * lib_path * " " *
    options * (N(options)==0? "": " ") *
    "$(LDBPATH)" * lib_path * " " *
    in * " -o " * out;
#endif
}

/******************************************************************************
* other routines
******************************************************************************/

bool
test_os (string s) {
  int i;
  for (i=0; i<N(s); i++) {
    if (s ("all", i)) return TRUE;
    if ((N(s)>=i+N(os)) && (s(i,i+N(os))==os)) return TRUE;
    while ((i<N(s)) && (s[i]!=' ') && (s[i]!='\t')) i++;
  }
  return FALSE;
}

bool
option_contains (string s, string special) {
  int i;
  for (i=0; i<N(s); i++) {
    if ((N(s)>=i+N(special)) && (s(i,i+N(special))==special))
      return TRUE;
    while ((i<N(s)) && (s[i]!=' ') && (s[i]!='\t')) i++;
  }
  return FALSE;
}

string
replace_special_option (string s, string special, string value) {
  int i;
  for (i=0; i<N(s); i++) {
    if ((N(s)>=i+N(special)) && (s(i,i+N(special))==special))
      return s(0, i)* value*
	replace_special_option (s (i+N(special), N(s)), special, value);
    while ((i<N(s)) && (s[i]!=' ') && (s[i]!='\t')) i++;
  }
  return s;
}

string
special_object_option (string s) {
  s= replace_special_option (s, "+optimize", "$(CXXOPTIMIZE)");
  s= replace_special_option (s, "+X-windows", "$(CXXXWINDOWS)");
  s= replace_special_option (s, "+X-Windows", "$(CXXXWINDOWS)");
  return s;
}

string
special_library_option (string s) {
  s= replace_special_option (s, "+X-windows", "$(LDXWINDOWS)");
  s= replace_special_option (s, "+X-Windows", "$(LDXWINDOWS)");
  s= replace_special_option (s, "+static", "");
  s= replace_special_option (s, "+shared", "");
  return s;
}

string
special_binary_option (string s) {
  s= replace_special_option (s, "+X-windows", "$(LDBXWINDOWS)");
  s= replace_special_option (s, "+X-Windows", "$(LDBXWINDOWS)");
  s= replace_special_option (s, "+static", "");
  s= replace_special_option (s, "+shared", "");
  return s;
}
