
/******************************************************************************
* MODULE     : fp_real.gen.cc
* DESCRIPTION: fp_reals based on gmp
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <Arithmetic/fp_real.gen.h>

#module code_fp_real
#import fp_real
#include <math.h>

/******************************************************************************
* conversion
******************************************************************************/

static int fp_real_precision=0;

void
set_fp_real_precision (int bits) {
  fp_real_precision= bits;
  mpf_set_default_prec (bits);
}

fp_real::fp_real () {
  rep= new fp_real_rep();
}

fp_real::fp_real (int i) {
  rep= new fp_real_rep();
  mpf_set_si (rep->F, i);
}

fp_real::fp_real (string s, int base) {
  rep= new fp_real_rep();
  char* ptr= as_charp (s);
  mpf_set_str (rep->F, ptr, base);
  delete[] ptr;
}

fp_real::operator string () {
  int n= (int) (fp_real_precision * (log(10)/log(2.0)));
  char s[n+2];
  long int exp;
  (void) mpf_get_str (s, &exp, 10, n, rep->F);
  string mantissa;
  if (s[0]=='\0') return ("0");
  if (s[0]=='-') mantissa= "-" * string (s[1]) * "." * string (s+2);
  else mantissa= string (s[0]) * "." * string (s+1);
  if (mantissa[N(mantissa)-1]=='.') mantissa= mantissa * "0";
  if (exp==1) return mantissa;
  return mantissa * "e" * as_string ((int) (exp-1));
}

fp_real::operator tree () {
  return tree ((string) (*this));
}

ostream&
operator << (ostream& out, fp_real x) {
  return out << ((string) x);
}

/******************************************************************************
* basic arithmetics
******************************************************************************/

fp_real
operator + (fp_real x, fp_real y) {
  fp_real z;
  mpf_add (z->F, x->F, y->F);
  return z;
}

fp_real
operator - (fp_real x) {
  fp_real y;
  mpf_neg (y->F, x->F);
  return y;
}

fp_real
operator - (fp_real x, fp_real y) {
  fp_real z;
  mpf_sub (z->F, x->F, y->F);
  return z;
}

fp_real
operator * (fp_real x, fp_real y) {
  fp_real z;
  mpf_mul (z->F, x->F, y->F);
  return z;
}

fp_real
operator / (fp_real x, fp_real y) {
  fp_real z;
  mpf_div (z->F, x->F, y->F);
  return z;
}

/******************************************************************************
* tests on sign
******************************************************************************/

int
sign (fp_real x) {
  return mpf_sgn (x->F);
}

bool
operator == (fp_real x, fp_real y) {
  return mpf_cmp (x->F, y->F) == 0;
}

bool
operator != (fp_real x, fp_real y) {
  return mpf_cmp (x->F, y->F) != 0;
}

bool
operator <= (fp_real x, fp_real y) {
  return mpf_cmp (x->F, y->F) <= 0;
}

bool
operator >= (fp_real x, fp_real y) {
  return mpf_cmp (x->F, y->F) >= 0;
}

bool
operator < (fp_real x, fp_real y) {
  return mpf_cmp (x->F, y->F) < 0;
}

bool
operator > (fp_real x, fp_real y) {
  return mpf_cmp (x->F, y->F) > 0;
}

#endmodule // code_fp_real
