
/******************************************************************************
* MODULE     : rational.gen.cc
* DESCRIPTION: rationals based on gmp
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <Arithmetic/rational.gen.h>

#module code_rational
#import rational

/******************************************************************************
* conversion
******************************************************************************/

rational::rational () {
  rep= new rational_rep();
}

rational::rational (int i) {
  rep= new rational_rep();
  mpq_set_si (rep->Q, i, 1);
}

rational::rational (integer i) {
  rep= new rational_rep();
  mpq_set_z (rep->Q, i->I);
}

rational::operator string () {
  int n= mpz_sizeinbase (mpq_numref (rep->Q), 10);
  char num[n+2];
  mpz_get_str (num, 10, mpq_numref (rep->Q));
  n= mpz_sizeinbase (mpq_denref (rep->Q), 10);
  char den[n+2];
  mpz_get_str (den, 10, mpq_denref (rep->Q));
  if (string (den) == "1") return string (num);
  return string (num) * "/" * string (den);
}

rational::operator tree () {
  return tree ((string) (*this));
}

ostream&
operator << (ostream& out, rational i) {
  return out << ((string) i);
}

/******************************************************************************
* basic arithmetics
******************************************************************************/

rational
operator + (rational i, rational j) {
  rational k;
  mpq_add (k->Q, i->Q, j->Q);
  return k;
}

rational
operator - (rational i) {
  rational j;
  mpq_neg (j->Q, i->Q);
  return j;
}

rational
operator - (rational i, rational j) {
  rational k;
  mpq_sub (k->Q, i->Q, j->Q);
  return k;
}

rational
operator * (rational i, rational j) {
  rational k;
  mpq_mul (k->Q, i->Q, j->Q);
  return k;
}

rational
operator / (rational i, rational j) {
  rational k;
  mpq_div (k->Q, i->Q, j->Q);
  return k;
}

#endmodule // code_rational
