
/******************************************************************************
* MODULE     : rational.gen.cc
* DESCRIPTION: rationals based on gmp
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <Arithmetic/rational.gen.h>

#module code_rational
#import rational

/******************************************************************************
* conversion
******************************************************************************/

rational::rational () {
  rep= new rational_rep (0, 1);
}

rational::rational (int i) {
  rep= new rational_rep (i, 1);
}

rational::rational (integer p, integer q) {
  integer g= gcd (p, q);
  if (sign(g)*sign(q)<0) g=-g;
  rep= new rational_rep (p/g, q/g);
}

rational::operator string () {
  if (rep->q == 1) return (string) rep->p;
  return ((string) rep->p) * "/" * ((string) rep->q);
}

rational::operator tree () {
  return tree ((string) (*this));
}

ostream&
operator << (ostream& out, rational i) {
  return out << ((string) i);
}

/******************************************************************************
* basic arithmetics
******************************************************************************/

rational
operator + (rational i, rational j) {
  return rational (i->p*j->q+ i->q*j->p, i->q*j->q);
}

rational
operator - (rational i) {
  return rational (-i->p, i->q);
}

rational
operator - (rational i, rational j) {
  return rational (i->p*j->q- i->q*j->p, i->q*j->q);
}

rational
operator * (rational i, rational j) {
  return rational (i->p*j->p, i->q*j->q);
}

rational
operator / (rational i, rational j) {
  return rational (i->p*j->q, i->q*j->p);
}

#endmodule // code_rational
