
/******************************************************************************
* MODULE     : viewer.gen.cc
* DESCRIPTION: Visual interface for analytic functions
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <dir.gen.h>
#include <window.gen.h>
#include <Widget/basic_widget.gen.h>

#module code_viewer
#import dir
#import window
#import basic_widget

#define WIDTH 701
#define HEIGHT 501

/******************************************************************************
* Test widget
******************************************************************************/

class viewer_widget_rep: public basic_widget_rep {
  widget footer;
  analytic_function f;

  C    origin;
  R    unit;
  R    scale;
  R    si;
  int  mode;
  C*   raster;
  bool recompute_flag;

public:
  viewer_widget_rep (widget footer, analytic_function f, C origin);
  ~viewer_widget_rep ();
  operator tree ();

  void  recompute ();
  C     get (SI x, SI y);
  color get_color (SI x, SI y);
  void  execute (string s);
  void  display_axes (repaint_event ev);
  void  display_function (repaint_event ev);

  void handle_get_size (get_size_event ev);
  void handle_repaint (repaint_event ev);
  void handle_keypress (keypress_event ev);
  void handle_mouse (mouse_event ev);
};

viewer_widget_rep::viewer_widget_rep (widget footer2,
  analytic_function f2, C origin2):
  basic_widget_rep (footer2->dis, center), footer (footer2),
  f (f2), origin (origin2)
{
  unit  = 100*PIXEL;
  scale = 1.0;
  si    = scale/unit;
  mode  = 0;
  raster= new C[WIDTH*HEIGHT];
  recompute_flag= TRUE;
}

viewer_widget_rep::~viewer_widget_rep () {
  delete[] raster;
}

viewer_widget_rep::operator tree () {
  return "viewer";
}

void
viewer_widget_rep::handle_get_size (get_size_event ev) {
  ev->w= (WIDTH-1)*PIXEL;
  ev->h= (HEIGHT-1)*PIXEL;
}

/******************************************************************************
* Compute values of the function
******************************************************************************/

void
viewer_widget_rep::recompute () {
  int d=10;
  int i, j;
  for (i=0; i<WIDTH; i+=d) {
    if (i%25==0) cout << i << "\n";
    for (j=0; j<HEIGHT; j+=d) {
      R x= (i-(WIDTH >>1))*PIXEL*si;
      R y= (j-(HEIGHT>>1))*PIXEL*si;
      raster[j*WIDTH+i]= evaluate_straight (f, C (x, y));
    }
  }

  int i0, j0;
  for (i0=0; i0<WIDTH; i0+=d) {
    if (i0%100==0) cout << i0 << "\n";
    for (j0=0; j0<HEIGHT; j0+=d) {
      C z00= raster[j0*WIDTH+i0];
      C z01= raster[j0*WIDTH+ min (i0+d, WIDTH-1)];
      C z10= raster[min (j0+d,HEIGHT-1)*WIDTH+i0];
      C z11= raster[min (j0+d,HEIGHT-1)*WIDTH+ min (i0+d, WIDTH-1)];
      for (i=0; i<min(d,WIDTH-i0); i++) {
	C z0= (z00*(d-i)+ z01*i)/C(d);
	C z1= (z10*(d-i)+ z11*i)/C(d);
	for (j=0; j<min(d,HEIGHT-j0); j++)
	  raster[(j0+j)*WIDTH+(i0+i)]= (z0*(d-j)+ z1*j)/C(d);
      }
    }
  }
}

C
viewer_widget_rep::get (SI x, SI y) {
  x = (x/PIXEL)+(WIDTH >>1);
  y = (y/PIXEL)+(HEIGHT>>1);
  if (x<0) x=0; if (x>=WIDTH) x=WIDTH-1;
  if (y<0) y=0; if (y>=HEIGHT) y=HEIGHT-1;
  return raster[y*WIDTH+x];
}

/******************************************************************************
* Displaying the function
******************************************************************************/

color
viewer_widget_rep::get_color (SI x, SI y) {
  x = (x/PIXEL)+(WIDTH >>1);
  y = (y/PIXEL)+(HEIGHT>>1);
  if (x<0) x=0; if (x>=WIDTH-1) x=WIDTH;
  if (y<0) y=0; if (y>=HEIGHT-1) y=HEIGHT;

  switch (mode) {
  case 0:
    {
      C fz0= raster[y*WIDTH+x];
      C fz1= raster[y*WIDTH+x+1];
      C fz2= raster[(y+1)*WIDTH+x];
      if ((abs(fz0)>1000000) || (abs(fz1)>1000000) || (abs(fz2)>1000000)) {
	if ((fz0.imag()*fz1.imag()<0) || (fz0.imag()*fz2.imag()<0))
	  return dis->blue;
	return dis->red;
      }
      else {
	int zx0= (int) floor (fz0.real()/scale);
	int zy0= (int) floor (fz0.imag()/scale);
	int zx1= (int) floor (fz1.real()/scale);
	int zy1= (int) floor (fz1.imag()/scale);
	int zx2= (int) floor (fz2.real()/scale);
	int zy2= (int) floor (fz2.imag()/scale);
	if ((zx1==zx0) && (zx2==zx0) && (zy1==zy0) && (zy2==zy0)) {
	  if (fz0.imag()<0) return dis->white;
	  else return dis->blue;
	}
	if ((fz0.imag()*fz1.imag()<0) || (fz0.imag()*fz2.imag()<0))
	  return dis->blue;
	return dis->red;
      }
    }
  case 1:
    {
      C fz0= (raster[y*WIDTH+x]==0? C(0): log (-raster[y*WIDTH+x]));
      C fz1= (raster[y*WIDTH+x+1]==0? C(0): log (-raster[y*WIDTH+x+1]));
      C fz2= (raster[(y+1)*WIDTH+x]==0? C(0): log (-raster[(y+1)*WIDTH+x]));
      if ((abs(fz0)>1000000) || (abs(fz1)>1000000) || (abs(fz2)>1000000))
	return dis->red;
      int zx0= (int) floor (fz0.real()*8.0);
      int zy0= (int) floor (48.0*fz0.imag()/6.284);
      int zx1= (int) floor (fz1.real()*8.0);
      int zy1= (int) floor (48.0*fz1.imag()/6.284);
      int zx2= (int) floor (fz2.real()*8.0);
      int zy2= (int) floor (48.0*fz2.imag()/6.284);
      if ((zx1==zx0) && (zx2==zx0) && (zy1==zy0) && (zy2==zy0))
	return dis->white;
      if ((zy1!=zy0) || (zy2!=zy0)) {
	if ((fz0.imag()*fz1.imag()<0) || (fz0.imag()*fz2.imag()<0))
	  return dis->blue;
	return dis->rgb (128, 128, 255);
      }
      return dis->red;
    }
  case 2:
    {
      R v = atan (log (abs (raster[y*WIDTH+x])+1e-50));
      int grey= (int) (255*(v+1.575)/3.15);
      return dis->rgb (grey, grey, grey);
    }
  }
  return dis->white;
}

void
viewer_widget_rep::display_function (repaint_event ev) {
  SI  x, y;
  for (y=ev->y1; y<ev->y2; y+=PIXEL)
    for (x=ev->x1; x<ev->x2; x+=PIXEL) {
      win->set_color (get_color (x, y));
      win->fill (x, y, x+PIXEL, y+PIXEL);
    }
}

void
viewer_widget_rep::display_axes (repaint_event ev) {
  int i, j;
  SI  ox= -(((SI) (origin.real()/si))/PIXEL)*PIXEL;
  SI  oy= -(((SI) (origin.imag()/si))/PIXEL)*PIXEL;
  int i1= (int) ((ev->x1-ox)/unit);
  int i2= (int) ((ev->x2-ox)/unit);
  int j1= (int) ((ev->y1-oy)/unit);
  int j2= (int) ((ev->y2-oy)/unit);
  win->set_color (dis->black);
  server_font fn= dis->server_ps_font ("lucidasans-12");
  win->line (ev->x1, oy, ev->x2, oy);
  for (i=i1-1; i<=i2+1; i++) {
    win->text (as_string (i* scale), fn,
	       ((int) (i*unit))+2*PIXEL+ ox, -fn->y1+ oy);
    win->line ((SI) (i*unit)+ ox, -2*PIXEL+ oy,
	       (SI) (i*unit)+ ox, 2*PIXEL+ oy);
  }
  win->line (ox, ev->y1, ox, ev->y2);
  for (j=j1-1; j<=j2+1; j++) {
    win->text (as_string (j* scale), fn,
	       2*PIXEL+ ox, ((SI) (j*unit))- fn->y1+ oy);
    win->line (-2*PIXEL+ ox, (SI) (j*unit)+ oy,
	       2*PIXEL+ ox, (SI) (j*unit)+ oy);
  }
}

void
viewer_widget_rep::handle_repaint (repaint_event ev) {
  if (recompute_flag) {
    recompute ();
    recompute_flag= FALSE;
  }
  win->set_color (dis->white);
  win->fill (ev->x1, ev->y1, ev->x2, ev->y2);
  display_axes (ev);
  display_function (ev);
  display_axes (ev);
}

/******************************************************************************
* Other routines
******************************************************************************/

void
viewer_widget_rep::execute (string s) {
  if (s=="cartesian") mode=0;
  if (s=="polar") mode=1;
  if (s=="norm") mode=2;
  widget (this) << emit_invalidate_all ();
}

void
viewer_widget_rep::handle_keypress (keypress_event ev) {
  if (ev->key == "+") scale /= 2;
  if (ev->key == "-") scale *= 2;
  if (ev->key == "<page-up>") scale /= 10;
  if (ev->key == "<page-down>") scale *= 10;
  si= scale/unit;
  recompute_flag= TRUE;
  widget (this) << emit_invalidate_all ();
}

string
as_string (C z) {
  if (z.real()==0) {
    if (z.imag()==0) return "0";
    else return as_string (z.imag()) * "i";
  }
  else {
    string re= as_string (z.real());
    if (z.imag()==0) return re;
    if (z.imag()<0) return re * "-" * as_string (-z.imag()) * "i";
    return re * "+" * as_string (z.imag()) * "i";
  }
}

void
viewer_widget_rep::handle_mouse (mouse_event ev) {
  if (ev->type == "press-left") {
    string pos = "position: " * as_string (C (ev->x*si, ev->y*si)+ origin);
    string val = "value: " * as_string (get (ev->x, ev->y));
    footer << set_widget ("left", text_widget (dis, pos));
    footer << set_widget ("right", text_widget (dis, val));
    if (attached ()) footer << emit_update ();
  }
  if (ev->type == "press-right") {
    f= continuate_straight (f, C (ev->x*si, ev->y*si));
    origin += C (ev->x*si, ev->y*si);
    recompute_flag= TRUE;
    widget (this) << emit_invalidate_all ();
  }
}

/******************************************************************************
* The viewer widget with menus and so
******************************************************************************/

class menu_command_rep: public command_rep {
  viewer_widget_rep* w;
  string s;

public:
  menu_command_rep (viewer_widget_rep* w2, string s2): w (w2), s(s2) {}
  void apply () { w->execute (s); }
};

command
menu_do (viewer_widget_rep* w, string s) {
  return new menu_command_rep (w, s);
}

class redirect_widget_rep: public basic_widget_rep {
  widget_rep* kbd_focus;

public:
  redirect_widget_rep (widget w, widget_rep* kbd_focus);
  operator tree ();
  void handle_keypress (keypress_event ev);
  void handle_keyboard_focus (keyboard_focus_event ev);
};

redirect_widget_rep::redirect_widget_rep (widget w, widget_rep* kf):
  basic_widget_rep (w->dis, 1), kbd_focus (kf) { a[0]= w; }

redirect_widget_rep::operator tree () {
  return tree ("redirect keyboard", a[0], widget(kbd_focus)); }

void
redirect_widget_rep::handle_keypress (keypress_event ev) {
  kbd_focus << ev; }

void
redirect_widget_rep::handle_keyboard_focus (keyboard_focus_event ev) {
  kbd_focus << ev; }

widget
redirect_widget (widget w, widget_rep* kbd_focus) {
  return new redirect_widget_rep (w, kbd_focus);
}

widget
viewer_widget (display dis) {
  array<widget> footer (3);
  array<string> footer_name (3);
  footer[0]= text_widget (dis, "Welcome to the analytic function shower");
  footer[1]= glue_widget (dis, TRUE, FALSE);
  footer[2]= text_widget (dis, "");
  footer_name[0]= "left";
  footer_name[1]= "middle";
  footer_name[2]= "right";
  widget ftr= horizontal_array (dis, footer, footer_name, 1);

  viewer_widget_rep* cw =
    new viewer_widget_rep (ftr, test_function(), C (2.0));

  widget mode_menu= vertical_menu (dis);
  mode_menu << emit_insert ("cartesian",
    command_button (text_widget (dis, "cartesian"), menu_do (cw, "cartesian")));
  mode_menu << emit_insert ("polar",
    command_button (text_widget (dis, "polar"), menu_do (cw, "polar")));
  mode_menu << emit_insert ("norm",
    command_button (text_widget (dis, "norm"), menu_do (cw, "norm")));

  widget main_menu= horizontal_menu (dis);
  main_menu << emit_insert ("Mode",
    pulldown_button (text_widget (dis, "Mode"), mode_menu));

  array<widget> header (2);
  header[0]= main_menu;
  header[1]= glue_widget (dis, TRUE, FALSE);

  array<widget> viewer_main (3);
  viewer_main[0]= horizontal_array (dis, header);
  viewer_main[1]= cw;
  viewer_main[2]= ftr;

  return redirect_widget (vertical_list (dis, viewer_main), cw);
}

/******************************************************************************
* The main program
******************************************************************************/

int
main (int argc, char** argv) {
  install_tex ();
  display dis= open_display (argc, argv);
  window w1= plain_window (viewer_widget (dis), "analytic function shower");
  w1->map ();
  dis->event_loop ();
  close_display (dis);
  return 0;
}

#endmodule // code_viewer
