
/******************************************************************************
* MODULE     : diffeq.gen.cc
* DESCRIPTION: Show a first orderdifferential equation
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <tree.gen.h>
#include <dir.gen.h>
#include <window.gen.h>
#include <Widget/basic_widget.gen.h>

#module code_diffeq
#import tree
#import dir
#import window
#import basic_widget

#include <math.h>

#define WIDTH 1001
#define HEIGHT 801
#define DENSITY 10
#define R double

/******************************************************************************
* Diffeq shower widget
******************************************************************************/

class diffeq_widget_rep: public basic_widget_rep {
  widget footer;

  R    origin_x, origin_y;
  R    unit;
  R    scale;
  R    si;

  bool redraw_flag;
  R    point_x, point_y;

public:
  diffeq_widget_rep (widget footer);
  ~diffeq_widget_rep ();
  operator tree ();


  R     f (R x, R y);
  void  display_axes (repaint_event ev);
  void  display_diffeq (repaint_event ev);
  bool  display_solution (repaint_event ev, R& x, R& y, R d);

  void handle_get_size (get_size_event ev);
  void handle_repaint (repaint_event ev);
  void handle_keypress (keypress_event ev);
  void handle_mouse (mouse_event ev);
};

diffeq_widget_rep::diffeq_widget_rep (widget footer2):
  basic_widget_rep (footer2->dis, center), footer (footer2)
{
  origin_x= 0.0;
  origin_y= 0.0;
  unit    = 100*PIXEL;
  scale   = 1.0;
  si      = scale/unit;

  redraw_flag= TRUE;
}

diffeq_widget_rep::~diffeq_widget_rep () {}

diffeq_widget_rep::operator tree () {
  return "diffeq";
}

void
diffeq_widget_rep::handle_get_size (get_size_event ev) {
  ev->w= (WIDTH-1)*PIXEL;
  ev->h= (HEIGHT-1)*PIXEL;
}

/******************************************************************************
* Displaying the diffeq
******************************************************************************/

R
diffeq_widget_rep::f (R x, R y) {
  return exp(x)*y+ exp(x-x*x)*y*y;
}

void
diffeq_widget_rep::display_axes (repaint_event ev) {
  int i, j;
  SI  ox= -(((SI) (origin_x/si))/PIXEL)*PIXEL;
  SI  oy= -(((SI) (origin_y/si))/PIXEL)*PIXEL;
  int i1= (int) ((ev->x1-ox)/unit);
  int i2= (int) ((ev->x2-ox)/unit);
  int j1= (int) ((ev->y1-oy)/unit);
  int j2= (int) ((ev->y2-oy)/unit);
  win->set_color (dis->black);
  server_font fn= dis->server_ps_font ("lucidasans-12");
  win->line (ev->x1, oy, ev->x2, oy);
  for (i=i1-1; i<=i2+1; i++) {
    win->text (as_string (i* scale), fn,
	       ((int) (i*unit))+2*PIXEL+ ox, -fn->y1+ oy);
    win->line ((SI) (i*unit)+ ox, -2*PIXEL+ oy,
	       (SI) (i*unit)+ ox, 2*PIXEL+ oy);
  }
  win->line (ox, ev->y1, ox, ev->y2);
  for (j=j1-1; j<=j2+1; j++) {
    win->text (as_string (j* scale), fn,
	       2*PIXEL+ ox, ((SI) (j*unit))- fn->y1+ oy);
    win->line (-2*PIXEL+ ox, (SI) (j*unit)+ oy,
	       2*PIXEL+ ox, (SI) (j*unit)+ oy);
  }
}

void
diffeq_widget_rep::display_diffeq (repaint_event ev) {
  int i, j;
  SI  ll= (SI) (unit/(3*DENSITY));
  SI  ox= -(((SI) (origin_x/si))/PIXEL)*PIXEL;
  SI  oy= -(((SI) (origin_y/si))/PIXEL)*PIXEL;
  int i1= (int) (DENSITY*(ev->x1-ox)/unit);
  int i2= (int) (DENSITY*(ev->x2-ox)/unit);
  int j1= (int) (DENSITY*(ev->y1-oy)/unit);
  int j2= (int) (DENSITY*(ev->y2-oy)/unit);
  win->set_color (dis->dark_grey);
  for (i=i1-1; i<=i2+1; i++)
    for (j=j1-1; j<=j2+1; j++) {
      SI  x= (SI) (i*unit/DENSITY)+ ox;
      SI  y= (SI) (j*unit/DENSITY)+ oy;
      R  xx= x*si+origin_x;
      R  yy= y*si+origin_y;
      R  zz= f (xx, yy);
      R dxx= sqrt(1.0/(1.0+zz*zz));
      R dyy= zz*dxx;
      SI dx= ((SI) (dxx*ll));
      SI dy= ((SI) (dyy*ll));
      win->line (x-dx, y-dy, x+dx, y+dy);
    }
}

bool
diffeq_widget_rep::display_solution (repaint_event ev, R& x, R& y, R d) {
  R  x1= ev->x1*si+origin_x;
  R  x2= ev->x2*si+origin_x;
  R  y1= ev->y1*si+origin_y;
  R  y2= ev->y2*si+origin_y;
  if ((x<x1) || (x>x2) || (y<y1) || (y>y2)) return FALSE;
  SI X1= (SI) ((x-origin_x)/si);
  SI Y1= (SI) ((y-origin_y)/si);
  R  dx= d*scale;
  R  dy= f(x,y)*dx;
  x += dx;
  y += dy;
  SI X2= (SI) ((x-origin_x)/si);
  SI Y2= (SI) ((y-origin_y)/si);
  win->line (X1, Y1, X2, Y2);
  return TRUE;
}

void
diffeq_widget_rep::handle_repaint (repaint_event ev) {
  if (redraw_flag) {
    win->set_color (dis->white);
    win->fill (ev->x1, ev->y1, ev->x2, ev->y2);
    display_axes (ev);
    display_diffeq (ev);
    display_axes (ev);
  }
  else {
    R x, y;
    win->set_color (dis->red);
    x= point_x, y= point_y;
    while (display_solution (ev, x, y, 0.001));
    x= point_x, y= point_y;
    while (display_solution (ev, x, y, -0.001));
  }
}

/******************************************************************************
* Other routines
******************************************************************************/

void
diffeq_widget_rep::handle_keypress (keypress_event ev) {
  if (ev->key == "+") scale /= 2;
  if (ev->key == "-") scale *= 2;
  if (ev->key == "<page-up>") scale /= 10;
  if (ev->key == "<page-down>") scale *= 10;
  if (ev->key == "<right>") origin_x += scale;
  if (ev->key == "<left>") origin_x -= scale;
  if (ev->key == "<down>") origin_y -= scale;
  if (ev->key == "<up>") origin_y += scale;
  si= scale/unit;
  redraw_flag= TRUE;
  widget (this) << emit_invalidate_all ();
}

void
diffeq_widget_rep::handle_mouse (mouse_event ev) {
  if (ev->type == "press-left") {
    point_x= ev->x*si+origin_x;
    point_y= ev->y*si+origin_y;
    redraw_flag= FALSE;
    widget (this) << emit_invalidate_all ();
  }

  if (ev->type == "press-middle") {
    R x= ev->x*si+origin_x, y= ev->y*si+origin_y;
    string pos = "position: (" * as_string (x) * "," * as_string (y) * ")";
    string val = "value: " * as_string (f (x, y));
    footer << set_widget ("left", text_widget (dis, pos));
    footer << set_widget ("right", text_widget (dis, val));
    if (attached ()) footer << emit_update ();
  }
}

/******************************************************************************
* The diffeq widget with menus and so
******************************************************************************/

class redirect_widget_rep: public basic_widget_rep {
  widget_rep* kbd_focus;

public:
  redirect_widget_rep (widget w, widget_rep* kbd_focus);
  operator tree ();
  void handle_keypress (keypress_event ev);
  void handle_keyboard_focus (keyboard_focus_event ev);
};

redirect_widget_rep::redirect_widget_rep (widget w, widget_rep* kf):
  basic_widget_rep (w->dis, 1), kbd_focus (kf) { a[0]= w; }

redirect_widget_rep::operator tree () {
  return tree ("redirect keyboard", a[0], widget(kbd_focus)); }

void
redirect_widget_rep::handle_keypress (keypress_event ev) {
  kbd_focus << ev; }

void
redirect_widget_rep::handle_keyboard_focus (keyboard_focus_event ev) {
  kbd_focus << ev; }

widget
redirect_widget (widget w, widget_rep* kbd_focus) {
  return new redirect_widget_rep (w, kbd_focus);
}

widget
diffeq_widget (display dis) {
  array<widget> footer (3);
  array<string> footer_name (3);
  footer[0]= text_widget (dis, "Welcome to the diffeq shower");
  footer[1]= glue_widget (dis, TRUE, FALSE);
  footer[2]= text_widget (dis, "");
  footer_name[0]= "left";
  footer_name[1]= "middle";
  footer_name[2]= "right";
  widget ftr= horizontal_array (dis, footer, footer_name, 1);

  diffeq_widget_rep* fw =
    new diffeq_widget_rep (ftr);

  array<widget> diffeq_main (2);
  diffeq_main[0]= fw;
  diffeq_main[1]= ftr;

  return redirect_widget (vertical_list (dis, diffeq_main), fw);
}

/******************************************************************************
* The main program
******************************************************************************/

int
main (int argc, char** argv) {
  install_tex ();
  display dis= open_display ();
  window w1= plain_window (diffeq_widget (dis),
			   "differential equation shower");
  w1->map ();
  dis->event_loop ();
  close_display (dis);
  return 0;
}

#endmodule // code_diffeq
