
/******************************************************************************
* MODULE     : zealous_test.gen.cc
* DESCRIPTION: test for zealous power series
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <timer.gen.h>
#include <Arithmetic/rational.gen.h>
#include <Arithmetic/fp_real.gen.h>
#include <Univariate/Zealous/zealous.gen.cc>

#module code_zealous_test
#import timer
#import rational
#import fp_real

#define REPS         25
#define BITS         10000
#define CST          rational
#define SERIES       zealous<rational>
#define IMPL_SERIES  implicit_series<rational>
#define SET_TERMS    set_series<rational>_precision
#define MAX_TIME     3600000
#import code_zealous (rational)

void
FFT_test () {
  cout << "Test for FFT multiplication\n\n";
  int i, n= 2048;
  SERIES f(n), g(n);
  for (i=0; i<(n/2); i++) f[i]= g[i]= 1;
  cout << "f  = " << f << "\n";
  cout << "g  = " << g << "\n";
  cout << "f*g= " << cyclo_mul (f, g) << "\n";
}

void
test1 (int TERMS) {
  if (TERMS<4) return;
  int i;
  SERIES I (TERMS), z (TERMS), f (TERMS), g (TERMS);
  I[0]= z[1]= CST(1);
  for (i=0; i<TERMS; i++) f[i]= g[i]= 1;
  cout << "f= " << f << "\n";
  cout << "g= " << g << "\n";
  cout << "f+g= " << f+g << "\n";
  cout << "f-g= " << f-g << "\n";
  cout << "f*g= " << f*g << "\n";
  cout << "1/(1+z)= " << inv (I+z) << "\n";
  cout << "f/g= " << f/g << "\n\n";

  cout << "log(1+z)= " << log (I+z) << "\n";
  cout << "exp(z)= " << exp (z) << "\n\n";

  cout << "exp(z) o z^2= " << compose (exp (z), (z*z) (0, 3)) << "\n";
  cout << "z^3 o (z+z^2)= " << compose (z*z*z, (z+z*z) (0, 3)) << "\n";
  cout << "exp(z) o (z exp(z))= " << compose (exp(z), z*exp(z)) << "\n";
  cout << "exp(z exp(z))= " << exp(z*exp(z)) << "\n";
  cout << "revert(z exp (z))= " << revert (z*exp(z)) << "\n";
  cout << "(z exp (z)) o revert(z exp (z))= "
       << compose (z*exp(z), revert (z*exp(z))) << "\n";
}

void
test2 (int TERMS) {
  if (TERMS<4) return;
  SERIES I (TERMS), z (TERMS);
  I[0]= z[1]= 1;
  SERIES f= exp(z*exp(z));
  (void) f;
}

int
bench (void (*f) (int), int TERMS) {
  int i;
  timer t;
  t->start ();
  for (i=0; i< REPS; i++) f (TERMS);
  int millisec= t->stop ()/REPS;
  cout << "Terms= " << TERMS << ": \t" << millisec << " ms\n";
  return millisec;
}

void
bench (void (*f) (int)) {
  if (bench (f, 1) > MAX_TIME) return;
  if (bench (f, 2) > MAX_TIME) return;
  if (bench (f, 5) > MAX_TIME) return;
  if (bench (f, 10) > MAX_TIME) return;
  if (bench (f, 20) > MAX_TIME) return;
  if (bench (f, 50) > MAX_TIME) return;
  if (bench (f, 100) > MAX_TIME) return;
  if (bench (f, 200) > MAX_TIME) return;
  if (bench (f, 500) > MAX_TIME) return;
  if (bench (f, 1000) > MAX_TIME) return;
  // if (bench (f, 2000) > MAX_TIME) return;
  if (bench (f, 2010) > MAX_TIME) return;
  if (bench (f, 2020) > MAX_TIME) return;
  if (bench (f, 2100) > MAX_TIME) return;
  if (bench (f, 2150) > MAX_TIME) return;
  if (bench (f, 5000) > MAX_TIME) return;
  if (bench (f, 10000) > MAX_TIME) return;
  if (bench (f, 20000) > MAX_TIME) return;
  if (bench (f, 50000) > MAX_TIME) return;
  if (bench (f, 100000) > MAX_TIME) return;
}

int
main () {
  cout << "Testing zealous power series\n";
  cout << " | rational coefficients\n";
  cout << " | exp(z exp(z))\n\n";
  set_fp_real_precision (BITS);
  bench (test2);
  return 0;
}

#endmodule // zealous_test
