
/******************************************************************************
* MODULE     : multiply.gen.cc
* DESCRIPTION: truncated multiplication of relaxed power series
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#module code_truncated_multiply (C)

/******************************************************************************
* Naive multiplication
******************************************************************************/

struct product1_series_rep<C>: public series_rep<C> {
  series<C> f, g;
  product1_series_rep<C> (series<C> f2, series<C> g2):
    series_rep<C> (RELAXED), f(f2), g(g2) {}
  C next () {
    // cout << "Coefficient " << n << "\n";
    int i;
    C c (ZERO<C>);
    for (i=0; i<=n; i++) c += f[i]*g[n-i];
    // cout << "  -> " << c << "\n";
    return c;
  }
  void set_precision (int terms) {
    if (N(value)<terms) value= value (0, terms);
    f->set_precision (terms);
    g->set_precision (terms);
  }
  operator tree () { return tree ("*", f, g); }
};

/******************************************************************************
* Fast truncated relaxed multiplication
******************************************************************************/

struct product3_series_rep<C>: public series_rep<C> {
  series<C> f, g;
  product3_series_rep<C> (series<C> f2, series<C> g2):
    series_rep<C> (RELAXED), f(f2), g(g2) {}
  C next ();
  void set_precision (int terms) {
    if (N(value)<terms) value= value (0, terms);
    f->set_precision (terms);
    g->set_precision (terms);
  }
  operator tree () { return tree ("*", f, g); }
};

C
product3_series_rep<C>::next () {
  int k= (n+2), p= 0;
  value[n] += f[0]*g[n];
  if (k==2) return value[n];
  value[n] += f[n]*g[0];

  if ((n<(N(value)-4)) && ((k&1)==0)) {
    k= k>>1; p++;
    value[(k<<1)-2] += f[1]*g[(k<<1)-3];
    value[(k<<1)-1] += f[2]*g[(k<<1)-3]+
                       f[1]*g[(k<<1)-2];
    value[(k<<1)  ] += f[2]*g[(k<<1)-2];
    if (k==2) return value[n];
    value[(k<<1)-2] += g[1]*f[(k<<1)-3];
    value[(k<<1)-1] += g[2]*f[(k<<1)-3]+
                       g[1]*f[(k<<1)-2];
    value[(k<<1)  ] += g[2]*f[(k<<1)-2];
  }

  while ((k&1)==0) {
    k= k>>1; p++;
    value->accumulate (multiply (f ((1<<p)-1, (2<<p)-1),
				 g (((k-1)<<p)-1, (k<<p)-1),
				 N(value)- ((k<<p)-2)),
		       (k<<p)-2);
    if (k==2) return value[n];
    value->accumulate (multiply (f (((k-1)<<p)-1, (k<<p)-1),
				 g ((1<<p)-1, (2<<p)-1),
				 N(value)- ((k<<p)-2)),
		       (k<<p)-2);
  }
  return value[n];
}

/******************************************************************************
* interface
******************************************************************************/

series<C>
operator * (series<C> f, series<C> g) {
  return new product3_series_rep<C> (f, g);
}

#endmodule // code_truncated_multiply (C)
