
/******************************************************************************
* MODULE     : zealous.gen.h
* DESCRIPTION: zealous power series
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <basic.gen.h>
#include <category.gen.h>

#module zealous (C)
#import basic
#import category
#define test<C>
#ifdef test<double>
#define ZERO<C> 0.0
#define ONE<C> 1.0
#else
#define ZERO<C> 0
#define ONE<C> 1
#endif

#ifdef has_divisible<C>
#define has_divisible<zealous<C>>
#define has_special_functions<zealous<C>>
#endif

class zealous_rep<C>: concrete_struct {
  C*  a;   // the elements
  int n;   // the length of the truncated power series

  inline zealous_rep<C> (int n2):
    a (n2==0? (C*) NULL: new C[n2](ZERO<C>)), n (n2) {}
  inline ~zealous_rep<C> () { if (n>0) delete[] a; }

public:
  friend class zealous<C>;
  void   accumulate (zealous<C> f, int at=0);
  friend int N (zealous<C> f);
  friend zealous<C> multiply (zealous<C> f, zealous<C> g);
  friend zealous<C> multiply (zealous<C> f, zealous<C> g, int order);
  friend zealous<C> operator * (zealous<C> f, zealous<C> g);
};

class zealous<C> {
#import concrete (zealous<C>, zealous_rep<C>)
  inline zealous<C> () { rep= new zealous_rep<C> (0); }
  inline zealous<C> (int n) { rep= new zealous_rep<C> (n); }
  inline friend int N (zealous<C> f) { return f->n; }
  inline C& operator [] (int i) { return rep->a[i]; }
  zealous<C> operator () (int start, int end);
};
#import code_concrete (zealous<C>, zealous_rep<C>)

zealous<C> constant    (C c, int k);
zealous<C> id          (int n);
zealous<C> operator << (zealous<C> f, int k);
zealous<C> operator >> (zealous<C> f, int k);

zealous<C> operator + (zealous<C> f, zealous<C> g);
zealous<C> operator - (zealous<C> f, zealous<C> g);
zealous<C> operator * (C c, zealous<C> f);
zealous<C> multiply   (zealous<C> f, zealous<C> g);
zealous<C> multiply   (zealous<C> f, zealous<C> g, int order);
zealous<C> operator * (zealous<C> f, zealous<C> g);
zealous<C> inv        (zealous<C> f);
zealous<C> operator / (zealous<C> f, zealous<C> g);

#ifdef has_divisible<C>
zealous<C> operator / (zealous<C> f, C c);
zealous<C> d   (zealous<C> f);
zealous<C> I   (zealous<C> f);
zealous<C> log (zealous<C> f);
zealous<C> exp (zealous<C> f);
#endif

zealous<C> compose (zealous<C> f, zealous<C> g);
zealous<C> reverse (zealous<C> f);

ostream& operator << (ostream& out, zealous<C> f);

#undef test<C>
#endmodule // zealous (C)
