
/******************************************************************************
* MODULE     : grm_test.gen.cc
* DESCRIPTION: Test program for interactive grammars
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <lexer.gen.h>
#include <parser.gen.h>
#include <tiger.gen.h>
#include <dir.gen.h>
#include <window.gen.h>

#module code_grm_test
#import lexer
#import parser
#import tiger
#import dir
#import window

program lisp_prg_to_program (string s, location root);

string
input () {
  int i;
  char in[1000];
  cout << "] ";
  for (i=0; i<999; i++) {
    cin.get(in[i]);
    if (in[i]=='\n') break;
  }
  in[i]='\0';
  return in;
}

void
output (program p, string s) {
  cout << p << " -> ";
  if ((p->start<0) || p->end>N(s)) cout << "?\n";
  else cout << s (p->start, p->end) << "\n";
  if (p->status==COMPOSITE) {
    int i;
    for (i=0; i<N(p); i++)
      output (p[i], s);
  }
}

int
main (int argv, char** argc) {
  if (argv != 3) {
    cerr << "usage: grm_test -g <name>\n";
    cerr << "     : grm_test -t <name>\n";
    return 1;
  }

  debug (0, TRUE);
  install_texmacs ();

  string option= argc[1];
  string name  = argc[2];
  lexer  lx    = load_lexer (name);

  if (option == "-g") {
    parser pa= load_parser (name);
    string in;
    while (TRUE) {
      in= input ();
      if (in == "quit") break;
      lx->set_input (in, 0);
      pa->set_input (lx);
      program p= pa->parse ();
      cout << "> " << p << "\n";
      // output (p, in);
    }
  }

  if (option == "-t") {
    tiger  tg= load_tiger (name);
    string in;
    while (TRUE) {
      in= input ();
      if (in == "quit") break;
      tg->set_lexer (lx);
      program p= lisp_expr_to_program (in, 0);
      string  s= tg->convert (p);
      if (s != "") cout << "\n" << s << "\n\n";
    }
  }

  return 0;
}

#endmodule // grm_test
