
/******************************************************************************
* MODULE     : replace.gen.cc
* DESCRIPTION: query replace on several files
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <string.gen.h>
#include <file.gen.h>
#include <dir.gen.h>

#module code_replace
#import string
#import file
#import dir

string what, by;
bool   diagnosis = FALSE;
bool   kill_flag = FALSE;

string notice1= "* BY         : Joris van der Hoeven
* COPYRIGHT  : This software falls under the GNU general public licence;
*              see the file 'LICENCE', which is provided with this package.";

string notice2= "* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.";

string notice3= "* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.";

/******************************************************************************
* Replace
******************************************************************************/

void
replace (string file_name) {
  if (diagnosis) cout << "replace] " << file_name;
  else cerr << "replace] " << file_name;
  if (what == "") {
    cerr << "\nreplace] error: no action specified\n";
    return;
  }
  string s;
  if (load_string (file_name, s)) {
    cerr << "\nreplace] error: file did not open\n";
    return;
  }

  int match= 0;
  int i, j;
  string r;
  for (i=0; i<N(s); i++) {
    bool flag= (i<(N(s)-N(what)+1));
    if (flag)
      for (j=0; j<N(what); j++)
	if (s[i+j] != what[j]) {
	  flag= FALSE;
	  break;
	}
    if (flag) {
      match++;
      r << by;
      i += N(what)-1;
      if (kill_flag) while ((i < N(s)) && (s[i] != '\n')) i++;
    }
    else r << s[i];
  }
  if (diagnosis) {
    if (match==0) cout << ": no matches\n";
    else if (match==1) cout << ": " << match << " match\n";
    else cout << ": " << match << " matches\n";
  }
  else {
    if (match==0) cerr << ": no matches\n";
    else if (match==1) cerr << ": " << match << " match\n";
    else cerr << ": " << match << " matches\n";
    if (match>0)
      if (save_string (file_name, r))
	cerr << "replace] error: file did not open for write\n";
  }
}

void
replace (tree t, string prefix="") {
  if (N(t) == 0) {
    string s= t->label;
    int    n= N(s);
    if ((n>2) && (s(n-2,n) == ".c")) replace (prefix * s);
    if ((n>2) && (s(n-2,n) == ".h")) replace (prefix * s);
    if ((n>3) && (s(n-3,n) == ".cc")) replace (prefix * s);
  }
  else {
    int i;
    for (i=0; i<N(t); i++) replace (t[i], prefix * t->label * "/");
  }
}

/******************************************************************************
* The main program
******************************************************************************/

int
main (int argc, char** argv) {
  if (argc<2) {
    cout << "Options for replace :\n";
    cout << "  -action <what> <by>    replace <what> <by>\n";
    cout << "  -copyright             replace old copyright notice\n";
    cout << "  -version <what> <by>   modify version\n";
    cout << "  -in <files>            specify the files\n";
    cout << "  -all                   recursively, in all c and c++ files\n";
    cout << "  -diagnosis             only display a diagnosis\n";
    return 0;
  }

  int i;
  for (i=1; i<argc; i++) {
    string s= argv[i];
    if ((s == "-action") && ((i+2) < argc)) {
      what      = argv[++i];
      by        = argv[++i];
      kill_flag = FALSE;
      continue;
    }
    if (s == "-copyright") {
      what      = notice2;
      by        = notice3;
      kill_flag = FALSE;
      continue;
    }
    if ((s == "-version") && ((i+2) < argc)) {
      what      = argv[++i];
      by        = argv[++i];
      kill_flag = FALSE;
      replace (string ("~/project/Makefile"));
      replace (string ("~/project/SETUP"));
      replace (string ("~/project/TeXmacs/doc/html/Web/Download.html"));
      replace (string ("~/project/src/Basic/dir.gen.cc"));
      replace (string ("~/project/src/Window/PsDevice/printer.gen.cc"));
      replace (string ("~/project/src/Convert/Connect/connect.gen.cc"));
      replace (string ("~/project/src/Edit/edit_base.gen.h"));
      continue;
    }
    if (s == "-in") {
      while (((i+1) < argc) && (argv[i+1][0] != '-'))
	replace (string (argv[++i]));
      continue;
    }
    if (s == "-all") {
      bool flag;
      tree t= directory_tree (".", flag);
      if (flag) cerr << "replace] error: directory could not be opened\n";
      replace (t);
      continue;
    }
    if (s == "-diagnosis") {
      diagnosis= TRUE;
      continue;
    }
    cerr << "replace] error: unknown option " << s << "\n";
  }
}

#endmodule // code_xtexfont
