
/******************************************************************************
* MODULE     : ps_test.gen.cc
* DESCRIPTION: Display tex fonts on screen
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <dir.gen.h>
#include <window.gen.h>
#include <Widget/basic_widget.gen.h>

#module code_ps_test
#import dir
#import window
#import basic_widget

/******************************************************************************
* The widget
******************************************************************************/

class font_widget_rep: public basic_widget_rep {
  int         sfactor;
  int         pixel;
  server_font text_fn;
  server_font fn;

  int         fn_height;
  int         fn_depth;

public:
  font_widget_rep (display dis, int sf, server_font text_fn, server_font fn);
  operator tree ();
  void handle_get_size (get_size_event ev);
  void handle_repaint (repaint_event ev);
};

/******************************************************************************
* Setup game
******************************************************************************/

font_widget_rep::font_widget_rep (display dis, int sfactor2,
				  server_font f1, server_font f2):
  basic_widget_rep (dis), sfactor (sfactor2), pixel (sfactor2*PIXEL),
  text_fn (f1), fn (f2)
{
  SI x1, y1, x2, y2, x3, y3, x4, y4;
  text_fn->get_extents ("f", x1, y1, x2, y2, x3, y3, x4, y4);
  fn_height= y2-y1;
  fn_depth = -y1;
}

font_widget_rep::operator tree () {
  return "ps_test";
}

void
font_widget_rep::handle_get_size (get_size_event ev) {
  ev->w= 400*PIXEL;
  ev->h= 300*PIXEL;
}

/******************************************************************************
* Repainting
******************************************************************************/

void
font_widget_rep::handle_repaint (repaint_event ev) {
  win->set_background (dis->white);
  win->clear (ev->x1, ev->y1, ev->x2, ev->y2);
  win->set_color (dis->black);
  win->set_shrinking_factor (sfactor);

  int y, y1, y2;
  y1= ((ev->y1*sfactor)/fn_height) - 1;
  y2= ((ev->y2*sfactor)/fn_height) + 1;
  for (y=y1; y<=y2; y++) {
    int ypos= y*fn_height+ fn_depth;
    win->var_text ("This is a test to see how fast a TeX font is displayed on the screen. We can not expect TeXmacs to scroll faster than this window.", fn, 0, ypos);
  }

  win->set_shrinking_factor (1);
}

/******************************************************************************
* The main program
******************************************************************************/

int
main (int argc, char** argv) {
  if ((argc<3) || (argc>6)) {
    cout << "Usage: ps_test <family> <size>\n";
    cout << "       ps_test <family> <size> <dpi>\n";
    cout << "       ps_test <family> <size> <dpi> <sfactor>\n";
    return 0;
  }
  string name= argv[1];
  int size   = as_int (string (argv[2]));
  int dpi    = argc>3? as_int (string (argv[3])): 600;
  int sfactor= argc>4? as_int (string (argv[4])): 5;

  install_tex ();
  display dis= open_display (argc, argv);
  server_font text_fn= dis->server_tex_font ("cmr", size, dpi, 10);
  server_font fn     = dis->server_ps_font (name, size, dpi);
  widget wid= canvas_widget (new font_widget_rep (dis, sfactor, text_fn, fn));
  wid << set_extents (0, -3000*PIXEL, 800*PIXEL, 0);
  window win= plain_window (wid, "ps_test");
  win->map ();
  dis->event_loop ();
  close_display (dis);
  return 0;
}

#endmodule // code_ps_test
