
/******************************************************************************
* MODULE     : command.gen.cc
* DESCRIPTION: Abstract dynamic commands
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <command.gen.h>

#module code_command
#import command

/******************************************************************************
* standard commands without arguments
******************************************************************************/

class std_command_rep: public command_rep {
  void (*routine) (void);
public:
  std_command_rep (void (*routine) (void));
  void apply ();
};

std_command_rep::std_command_rep (void (*routine2) (void)):
  routine (routine2) {}
void std_command_rep::apply () { routine (); }

command::command (void (*routine) (void)):
  rep (new std_command_rep (routine)) { rep->ref_count++; }

/******************************************************************************
* standard commands with string arguments
******************************************************************************/

class string_command_rep: public command_rep {
  void (*routine) (string s);
  string s;
public:
  string_command_rep (void (*routine) (string), string s);
  void apply ();
};

string_command_rep::string_command_rep (void (*routine2) (string), string s2):
  routine (routine2), s (s2) {}
void string_command_rep::apply () { routine (s); }

command::command (void (*routine) (string), string s):
  rep (new string_command_rep (routine, s)) { rep->ref_count++; }

/******************************************************************************
* standard commands with tree arguments
******************************************************************************/

class tree_command_rep: public command_rep {
  void (*routine) (tree);
  tree t;
public:
  tree_command_rep (void (*routine) (tree), tree t);
  void apply ();
};

tree_command_rep::tree_command_rep (void (*routine2) (tree), tree t2):
  routine (routine2), t (t2) {}
void tree_command_rep::apply () { routine (t); }

command::command (void (*routine) (tree), tree t):
  rep (new tree_command_rep (routine, t)) { rep->ref_count++; }

#endmodule // code_command
