
/******************************************************************************
* MODULE     : file.gen.cc
* DESCRIPTION: file handling
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <file.gen.h>
#include <dir.gen.h>
#include <analyze.gen.h>
#include <hashmap.gen.h>

#module code_file
#import file
#import dir
#import analyze
#import hashmap (string, string)
#include <iostream.h>

string var_eval_system (string s);

/*****************************************************************************/
// Decode file name
/*****************************************************************************/

string
decode_file_name (string s) {
  int i;
  string r(0);
  for (i=0; i<N(s); ) {
    if ((s[i]=='~') && ((i==0) || (s[i-1]==':'))) {
      r << get_env ("HOME");
      i++;
      continue;
    }
    if (s[i]=='$') {
      int start= ++i;
      while ((i<N(s)) && (s[i]!='/') && (s[i]!=':') &&
             (s[i]!='.') && (s[i]!='$')) i++;
      r << get_env (s (start, i));
      continue;
    }
    r << s[i++];
  }
  return r;
}

/*****************************************************************************/
// Constructors and destructors
/*****************************************************************************/

static string
un_point (string s) {
  int i;
  string r;
  for (i=0; i<N(s); )
    if ((i<(N(s)-2)) && (s[i]=='/') && (s[i+1]=='.') && (s[i+2]=='/')) i += 2;
    else r << s[i++];
  return r;
}

file_rep::file_rep (string name2, char* rw_flag): dir (""), name (name2) {
  string s= decode_file_name (name);
  char* _s= as_charp (s);
  fp= fopen (_s, rw_flag);
  delete[] _s;
  if (is_open ()) {
    if ((N(s)!=0) && (s[0]!='/'))
      s= var_eval_system ("pwd") * "/" * s;
    os_name= un_point (s);
  }
}

file_rep::file_rep (string dir2, string name2, char* rw_flag):
  dir (dir2), name (name2)
{
  string Dir = decode_file_name (dir);
  string Name= decode_file_name (name);
  int i=0;

  do {
    int start= i;
    while ((i<N(Dir)) && (Dir[i]!=':')) i++;
    string s;
    string ss= Dir (start, i);
    if ((N(Name)>0) && (Name[0]=='/')) s= Name;
    else if (N(ss)==0) s= Name;
    else if ((N(ss)>0) && (ss[N(ss)-1]=='/')) s= ss * Name;
    else s= ss * "/" * Name;
    char* _s= as_charp (s);
    fp= fopen (_s, rw_flag);
    delete[] _s;
    if (is_open ()) {
      if ((N(s)!=0) && (s[0]!='/'))
	s= var_eval_system ("pwd") * "/" * s;
      os_name= un_point (s);
      break;
    }
    if (i<N(Dir)) i++;
  } while (i<N(Dir));
}

file_rep::~file_rep () {
  if (fp!=NULL) fclose (fp);
}

bool
file_rep::is_open () {
  return (fp!=NULL);
}

void
file_rep::check_open (char* routine, char* in) {
  if (!is_open ()) {
    cerr << "\nFatal error: file " << name << " not found";
    if (routine[0]!='\0') cerr << " in " << routine;
    cerr << "\n";
    if (N(dir)>0) cerr << "           : directory was " << dir << "\n";
    if (in[0]!='\0') cerr << "See file   : " << in << "\n";
    exit (1);
  }
}

file::file (string name, char* rw_flag) {
  rep= new file_rep (name, rw_flag);
}

file::file (string dir, string name, char* rw_flag) {
  rep= new file_rep (dir, name, rw_flag);
}

/*****************************************************************************/
// Routines for manipulating files
/*****************************************************************************/

bool
eof (file f) {
  return feof (f->fp);
}

file&
operator >> (file& f, QI& c) {
  c= getc (f->fp);
  return f;
}

file&
operator >> (file& f, QN& c) {
  c= getc (f->fp);
  return f;
}

file&
operator >> (file& f, HI& c) {
  QI c1= getc (f->fp);
  QN c2= getc (f->fp);
  c= (((HI) c1)<<8)+ c2;
  return f;
}

file&
operator >> (file& f, HN& c) {
  QN c1= fgetc (f->fp);
  QN c2= fgetc (f->fp);
  c= (((HN) c1)<<8)+ c2;
  return f;
}

file&
operator >> (file& f, SI& c) {
  QI c1= getc (f->fp);
  QN c2= getc (f->fp);
  QN c3= getc (f->fp);
  QN c4= getc (f->fp);
  c= (((((((SI) c1)<<8)+ ((SI) c2))<<8)+ ((SI) c3))<<8)+ c4;
  return f;
}

file&
operator >> (file& f, string& s) {
  s= string (0);
  while (!feof (f->fp)) {
    char c= getc (f->fp);
    if ((c==((char) -1)) && (feof (f->fp))) break;
    s << c;
  }
  return f;
}

file&
operator << (file& f, string s) {
  int i;
  for (i=0; i<N(s); i++) fputc (s[i], f->fp);
  return f;
}

/*****************************************************************************/
// Loading or saving an entire file
/*****************************************************************************/

bool
load_string (string file_name, string& s, bool fatal) {
  file fin (file_name, "r");
  if (fatal) fin->check_open ("load_string", "file.gen.cc");
  else if (!fin->is_open()) return TRUE;
  fin >> s;
  return FALSE;
}

bool
load_string (string dir, string file_name, string& s, bool fatal) {
  file fin (dir, file_name, "r");
  if (fatal) fin->check_open ("load_string", "file.gen.cc");
  else if (!fin->is_open()) return TRUE;
  fin >> s;
  return FALSE;
}

bool
save_string (string file_name, string s, bool fatal) {
  file fout (file_name, "w");
  if (fatal) fout->check_open ("save_string", "file.gen.cc");
  else if (!fout->is_open()) return TRUE;
  fout << s;
  return FALSE;
}

bool
save_string (string dir, string file_name, string s, bool fatal) {
  file fout (dir, file_name, "w");
  if (fatal) fout->check_open ("save_string", "file.gen.cc");
  else if (!fout->is_open()) return TRUE;
  fout << s;
  return FALSE;
}

/*****************************************************************************/
// Loading and saving trees
/*****************************************************************************/

static tree
string_to_tree (string s, int& pos) {
  string l("");
  while ((pos<N(s)) && (s[pos]!='(') && (s[pos]!=',') && (s[pos]!=')')) {
    if ((s[pos]=='\\') && (pos<N(s)-1)) pos++;
    l << s[pos++];
  }
  tree t(l);
  if ((pos<N(s)) && (s[pos]=='(')) {
    do {
      pos++;
      t << string_to_tree (s, pos);
    } while ((pos<N(s)) && (s[pos]==','));
    if ((pos<N(s)) && (s[pos]==')')) pos++;
  }
  return t;
}

static tree
string_to_tree (string s) {
  int pos=0;
  return string_to_tree (s, pos);
}

bool
load_tree (string file_name, tree& t, bool fatal) {
  string s;
  bool flag= load_string (file_name, s, fatal);
  t= string_to_tree (s);
  return flag;
}

bool
load_tree (string dir, string file_name, tree& t, bool fatal) {
  string s;
  bool flag= load_string (dir, file_name, s, fatal);
  t= string_to_tree (s);
  return flag;
}

string
tree_to_string (tree t) {
  int i;
  string s (""), l(t->label);
  for (i=0; i<N(l); i++) {
    if ((l[i]=='(') || (l[i]==',') || (l[i]==')') || (l[i]=='\\')) s << '\\';
    s << l[i];
  }
  if (N(t)!=0) {
    s << '(';
    for (i=0; i<N(t); i++) {
      s << tree_to_string (t[i]);
      if (i<N(t)-1) s << ',';
    }
    s << ')';
  }
  return s;
}

bool
save_tree (string file_name, tree t, bool fatal) {
  return save_string (file_name, tree_to_string (t), fatal);
}

bool
save_tree (string dir, string file_name, tree t, bool fatal) {
  return save_string (dir, file_name, tree_to_string (t), fatal);
}

/******************************************************************************
* Loading postscript files or conversion to postscript
******************************************************************************/

static string
load_postscript (string file_name) {
  string s;
  load_string (file_name, s);
  return s;
}

static string
load_tiff (string file_name) {
  return eval_system ("tiff2ps " * file_name);
}

static string
load_pdf (string file_name) {
  string temp_name= get_temp_file_name ();
  system ("pdf2ps " * file_name * " " * temp_name);
  string s= load_postscript (temp_name);
  system ("rm -f " * temp_name);
  return s;
}

static string
load_pnm (string file_name) {
  return eval_system ("pnmtops " * file_name);
}

static string
load_gif (string file_name) {
  string temp_name= get_temp_file_name ();
  system ("giftopnm " * file_name * " | cat > " * temp_name);
  string s= load_pnm (temp_name);
  system ("rm -f " * temp_name);
  return s;
}

static string
load_ppm (string file_name) {
  string temp_name= get_temp_file_name ();
  system ("ppmtogif " * file_name * " > " * temp_name);
  string s= load_gif (temp_name);
  system ("rm -f " * temp_name);
  return s;
}

static string
load_xpm (string file_name) {
  string temp_name= get_temp_file_name ();
  system ("xpmtoppm " * file_name * " > " * temp_name);
  string s= load_ppm (temp_name);
  system ("rm -f " * temp_name);
  return s;
}

string
ps_load (string file_name) {
  int i;
  string radical;
  for (i=N(file_name); i>0; i--)
    if (file_name[i-1]=='.') break;
  if (i==0) return "";
  radical= file_name (i, N (file_name));

  if (radical == "ps") return load_postscript (file_name);
  else if (radical == "eps") return load_postscript (file_name);
  else if (radical == "tif") return load_tiff (file_name);
  else if (radical == "pdf") return load_pdf (file_name);
  else if (radical == "pnm") return load_pnm (file_name);
  else if (radical == "gif") return load_gif (file_name);
  else if (radical == "ppm") return load_ppm (file_name);
  else if (radical == "xpm") return load_xpm (file_name);
  else return "";
}

static hashmap<string,string> bounding_box ("");

void
ps_bounding_box (string file_name, int& x1, int& y1, int& x2, int& y2) {
  if (!bounding_box->contains (file_name)) {
    int i;
    string s= ps_load (file_name);
    string r= "0 0 596 842";
    for (i=0; i<N(s); i++)
      if (read (s, i, "%%BoundingBox: ")) {
	read_line (s, i, r);
	break;
      }
    bounding_box (file_name)= r;
  }

  int i= 0;
  bool ok;
  string s= bounding_box[file_name];
  skip_spaces (s, i);
  ok= read_int (s, i, x1);
  skip_spaces (s, i);
  ok= read_int (s, i, y1) && ok;
  skip_spaces (s, i);
  ok= read_int (s, i, x2) && ok;
  skip_spaces (s, i);
  ok= read_int (s, i, y2) && ok;
  if (ok) return;
  x1= y1= 0; x2= 596; y2= 842;
}

/******************************************************************************
* Other useful functions for files
******************************************************************************/

bool
file_exists (string file_name) {
  file fin (file_name, "r");
  return fin->is_open();
}

bool
file_exists (string dir, string file_name) {
  file fin (dir, file_name, "r");
  return fin->is_open();
}

string
get_unique_file_name (string dir, string file_name, bool fatal) {
  file fin (dir, file_name, "r");
  if (fatal) fin->check_open ("get_unique_file_name", "file.gen.cc");
  else if (!fin->is_open ()) return file_name;
  return fin->os_name;
}

string
get_temp_file_name () {
  char buffer [L_tmpnam+1];
  if (tmpnam (buffer) == NULL)
    fatal_error ("Couldn't create a temporary file name",
		 "get_temp_file_name", "file.gen.cc");
  return string (buffer);
}

string
get_radical_file_name (string base) {
  int i;
  for (i=N(base)-1; i>=0; i--)
    if (base[i]=='/') return base (0, i);
  return base;
}

string
get_relative_file_name (string base, string name, bool fatal) {
  string r;
  if ((N(name)>0) && ((name[0]=='/') || (name[0]=='$') || (name[0]=='~')))
    r= name;
  else r= get_radical_file_name (base) * "/" * name;
  return get_unique_file_name (".", r, fatal);
}

/******************************************************************************
* System functions
******************************************************************************/

int
system (string s) {
  char* _s= as_charp (s);
  int r= system (_s);
  delete[] _s;
  return r;
}

string
eval_system (string s) {
  string temp= get_temp_file_name ();
  system (s * " > " * temp);
  string result;
  bool flag= load_string (temp, result, FALSE);
  system ("rm " * temp);
  if (flag) return "";
  return result;
}

string
var_eval_system (string s) {
  string r= eval_system (s);
  while ((N(r)>0) && (r[N(r)-1]=='\n')) r= r (0,N(r)-1);
  return r;
}

string
get_env (string var) {
  char* _var= as_charp (var);
  char* _ret= getenv (_var);
  delete[] _var;
  if (_ret==NULL) return "";
  string ret (_ret);
  return ret;
  // do not delete s !
}

void
set_env (string var, string with) {
#os sun
  char* _varw= as_charp (var * "=" * with);
  (void) putenv (_varw);
#os linux
  char* _var = as_charp (var);
  char* _with= as_charp (with);
  setenv (_var, _with, 1);
#os all
  // do not delete s and r !!!
}

#endmodule // code_file
