
/******************************************************************************
* MODULE     : tree.gen.cc
* DESCRIPTION: fixed size trees with reference counting
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <tree.gen.h>
#include <array.gen.cc>

#module code_tree (T)
#import tree (T)
#import code_array (tree<T>)
#define test<T>

#ifdef test<string>
tree<T>::tree<T> (char c):
  rep (new tree_rep<T>(string(c), array<tree<T>>())) {}

tree<T>::tree<T> (int i):
  rep (new tree_rep<T>(as_string(i), array<tree<T>>())) {}

tree<T>::tree<T> (double x):
  rep (new tree_rep<T>(as_string(x), array<tree<T>>())) {}
#endif

tree<T>::tree<T> (T l, tree<T> t1):
  rep (new tree_rep<T>(l, array<tree<T>>(1))) {
  rep->a[0]=t1;
}

tree<T>::tree<T> (T l, tree<T> t1, tree<T> t2):
  rep (new tree_rep<T>(l, array<tree<T>>(2))) {
  rep->a[0]=t1;
  rep->a[1]=t2;
}

tree<T>::tree<T> (T l, tree<T> t1, tree<T> t2, tree<T> t3):
  rep (new tree_rep<T>(l, array<tree<T>>(3))) {
  rep->a[0]=t1;
  rep->a[1]=t2;
  rep->a[2]=t3;
}

tree<T>::tree<T> (T l, tree<T> t1, tree<T> t2, tree<T> t3, tree<T> t4):
  rep (new tree_rep<T>(l, array<tree<T>>(4))) {
  rep->a[0]=t1;
  rep->a[1]=t2;
  rep->a[2]=t3;
  rep->a[3]=t4;
}

void
tree<T>::resize (int n2) {
  rep->a.resize(n2);
}

tree<T>
tree::operator () (int begin, int end) {
  int i;
  tree<T> r (rep->label, end-begin);
  for (i=begin; i<end; i++) r[i-begin]= rep->a[i];
  return r;
}

bool
operator == (tree<T> t, tree<T> u) {
  return (t->label==u->label) && (t->a==u->a);
}

bool
operator != (tree<T> t, tree<T> u) {
  return (t->label!=u->label) || (t->a!=u->a);
}

tree<T>
copy (tree<T> t) {
  int i, n=N(t);
  tree<T> t2 (copy (t->label), n);
  for (i=0; i<n; i++) t2[i]= copy (t[i]);
  return t2;
}

tree<T>&
operator << (tree<T>& t, tree<T> t2) {
  t->a << t2;
  return t;
}

tree<T>&
operator << (tree<T>& t, array<tree<T>> a) {
  t->a << a;
  return t;
}

ostream&
operator << (ostream& out, tree<T> t) {
  int i;
  
  out << t->label;
  if (N(t)==0) return out;
  out << " (";
  for (i=0; i< N(t)-1; i++)
    out << t[i] << ", ";
  out << t[i] << ")";
  return out;
}

#undef test<T>
#endmodule // code_tree (T)

#module code_tree
#import string

struct tree<string>;
struct array<tree<string>>;
typedef tree<string> tree;
typedef array<tree<string>> array<tree>;
#import code_tree (string)

#endmodule // code_tree
