
/******************************************************************************
* MODULE     : tree.gen.h
* DESCRIPTION: fixed size trees with reference counting
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <basic.gen.h>
#include <string.gen.h>
#include <array.gen.h>

#module tree (T)
#define test<T>
#import basic

/******************************************************************************
* The tree class 'tree<T>'
******************************************************************************/

class tree_rep<T>;
class tree<T> {
#import concrete (tree<T>, tree_rep<T>)
  inline tree<T> ();
  inline tree<T> (T l, int n=0);
#ifdef test<string>
  inline tree<T> (char* l);
  tree<T> (char c);
  tree<T> (int i);
  tree<T> (double x);
#endif
  tree<T> (T l, tree<T> t1);
  tree<T> (T l, tree<T> t1, tree<T> t2);
  tree<T> (T l, tree<T> t1, tree<T> t2, tree<T> t3);
  tree<T> (T l, tree<T> t1, tree<T> t2, tree<T> t3, tree<T> t4);
  inline tree<T>& operator [] (int i);
  void resize (int n2);
  tree<T> operator () (int start, int end);
};

#import array (tree<T>)
class tree_rep<T>: concrete_struct {
public:
  T label;
  array<tree<T>> a;
  inline tree_rep<T> (T l, array<tree<T>> a2);
  friend class tree<T>;
};

/******************************************************************************
* Routines for trees
******************************************************************************/

#import code_concrete (tree<T>, tree_rep<T>)
inline tree<T>::tree<T> ():
  rep (new tree_rep<T>(string (), array<tree<T>>())) {}
inline tree_rep<T>::tree_rep<T> (T l, array<tree<T>> a2):
  label(l), a(a2) {}
inline tree<T>::tree<T> (T l, int n):
  rep (new tree_rep<T>(l, array<tree<T>>(n))) {}
#ifdef test<string>
inline tree<T>::tree<T> (char* l):
  rep (new tree_rep<T>(string(l), array<tree<T>>())) {}
#endif
inline tree<T>& tree<T>::operator [] (int i) { return rep->a[i]; }
extern inline int N (tree<T> t) { return N(t->a); }

tree<T> copy (tree<T> t);
bool operator == (tree<T> t, tree<T> u);
bool operator != (tree<T> t, tree<T> u);
tree<T>& operator << (tree<T>& t, tree<T> t2);
tree<T>& operator << (tree<T>& t, array<tree<T>> a);
ostream& operator << (ostream& out, tree<T> t);

#ifdef test<string>
inline bool is_func (tree t, string s) {
  return (N(t)>0) && (t->label==s); }
inline bool is_func (tree t, string s, int i) {
  return (N(t)==i) && (t->label==s); }
inline bool is_func (tree t, char* s) {
  return (N(t)>0) && (t->label==s); }
inline bool is_func (tree t, char* s, int i) {
  return (N(t)==i) && (t->label==s); }
#endif

#undef test<T>
#endmodule // tree (T)

#module tree
struct tree<string>;
struct array<tree<string>>;
typedef tree<string> tree;
typedef array<tree<string>> array<tree>;
#import string
#import tree (string)
#endmodule // tree
