
/******************************************************************************
* MODULE     : command_line.gen.cc
* DESCRIPTION: routines for retrieving command line and output
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <convert.gen.h>

#module code_command_line
#import convert

/******************************************************************************
* Input
******************************************************************************/

#define e2s edit_to_string

string
e2s (tree t) {
  if (N(t)==0) // does not take into account large parentheses
    return copy (t->label);
  else {
    string s, buffer;
    int i, n= N(t);
    switch (math_primary (t->label)) {
    case Document:
      for (i=0; i<n; i++) buffer << e2s (t[i]) << "\n";
      return buffer;
    case Paragraph:
      return e2s (t[0]);
    case Concat:
      for (i=0; i<n; i++) buffer << e2s (t[i]);
      return buffer;
    case Format:
      if (t[0]->label == NEW_LINE) return "\n";
      if (t[0]->label == LINE_SEP) return "\t";
      if (t[0]->label == NEXT_LINE) return "\n";
      return "";
    case Hspace:
    case Space:
    case Htab:
      return " ";

    case Matrix:
    case Table:
      {
	int x, cols= as_int (t[n-2]->label);
	int y, rows= as_int (t[n-1]->label);
	if (math_primary (t->label)==Matrix) buffer << "<matrix>";
	else buffer << "<table>";
	for (y=0; y<rows; y++) {
	  if (y>0) buffer << "<,>";
	  buffer << "<row>";
	  for (x=0; x<cols; x++) {
	    if (x>0) buffer << "<,>";
	    buffer << e2s (t[i]);
	  }
	  buffer << "</row>";
	}
	if (math_primary (t->label)==Matrix) buffer << "</matrix>";
	else buffer << "</table>";
	return buffer;
      }

    default:
      buffer << "<" << t->label << ">";
      for (i=0; i<n-1; i++) buffer << e2s (t[i]) << "<,>";
      buffer << e2s (t[i]);
      buffer << "</" << t->label << ">";
      return buffer;
    }
  }
}

#undef e2s

program
edit_to_edit_lisp (tree t) {
  return lisp_expr_to_program (edit_to_string (t), 0);
}

/******************************************************************************
* Output
******************************************************************************/

static bool
starts (string s, int at, string with) {
  return ((at+N(with)<=N(s)) && (s(at,at+N(with))==with));
}

static tree
normalize (tree t) {
  if (N(t)==0) return "";
  if (N(t)==1) return t[0];
  return t;
}

static tree
string_to_edit_sub (string s, int& i) {
  tree t (CONCAT);
  while (i<N(s)) {
    if (s[i]=='<') {
      i++;
      int start= i;
      while ((i<N(s)) && (s[i]!='>')) i++;
      string r= s (start, i);
      if (s[i]=='>') i++;

      if ((r == ",") || (r == "/row")) return normalize (t);
      else if (math_primary->contains (r) || (r=="row")) {
        tree u (r);
	while (i<N(s)) {
	  u << string_to_edit_sub (s, i);
	  if ((i>0) && (s[i-1]=='>')) {
	    int j= i-1, end= i-1;
	    while ((j>1) && (s[j-1]!='<')) j--;
	    if (s (j, end) == ("/" * r)) break;
	  }
	}
	if (N(u)>0) {
	  if ((r=="matrix") || (r=="table")) {
	    int x, cols=N(u[0]);
	    int y, rows=N(u);
	    tree v (r, rows* cols+ 2);
	    for (y=0; y<rows; y++)
	      for (x=0; x<cols; x++)
		if (x<N(u[y])) v[y*cols+x]= u[y][x];
	    v[rows*cols]  = as_string (cols);
	    v[rows*cols+1]= as_string (rows);
	    t << v;
	  }
	  else  t << u; // CHECK ARITY HERE !!!
	}
      }
      else if (starts (r, 0, "/") && math_primary->contains (r (1, N(r))))
        return normalize (t);
      else {
	if ((N(t)==0) || (N(t[N(t)-1])!=0)) t << ("<" * r *">");
	else t[N(t)-1]->label << ("<" * r *">");
      }
    }
    else {
      if ((N(t)==0) || (N(t[N(t)-1])!=0)) t << s(i,i+1);
      else t[N(t)-1]->label << s[i];
      i++;
    }
  }
  return normalize (t);
}

tree
string_to_edit (string s) {
  int i, j;
  // cout << "Input : " << s << "\n";
  tree t (DOCUMENT);
  for (i=0; i<N(s); i++) {
    int start= i;
    while ((i<N(s)) && (s[i]!='\n')) i++;
    string r= s (start, i); j=0;
    t << string_to_edit_sub (r, j);
  }
  // cout << "Output: " << t << "\n";
  return t;
}

/******************************************************************************
* Verbatim output
******************************************************************************/

static string
un_tab (string s) {
  int i, j;
  string r;
  for (i=0, j=0; i<N(s); i++, j++)
    if (s[i]=='\t') {
      do {
	r << " "; j++;
      } while ((j&7) != 0);
      j--;
    }
    else r << s[i];
  return r;
}

tree
string_to_verbatim_edit (string s) {
  int i, j;
  for (i=0; i<N(s); i++)
    if (s[i]=='\n') {
      tree t (DOCUMENT);
      for (i=0, j=0; i<N(s); i++)
	if (s[i]=='\n') {
	  t << un_tab (s (j, i));
	  j= i+1;
	}
      return t;
    }
  return s;
}

#endmodule // code_command_line
