
/******************************************************************************
* MODULE     : lisp.gen.cc
* DESCRIPTION: typed lisp utilities
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <convert.gen.h>
#include <compiler.gen.h>

#module code_lisp
#import convert
#import compiler

/******************************************************************************
* Alternative conversion of trees to strings
******************************************************************************/

static string
var_tree_to_string (tree t, bool flag) {
  if (N(t)==0) return t->label;
  else if (flag && is_func (t, "()", 2) && is_func (t[0], "\\\'", 0)) {
    bool f= is_func (t[1], "()", 3) && is_func (t[1][0], "\\:", 0);
    return "'" * var_tree_to_string (t[1], !f);
  }
  else if (flag && is_func (t, "()", 3) && is_func (t[0], "\\:", 0)) {
    bool f1=
      (is_func (t[1], "()", 2) && is_func (t[1][0], "\\\'", 0)) ||
      (is_func (t[1], "()", 3) && is_func (t[1][0], "\\:", 0));
    bool f2=
      (is_func (t[2], "()", 2) && is_func (t[2][0], "\\\'", 0)) ||
      (is_func (t[2], "()", 3) && is_func (t[2][0], "\\:", 0));
    return
      var_tree_to_string (t[1], !f1) * ":" *
      var_tree_to_string (t[2], !f2);
  }
  else {
    string s= t->label, buffer;
    if ((s == "") || (s == "(,)") || (s == "()")) buffer << "(";
    else if ((s == "[,]") || (s == "[]")) buffer << "[";
    else if ((s == "{,}") || (s == "{}")) buffer << "{";
    else buffer << s << " (";

    int i;
    for (i=0; i<(N(t)-1); i++) {
      buffer << var_tree_to_string (t[i]);
      if ((s == "()") || (s == "[]") || (s == "{}")) buffer << " ";
      else buffer << ", ";
    }
    buffer << var_tree_to_string (t[i]);
    
    if ((s == "[,]") || (s == "[]")) buffer << "]";
    else if ((s == "{,}") || (s == "{}")) buffer << "}";
    else buffer << ")";
    return buffer;
  }
}

string
var_tree_to_string (tree t) {
  return var_tree_to_string (t, TRUE);
}

/******************************************************************************
* Conversion to a lisp program
******************************************************************************/

static tree
error_tree (string s) {
  return tree (CONCAT, tree (SET, COLOR, "red"), s, tree (RESET, COLOR));
}

tree
compile_lisp (compiler cpl, tree t) {
  program p= edit_to_edit_lisp (t);
  list<typed_program> ltp= cpl->compile (p);
  if (nil (ltp))
    return tree (DOCUMENT, error_tree ("Compilation failed"));
  else {
    tree doc= tree (DOCUMENT);
    bool flag=TRUE;
    while (!nil (ltp)) {
      program P (ltp->item->p);
      type    T (ltp->item->t);
      tree    l (CONCAT, 5);
      l[0]= var_tree_to_string (P);
      l[1]= tree (HTAB, "1cm");
      l[2]= tree (SET, COLOR, "brown");
      l[3]= var_tree_to_string (T);
      l[4]= tree (RESET, COLOR);
      doc << l;
      ltp = ltp->next;
      flag= FALSE;
    }
    return doc;
  }
}

tree
eval_lisp (compiler cpl, tree t) {
  program p= edit_to_edit_lisp (t);
  list<typed_program> ltp= cpl->compile (p);
  if (nil (ltp)) return error_tree ("Compilation failed");
  else if (!atom (ltp)) return error_tree ("Ambiguous expression");

  program P (ltp->item->p);
  type    T (ltp->item->t);
  program Q;
  if (is_string (T, "Void")) Q= P;
  else Q= cpl->coerce (P, T, "Tree");
  tree* ptr= (tree*) execute (Q);
  if (is_string (T, "Void")) return tree ();
  tree u= *ptr;
  delete ptr;

  tree r (CONCAT, 5);
  r[0]= var_tree_to_string (u);
  r[1]= tree (HTAB, "1cm");
  r[2]= tree (SET, COLOR, "brown");
  r[3]= var_tree_to_string (T);
  r[4]= tree (RESET, COLOR);
  return r;
}

#endmodule // code_lisp
