
/******************************************************************************
* MODULE     : scheme.gen.cc
* DESCRIPTION: Execution of scheme commands via guile
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <connect.gen.h>

#module code_scheme
#import connect

typedef long SCM;

static bool     guile_installed     = FALSE;
static bool     guile_extra         = FALSE;
static bool     guile_initialized   = FALSE;
static string   guile_install_error = "";
static void   (*guile_enter) (int, char**, void (*call_back) (int, char**));
static SCM    (*guile_eval_str) (char* s);
static char*  (*guile_scm2newstr) (SCM s, int* len);
static void   (*guile_glue) (pointer);

/******************************************************************************
* Installation of guile
******************************************************************************/

#define get_symbol(r,p) \
  (guile_install_symbol (r,*((pointer*) ((void*)(&p)) )))

#define get_glue(r,p) \
  (guile_install_glue (r,*((pointer*) ((void*)(&p)) )))

static bool
guile_install_symbol (string routine, pointer& p) {
  guile_install_error= symbol_install ("libguile.so", routine, p);
  if (p==NULL) cerr << guile_install_error << "\n";
  return (p==NULL);
}

static bool
guile_install_glue (string routine, pointer& p) {
  guile_install_error= symbol_install ("libglue.so", routine, p);
  if (p==NULL) cerr << guile_install_error << "\n";
  return (p==NULL);
}

void
install_guile (int argc, char** argv, void (*call_back) (int, char**)) {
  if (get_symbol ("gh_enter", guile_enter)) return;
  if (get_symbol ("gh_eval_str", guile_eval_str)) return;
  if (get_symbol ("gh_scm2newstr", guile_scm2newstr)) return;
  guile_installed= TRUE;
  guile_extra    = !get_glue ("initialize_glue", guile_glue);
  guile_enter (argc, argv, call_back);
}

/******************************************************************************
* Initialization of guile
******************************************************************************/

void
initialize_guile (pointer ptr_sv) {
  if (!guile_initialized) {
    char* init_prg =
      "(define TeXmacs-guile-error-handler
         (lambda args
           (object->string args)))

       (define object->string
         (lambda (obj)
           (call-with-output-string
             (lambda (port) (write obj port)))))

       (define TeXmacs-guile-eval
         (lambda (s)
           (catch
             #t
             (lambda ()
               (object->string (eval-string s)))
             TeXmacs-guile-error-handler)))";

    guile_eval_str (init_prg);
    if (guile_extra) guile_glue (ptr_sv);
    guile_initialized= TRUE;
  }
}

/******************************************************************************
* Evaluation of scheme commands
******************************************************************************/

static string
back_slash (string s) {
  int i;
  string r;
  for (i=0; i<N(s); i++)
    switch (s[i]) {
    case '\\': r << "\\\\"; break;
    case '\"': r << "\\\""; break;
    default: r << s[i];
    }
  return r;
}

string
eval_scheme (string s) {
  if (!guile_installed) {
    if (N (guile_install_error) == 0)
      return "You should run TeXmacs with the -g flag in order to use guile";
    else return guile_install_error;
  }
  if (!guile_initialized)
    return "TeXmacs failed to initialize guile correctly";

  string prg= "(TeXmacs-guile-eval \"" * back_slash (s) * "\")";
  char* _s= as_charp (prg);
  SCM result= guile_eval_str (_s);
  delete[] _s;

  int len_r;
  char* _r= guile_scm2newstr (result, &len_r);
  string r= _r;
  free (_r);

  if (r == "#<unspecified>") r= "";
  return r;
}

#endmodule // code_scheme
