
/******************************************************************************
* MODULE     : text.gen.cc
* DESCRIPTION: routines for converting between TeXmacs and text
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <convert.gen.h>

#module code_text
#import convert

/******************************************************************************
* Verbatim input
******************************************************************************/

string
edit_to_verbatim (tree t) {
  int n= N(t);
  if (n==0) return copy (t->label);
  switch (math_primary (t->label)) {
  case Document:
    {
      int i;
      string buffer;
      for (i=0; i<n; i++) {
	if (i>0) buffer << '\n';
	buffer << edit_to_verbatim (t[i]);
      }
      return buffer;
    }
  case Concat:
    {
      int i;
      string buffer;
      for (i=0; i<n; i++)
	buffer << edit_to_verbatim (t[i]);
      return buffer;
    }
  default:
    return "";
  }
}

/******************************************************************************
* Verbatim output
******************************************************************************/

static string
un_tab (string s) {
  int i, j;
  string r;
  for (i=0, j=0; i<N(s); i++, j++)
    if (s[i]=='\t') {
      do {
	r << " "; j++;
      } while ((j&7) != 0);
      j--;
    }
    else r << s[i];
  return r;
}

tree
verbatim_to_edit (string s) {
  int i, j;
  for (i=0; i<N(s); i++)
    if (s[i]=='\n') {
      tree t (DOCUMENT);
      for (i=0, j=0; i<N(s); i++)
	if (s[i]=='\n') {
	  t << un_tab (s (j, i));
	  j= i+1;
	}
      return t;
    }
  return s;
}

/******************************************************************************
* Conversion of edit trees to edit programs
******************************************************************************/

program
edit_to_edit_program (tree t) {
  int n= N(t);
  if (n==0) return program (copy (t->label));

  switch (math_primary (t->label)) {

    /*
      case Format:
      if (t[0]->label == NEW_LINE) return program (string ("\n"));
      if (t[0]->label == LINE_SEP) return program (string ("\t"));
      if (t[0]->label == NEXT_LINE) return program (string ("\n"));
      return "";
      case Hspace:
      case Space:
      case Htab:
      return program (string (" "));
    */

  case Matrix:
  case Table:
    {
      int x, cols= as_int (t[n-2]->label);
      int y, rows= as_int (t[n-1]->label);
      array<program> a (rows+1);
      a[0]= program (copy (t->label));
      for (y=0; y<rows; y++) {
	array<program> b (cols+1);
	b[0]= program (string ("row"));
	for (x=0; x<cols; x++)
	  b[x+1]= edit_to_edit_program (t[y*cols+x]);
	a[y+1]= program (b);
      }
      return program (a);
    }
    
  default:
    {
      int i;
      array<program> a (n+1);
      a[0]= program (copy (t->label));
      for (i=0; i<n; i++)
	a[i+1]= edit_to_edit_program (t[i]);
      return program (a);
    }
  }
}

/******************************************************************************
* Conversion of edit programs to edit trees
******************************************************************************/

tree
edit_program_to_edit (program p) {
  if (is_string (p)) return copy (p->s);
  if (N(p)<2) return "";
  if (!is_string (p[0])) return "";
  if (!math_primary->contains (p[0]->s)) return "";

  switch (math_primary (p[0]->s)) {
  case Matrix:
  case Table:
    {
      int x, cols= N(p[1])-1;
      int y, rows= N(p)-1;
      if ((cols<=0) || (rows<=0)) return "";
      tree t (copy (p[0]->s), rows* cols+ 2);
      for (y=0; y<rows; y++)
	for (x=0; x<cols; x++)
	  if ((x+1)<N(p[y+1]))
	    t[y*cols+x]= edit_program_to_edit (p[y+1][x+1]);
      t[rows*cols]  = as_string (cols);
      t[rows*cols+1]= as_string (rows);
      return t;
    }
    
  default:
    {
      int i;
      tree t (copy (p[0]->s), N(p)-1);
      for (i=0; i<(N(p)-1); i++)
	t[i]= edit_program_to_edit (p[i+1]);
      return t;
    }
  }
}

/******************************************************************************
* Convertion of edit programs to tag strings
******************************************************************************/

static string
slash (string s) {
  int i;
  string r;
  for (i=0; i<N(s); i++)
    switch (s[i]) {
    case '{':
    case '}':
    case '\\':
      r << "\\" << s[i];
      break;
    case '\n':
      r << "\\N";
      break;
    default:
      r << s[i];
    }
  return r;
}

string
edit_program_to_string (program p) {
  if (N(p)==0) {
    if (is_string (p)) return slash (p->s);
    fatal_error ("program is not an edit_program", "edit_program_to_string");
  }

  int i;
  string buffer;

  if (is_string (p[0], "document")) {
    for (i=1; i<N(p); i++) {
      if (i>1) buffer << "\n";
      buffer << edit_program_to_string (p[i]);
    }
  }

  else if (is_string (p[0], "concat"))
    for (i=1; i<N(p); i++)
      buffer << edit_program_to_string (p[i]);

  else {
    buffer << "\\";
    if ((!is_string (p[0])) || (slash (p[0]->s) != p[0]->s))
      fatal_error ("program is not an edit_program", "edit_program_to_string");
    buffer << copy (p[0]->s);
    buffer << "{";
    for (i=1; i<N(p); i++) {
      if (i>1) buffer << "}{";
      buffer << edit_program_to_string (p[i]);
    }
    buffer << "}";
  }

  return buffer;
}

/******************************************************************************
* Convertion of tag strings to edit programs
******************************************************************************/

static string
unslash (string s) {
  int i;
  string r;
  for (i=0; i<N(s); i++)
    if ((s[i]=='\\') && (i+1<N(s))) {
      if (s[i+1]=='N') r << '\n';
      else r << s[i+1];
      i++;
    }
    else r << s[i];
  return r;
}

program
string_to_edit_program (string s, int pos, int& i) {
  array<program> a (1);
  a[0]= "concat";
  int begin= i;
  while (TRUE) {
    int start= i;
    for (; i<N(s); i++) {
      if ((s[i]=='\\') && ((i+1)<N(s)) &&
	  ((s[i+1]=='\\') || (s[i+1]=='N') ||
	   (s[i+1]=='{') || (s[i+1]=='}'))) { i++; continue; }
      if ((s[i]=='\\') || (s[i]=='{') || (s[i]=='}')) break;
    }
    if (start<i) a << program (unslash (s (start, i)), pos+start, pos+i);
    if ((i==N(s)) || (s[i]=='{') || (s[i]=='}')) break;

    start= i;
    array<program> b;
    for (i++; TRUE; i++) {
      program q= string_to_edit_program (s, pos, i);
      if (!is_string (q, "")) b << q;
      if (i==N(s)) break;
      if (s[i]=='}') {
	i++;
	if (i==N(s)) break;
	if (s[i]!='{') break;
      }
    }
    a << program (b, pos+start, pos+i+(i==N(s)? 0: 1));
  }
  if (N(a)==1) return program (string (""));
  if (N(a)==2) return a[1];
  return program (a, pos+begin, pos+i);
}

program
string_to_edit_program (string s, int pos=0) {
  array<program> a (1);
  a[0]= program (string ("document"));
  int i;
  for (i=0; i<N(s); i++) {
    int start= i;
    while ((i<N(s)) && (s[i]!='\n')) i++;
    int j= 0;
    a << string_to_edit_program (s (start, i), pos+start, j);
  }
  return program (a, pos, pos+N(s));
}

/******************************************************************************
* Derived routines
******************************************************************************/

string
edit_to_string (tree t) {
  // cout << "t= [" << t << "]\n";
  program p= edit_to_edit_program (t);
  // cout << "p= [" << p << "]\n";
  string s= edit_program_to_string (p);
  // cout << "s= [" << s << "]\n";
  return s;
}

tree
string_to_edit (string s) {
  // cout << "s= [" << s << "]\n";
  program p= string_to_edit_program (s);
  // cout << "p= [" << p << "]\n";
  tree t= edit_program_to_edit (p);
  // cout << "t= [" << t << "]\n";
  return t;
}

program
edit_to_edit_lisp (tree t) {
  return lisp_expr_to_program (edit_to_verbatim (t), 0);
}

#endmodule // code_text
