
/******************************************************************************
* MODULE     : lexer.gen.h
* DESCRIPTION: dynamic lexer
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <program.gen.h>
#include <list.gen.h>
#include <path.gen.h>
#include <hashmap.gen.h>
#include <hashset.gen.h>
#include <resource.gen.h>

#module lexer
#import program
#import path
#import array (string)
#import hashmap (string, program)
#import resource (lexer_rep, lexer)

/******************************************************************************
* Lexical states
******************************************************************************/

class lex_state_rep;
class lex_state {
#import concrete_null (lex_state, lex_state_rep)
  lex_state (program name, int p, string jump);
  friend int hash (lex_state st);
  friend bool operator == (lex_state st1, lex_state st2);
  friend bool operator != (lex_state st1, lex_state st2);
  friend ostream& operator << (ostream& out, lex_state st);
};

#define no_tree_converter<lex_state>
#import array (lex_state)
#import list (lex_state)
#import hashset (lex_state)
#import hashmap (string, lex_state)
#import hashmap (string, array<lex_state>)
#import hashmap (lex_state, path)
typedef hashmap<string,array<lex_state>> lex_table;

class lex_state_rep: public concrete_struct {
  lex_table next;
  program name;
  int priority;
  string jump;

public:
  lex_state_rep (program name, int p, string jump);
  friend class lex_state;
  friend class lexer_rep;
  friend ostream& operator << (ostream& out, lex_state st);
};
#import code_concrete_null (lex_state, lex_state_rep)

/******************************************************************************
* The lexer
******************************************************************************/

struct lexer_rep: rep<lexer> {
  hashset<lex_state>        table;
  hashmap<string,lex_state> init;
  hashmap<string,lex_state> term;
  hashmap<string,program>   def;

  string                  lex_s;    // the input string
  string                  lex_symb; // symbol being lexed
  int                     lex_last; // the tokens have been flushed until here
  int                     lex_pos;  // position of next input token
  bool                    lex_eof;  // already lexed end of file token ?
  location                lex_off;  // location of first input char in source
  hashmap<lex_state,path> lex_H;    // best path which attains lex_state
  string                  lex_r;    // the (partial) next output value
  int                     lex_rst;  // the corresponding start in the source
  string                  lex_t;    // the next output type
  bool                    lex_flag; // display error messages

public:
  lexer_rep (string name, program p);
  ~lexer_rep ();

  void      insert_definition (string name, program p);
  lex_state insert_state (program name, int priority=-1, string jump= "");
  void      insert_transition (lex_state st1, lex_state st2, string s);
  void      insert_rule (string name, program p, int pri);
  void      insert_symbol (string name, program p);
  void      convert (program p, lex_state st1, lex_state st2);

  void set_input (string in, location offset,
		  string lex_symb= "main", bool flag= TRUE);
  void lex (string c);
  bool produce ();
  void get (string& s, string& type, location& start, location& end);

  void saturate (hashset<lex_state>& S, string c);
  bool recognizes (string s, string type);
  bool recognizes (string s, string type, string& result);

  friend class lexer;
  friend class parser_rep;
  friend class tiger_rep;
};

lexer load_lexer (string name);

#endmodule // lexer
