
/******************************************************************************
* MODULE     : tiger.gen.h
* DESCRIPTION: dynamic converter of programs to strings
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <program.gen.h>
#include <hashmap.gen.h>
#include <hashset.gen.h>
#include <resource.gen.h>

#module tiger
#import program
#import hashmap (string, program)
#import resource (tiger_rep, tiger)

struct tiger_rep: rep<tiger> {
  hashmap<string,program> rules_table; // the grammar

  lexer  tiger_lx;  // the lexer for conversions
  string tiger_out; // the result of the conversion

public:
  /*********** Building the tiger *************/
  tiger_rep (string name, program p);
  void insert_symbol (string name);
  void insert_method (string name, program p1, program p2);

  /****************** Parsing  *****************/
  void    set_lexer (lexer lx);
  bool    match (program p, program l);
  program get_sub_program (program p, program l, program where, int offset);
  void    produce (program p, program l, program r, hashset<string>& X);
  void    convert (program p, string type, hashset<string>& X);
  string  convert (program p, string type= "main");

  /************** Error handling ***************/
  void grammar_error (string s);
  void error (string s, program p);
};

tiger load_tiger (string name);

#endmodule // tiger
