
/******************************************************************************
* MODULE     : cursor.gen.cc
* DESCRIPTION: cursor handling
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <edit_cursor.gen.h>
#include <data.gen.h>

#module code_edit_cursor
#import edit_cursor
#import data

path inner_paragraph (tree t, path p);

/******************************************************************************
* Subroutines for cursor paths in trees
******************************************************************************/

bool
valid_cursor (tree t, path p, bool start_flag=FALSE) {
  if ((!nil (p)) && (!atom (p)) && ((p->item<0) || (p->item>=N(t)))) {
    cerr << "TeXmacs] testing valid cursor " << p << " in " << t << "\n";
    fatal_error ("bad path", "valid_cursor", "edit_cursor.gen.cc");
  }

  if (nil (p)) return FALSE;
  if (atom (p)) {
    if (start_flag && is_concat (t)) return (p->item!=0);
    if (is_format (t)) return FALSE;
    return TRUE;
  }
  if (is_concat (t))
    return valid_cursor (t[p->item], p->next, start_flag || (p->item!=0));
  if (is_func (t, INACTIVE))
    return (N(t[0])==0) || (!atom (p->next));
  return valid_cursor (t[p->item], p->next, FALSE);
}

path
pre_correct (tree t, path p) {
  if ((!nil (p)) && (!atom (p)) && ((p->item<0) || (p->item>=N(t)))) {
    cerr << "TeXmacs] precorrecting " << p << " in " << t << "\n";
    fatal_error ("bad path", "pre_correct", "edit_cursor.gen.cc");
  }

  if (nil (p)) return pre_correct (t, path(0));
  if (atom (p)) {
    if (is_format (t)) {
      if (p->item==0) return path (0, pre_correct (t[0], path (0)));
      else {
	int l=N(t)-1;
	return path (l, pre_correct (t[l], path (right_index (t[l]))));
      }
    }
    return p;
  }
  if (is_func (t, INACTIVE) && (N(t[0])!=0) && atom (p->next)) {
    t= t[0]; p= p->next;
    if (p->item==0) return path (0, path (0, pre_correct (t[0], path (0))));
    else {
      int l=N(t)-1;
      return path (0, path (l, pre_correct (t[l], path (right_index (t[l])))));
    }
  }
  return path (p->item, pre_correct (t[p->item], p->next));
}

bool
left_most (tree t, path p) {
  if (nil (p))
    fatal_error ("invalid nil path", "left_most", "edit_cursor.gen.cc");
  if ((!atom (p)) && ((p->item<0) || (p->item>=N(t)))) {
    cerr << "TeXmacs] left most " << p << " in " << t << "\n";
    fatal_error ("bad path", "left_most", "edit_cursor.gen.cc");
  }

  int i=p->item;
  if (atom (p)) return i==0;
  if (is_concat (p)) return (i==0) && left_most (t[0], p->next);
  return FALSE;
}

path
left_correct (tree t, path p) {
  if (nil (p))
    fatal_error ("invalid nil path", "left_correct", "edit_cursor.gen.cc");
  if ((!atom (p)) && ((p->item<0) || (p->item>=N(t)))) {
    cerr << "TeXmacs] left correcting " << p << " in " << t << "\n";
    fatal_error ("bad path", "left_correct", "edit_cursor.gen.cc");
  }

  int i=p->item;
  if (atom (p)) return p;
  if (is_concat (t) && (i>0) && left_most (t[i], p->next))
    return path (i-1, pre_correct (t[i-1], path (right_index (t[i-1]))));
  return path (i, left_correct (t[i], p->next));
}

bool
right_most (tree t, path p) {
  if (nil (p))
    fatal_error ("invalid nil path", "right_most", "edit_cursor.gen.cc");
  if ((!atom (p)) && ((p->item<0) || (p->item>=N(t)))) {
    cerr << "TeXmacs] right most " << p << " in " << t << "\n";
    fatal_error ("bad path", "right_most", "edit_cursor.gen.cc");
  }

  int i=p->item;
  if (atom (p)) return i==right_index (t);
  if (is_concat (p)) return (i==1) && right_most (t[N(t)-1], p->next);
  return FALSE;
}

path
right_correct (tree t, path p) {
  if (nil (p))
    fatal_error ("invalid nil path", "right_correct", "edit_cursor.gen.cc");
  if ((!atom (p)) && ((p->item<0) || (p->item>=N(t)))) {
    cerr << "TeXmacs] right correcting " << p << " in " << t << "\n";
    fatal_error ("bad path", "right_correct", "edit_cursor.gen.cc");
  }

  int i=p->item;
  if (atom (p)) return p;
  if (is_concat (t) && (i<N(t)-1) && right_most (t[i], p->next))
    return path (i+1, pre_correct (t[i-1], path (0)));
  return path (i, right_correct (t[i], p->next));
}

/******************************************************************************
* Exported routines for cursor paths in trees
******************************************************************************/

path
correct_cursor (tree t, path p) {
  return left_correct (t, pre_correct (t, p));
}

path
start (tree t, path p) {
  return correct_cursor (t, path_down (p, 0));
}

path
end (tree t, path p) {
  return correct_cursor (t, path_down (p, right_index (subtree (t, p))));
}

path start (tree t) { return start (t, path ()); }
path end (tree t) { return end (t, path ()); }

/******************************************************************************
* Constructor and destructor
******************************************************************************/

edit_cursor::edit_cursor (tm_buffer buf):
  edit_base (buf),
  cu (0, 0), mv (0, 0), oc (0, 0), mv_status (0),
  dragging (FALSE), made_selection (FALSE) {}

edit_cursor::~edit_cursor () {}

/******************************************************************************
* Cursor movement
******************************************************************************/

#define DELTA (1<<23)

path
edit_cursor::tree_path (SI x, SI y, SI delta) {
  path p= eb->get_path (x, y, delta);
  p= correct_cursor (et, p);
  return p;
}

bool
edit_cursor::cursor_move_sub (SI& x0, SI& y0, SI& d0, SI dx, SI dy) {
  int i,d;
  path ref_p= tree_path (x0, y0, d0);
  
  // cout << "ref_p = " << ref_p << "\n";
  if (ref_p == tree_path (x0, y0, d0+ dx*DELTA)) {
    for (i=1; i<DELTA; i=i<<1)
      if (ref_p != tree_path (x0+ dx*i, y0+ dy*i, d0+ dx*DELTA)) break;
    if (i>=DELTA) return FALSE;
    for (d=i>>2; d>=1; d=d>>1)
      if (ref_p != tree_path (x0+ dx*(i-d), y0+ dy*(i-d), d0+ dx*DELTA)) i-=d;

    x0 += dx*i;
    y0 += dy*i;
  }
  
  // cout << "path  = " << tree_path (x0, y0, d0) << "\n";
  if (dx!=0) {
    if (ref_p == tree_path (x0, y0, d0)) {
      for (i=1; i<DELTA; i=i<<1)
	if (ref_p != tree_path (x0, y0, d0+ dx*i)) break;
      if (i>=DELTA)
	fatal_error ("inconsistent cursor handling",
		     "edit_cursor::cursor_move_sub");
      for (d=i>>2; d>=1; d=d>>1)
	if (ref_p != tree_path (x0, y0, d0+ dx*(i-d))) i-=d;
      d0 += dx*i;
    }
    else {
      for (i=1; i<DELTA; i=i<<1)
	if (ref_p == tree_path (x0, y0, d0- dx*i)) break;
      if (i<DELTA) {
	for (d=i>>2; d>=1; d=d>>1)
	  if (ref_p == tree_path (x0, y0, d0- dx*(i-d))) i-=d;
	i--;
	d0 -= dx*i;
      }
      else {  // exceptional case
	ref_p= tree_path (x0, y0, d0- dx*DELTA);
	for (i=1; i<DELTA; i=i<<1)
	  if (ref_p == tree_path (x0, y0, d0- dx*i)) break;
	for (d=i>>2; d>=1; d=d>>1)
	  if (ref_p == tree_path (x0, y0, d0- dx*(i-d))) i-=d;
	d0 -= dx*i;
      }
    }
  }

  tp= tree_path (x0, y0, d0);
  return TRUE;
}

void
edit_cursor::cursor_move (SI dx, SI dy) {
  cursor_move_sub (mv->x0, mv->y0, mv->delta, dx, dy);
}

/******************************************************************************
* Routines affecting both the cursor and the ghost cursor
******************************************************************************/

void
edit_cursor::adjust_ghost_cursor (int status) {
  if (status==mv_status) {
    if (status!=HORIZONTAL) {
      mv->x0   = cu->x0;
      mv->delta= cu->delta;
    }
    if (status!=VERTICAL)
      mv->y0= cu->y0;
  }
}

void
edit_cursor::notify_cursor_moved (int status) {
  mv_status= status;
  cu= eb->get_cursor (tp);
  notify_change (THE_CURSOR);
}

void
edit_cursor::set_cursor (SI x, SI y) {
  if (has_changed (THE_TREE+THE_ENVIRONMENT)) return;
  tp= tree_path (x- eb->x0, y- eb->y0, 0);
  notify_cursor_moved (CENTER);
  mv->x0   = x- eb->x0;
  mv->y0   = y- eb->y0;
  mv->delta= 0;
}

void
edit_cursor::go_left () {
  if (has_changed (THE_TREE+THE_ENVIRONMENT)) return;
  adjust_ghost_cursor (VERTICAL);
  cursor_move (-1, 0);
  notify_cursor_moved (HORIZONTAL);
}

void
edit_cursor::go_right () {
  if (has_changed (THE_TREE+THE_ENVIRONMENT)) return;
  adjust_ghost_cursor (VERTICAL);
  cursor_move (1, 0);
  notify_cursor_moved (HORIZONTAL);
}

void
edit_cursor::go_up () {
  if (has_changed (THE_TREE+THE_ENVIRONMENT)) return;
  adjust_ghost_cursor (HORIZONTAL);
  cursor_move (0, 1);
  notify_cursor_moved (VERTICAL);
}

void
edit_cursor::go_down () {
  if (has_changed (THE_TREE+THE_ENVIRONMENT)) return;
  adjust_ghost_cursor (HORIZONTAL);
  cursor_move (0, -1);
  notify_cursor_moved (VERTICAL);
}

void
edit_cursor::go_page_up () {
  if (has_changed (THE_TREE+THE_ENVIRONMENT)) return;
  if (mv->y0+ eb->y0 >= eb->y2) return;
  set_cursor (mv->x0+ eb->x0, mv->y0+ eb->y0+ get_window_height ());
}

void
edit_cursor::go_page_down () {
  if (has_changed (THE_TREE+THE_ENVIRONMENT)) return;
  if (mv->y0+ eb->y0 < eb->y1) return;
  set_cursor (mv->x0+ eb->x0, mv->y0+ eb->y0- get_window_height ());
}

/******************************************************************************
* Setting the cursor from a path
******************************************************************************/

void
edit_cursor::set_cursor (path p) {
  tp= p;
  mv_status= DIRECT;
  if (!has_changed (THE_TREE+THE_ENVIRONMENT)) {
    cu= eb->get_cursor (tp);
    mv= copy (cu);
  }
  notify_change (THE_CURSOR);
}

void
edit_cursor::go_home () {
  set_cursor (correct_cursor (et, path (0)));
}

void
edit_cursor::go_end () {
  set_cursor (correct_cursor (et, path (1)));
}

void
edit_cursor::go_home_line () {
  set_cursor (correct_cursor (et, path_down (inner_paragraph (et, tp), 0)));
}

void
edit_cursor::go_end_line () {
  path p= inner_paragraph (et, tp);
  tree t= subtree (et, p);
  set_cursor (correct_cursor (et, path_down (p, right_index (t))));
}

#endmodule // code_edit_cursor
