
/******************************************************************************
* MODULE     : edit_footer.gen.cc
* DESCRIPTION: display interesting information for the user in the footer
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#module code_edit_footer

extern string compute_context (tree t, path p);

/******************************************************************************
* Set left footer with information about environment variables
******************************************************************************/

void
edit_interface::set_left_footer (string s) {
  SERVER (set_left_footer (s));
}

void
edit_interface::append_left_footer (string& s, string env_var) {
  string i= get_init_value (env_var)->label;
  string c= get_env_value  (env_var)->label;
  if (c != i) s= s * "#" * c;
}

void
edit_interface::set_left_footer () {
  int i;
  string s, r, e;
  double base_sz= as_int (get_env_value (FONT_BASE_SIZE)->label);
  double sz= as_double (get_env_value (FONT_SIZE)->label);
  for (i=0; i<N(the_style); i++) s= s * "#" * the_style[i]->label;
  string mode= get_env_value (MODE)->label;
  string lan = get_env_value (LANGUAGE (mode))->label;
  if (get_init_value (LANGUAGE (mode))->label != lan) s= s * "#" * lan;
  else s= s * "#" * mode;
  if (mode == "text") {
    s= s * "#" * get_env_value (TEXT_FONT)->label;
    append_left_footer (s, TEXT_FAMILY);
    s= s * "#" * as_string ((int) ((base_sz+0.5)*sz));
    append_left_footer (s, TEXT_SERIES);
    append_left_footer (s, TEXT_SHAPE);
  }
  else if (mode == "math") {
    s= s * "#" * get_env_value (MATH_FONT)->label;
    append_left_footer (s, MATH_FAMILY);
    s= s * "#" * as_string ((int) ((base_sz+0.5)*sz));
    append_left_footer (s, MATH_SERIES);
    append_left_footer (s, MATH_SHAPE);
  }
  else {
    s= s * "#" * get_env_value (PROG_FONT)->label;
    append_left_footer (s, PROG_FAMILY);
    s= s * "#" * as_string ((int) ((base_sz+0.5)*sz));
    append_left_footer (s, PROG_SERIES);
    append_left_footer (s, PROG_SHAPE);
  }
  r= get_env_value (COLOR)->label;
  if (r != "black") s= s * "#" * r;
  if ((N(s)>0) && (s[0] == '#')) s= s (1, N(s));
  set_left_footer (s);
}

/******************************************************************************
* Set right footer with information about cursor position
******************************************************************************/

void
edit_interface::set_right_footer (string s) {
  SERVER (set_right_footer (s));
}

string
edit_interface::compute_text_footer (tree st) {
  string r;
  string mode= get_env_value (MODE)->label;
  language lan;
  if (mode == "text")
    lan= text_language (get_env_value (TEXT_LANGUAGE)->label);
  else if (mode == "math")
    lan= math_language (get_env_value (MATH_LANGUAGE)->label);
  else
    lan= prog_language (get_env_value (PROG_LANGUAGE)->label);
  int end  = last_item (tp);
  int start= end;
  if (lan->enc->token_backward (st->label, start))
    fatal_error ("bad cursor position in string",
		 "edit_interface::set_footer");
  r= st->label (start, end);
  if (r == "") r= "start";
  if (r == " ") r= "space";
  if (r == "#") r= "sharp";
  return r;
}

static string
get_accent_type (string s) {
  if (s == "^") return "hat";
  if (s == "~") return "tilde";
  if ((N(s)>=2) && (s[0]=='<') && (s[N(s)-1]=='>')) return s (1, N(s)-1);
  return "unknown accent";
}

string
edit_interface::compute_operation_footer (tree st) {
  string r;
  switch (math_primary[st->label]) {
  case Format: r= st[0]->label; break;
  case Hspace: r= "horizontal space"; break;
  case Vspace_before: r= "vertical space before"; break;
  case Vspace_after: r= "vertical space"; break;
  case Space: r= "rigid space"; break;
  case Htab: r= "tab"; break;
  case Split: r= "split"; break;
  case Move: r= "move"; break;
  case Resize: r= "resize"; break;
  case Left_sub: r= "left subscript"; break;
  case Left_sup: r= "left superscript"; break;
  case Right_sub: r= "subscript"; break;
  case Right_sup: r= "superscript"; break;
  case Frac: r= "fraction"; break;
  case Sqrt: r= (char*) ((N(st)==1)? "square root": "n-th root"); break;
  case Wide: r=  get_accent_type (st[1]->label); break;
  case Neg: r= "negation"; break;
  case Tree: r= "tree"; break;
  case Matrix: r= "matrix"; break;
  case Table: r= "table"; break;
  case Mosaic: r= "mosaic"; break;
  case Inactive: r= "inactive#" * st[0]->label; break;
  case Symbol: r= "symbol"; break;
  case Latex: r= "latex"; break;
  case Hybrid: r= "hybrid"; break;
  case Assign: r= "assign#" * st[0]->label; break;
  case Set: r= "set#" * st[0]->label; break;
  case Reset: r= "reset#" * st[0]->label; break;
  case Apply: r= "apply#" * st[0]->label; break;
  case Begin: r= "begin#" * st[0]->label; break;
  case End: r= "end#" * st[0]->label; break;
  case Label: r= "label: " * st[0]->label; break;
  case Write: r= "write to " * st[0]->label; break;
  case Reference: r= "reference: " * st[0]->label; break;
  case Function: r= "function"; break;
  case Environment: r= "environment"; break;
  case Plus: r= "plus"; break;
  case Minus: r= "minus"; break;
  case Times: r= "times"; break;
  case Merge: r= "merge"; break;
  case Number: r= "number"; break;
  case Graphics: r= "graphics"; break;
  case Point: r= "point"; break;
  case Line: r= "line"; break;
  case Arc: r= "arc"; break;
  case Bezier: r= "Bezier"; break;
  case Postscript: r= "postscript image"; break;
  default: r= as_string (mem_used ())*" bytes used";
  }
  if (last_item (tp) == 0) r= "before#" * r;
  return r;
}

string
edit_interface::compute_compound_footer (tree t, path p) {
  if (nil (p) || atom (p)) return "";
  string up= compute_compound_footer (t, path_up (p));
  tree st= subtree (t, path_up (p));
  int  l = last_item (p);
  switch (math_primary[st->label]) {
  case Move:
    if (l==0) return up * "move#";
    else return up;
  case Resize:
    if (l==0) return up * "resize#";
    else return up;
  case Left_sub:
    return up * "left subscript#";
  case Left_sup:
    return up * "left superscript#";
  case Right_sub:
    return up * "subscript#";
  case Right_sup:
    return up * "superscript#";
  case Frac:
    if (l==0) return up * "numerator#";
    else return up * "denominator#";
  case Sqrt:
    if (N(st)==1) return up * "square root#";
    if (l==0) return up * "root#";
    else return up * "index#";
  case Wide:
    return up * get_accent_type (st[1]->label) * "#";
  case Neg:
    return up * "negation#";
  case Tree:
    if (l==0) return up * "root#";
    else return up * "branch(" * as_string (l) * ")#";
  case Matrix:
  case Table:
    {
      string mt= (char*) ((math_primary[st->label]==Matrix)?"matrix":"table");
      int cols = as_int (st[N(st)-2]->label);
      int i    = (l/cols)+1;
      int j    = (l%cols)+1;
      return up * mt * "(" * as_string (i) * "," * as_string (j) * ")#";
    }
  case Mosaic:
    return up * "mosaic#";
  case Symbol:
    return up * "symbol#";
  case Latex:
    return up * "latex#";
  case Hybrid:
    return up * "hybrid#";
  default:
    return up;
  }
}

bool active_envs= FALSE;

void
edit_interface::set_right_footer () {
  string s, r;
  tree st= subtree (et, path_up (tp));
  if (N(st)==0) r= compute_text_footer (st);
  else r= compute_operation_footer (st);
  r= compute_compound_footer (et, path_up (tp)) * r;
  if (active_envs) r= compute_context (et, tp) * r;
  set_right_footer (r);
}

/******************************************************************************
* Set footer with information about execution of latex command
******************************************************************************/

bool
edit_interface::set_latex_footer (tree st) {
  if (is_func (subtree (et, path_up (path_up (tp))), LATEX, 1) ||
      is_func (subtree (et, path_up (path_up (tp))), HYBRID, 1)) {
    string s= st->label;
    string help;
    program p;
    if (sv->get_latex_command (s, help, p)) {
      set_left_footer ("<return>:#" * help);
      set_right_footer ("latex command");
      return TRUE;
    }
  }
  return FALSE;
}

/******************************************************************************
* Update footer
******************************************************************************/

DEBUG
(
  int concrete_count = 0;
  int abstract_count = 0;
  int box_count      = 0;
  int event_count    = 0;
  int widget_count   = 0;
  int line_item_count= 0;
  int list_count     = 0;
  int command_count  = 0;
  int iterator_count = 0;
  int function_count = 0;
  int instance_count = 0;
)

void
edit_interface::set_footer () {
  DEBUG
  (
    cout << "--------------------------------------------------------------\n";
    cout << "concrete  " << concrete_count << "\n";
    cout << "abstract  " << abstract_count << "\n";
    cout << "widget    " << widget_count << "\n";
    cout << "box       " << box_count << "\n";
    cout << "event     " << event_count << "\n";
    cout << "line item " << line_item_count << "\n";
    cout << "list      " << list_count << "\n";
    cout << "command   " << command_count << "\n";
    cout << "iterator  " << iterator_count << "\n";
    cout << "function  " << function_count << "\n";
    cout << "instance  " << instance_count << "\n";
  )

  if ((N(message_l) == 0) && (N(message_r) == 0)) {
    tree st= subtree (et, path_up (tp));
    if (set_latex_footer (st)) return;
    set_left_footer();
    set_right_footer();
  }
  else {
    set_left_footer (message_l);
    set_right_footer (message_r);
    message_l= message_r= "";
  }
}

/******************************************************************************
* Interactive commands
******************************************************************************/

class interactive_command_rep: public command_rep {
  edit_interface* ed;
  program p;    // the interactive arguments
  program q;    // the function which is applied to the arguments
  int     i;    // counter where we are
  string* s;    // feedback from interaction with user

public:
  interactive_command_rep (edit_interface* ed2, program p2, program q2):
    ed (ed2), p (p2), q (q2), i (0), s (new string [N(p)]) {}
  ~interactive_command_rep () { delete[] s; }
  void apply ();
  ostream& print (ostream& out) {
    return out << "interactive command " << p; }
};

void
interactive_command_rep::apply () {
  if ((i>0) && (s[i-1] == "cancel")) return;
  if (i==N(p)) {
    array<program> prg (N(p)+1);
    prg[0]= q;
    for (i=0; i<N(p); i++) prg[i+1]= program (s[i]);
    program r= ed->compile (program (prg));
    if (is_compiled (r)) ::execute (r);
    else ed->set_message ("Error: TeXmacs-lisp compilation error",
			  "interactive command");
  }
  else {
    if ((!is_string (p[i])) || (!is_string (p[i]->s))) return;
    s[i]= string ("");
    tm_view temp_vw= ed->sv->get_view (FALSE);
    ed->focus_on_this_editor ();
    ed->sv->interactive (as_string (p[i]->s), s[i], this);
    ed->sv->set_view (temp_vw);
    i++;
  }
}

/******************************************************************************
* Exported routines
******************************************************************************/

void
edit_interface::set_message (string l, string r) {
  message_l= l;
  message_r= r;
  notify_change (THE_FOOTER);
}

void
edit_interface::interactive (program p, program q) {
  command interactive_cmd= new interactive_command_rep (this, p, q);
  interactive_cmd ();
}

#endmodule // code_edit_footer
