
/******************************************************************************
* MODULE     : edit_interface.gen.cc
* DESCRIPTION: interface between the editor and the window manager
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <edit_interface.gen.h>
#include <string.gen.h>
#include <hashmap.gen.h>
#include <file.gen.h>
#include <Atom/typesetter.gen.h>
#include <convert.gen.h>
#include <server.gen.h>
#include <tm_buffer.gen.h>

#module code_edit_interface
#import edit_interface
#import hashmap (string, string)
#import hashmap (string, program)
#import file
#import typesetter
#import convert
#import server
#import tm_buffer

extern void   (*env_next_prog)(void);
extern void   clear_rectangles (ps_device dev, rectangles l);

static string
LANGUAGE (string mode) {
  if (mode == "text") return TEXT_LANGUAGE;
  if (mode == "math") return MATH_LANGUAGE;
  if (mode == "prog") return PROG_LANGUAGE;
  cerr << "Mode = " << mode << "\n";
  fatal_error ("invalid mode", "the_language", "edit_interface.gen.cc");
}

/******************************************************************************
* Main edit_interface routines
******************************************************************************/

edit_interface::edit_interface (server_rep* sv2, display dis, tm_buffer buf):
  edit_base (buf), edit_cursor (buf), edit_typeset (buf),
  basic_widget_rep (dis),
  got_focus (FALSE), sh_s (""), sh_len (0),
  main_menu (array<program> ()),
  popup_menu (array<program> ()), popup_win (NULL),
  sfactor (5), pixel (5*PIXEL), sv (sv2) {}

edit_interface::~edit_interface () {}

edit_interface::operator tree () {
  return tree ("edit", get_name ());
}

void
edit_interface::suspend () {
  got_focus= FALSE;
  notify_change (THE_FOCUS);
}

void
edit_interface::resume () {
  got_focus= TRUE;
  SERVER (menu_main (main_menu));
  notify_change (THE_FOCUS + THE_EXTENTS);
}

/******************************************************************************
* some routines for dealing with shrinked coordinates
******************************************************************************/

void
edit_interface::set_shrinking_factor (int sf) {
  sfactor= sf;
  pixel  = sf*PIXEL;
  notify_change (THE_ENVIRONMENT);
}

void
edit_interface::invalidate (SI x1, SI y1, SI x2, SI y2) {
  this << emit_invalidate ((x1-sfactor+1)/sfactor, (y1-sfactor+1)/sfactor,
			   (x2+sfactor-1)/sfactor, (y2+sfactor-1)/sfactor);
}

void
edit_interface::get_visible (SI& x1, SI& y1, SI& x2, SI& y2) {
  SERVER (get_visible (x1, y1, x2, y2));
  x1*=sfactor; y1*=sfactor; x2*=sfactor; y2*=sfactor;
}

SI
edit_interface::get_window_height () {
  SI x1, y1, x2, y2;
  get_visible (x1, y1, x2, y2);
  return (y2-y1);
}

void
edit_interface::scroll_to (SI x, SI y) {
  SERVER (scroll_to (x/sfactor, y/sfactor));
}

void
edit_interface::set_extents (SI x1, SI y1, SI x2, SI y2) {
  SERVER (set_extents ((x1-sfactor+1)/sfactor, (y1-sfactor+1)/sfactor,
		       (x2+sfactor-1)/sfactor, (y2+sfactor-1)/sfactor));
}

/******************************************************************************
* repainting the window
******************************************************************************/

extern int nr_painted;

void
edit_interface::draw_text (repaint_event ev) {
  ps_device dev= win->window_to_shadow (
    ev->x1/sfactor, ev->y1/sfactor,
    ev->x2/sfactor, ev->y2/sfactor);
  dev->set_shrinking_factor (sfactor);
  rectangle r (
    ev->x1+ dev->ox, ev->y1+ dev->oy,
    ev->x2+ dev->ox, ev->y2+ dev->oy);
  clear_rectangles (dev, rectangles (r));
  rectangles l;

  nr_painted=0;
  dev->set_background (dis->white);
  eb->redraw (dev, tp, l);
  draw_cursor (dev);
  draw_selection (dev);
  if (dev->check_event (EVENT_STATUS)) {
    ev->stop= TRUE;
    simplify (l);
  }
  else l= rectangles (r);

  SI extra= 3*as_int(get_init_value(FONT_BASE_SIZE)->label)*PIXEL/(2*sfactor);
  dev->set_shrinking_factor (1);
  while (!nil(l)) {
    SI x1= (l->item->x1)/sfactor -dev->ox;
    SI y1= (l->item->y1)/sfactor -dev->oy;
    SI x2= (l->item->x2)/sfactor -dev->ox;
    SI y2= (l->item->y2)/sfactor -dev->oy;
    dev->outer_round (x1, y1, x2, y2);
    win->shadow_to_window (x1-extra, y1-extra, x2+extra, y2+extra);
    l= l->next;
  }
}

void
edit_interface::draw_cursor (ps_device dev) {
  if (got_focus) {
    SI x1= cu->x0 + cu->x1 + eb->x0, y1= cu->y0 + cu->y1 + eb->y0;
    SI x2= cu->x0 + cu->x2 + eb->x0, y2= cu->y0 + cu->y2 + eb->y0;
    dev->set_line_style (pixel);
    dev->set_color (dis->red);
    dev->line (x1-pixel, y1, x1+pixel, y1);
    if (y1<=y2-pixel) {
      dev->line (x1, y1, x2, y2-pixel);
      dev->line (x2-pixel, y2-pixel, x2+pixel, y2-pixel);
    }
  }
}

void
edit_interface::draw_context (repaint_event ev) {
  int i;
  win->set_color (dis->light_grey);
  win->set_line_style (pixel);
  for (i=1; i<N(eb[0]); i++)
    win->line (ev->x1, eb[0]->y0+ eb[0][i]->y0+ eb[0][i]->y2,
	       ev->x2, eb[0]->y0+ eb[0][i]->y0+ eb[0][i]->y2);

  string medium= get_init_value (PAGE_MEDIUM)->label;
  win->fill (max (eb->x2, ev->x1), ev->y1,
	     ev->x2, min (eb->y2+ 2*pixel, ev->y2));
  if (medium == "paper")
    win->fill (ev->x1, ev->y1, ev->x2, min (eb->y1, ev->y2));
}

void
edit_interface::draw_selection (ps_device dev) {
  if (made_selection) {
    rectangles rs= selection_rects;
    while (!nil (rs)) {
      dev->set_color (dis->red);
      dev->fill (rs->item->x1, rs->item->y1, rs->item->x2, rs->item->y2);
      rs= rs->next;
    }
  }
}

void
edit_interface::handle_repaint (repaint_event ev) {
  if (env_change != 0)
    fatal_error ("Invalid situation", "edit_interface::handle_repaint");

  // cout << "Repainting\n";
  // Repaint slightly more in order to hide trace of moving cursor
  SI extra= 3*as_int(get_init_value(FONT_BASE_SIZE)->label)*PIXEL/(2*sfactor);
  event sev= ::emit_repaint
    ((ev->x1-extra)*sfactor, (ev->y1-extra)*sfactor,
     (ev->x2+extra)*sfactor, (ev->y2+extra)*sfactor, ev->stop);
  draw_text (sev);
  win->set_shrinking_factor (sfactor);
  draw_context (sev);
  draw_cursor (win);
  draw_selection (win);
  win->set_shrinking_factor (1);
  // cout << "Repainted\n";
}

/******************************************************************************
* handling changes
******************************************************************************/

void
edit_interface::notify_change (int change) {
  env_change= env_change | change;
}

bool
edit_interface::has_changed (int question) {
  return (env_change & question) != 0;
}

void
edit_interface::process_style (tree style) {
  if (style->label == "none") return;
  if (style->label != "style") style= tree ("style", style);

  int i;
  for (i=0; i<N(style); i++) {
    tree   t;
    string path= "$TEXMACS_STYLE_PATH";
    string name= style[i]->label * ".ts";
    if (!load_tree (path, name, t, FALSE)) {
      if (N(t)<4) {
	process_style (t[0]);
	typesetter T= new typesetter_rep (env);
	box the_box= T->typeset (t[2]);
	delete T;
      }
      else if (N(t)<7) {
	process_style (t[1]);
	typesetter T= new typesetter_rep (env);
	box the_box= T->typeset (t[0]);
	delete T;
      }
      else {
	process_style (t[2]);
	typesetter T= new typesetter_rep (env);
	box the_box= T->typeset (t[0]);
	delete T;
      }
    }
  }
}

void
edit_interface::cursor_visible () {
  SI x1, y1, x2, y2;
  get_visible (x1, y1, x2, y2);
  if ((cu->x0+ cu->x1+ eb->x0 <  x1) ||
      (cu->x0+ cu->x2+ eb->x0 >= x2) ||
      (cu->y0+ cu->y1+ eb->y0 <  y1) ||
      (cu->y0+ cu->y2+ eb->y0 >= y2))
    {
      scroll_to (cu->x0+ eb->x0- ((x2-x1)>>1), cu->y0+ eb->y0+ ((y2-y1)>>1));
      this << emit_invalidate_all ();
    }
}

void
edit_interface::selection_visible () {
  SI x1, y1, x2, y2;
  get_visible (x1, y1, x2, y2);
  if ((x2-x1 <= 80*pixel) || (y2-y1 <= 80*pixel)) return;

  bool scroll_x=
    (end_x+ eb->x0 <  x1+20*pixel) ||
    (end_x+ eb->x0 >= x2-20*pixel);
  bool scroll_y=
    (end_y+ eb->y0 <  y1+20*pixel) ||
    (end_y+ eb->y0 >= y2-20*pixel);
  SI new_x= x1;
  if (scroll_x) new_x= end_x+ eb->x0- ((x2-x1)>>1);
  SI new_y= y2;
  if (scroll_y) new_y= end_y+ eb->y0+ ((y2-y1)>>1);

  if (scroll_x || scroll_y) {
    scroll_to (new_x, new_y);
    this << emit_invalidate_all ();
    SI X1, Y1, X2, Y2;
    get_visible (X1, Y1, X2, Y2);
    end_x += X1- x1;
    end_y += Y1- y1;
  }
}

void
edit_interface::apply_changes () {
  if (env_change == 0) return;

  // cout << "Applying changes " << env_change << " to " << get_name() << "\n";
  // cout << "Handling selection\n";
  if (env_change & (THE_TREE+THE_ENVIRONMENT+THE_SELECTION)) {
    rectangles rs= selection_rects;
    if (made_selection)
      while (!nil (rs)) {
	invalidate (rs->item->x1-pixel, rs->item->y1-pixel,
		    rs->item->x2+pixel, rs->item->y2+pixel);
	rs= rs->next;
      }
  }

  // cout << "Handling environment\n";
  if (env_change & THE_ENVIRONMENT) {
    tree t;
    env= edit_env (buf->ref, (buf->prj==NULL? buf->ref: buf->prj->ref),
		   buf->aux, (buf->prj==NULL? buf->aux: buf->prj->aux));
    env->set_display (dis);
    process_style (the_style);
    env->env->change (init);
  }

  // cout << "Handling tree\n";
  if (env_change & (THE_TREE+THE_ENVIRONMENT)) {
    made_selection= FALSE;
    start_p= end_p= tp;
    selection_rects= rectangles ();
    SI x1, y1, x2, y2;
    typeset (x1, y1, x2, y2);
    invalidate (x1- 2*pixel, y1- 2*pixel, x2+ 2*pixel, y2+ 2*pixel);
    check_data_integrety ();
    mv= copy (eb->get_cursor (tp));
  }

  // cout << "Handling extents\n";
  if (env_change & (THE_TREE+THE_ENVIRONMENT+THE_EXTENTS))
    set_extents (eb->x1, eb->y1, eb->x2, eb->y2);

  // cout << "Cursor\n";
  if (env_change & (THE_TREE+THE_ENVIRONMENT+THE_EXTENTS+
		    THE_CURSOR+THE_FOCUS)) {
    SI P1= pixel, P2= 2*pixel;
    cu= eb->get_cursor (tp);
    if (mv_status==DIRECT) mv= copy (cu);
    if (env_change != THE_FOCUS) cursor_visible ();
    invalidate (oc->x0+ oc->x1+ eb->x0- P2, oc->y0+ oc->y1+ eb->y0- P1,
		oc->x0+ oc->x2+ eb->x0+ P2, oc->y0+ oc->y2+ eb->y0+ P1);
    invalidate (cu->x0+ cu->x1+ eb->x0- P2, cu->y0+ cu->y1+ eb->y0- P1,
		cu->x0+ cu->x2+ eb->x0+ P2, cu->y0+ cu->y2+ eb->y0+ P1);
    oc= copy (cu);
  }

  // cout << "Handling selection\n";
  if (env_change & THE_SELECTION) {
    made_selection = (start_p!=end_p);
    if (made_selection) {
      selection sel= eb->select (start_p, end_p);
      selection_rects= simplify (correct (outline (sel->rs, pixel)));
      rectangles rs= selection_rects;
      while (!nil (rs)) {
	invalidate (rs->item->x1-pixel, rs->item->y1-pixel,
		    rs->item->x2+pixel, rs->item->y2+pixel);
	rs= rs->next;
      }
    }
  }

  // cout << "Handling footer\n";
  if (env_change & THE_FOOTER)
    set_footer ();

  // cout << "Applied changes\n";
  env_change= 0;
}

/******************************************************************************
* miscellaneous routines
******************************************************************************/

bool
edit_interface::get_latex_command (string which, string& help, program& p) {
  return sv->get_latex_command (which, help, p);
}

void
edit_interface::set_main_menu (program p) {
  main_menu= p;
  if (got_focus) SERVER (menu_main (main_menu));
}

void
edit_interface::set_popup_menu (program p) {
  popup_menu= p;
}

void
edit_interface::automatic_resize () {
  if (!attached())
    fatal_error ("Window not attached", "edit_interface::handle_resize");
  SI wx, wy;
  win->get_size (wx, wy);
  init_env (PAGE_WIDTH, as_string ((wx-20*PIXEL)*sfactor) * "unit");
  init_env (PAGE_HEIGHT, as_string (wy*sfactor) * "unit");
  notify_change (THE_ENVIRONMENT);
}

/******************************************************************************
* event handlers
******************************************************************************/

void
edit_interface::handle_get_size (get_size_event ev) {
  dis->get_extents (ev->w, ev->h);
}

void
edit_interface::handle_attach_window (attach_window_event ev) {
  basic_widget_rep::handle_attach_window (ev);
  env->set_display (dis);
}

void
edit_interface::handle_resize (resize_event ev) { (void) ev;
  if (get_init_value (PAGE_MEDIUM)->label == "automatic")
    automatic_resize ();
  notify_change (THE_TREE);
}

#endmodule // code_edit_interface
