
/******************************************************************************
* MODULE     : edit_keyboard.gen.cc
* DESCRIPTION: Keyboard handling
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#module code_edit_keyboard

/******************************************************************************
* Keyboard
******************************************************************************/

void
edit_interface::key_press (string key) {
  string  mode= get_env_value (MODE)->label;
  string  lan = get_env_value (LANGUAGE (mode))->label;
  int     status;
  program keym;
  string  shorth;
  string  help;

  // ----------------- key combinations ---------------------
  sv->get_keycomb (mode, lan, sh_s * key, status, keym, shorth, help);
  if ((status & 1) == 1) {
    if (sh_len>0) {
      tp= path_add (tp, -sh_len);
      remove (et, tp, sh_len);
    }
    program p= compile (keym);
    if (is_compiled (p)) ::execute (p);
    else set_message ("Error: TeXmacs-lisp compilation error",
		      "keyboard#" * sh_s * key);
    sh_s  = string ("");
    sh_len= 0;
    return;
  }
  sv->get_keycomb (mode, lan, key, status, keym, shorth, help);
  if ((status & 1) == 1) {
    program p= compile (keym);
    if (is_compiled (p)) ::execute (p);
    else set_message ("Error: TeXmacs-lisp compilation error",
		      "keyboard#" * key);
    sh_s  = string ("");
    sh_len= 0;
    return;
  }

  // ---------- variants and uncompleted shorthands ---------
  sv->get_keycomb (mode, lan, sh_s * key, status, keym, shorth, help);
  if ((status & 2) == 0) {
    if ((key == "*") && (N(sh_s)>0) && (sh_s[N(sh_s)-1]=='*')) {
      int i;
      for (i=N(sh_s)-1; i>0; i--)
	if (sh_s[i-1]!='*') break;
      tp= path_add (tp, -sh_len);
      if (sh_len>0) remove (et, tp, sh_len);
      sh_s= sh_s (0, i);
      sv->get_keycomb (mode, lan, sh_s, status, keym, shorth, help);
      if (N(shorth)>0) insert (shorth);
      sh_len= N (shorth);
      if (N(help)>0) set_message (help, sh_s);
      return;
    }
    else {
      sh_s  = string ("");
      sh_len= 0;
    }
  }

  // -------------------- shorthands ------------------------
  sv->get_keycomb (mode, lan, sh_s * key, status, keym, shorth, help);
  if ((status & 2) == 2) {
    sh_s  = sh_s * key;
    if (sh_len>0) {
      tp= path_add (tp, -sh_len);
      if (sh_len>0) remove (et, tp, sh_len);
    }
    if (N(shorth)>0) insert (shorth);
    sh_len= N (shorth);
    if (N(help)>0) set_message (help, sh_s);
    return;
  }

  // ----------------------- default ------------------------
  if (N(key)==1) {
    int i ((unsigned char) key[0]);
    if (((i>=32) && (i<=127)) ||
	((i>=160) && (i <=255))) insert (key);
    sh_s  = string ("");
    sh_len= 0;
  }
}

void
edit_interface::emulate_keyboard (string keys, string action) {
  int i;
  string keys_s;
  for (i=0; i<N(keys); i++)
    if (keys[i] == '<') {
      int start=i;
      for (; (i<N(keys)) && (keys[i]!='>'); i++);
      string s= keys (start, i+1);
      key_press (s);
      if (s == "<less>") keys_s << "< ";
      else if (s == "<gtr>") keys_s << "> ";
      else keys_s << s << " ";
    }
    else {
      key_press (keys (i, i+1));
      keys_s << keys[i] << " ";
    }
  keys_s= keys_s (0, N(keys_s)-1);
  if (N(action) != 0)
    set_message ("You can also obtain#" * action *
		 "#by typing#" * keys_s, action);
}

/******************************************************************************
* Event handlers
******************************************************************************/

void
edit_interface::show_keymaps () {
  fatal_error ("no longer supported", "edit_interface::show_keymaps");
}

void
edit_interface::handle_keypress (keypress_event ev) {
  key_press (ev->key);
  if (attached ())
    if (!win->check_event (INPUT_EVENT))
      notify_change (THE_FOOTER);
}

void
edit_interface::handle_keyboard_focus (keyboard_focus_event ev) {
  got_focus= ev->flag;
  notify_change (THE_FOCUS);
  if (got_focus) {
    focus_on_this_editor ();
    SERVER (menu_main (main_menu));
  }
}

#endmodule // code_edit_interface
