
/******************************************************************************
* MODULE     : editor.gen.cc
* DESCRIPTION: routines for the editor
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <data.gen.h>
#include <edit_main.gen.h>
#include <tm_widget.gen.h>
#include <tm_buffer.gen.h>
#include <file.gen.h>
#include <dir.gen.h>
#include <PsDevice/printer.gen.h>
#include <PsDevice/page_type.gen.h>
#include <convert.gen.h>
#include <connect.gen.h>

#module code_edit_main
#import data
#import edit_main
#import tm_widget
#import tm_buffer
#import file
#import dir
#import printer
#import page_type
#import convert
#import connect

#include <dlfcn.h>
#include <setjmp.h>

extern void   initialize_data ();
extern string tree_to_tex (tree t);
extern tree   compile_lisp (compiler cpl, tree t);
extern tree   eval_lisp (compiler cpl, tree t);
extern string eval_scheme (string s);

/******************************************************************************
* Constructors and destructor
******************************************************************************/

edit_base::edit_base (tm_buffer buf2): buf (buf2), et (buf2->t) {}

edit_main::edit_main (server_rep* sv, display dis, tm_buffer buf):
  edit_base (buf), edit_modify (buf),
  edit_cursor (buf), edit_typeset (buf), edit_interface (sv, dis, buf)
{
  initialize_data ();
  notify_change (THE_TREE);
  tp= correct_cursor (et, path (0));
}

edit_main::~edit_main () {}

/******************************************************************************
* Global routines
******************************************************************************/

void
edit_main::clear_buffer () {
  assign (et, path (), tree (DOCUMENT, tree ("")));
}

void
edit_main::new_window () {
}

void
edit_main::clone_window () {
}

void
edit_main::tex_buffer () {
  // cout << tree_to_tex (et) << "\n";
  // dis->set_selection (widget (this), tree_to_tex (et));
}

string
edit_main::get_name () {
  return buf->name;
}

void
edit_main::focus_on_this_editor () {
  sv->focus_on_editor (this);
}

/******************************************************************************
* Printing
******************************************************************************/

void
edit_main::set_page_parameters () {
  string medium     = get_init_value (PAGE_MEDIUM)->label;
  string type       = get_init_value (PAGE_TYPE)->label;
  string orientation= get_init_value (PAGE_ORIENTATION)->label;
  bool   landscape  = (orientation == "landscape");

  if (medium == "automatic") {
    init_env (PAGE_ODD, "5mm");
    init_env (PAGE_EVEN, "5mm");
    init_env (PAGE_RIGHT, "5mm");
    init_env (PAGE_TOP, "5mm");
    init_env (PAGE_BOT, "5mm");
    automatic_resize ();
    return;
  }
  if (type == "user") return;

#define PAGE_INIT(feature) \
  init_env (feature, page_get_feature (type, feature, landscape))
  PAGE_INIT (PAR_WIDTH);
  PAGE_INIT (PAGE_ODD);
  PAGE_INIT (PAGE_EVEN);
  PAGE_INIT (PAGE_RIGHT);
  PAGE_INIT (PAGE_TOP);
  PAGE_INIT (PAGE_BOT);
  PAGE_INIT (PAGE_REDUCE_LEFT);
  PAGE_INIT (PAGE_REDUCE_RIGHT);
  PAGE_INIT (PAGE_REDUCE_TOP);
  PAGE_INIT (PAGE_REDUCE_BOT);
#undef PAGE_INIT
}

void
edit_main::set_page_medium (string medium) {
  init_env (PAGE_MEDIUM, medium);
  set_page_parameters ();
}

void
edit_main::set_page_type (string type) {
  init_env (PAGE_TYPE, type);
  set_page_parameters ();
}

void
edit_main::set_page_orientation (string orientation) {
  init_env (PAGE_ORIENTATION, orientation);
  set_page_parameters ();
}

/******************************************************************************
* Printing
******************************************************************************/

static string printing_dpi ("600");
static string printing_cmd ("lpr");
static string printing_on ("a4");

void
edit_main::set_printing_command (string cmd) {
  printing_cmd= cmd;
}

void
edit_main::set_printer_page_type (string type) {
  printing_on= type;
}

void
edit_main::set_printer_dpi (string dpi) {
  printing_dpi= dpi;
}

void
edit_main::print (string ps_name, bool conform, int first, int last) {
  string medium = env->get_string (PAGE_MEDIUM);
  if ((!conform) && (medium == "automatic")) {
    set_message (
      "Error: you should switch to ``paper'' or ``papyrus'' page type",
      "print");
    return;
  }

  /* Set environment variables for printing */

  if (!conform) env->set (PAGE_MEDIUM, "paper");
  env->set (DPI, printing_dpi);
  env->set (SFACTOR, "1");
  env->set (PAGE_REDUCE_LEFT, "0cm");
  env->set (PAGE_REDUCE_RIGHT, "0cm");
  env->set (PAGE_REDUCE_TOP, "0cm");
  env->set (PAGE_REDUCE_BOT, "0cm");
  env->set (PAGE_SHOW_HF, "true");

  /* Typeset pages for printing */

  env->style_init_env ();
  env->update ();
  env->save ();
  typesetter T= new typesetter_rep (env);
  box the_box= T->typeset (et);
  delete T;
  env->recover ();

  /* Determine parameters for printer device */

  string page_type = printing_on;
  double w         = env->get_length (PAGE_WIDTH);
  double h         = env->get_length (PAGE_HEIGHT);
  double cm        = env->decode_length ("1cm");
  bool   landsc    = (env->get_string (PAGE_ORIENTATION) == "landscape");
  int    dpi       = as_int (printing_dpi);
  int    start     = max (0, first-1);
  int    end       = min (N(the_box[0]), last);
  int    pages     = end-start;
  if (conform) {
    page_type= "user";
    SI bw= the_box[0][0]->w();
    SI bh= the_box[0][0]->h();
    string bws= as_string (bw) * "unit";
    string bhs= as_string (bh) * "unit";
    w= env->decode_length (bws);
    h= env->decode_length (bhs);
  }

  /* Print pages */

  int i;
  ps_device dev=
    printer (dis, ps_name, dpi, pages, page_type, landsc, w/cm, h/cm);
  for (i=start; i<end; i++) {
    rectangles rs;
    box b= the_box[0][i];
    b->x0= b->y0= 0;
    b->redraw (dev, path (0), rs);
    if (i<end-1) dev->next_page ();
  }
  delete dev;

  /* Reset environment */

  env->reset (DPI);
  env->reset (SFACTOR);
  env->reset (PAGE_REDUCE_LEFT);
  env->reset (PAGE_REDUCE_RIGHT);
  env->reset (PAGE_REDUCE_TOP);
  env->reset (PAGE_REDUCE_BOT);
  env->reset (PAGE_SHOW_HF);
  if (!conform) env->reset (PAGE_MEDIUM);
  env->style_init_env ();
  env->update ();
}

void
edit_main::print_to_file (string ps_name, string first, string last) {
  print (ps_name, FALSE, as_int (first), as_int (last));
}

void
edit_main::print_buffer (string first, string last) {
  string temp= get_temp_file_name () * ".ps";
  print (temp, FALSE, as_int (first), as_int (last));
  system (printing_cmd * " " * temp);
  system ("rm " * temp);
}

void
edit_main::export_ps (string ps_name, string first, string last) {
  print (ps_name, TRUE, as_int (first), as_int (last));
}

/******************************************************************************
* Evaluation of expressions
******************************************************************************/

program
edit_main::compile (program p) {
  compiler& cpl (sv->get_compiler());
  return cpl->filter (cpl->compile (p), type ("Void"), p);
}

void
edit_main::execute_edit_lisp (string s) {
  compiler& cpl (sv->get_compiler());
  program p= lisp_expr_to_program (s, 0);
  list<typed_program> ltp= cpl->compile (p);
  if (nil (ltp))
    set_message ("Error: compilation failed", "evaluate expression");
  else if (!atom (ltp))
    set_message ("Error: Ambiguous expression", "evaluate expression");
  else {
    program P (ltp->item->p);
    type    T (ltp->item->t);
    program Q;
    if (is_string (T, "Void")) Q= P;
    else Q= cpl->coerce (P, T, "Tree");
    tree* ptr= (tree*) ::execute (Q);
    if (is_string (T, "Void")) {
      set_message ("", "Void");
      return;
    }
    tree u= *ptr;
    delete ptr;
    set_message (var_tree_to_string (u), var_tree_to_string (T));
  }
}

tree
edit_main::the_line () {
  return copy (subtree (et, inner_paragraph (et, path_up (tp))));
}

tree
edit_main::the_buffer () {
  return copy (et);
}

tree
edit_main::prog_eval (tree t) {
  string lan= get_env_value (PROG_LANGUAGE)->label;
  if (lan == "shell") return shell_eval (t);
  if (lan == "scheme") return scheme_eval (t);
  if (lan == "TeXmacs lisp") return edit_lisp_eval (t);
  if (!package_declared (lan)) {
    set_message ("Programming language not declared",
		 "Evaluate#'" * lan * "'#expression");
  }
  else {
    string  s1= edit_to_string (t);
    string  s2= sv->evaluate (lan, "default", s1); if (s2=="") return "";
    return  string_to_edit (s2);
  }
}

tree
edit_main::shell_eval (tree t) {
  string s= edit_to_verbatim (t);
  string e= eval_system (s);
  return verbatim_to_edit (e);
}

tree
edit_main::scheme_eval (tree t) {
  string s= edit_to_verbatim (t);
  string e= eval_scheme (s);
  return verbatim_to_edit (e);
}

tree
edit_main::edit_lisp_compile (tree t) {
  compiler& cpl (sv->get_compiler());
  return compile_lisp (cpl, t);
}

tree
edit_main::edit_lisp_eval (tree t) {
  compiler& cpl (sv->get_compiler());
  return eval_lisp (cpl, t);
}

/******************************************************************************
* Miscellaneous
******************************************************************************/

void
edit_main::show_tree () {
  cout << et << "\n";
}

void
edit_main::show_env () {
  cout << env->env;
}

void
edit_main::show_path () {
  cout << tp << "\n";
}

void
edit_main::show_cursor () {
  cout << "Principal cursor: "
       << cu->x0 << ", " << cu->y0 << " [" << cu->delta << "]\n";
  cout << "Ghost cursor    : "
       << mv->x0 << ", " << mv->y0 << " [" << mv->delta << "]\n";
}

void
edit_main::show_selection () {
  selection sel= eb->select (start_p, end_p);
  cout << "physical selection: " << start_p << " --- " << end_p << "\n";
  cout << "logical selection : " << sel->start << " --- " << sel->end << "\n";
}

void
edit_main::show_meminfo () {
  mem_info ();
}

void
edit_main::edit_special () {
}

void
edit_main::edit_test () {
  bool error_flag;
  tree t= directory_tree (".", error_flag, TRUE);
  cout << make_path (t) << "\n";
  // cout << "Test !\n";
}

#endmodule // code_edit_main
