
/******************************************************************************
* MODULE     : edit_mouse.gen.cc
* DESCRIPTION: Mouse handling
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#module code_edit_mouse

/******************************************************************************
* Routines for the mouse
******************************************************************************/

void
edit_interface::mouse_press (string s, SI x, SI y) {
  if (s == "<mouse-select>")
    mouse_select (x, y);
  if (s == "<mouse-paste>")
    mouse_paste (x, y);
  if (s == "<mouse-adjust>")
    mouse_adjust (x, y);
}

void
edit_interface::mouse_select (SI x, SI y) {
  set_cursor (x, y);
}

void
edit_interface::mouse_paste (SI x, SI y) { (void) x; (void) y;
  selection_copy ();
  selection_paste ();
}

void
edit_interface::mouse_drag (SI x, SI y) {
  end_x  = x;
  end_y  = y;
  selection_visible ();
  start_p= tree_path (start_x- eb->x0, start_y- eb->y0, 0);
  end_p  = tree_path (end_x  - eb->x0, end_y  - eb->y0, 0);
  if ((start_p==end_p) && start_drag) return;
  start_drag= FALSE;
  if (path_inf (end_p, start_p)) {
    path temp= start_p;
    start_p= end_p;
    end_p= temp;
  }
  notify_change (THE_SELECTION);
}

void
edit_interface::mouse_adjust (SI x, SI y) {
  x /= sfactor; y /= sfactor;
  abs_round (x, y);
  if (popup_win == NULL) {
    SI wx, wy;
    win->get_position (wx, wy);
    widget wid; SERVER (menu_widget (popup_menu, wid));
    widget popup_wid= popup_widget (wid, center);
    popup_win= popup_window (popup_wid, wx+ ox+ x, wy+ oy+ y);
    popup_win->map ();
    this << emit_mouse_grab (TRUE);
    popup_wid << set_integer ("grabbed", 1);
    popup_wid << emit_mouse_grab (TRUE);
  }
}

/******************************************************************************
* Copy and paste
******************************************************************************/

void
edit_interface::selection_set_start () {
  start_p= tp;
  if (path_var_inf_eq (end_p, start_p)) end_p= start_p;
  notify_change (THE_SELECTION);
}

void
edit_interface::selection_set_end () {
  end_p= tp;
  if (path_var_inf_eq (end_p, start_p)) start_p= end_p;
  notify_change (THE_SELECTION);
}

bool
edit_interface::selection_active () {
  return made_selection;
}

void
edit_interface::selection_get (path& start, path& end) {
  selection sel= eb->select (start_p, end_p);
  start= sel->start;
  end  = sel->end;
}

void
edit_interface::selection_set (tree t) {
  string mode= get_env_value (MODE)->label;
  string lan = get_env_value (LANGUAGE (mode))->label;
  tree sel ("edit", t, mode, lan);
  string s= edit_to_latex (t, mode);
  if (is_func (sel[1], "math", 0)) s= "$" * s * "$";
  if (dis->set_selection (widget (this), sel, s)) {
    end_p= start_p;
    notify_change (THE_SELECTION);
  }
}

void
edit_interface::selection_copy () {
  if (made_selection) selection_set (the_selection ());  
}

void
edit_interface::selection_paste () {
  tree t= copy (dis->get_selection (widget (this)));
  // if (is_func (t, "none", 0)) return;
  if (is_func (t, "extern", 1))
    insert (latex_to_edit (t[0]->label, get_env_value (MODE)->label));
  if (is_func (t, "edit", 3)) {
    string mode= get_env_value (MODE)->label;
    string lan = get_env_value (LANGUAGE (t[1]->label))->label;
    if (t[1] != mode)
      insert_tree (tree (CONCAT, tree (SET, copy (MODE), copy (t[1])),
			 tree (RESET, copy (MODE))), path (0, path (1)));
    if (t[2] != lan) {
      string SEL_LAN= copy (LANGUAGE (t[1]->label));
      insert_tree (tree (CONCAT, tree (SET, copy (SEL_LAN), copy (t[2])),
			 tree (RESET, copy (SEL_LAN))), path (0, path (1)));
    }
    insert (t[0]);
  }
}

/******************************************************************************
* event handlers
******************************************************************************/

void
edit_interface::handle_mouse (mouse_event ev) {
  string type= ev->type;
  SI     x   = ev->x*sfactor;
  SI     y   = ev->y*sfactor;

  if ((popup_win != NULL) && (type != "leave")) {
    popup_win->map ();
    delete popup_win;
    popup_win= NULL;
    this << emit_mouse_grab (FALSE);
  }
  if (type == "press-left") {
    start_x   = x;
    start_y   = y;
    start_drag= TRUE;
    dragging  = TRUE;
    this << emit_mouse_grab (TRUE);
  }
  if (dragging && (type == "move")) {
    if (attached () && win->check_event (DRAG_EVENT)) return;
    mouse_drag (x, y);
  }
  if (type == "release-left") {
    dragging= FALSE;
    this << emit_mouse_grab (FALSE);
    mouse_select (x, y);
  }
  if (type == "press-middle") mouse_paste (x, y);
  if (type == "press-right") mouse_adjust (x, y);

  if ((type == "press-left") ||
      (type == "release-left") ||
      (type == "press-middle") ||
      (type == "press-right"))
    notify_change (THE_FOOTER);
}

#endmodule // code_edit_mouse
