
/******************************************************************************
* MODULE     : typeset.gen.cc
* DESCRIPTION: typeset the tree being edited
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <data.gen.h>
#include <edit_typeset.gen.h>
#include <Document/document.gen.h>
#include <Atom/typesetter.gen.h>
#include <tm_buffer.gen.h>

#module code_edit_typeset
#import data
#import edit_typeset
#import document
#import typesetter
#import tm_buffer

/******************************************************************************
* Contructors, destructors and notification of modifications
******************************************************************************/

edit_typeset::edit_typeset (tm_buffer buf):
  edit_base (buf), the_style ("style"), init ("?"), fin ("?"),
  env (buf->ref, (buf->prj==NULL? buf->ref: buf->prj->ref),
       buf->aux, (buf->prj==NULL? buf->aux: buf->prj->aux)),
  doc (new document_rep (et, env)) {}
edit_typeset::~edit_typeset () { delete doc; }
void edit_typeset::typeset_notify_modify (int pos) {
  doc->notify_modify (pos); }
void edit_typeset::typeset_notify_assign (int nr) {
  doc->notify_assign (nr); }
void edit_typeset::typeset_notify_insert (int pos, int nr) {
  doc->notify_insert (pos, nr); }
void edit_typeset::typeset_notify_remove (int pos, int nr) {
  doc->notify_remove (pos, nr); }

/******************************************************************************
* Routines for getting information
******************************************************************************/

tree
edit_typeset::get_env_value (string var) {
  return doc->get_env (var, tp);
}

tree
edit_typeset::get_init_value (string var) {
  if (init->contains (var)) return init [var];
  return doc->get_env (var, path (0));
}

/******************************************************************************
* Initialization
******************************************************************************/

void
edit_typeset::init_style (string name) {
  int i;
  if ((name == "none") || (name == "style")) the_style= "style";
  else the_style= tree ("style", name);
  for (i=0; i<N(et); i++)
    doc->notify_modify (i);
  notify_change (THE_ENVIRONMENT);
  buf->modification_flag= TRUE;
}

void
edit_typeset::init_extra_style (string name) {
  int i;
  the_style << tree (name);
  for (i=0; i<N(et); i++)
    doc->notify_modify (i);
  notify_change (THE_ENVIRONMENT);
  buf->modification_flag= TRUE;
}

void
edit_typeset::init_env (hashmap<string,tree> H) {
  int i;
  init= hashmap<string,tree> ("?");
  init->join (H);
  for (i=0; i<N(et); i++)
    doc->notify_modify (i);
  notify_change (THE_ENVIRONMENT);
}

void
edit_typeset::init_env (string var, tree by) {
  int i;
  if (init (var) == by) return;
  init (var)= by;
  for (i=0; i<N(et); i++)
    doc->notify_modify (i);
  notify_change (THE_ENVIRONMENT);
}

void
edit_typeset::init_default (string var) {
  int i;
  if (!init->contains (var)) return;
  init->reset (var);
  for (i=0; i<N(et); i++)
    doc->notify_modify (i);
  notify_change (THE_ENVIRONMENT);
}

/******************************************************************************
* Actual typesetting
******************************************************************************/

void
edit_typeset::typeset (SI& x1, SI& y1, SI& x2, SI& y2) {
  env->style_init_env ();
  env->update ();
  eb= doc->typeset (x1, y1, x2, y2);
}

void
edit_typeset::reprocess_all () {
  int i;
  for (i=0; i<N(et); i++)
    doc->notify_modify (i);
  notify_change (THE_ENVIRONMENT);
}

#endmodule // code_edit_typeset
