
/******************************************************************************
* MODULE     : correct.gen.cc
* DESCRIPTION: maintains the correctness of the data in the math-editor
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <data.gen.h>
#include <edit_modify.gen.h>

#module code_correct
#import data
#import edit_modify

/******************************************************************************
* Correction routines for abstract trees
******************************************************************************/

tree math_correct (tree);

void
process_concat (tree& r, tree t) {
  int i;
  tree u (t->label, N(t));
  for (i=0; i<N(t); i++)
    u[i]= math_correct (t[i]);
  for (i=0; i<N(u); i++) {   
    if (is_concat (u[i])) process_concat (r, u[i]);
    else if ((N(u[i])==0) && (N(r)>0) && (N(r[N(r)-1])==0))
      r[N(r)-1]= tree (r[N(r)-1]->label * u[i]->label);
    else r << u[i];
  }
}

tree
math_correct (tree t) {
  int i;
  if (N(t)==0) return t;
  if (is_concat (t)) {
    tree r (CONCAT);
    process_concat (r, t);
    if (N(r)==0) return tree ("");
    if (N(r)==1) return r[0];
    return r;
  }

  tree r (t->label, N(t));
  for (i=0; i<N(t); i++) r[i]= math_correct (t[i]);

  return r;
}

/*****************************************************************************/
// Correction routines for the edit tree
/*****************************************************************************/

void
edit_modify::correct_concat (path p, int done) {
  tree t (subtree (et, p));
  if (t->label!=CONCAT)
    fatal_error ("concat expected", "ME::correct_concat");
  if (N(t)==0) { assign (et, p, string ("")); return; }
  if (N(t)==1) { rem_unary (et, p); return; }
  int i;
  for (i=done; i<N(t); i++) {
    if (is_func (t[i], "", 0)) {
      remove (et, path_down (p, i), 1);
      correct_concat (p, i);
      return;
    }
    if ((i<N(t)-1) && (N(t[i])==0) && (N(t[i+1])==0)) {
      join (et, path_down (p, i));
      correct_concat (p, i);
      return;
    }
    if (is_concat (t[i])) {
      ins_unary (et, p, CONCAT);
      split (et, path_down (path_down (p, 0), i));
      split (et, path_down (path_down (p, 1), 1));
      rem_unary (et, path_down (p, 1));
      if (subtree (et, path_down (p, 0)) == tree (CONCAT))
	remove (et, path_down (p, 0), 1);
      else join (et, path_down (p, 0));
      if (subtree (et, path_down (p, 1)) == tree (CONCAT))
	remove (et, path_down (p, 1), 1);
      else join (et, path_down (p, 0));
      rem_unary (et, p);
    }
  }
}

void
edit_modify::correct (path p) {
  tree t (subtree (et, p));
  if (is_concat (t)) correct_concat (p);
  if (is_func (t, "", 0)) correct (path_up (p));
}

/*****************************************************************************/
// Checks for data consistency
/*****************************************************************************/

void
warning (tree t, char* message) {
  cerr << "Warning: " << message << "\n";
  cerr << "--------------------------------------------------------------------------\n";
  cerr << t << "\n";
  cerr << "--------------------------------------------------------------------------\n";
}

void
check_correct (tree t) {
  if (N(t)==0) {
    if (t->label==CONCAT)
      warning (t, "empty concat");
  }
  else {
    int i;
    if (is_concat (t)) {
      if (N(t)==1) warning (t, "single concat");
      for (i=0; i<N(t); i++)
	if (is_concat (t[i]))
	  warning (t, "nested concat");
    }
    for (i=0; i<N(t); i++) check_correct (t[i]);
  }
}

void
edit_modify::check_data_integrety () {
  check_correct (et);
}

#endmodule // code_correct
