
/******************************************************************************
* MODULE     : modify_aux.gen.cc
* DESCRIPTION: incorporate automatically generated data into text
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <file.gen.h>
#include <convert.gen.h>
#include <tm_buffer.gen.h>

#module code_modify_aux
#import file
#import convert
#import tm_buffer

/******************************************************************************
* Saerch an environment
******************************************************************************/

bool
search_environment (tree t, path p, path& begin, path& end) {
  while (!nil (p)) {
    tree u= subtree (t, p);
    if (is_func (u, BEGIN) && (u[0] == "bibliography")) {
      begin= p;
      end  = right_match (t, p, u);
      if (!nil (end)) return TRUE;
    }
    p= increment (t, p);
  }
  return FALSE;
}

/******************************************************************************
* Automatically generate auxiliairy data and replace in text
******************************************************************************/

void
edit_modify::generate_bibliography (string bib, string style, string fname) {
  if (debug (0)) {
    cout << "TeXmacs] Generating bibliography\n";
    cout << "TeXmacs]   Tag  = " << bib << "\n";
    cout << "TeXmacs]   Style= " << style << "\n";
    cout << "TeXmacs]   File = " << fname << "\n";
  }

  int i;
  string bib_s= "\\bibstyle{" * style * "}\n";
  tree   bib_t= ((buf->prj==NULL)? buf->aux[bib]: buf->prj->aux[bib]);
  for (i=0; i<N(bib_t); i++)
    bib_s << "\\citation{" << bib_t[i]->label << "}\n";
  string rel_name= get_relative_file_name (buf->name, fname, FALSE);
  if ((N(rel_name) >= 4) && (rel_name (N(rel_name)-4, N(rel_name)) == ".bib"))
    rel_name= rel_name (0, N(rel_name)- 4);
  bib_s << "\\bibdata{" << rel_name << "}\n";
  save_string ("$TEXMACS_HOME_PATH/system/temp.aux", bib_s);
  system ("cd $TEXMACS_HOME_PATH/system; bibtex temp");
  string result;
  if (load_string ("$TEXMACS_HOME_PATH/system/temp.bbl", result)) {
    set_message ("Error: bibtex failed to create bibliography",
	     "compile bibliography");
    return;
  }

  int count=1;
  tree t= latex_to_edit (result, "text");
  tree u (DOCUMENT);
  for (i=0; i<N(t); i++)
    if (is_concat (t[i]) &&
	is_func (t[i][0], APPLY) &&
	((t[i][0][0]->label == "bibitem") ||
	 (t[i][0][0]->label == "bibitem*")))
      {
	tree item= t[i][0];
	if (item[0]->label == "bibitem")
	  item= tree (APPLY, "bibitem*", as_string (count++), item[1]);
	t[i][0]= item;
	u << t[i];
      }
  if (N(u)>0) insert_document (u);
}

void
edit_modify::generate_aux () {
  path p= start_forward (et);
  path bp, ep;

  while (search_environment (et, p, bp, ep)) {
    tree t= subtree (et, bp);
    int first_ln= bp->item;
    int nr_lines= ep->item- bp->item;

    set_cursor (path_down (bp, 1));
    insert_return ();
    bp= path (first_ln+1);
    if (is_concat (subtree (et, bp))) bp= path_down (bp, 0);
    bp= decrement (et, bp);
    ep= right_match (et, bp, t);
    set_cursor (path_down (ep, 0));
    insert_return ();
    remove (et, path (first_ln+1), nr_lines+1);
    set_cursor (start (et, path (first_ln+1)));
    backspace ();

    /*
    cout << "et= " << et << "\n";
    cout << "tp= " << tp << "\n";
    cout << "----------------------------------------------------------\n";
    */
    if (N(t)>=4) generate_bibliography (t[1]->label, t[2]->label, t[3]->label);
    /*
    cout << "et= " << et << "\n";
    cout << "tp= " << tp << "\n";
    cout << "----------------------------------------------------------\n\n\n";
    */

    p= path (tp->item==first_ln? tp->item+1: tp->item);
    if (p->item >= N(et)) return;
    if (is_concat (subtree (et, p))) p= path_down (p, 0);
  }
}

#endmodule // code_modify_aux
