
/******************************************************************************
* MODULE     : modify_del.gen.cc
* DESCRIPTION: treat deletions
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <env.gen.h>

#module code_modify_del
#import env

/******************************************************************************
* Deletion of one character
******************************************************************************/

void
edit_modify::backspace () {
  path p= tp;
  int  last, node;
  tree t, u;

  /********************* get the position where to delete ********************/
  
  do {
    last= last_item (p);
    p   = path_up (p);
    t   = subtree (et, p);
  } while ((last==0) && (!nil (p)) && is_format (subtree (et, path_up (p))));
  if (nil (p)) {
    if (last==0) return;
    remove_return (path (last-1));
    return;
  }
  node= last_item (p);
  u= subtree (et, path_up (p));

  // cout << "Backspace\n-----------------------";
  // cout << "t= " << t << "\n";
  // cout << "u= " << u << "\n";
  // cout << "p= " << p << "\n";

  /**************************** deleting text ********************************/

  if ((N(t)==0) && (last!=0)) {
    string mode= get_env_value (MODE)->label;
    language lan;
    if (mode == "text")
      lan= text_language (get_env_value (TEXT_LANGUAGE)->label);
    else if (mode == "math")
      lan= math_language (get_env_value (MATH_LANGUAGE)->label);
    else
      lan= prog_language (get_env_value (PROG_LANGUAGE)->label);
    int start= last;
    if (lan->enc->token_backward (t->label, start))
      fatal_error ("bad cursor position in string", "edit_modify::backspace");
    remove (et, path_down (p, start), last-start);
    correct (path_up (p));
    return;
  }

  /************************ deletion governed by t ***************************/

  if (last==1)
    switch (math_primary [t->label]) {
    case Format:
    case Hspace:
    case Vspace_before:
    case Vspace_after:
    case Space:
    case Htab:
    case Split:
      if (!is_concat (subtree (et, path_up (p)))) assign (et, p, "");
      else remove (et, p, 1);
      correct (path_up (p));
      return;
    case Move:
    case Resize:
      assign (et, p, subtree (et, path_down (p, 0)));
      set_cursor (end (et, p));
      correct (path_up (p));
      return;
    case Left_sub:
    case Left_sup:
    case Right_sub:
    case Right_sup:
      set_cursor (end (et, path_down (p, 0)));
      return;
    case Frac:
      set_cursor (end (et, path_down (p, 1)));
      return;
    case Sqrt:
    case Wide:
    case Neg:
      set_cursor (end (et, path_down (p, 0)));
      return;
    case Tree:
      set_cursor (end (et, path_down (p, N(t)-1)));
      return;
    case Matrix:
    case Table:
      set_cursor (end (et, path_down (p, N(t)-3)));
      return;
    case Mosaic:
      mosaic_set_cursor (path_down (p, N(t)-3));
      return;
    case Inactive:
      set_cursor (end (et, path_down (p, N(t)-1)));
      return;
    case Symbol:
    case Latex:
    case Hybrid:
    case Quote:
    case Assign:
      back_dynamic (p);
      return;
    case Set:
      back_env (u, p, RESET);
      return;
    case Reset:
      set_cursor (start (et, p));
      return;    
    case Apply:
      back_dynamic (p);
      return;    
    case Begin:
      back_env (u, p, END);
      return;
    case End:
      set_cursor (start (et, p));
      return;
    case Function:
    case Environment:
    case Label:
    case Reference:
    case Write:
    case Plus:
    case Minus:
    case Times:
    case Merge:
    case Number:
    case Translate:
      back_dynamic (p);
      return;
    case Graphics:
    case Point:
    case Line:
    case Arc:
    case Bezier:
      return;
    case Postscript:
      back_dynamic (p);
      return;
    }

  /************************* deletion depends on u ***************************/

  if (last==0) {
    switch (math_primary [u->label]) {
    case Split:
      back_in_dynamic (u, p, 1);
      return;
    case Move:
    case Resize:
      set_cursor (start (et, path_up (p)));
      return;
    case Frac:
      back_in_math (u, p);
      return;
    case Wide:
      back_in_math_accent (u, p);
      return;
    case Left_sub:
    case Left_sup:
    case Right_sub:
    case Right_sup:
      back_in_math (u, p);
      return;
    case Sqrt:
      back_in_dynamic (u, p, 1);
      return;    
    case Neg:
      back_in_math (u, p);
      return;
    case Tree:
      back_in_tree (u, p);
      return;
    case Matrix:
    case Table:
      back_in_matrix (u);
      return;
    case Mosaic:
    case Mosaic_item:
      back_in_mosaic ();
      return;
    case Inactive:
    case Symbol:
    case Latex:
    case Hybrid:
    case Quote:
      back_in_dynamic (u, p, 1);
      return;
    case Assign:
      back_in_dynamic (u, p, 2);
      return;    
    case Set:
    case Apply:
    case Begin:
    case Function:
      back_in_dynamic (u, p, 1);
      return;    
    case Environment:
      back_in_environment (u, p);
      return;
    case Label:
    case Reference:
    case Write:
      back_in_dynamic (u, p, 1);
      return;
    case Plus:
    case Minus:
    case Times:
    case Merge:
    case Number:
      back_in_dynamic (u, p, 2);
      return;
    case Translate:
      back_in_dynamic (u, p, 3);
      return;
    case Graphics:
    case Point:
    case Line:
    case Arc:
    case Bezier:
      return;
    case Postscript:
      back_in_dynamic (u, p, 7);
      return;
    }
  }
}

/******************************************************************************
* Deletion of an object
******************************************************************************/

path left_match (tree t, path p, tree which, int level= 0);
path right_match (tree t, path p, tree which, int level= 0);

void
edit_modify::remove_object () {
  path p= tp;
  int  last, node;
  tree t, u;

  /********************* get the position where to delete ********************/
  
  do {
    last= last_item (p);
    p   = path_up (p);
    t   = subtree (et, p);
  } while ((last==0) && (!nil (p)) && is_format (subtree (et, path_up (p))));
  if (nil (p)) {
    if (last==0) return;
    remove_return (path (last-1));
    return;
  }
  node= last_item (p);
  u= subtree (et, path_up (p));

  /**************************** deleting text ********************************/

  if ((N(t)==0) && (last!=0)) {
    string mode= get_env_value (MODE)->label;
    language lan;
    if (mode == "text")
      lan= text_language (get_env_value (TEXT_LANGUAGE)->label);
    else if (mode == "math")
      lan= math_language (get_env_value (MATH_LANGUAGE)->label);
    else
      lan= prog_language (get_env_value (PROG_LANGUAGE)->label);
    int start= last, end= last;
    string s= t->label;
    while (TRUE) {
      int pos= max (start-1, 0), d1;
      space d2;
      lan->advance (s, pos, d2, d2, d2, d2, d1, d1);
      while ((pos < N(s)) && (s[pos] == ' ')) pos++;
      if (pos < last) break;
      end= pos;
      if (start == 0) break;
      start--;
    }
    remove (et, path_down (p, start), end-start);
    correct (path_up (p));
    return;
  }

  /************************ deletion governed by t ***************************/

  if (last==1)
    switch (math_primary [t->label]) {
    case Set:
    case Reset:
    case Begin:
    case End:
      rem_env (t, p);
      return;
    default:
      if (!is_concat (u)) assign (et, p, "");
      else remove (et, p, 1);
      correct (path_up (p));
      return;
    }

  /************************* deletion depends on u ***************************/

  if (last==0)
    switch (math_primary [t->label]) {
    case Mosaic:
    case Mosaic_item:
      return;
    default:
      assign (et, path_up (p), subtree (et, p));
      set_cursor (end (et, path_up (p)));
      correct (path_up (p, 2));
      return;
    }
}

#endmodule // code_modify_del
