
/******************************************************************************
* MODULE     : modify_dynamic.gen.cc
* DESCRIPTION: modify the tree being edited
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#module code_modify_dynamic

/******************************************************************************
* Routines for potentially deactivated structures
******************************************************************************/

path
edit_modify::find_dynamic (path p) {
  path parent= path_up (p);
  if (nil (parent)) return parent;
  if (is_dynamic (subtree (et, parent))) return p;
  return find_dynamic (parent);
}

path
edit_modify::find_deactivated (path p) {
  path parent= path_up (p);
  if (nil (parent)) return parent;
  if (is_func (subtree (et, parent), INACTIVE)) return parent;
  return find_deactivated (parent);
}

bool
edit_modify::in_preamble_mode () {
  return get_env_value (PREAMBLE)->label == "true";
}

bool
edit_modify::is_deactivated () {
  return !nil (find_deactivated (tp));
}

void
edit_modify::activate () {
  path p= find_deactivated (tp);
  if (nil (p)) return;
  tree st= subtree (et, path_down (p, 0));

  if ((is_func (st, LATEX, 1) || (is_func (st, HYBRID, 1))) &&
      (N(st[0])==0)) {
    string  s= st[0]->label;
    string  help;
    program cmd;
    if (get_latex_command (s, help, cmd)) {
      cut (path_down (p, 0), path_down (p, 1));
      program comp_cmd= compile (cmd);
      if (is_compiled (comp_cmd)) ::execute (comp_cmd);
      else set_message ("Error: TeXmacs-lisp compilation error", s);
      return;
    }
    else if (is_func (st, HYBRID, 1)) {
      st= tree (APPLY, copy (st[0]));
      assign (et, path_down (p, 0), st);
    }
  }

  if (is_func (st, SYMBOL, 1) && (N(st[0])==0)) {
    int i;
    string s= st[0]->label;
    for (i=0; i<N(s); i++)
      if ((s[i]=='<') || (s[i]=='>'))
	{ s= ""; break; }
    assign (et, p, "<" * s * ">");
    set_cursor (end (et, p));
    correct (path_up (p));
    return;
  }

  assign (et, p, st);
  set_cursor (end (et, p));
  correct (path_up (p));
}

/******************************************************************************
* Making dynamic objects
******************************************************************************/

void
edit_modify::make_deactivated (tree t, path p) {
  if (in_preamble_mode ()) insert_tree (t, p);
  else insert_tree (tree (INACTIVE, t), path (0, p));
}

void
edit_modify::make_symbol () {
  make_deactivated (tree (SYMBOL, ""), path (0, 0));
  set_message ("<return>: activate", "symbol");
}

void
edit_modify::make_latex () {
  make_deactivated (tree (LATEX, ""), path (0, 0));
  set_message ("Type a latex command followed by <return>", "latex");
}

void
edit_modify::make_hybrid () {
  make_deactivated (tree (HYBRID, ""), path (0, 0));
  set_message
    ("<tab>: insert argument, <return>: activate or execute latex command",
     "");
}

void
edit_modify::make_quote () {
  make_deactivated (tree (QUOTE, ""), path (0, 0));
  set_message ("<return>: activate", "quote");
}

void
edit_modify::make_assign () {
  make_deactivated (tree (ASSIGN, "", ""), path (0, 0));
  set_message ("<return>: activate", "assign");
}

void
edit_modify::make_set (string s) {
  make_deactivated (tree (CONCAT, tree (SET, s, ""), tree (RESET, s)),
	       path (0, path (1, 0)));
  set_message ("<return>: activate", "set");
}

void
edit_modify::make_apply () {
  make_deactivated (tree (APPLY, ""), path (0, 0));
  set_message ("<tab>: insert argument, <return>: activate", "apply");
}

void
edit_modify::make_function () {
  make_deactivated (tree (FUNCTION, ""), path (0, 0));
  set_message ("<tab>: insert argument, <return>: activate", "function");
}

void
edit_modify::make_environment () {
  if (!is_concat (subtree (et, path_up (tp, 2)))) {
    if (!is_func (subtree (et, path_up (tp, 2)), FUNCTION)) return;
    ins_unary (et, path_up (tp), CONCAT);
  }
  int pos = last_item (path_up (tp));
  if (last_item (tp)>0) pos++;
  if (!is_concat (subtree (et, path_up (tp, 2)))) return;
  tree f= subtree (et, path_up (tp, 3));
  if (!is_func (f, FUNCTION, last_item (path_up (tp, 2))+1)) return;
  
  int i, n= N(f)-1, l= N(f[n]);
  tree r (ENVIRONMENT, n+2);
  for (i=0; i<n; i++) r[i]= f[i];
  tree c1 (CONCAT, pos);
  for (i=0; i<pos; i++) c1[i]= f[n][i];
  tree c2 (CONCAT, l- pos);
  for (i=pos; i<l; i++) c2[i-pos]= f[n][i];
  r[n]  = c1;
  r[n+1]= c2;

  path p= path_up (tp, 3);
  assign (et, p, r);
  correct_concat (path_down (p, n));
  correct_concat (path_down (p, n+1));
}

void
edit_modify::make_label () {
  make_deactivated (tree (LABEL, ""), path (0, 0));
  set_message ("<tab>: insert argument, <return>: activate", "label");
}

void
edit_modify::make_reference () {
  make_deactivated (tree (REFERENCE, ""), path (0, 0));
  set_message ("<tab>: insert argument, <return>: activate", "reference");
}

void
edit_modify::make_write () {
  make_deactivated (tree (WRITE, ""), path (0, 0));
  set_message ("<tab>: insert argument, <return>: activate", "reference");
}

void
edit_modify::make_plus () {
  make_deactivated (tree (PLUS, "", ""), path (0, 0));
  set_message ("<return>: activate", "plus");
}

void
edit_modify::make_minus () {
  make_deactivated (tree (MINUS, "", ""), path (0, 0));
  set_message ("<return>: activate", "minus");
}

void
edit_modify::make_times () {
  make_deactivated (tree (TIMES, "", ""), path (0, 0));
  set_message ("<return>: activate", "times");
}

void
edit_modify::make_merge () {
  make_deactivated (tree (MERGE, "", ""), path (0, 0));
  set_message ("<return>: activate", "merge");
}

void
edit_modify::make_number () {
  make_deactivated (tree (NUMBER, "", ""), path (0, 0));
  set_message ("<return>: activate", "number");
}

void
edit_modify::make_translate () {
  make_deactivated (tree (TRANSLATE, "", "", ""), path (0, 0));
  set_message ("<return>: activate", "translate");
}

void
edit_modify::insert_argument () {
  if ((last_item (tp)==1) && is_func (subtree (et, path_up (tp)), BEGIN)) {
    path p= path_up (tp);
    tree t= subtree (et, path_up (tp));
    insert (et, path_down (p, N(t)), tree (t->label, ""));
    set_cursor (path_down (path_down (p, N(t)), 0));
    return;
  }

  path p= find_dynamic (tp);
  if (nil (p)) return;
  if (p==tp) {
    p= find_dynamic (path_up (tp));
    if (nil (p)) return;
  }
  tree t= subtree (et, path_up (p));
  if (is_func (t, SYMBOL) || is_func (t, QUOTE) || is_func (t, ASSIGN) ||
      is_func (t, PLUS) || is_func (t, MINUS) || is_func (t, TIMES) ||
      is_func (t, NUMBER) || is_func (t, TRANSLATE)) return;
  if (is_func (t, HYBRID))
    assign (et, path_up (p), tree (APPLY, copy (t[0])));
  insert (et, path_inc (p), tree (t->label, ""));
  set_cursor (path_down (path_inc (p), 0));
}

/******************************************************************************
* Inserting active dynamic objects
******************************************************************************/

bool
edit_modify::make_return_before () {
  bool flag;
  path q= tp;
  while (!is_document (subtree (et, path_up (q)))) q= path_up (q);
  flag= (N (subtree (et, path_up (q))) == (q->item+1)) || (tp != end (et, q));
  if (flag) {
    flag= insert_return ();
    set_cursor (end (et, q));
  }
  return flag;
}

bool
edit_modify::make_return_after () {
  path q= tp;
  while (!is_document (subtree (et, path_up (q)))) q= path_up (q);
  if (tp == start (et, q)) return FALSE;
  return insert_return ();
}

void
edit_modify::make_active_assign (tree var, tree by) {
  insert_tree (tree (CONCAT, tree (ASSIGN, var, by)));
}

void
edit_modify::make_active_set (tree var, tree by) {
  tree ins= tree (CONCAT, tree (SET, var, by), tree (RESET, var));
  if (selection_active ()) {
    tree t= the_selection ();
    selection_cut ();
    insert_tree (ins, path (0, 1));
    insert (remove_changes_in (t, var->label));
  }
  else insert_tree (ins, path (0, 1));
}

void
edit_modify::make_active_begin_from_apply () {
  path p= path_up (tp);
  tree t= subtree (et, p);
  if ((is_func (t, APPLY) || is_func (t, HYBRID)) &&
      (!is_func (subtree (et, path_up (p)), INACTIVE))) {
    tree begin (BEGIN); begin << copy (t->a);
    tree end (END, copy (t[0]));
    assign (et, p, tree (CONCAT, begin, end));
    set_cursor (path_down (path_down (p, 0), 1));
    correct (path_up (p));
  }
  else {
    path p= find_deactivated (tp);
    tree t= subtree (et, path_down (p, 0));
    if ((!nil (p)) && (is_func (t, APPLY) || (is_func (t, HYBRID)))) {
      activate ();
      make_active_begin_from_apply ();
    }
  }
}

bool
edit_modify::make_active_begin (tree t) {
  tree sel;
  bool active= selection_active ();
  if (active) { sel= the_selection (); selection_cut (); }
  if (make_return_after ()) return TRUE;
  if (make_return_before ()) return TRUE;

  int i;
  tree b (BEGIN, t->label);
  for (i=0; i<N(t); i++) b << t[i];
  insert_tree (tree (CONCAT, b, tree (END, t->label)), path (0, 1));
  if (active) insert (sel);
  return FALSE;
}

void
edit_modify::make_active_apply (tree t) {
  if (make_return_after ()) return;

  int i;
  tree a (APPLY, t->label);
  for (i=0; i<N(t); i++) a << t[i];
  insert_tree (a);
  (void) make_return_before ();
}

void
edit_modify::make_active_list (string s, string first) {
  if (make_active_begin (s)) return;
  if (first == "") insert_tree (tree (APPLY, "item"));
  else insert_tree (tree (APPLY, "item*", first));
}

void
edit_modify::make_active_item (tree t) {
  if (make_return_after ()) return;

  int i;
  tree a (APPLY, t->label);
  for (i=0; i<N(t); i++) a << t[i];
  insert_tree (a);
}

void
edit_modify::make_active_equation () {
  insert_tree (tree (CONCAT, tree (FORMAT, LINE_SEP),
		     tree (FORMAT, LINE_SEP)), path (0, 0));
}

/******************************************************************************
* Deleting dynamic objects
******************************************************************************/

void
edit_modify::back_env (tree t, path p, string close) {
  int node= last_item (p);
  if (is_concat (t) && (N(t)>node+1) &&
      is_func (t[node+1], close) &&
      (t[node][0]->label == t[node+1][0]->label)) {
    remove (et, p, 2);
    correct_concat (path_up (p));
  }
  else set_cursor (start (et, p));
}

void
edit_modify::back_dynamic (path p) {
  if (is_func (subtree (et, path_up (p)), INACTIVE))
    set_cursor (end (et, path_down (p, N (subtree (et, p))- 1)));
  else {
    string s= subtree (et, p)->label;
    ins_unary (et, p, INACTIVE);
    set_message ("<return>: reactivate", "delete#" * s);
  }
}

void
edit_modify::back_in_dynamic (tree t, path p, int min_args) {
  int node= last_item (p);
  if (node>0) {
    set_cursor (end (et, path_down (path_up (p), node-1)));
    if (is_func (t, SET) || is_func (t, BEGIN))
      if (node==1) set_cursor (start (et, path_up (p)));
    if ((N(t) > min_args) && (t[node] == tree ("")))
      remove (et, path_down (path_up (p), node), 1);
  }
  else {
    int i;
    for (i=0; i<N(t); i++)
      if (t[i] != tree("")) {
	set_cursor (start (et, path_up (p)));
	return;
      }
    assign (et, path_up (p), "");
    if (subtree (et, path_up (p, 2)) == tree (INACTIVE, "")) {
      assign (et, path_up (p, 2), "");
      correct (path_up (p, 3));
    }
    else correct (path_up (p, 2));
  }
}

void
edit_modify::back_in_environment (tree t, path p) {
  int node= last_item (p);
  if (node == N(t)-1) {
    int i, n= N(t)-2;
    tree f (FUNCTION, n+1);
    for (i=0; i<n; i++) f[i]= t[i];
    tree t1= t[n]  ; if (!is_concat (t1)) t1= tree (CONCAT, t1);
    tree t2= t[n+1]; if (!is_concat (t2)) t2= tree (CONCAT, t2);
    tree r (CONCAT, N(t1)+N(t2));
    for (i=0; i<N(t1); i++) r[i]= t1[i];
    for (i=0; i<N(t2); i++) r[i+N(t1)]= t2[i];
    f[n]= r;
    assign (et, path_up (p), f);
    correct_concat (path_down (path_up (p), n));
    set_message ("<esc>e: remake the environment", "delete#environment");
  }
  else back_in_dynamic (t, p, 2);
}

void
edit_modify::rem_env (tree t, path p) {
  path p1, p2;
  int type= math_primary [t->label];
  if ((type==SET) || (type==BEGIN)) {
    p1= p;
    p2= right_match (et, p1, t);
    if (nil (p2)) return;
  }
  else {
    p2= p;
    p1= left_match (et, p2, t);
    if (nil (p1)) return;
  }
  if (!is_concat (subtree (et, path_up (p2)))) assign (et, p2, "");
  else remove (et, p2, 1);
  if (path_up (p2) != path_up (p1)) correct (path_up (p2));
  if (!is_concat (subtree (et, path_up (p1)))) assign (et, p1, "");
  else remove (et, p1, 1);
  correct (path_up (p1));
}

#endmodule // code_modify_dynamic
