
/******************************************************************************
* MODULE     : modify_math.gen.cc
* DESCRIPTION: modify mathematical structures
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#module code_modify_math

/******************************************************************************
* Making mathematical objects
******************************************************************************/

void
edit_modify::make_script (bool sup, bool right) {
  string s (sup? SUP (right): SUB (right));
  if (math_selection_active ())
    insert_tree (tree (s, math_selection ()));
  else {
    path   p= path_up (tp);
    tree   t= subtree (et, p);
    bool   flag;

    if (is_format (p))
      fatal_error ("bad cursor position", "edit_modify::make_script");
    if (is_script (t, flag) && (flag==right) && (t->label==s)) {
      set_cursor (end (et, path_down (p, 0)));
      return;
    }
    insert_tree (tree (s, ""), path (0, 0));
    set_message ("move to the right when finished",
		 (char*) (sup? (right? "superscript": "left superscript"):
			       (right? "subscript": "left subscript")));
  }
}

void
edit_modify::make_fraction () {
  if (math_selection_active ()) {
    insert_tree (tree (FRAC, math_selection (), ""), path (1, 0));
    set_message ("type denomenator, move right", "fraction");
  }
  else {
    insert_tree (tree (FRAC, "", ""), path (0, 0));
    set_message ("type numerator, move down, type denomenator", "fraction");
  }
}

void
edit_modify::make_sqrt () {
  if (math_selection_active ())
    insert_tree (tree (SQRT, math_selection ()));
  else {
    insert_tree (tree (SQRT, ""), path (0, 0));
    set_message ("move to the right when finished", "square root");
  }
}

void
edit_modify::make_var_sqrt () {
  if (math_selection_active ()) {
    tree t= math_selection ();
    if (is_func (t, SQRT, 1))
      insert_tree (tree (SQRT, t[0], ""), path (1, 0));
    else insert_tree (tree (SQRT, t, ""), path (1, 0));
  }
  else {
    insert_tree (tree (SQRT, "", ""), path (0, 0));
    set_message ("<left>: set n, <right>: when finished", "n-th root");
  }
}

void
edit_modify::make_wide (string wide) {
  if (math_selection_active ())
    insert_tree (tree (WIDE, math_selection (), wide));
  else {
    insert_tree (tree (WIDE, "", wide), path (0, 0));
    set_message ("move to the right when finished", "wide accent");
  }
}

void
edit_modify::make_neg () {
  if (math_selection_active ())
    insert_tree (tree (NEG, math_selection ()));
  else {
    insert_tree (tree (NEG, ""), path (0, 0));
    set_message ("move to the right when finished", "negation");
  }
}

/******************************************************************************
* Deleting mathematical objects
******************************************************************************/

void
edit_modify::back_in_math (tree t, path p) {
  int node= last_item (p);
  if (node>0) {
    set_cursor (end (et, path_down (path_up (p), node-1)));
    return;
  }

  int i;
  bool flag=TRUE;
  for (i=0; i<N(t); i++)
    flag= flag && is_func (t[i], "", 0);
  if (flag) {
    assign (et, path_up (p), "");
    correct (path_up (p, 2));
    return;
  }

  set_cursor (start (et, path_up (p)));
}

void
edit_modify::back_in_math_accent (tree t, path p) {
  if (t[0] == "") {
    assign (et, path_up (p), "");
    correct (path_up (p, 2));
  }
  else set_cursor (start (et, path_up (p)));
}

/******************************************************************************
* Trees
******************************************************************************/

void
edit_modify::make_tree () {
  if (math_selection_active ())
    insert_tree (tree (TREE, math_selection (), ""), path (1, 0));
  else {
    insert_tree (tree (TREE, "", ""), path (0, 0));
    set_message ("<esc>-<right>: insert a new branch", "tree");
  }
}

bool
edit_modify::inside_tree () {
  int i;
  return !nil (get_tree (i));
}

path
edit_modify::get_tree (int& i) {
  path p= tp;
  while (TRUE) {
    path q= path_up (p);
    if (nil (q)) return path ();
    tree t= subtree (et, q);
    if (is_func (t, MATRIX) || is_func (t, TABLE) || is_func (t, MOSAIC))
      return path ();
    if (is_func (t, TREE)) {
      i= last_item (p);
      return q;
    }
    p= q;
  }
}

void
edit_modify::branch_insert (bool at_right) {
  int i;
  path p= get_tree (i);
  if (nil (p)) return;
  if (i==0) {
    if (at_right) i=N(subtree(et,p));
    else i=1;
  }
  else if (at_right) i++;
  insert (et, path_down (p, i), tree (TREE, ""));
  set_cursor (path_down (path_down (p, i), 0));
}

void
edit_modify::branch_delete () {
  int i;
  path p= get_tree (i);
  if (nil (p) || (i==0)) return;
  if (N (subtree (et, p)) == 2) {
    assign (et, p, subtree (et, path_down (p, 0)));
    correct (path_up (p));
  }
  else remove (et, path_down (p, i), 1);
}

void
edit_modify::back_in_tree (tree t, path p) {
  int i= last_item (p);
  if (i>0) {
    if (t[i] == "") branch_delete ();
    else set_cursor (end (et, path_down (path_up (p), i-1)));
  }
  else {
    if (t == tree (TREE, "", "")) {
      p= path_up (p);
      assign (et, p, "");
      correct (path_up (p));
    }
    else set_cursor (start (et, path_up (p)));
  }
}

#endmodule // code_modify_math
