
/******************************************************************************
* MODULE     : modify_mosaic.gen.cc
* DESCRIPTION: modify complex mosaics
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#module code_modify_mosaic

/******************************************************************************
* Making matrices, tables and mosaics
******************************************************************************/

void
edit_modify::make_matrix () {
  tree M (MATRIX);
  M << tree ("") << tree ("1") << tree ("1");
  insert_tree (M, path (0, 0));
  set_message ("<shift-down>: new row, <shift-right>: new column", "matrix");
}

void
edit_modify::make_table () {
  tree T (TABLE);
  T << tree ("") << tree ("1") << tree ("1");
  insert_tree (T, path (0, 0));
  set_message ("<shift-down>: new row, <shift-right>: new column", "table");
}

static tree
get_default (tree t) {
  if (t->label == MOSAIC) {
    tree empty (MOSAIC_ITEM, 5);
    empty[0]= "";
    empty[1]= "0w";
    empty[2]= "1";
    empty[3]= "1";
    empty[4]= "none";
    return empty;
  }
  return "";
}

void
edit_modify::make_mosaic () {
  tree M (MOSAIC);
  M << get_default (M) << tree ("1") << tree ("1");
  insert_tree (M, path (0, path (0, 0)));
  set_message ("<shift-down>: new row, <shift-right>: new column", "mosaic");
}

/******************************************************************************
* General subroutines
******************************************************************************/

path
edit_modify::get_matrix (int& i, int& j, int& n, int& m) {
  path p= tp;
  while (TRUE) {
    path q= path_up (p);
    if (nil (q)) return path ();
    tree t=subtree (et, q);
    if (is_func (t, MATRIX) || is_func (t, TABLE) || is_func (t, MOSAIC)) {
      n= as_int (t[N(t)-2]->label);
      m= as_int (t[N(t)-1]->label);
      i= last_item (p);
      j= i%n;
      i= i/n;
      return q;
    }
    p= q;
  }
}

void
edit_modify::mosaic_set_cursor (path p) {
  int k= last_item (p);
  p= path_up (p);
  tree t= subtree (et, p);
  while (as_int (t[k][2]->label) * as_int (t[k][3]->label) == 0) k--;
  set_cursor (end (et, path_down (path_down (p, k), 0)));
}

bool
edit_modify::row_safe (int i0, int j0, bool flag) {
  int i, j, n, m; (void) j0;
  path p= get_matrix (i, j, n, m);
  tree t= subtree (et, p);
  if (!is_func (t, MOSAIC)) return TRUE;
  if (i0==m) return TRUE;
  for (j=0; j<n; j++)
    if ((as_int (t[i0*n+j][3]->label) == 0) ||
	((flag && (as_int (t[i0*n+j][3]->label) != 1))))
      {
	set_message ("Error: row contains glued items", "mosaic");
	return FALSE;
      }
  return TRUE;
}

bool
edit_modify::column_safe (int i0, int j0, bool flag) {
  int i, j, n, m; (void) i0;
  path p= get_matrix (i, j, n, m);
  tree t= subtree (et, p);
  if (!is_func (t, MOSAIC)) return TRUE;
  if (j0==n) return TRUE;
  for (i=0; i<m; i++)
    if ((as_int (t[i*n+j0][2]->label) == 0) ||
        (flag && (as_int (t[i*n+j0][2]->label) != 1)))
      {
	set_message ("Error: column contains glued items", "mosaic");
	return FALSE;
      }
  return TRUE;
}

/******************************************************************************
* Inserting new rows and columns
******************************************************************************/

void
edit_modify::row_insert (bool d) {
  int i, j, i0, j0, n, m;
  path p= get_matrix (i0, j0, n, m);
  if (nil (p)) return;
  tree t= subtree (et, p);
  tree empty= get_default (t);
  if (d) i0++;
  if (!row_safe (i0, j0, FALSE)) return;
  tree M (t->label, n*(m+1)+2);
  M[N(M)-2]= n;
  M[N(M)-1]= m+1;
  for (i=0; i<m+1; i++)
    for (j=0; j<n; j++) {
      if (i< i0) M[i*n+j]= t[i*n+j];
      if (i==i0) M[i*n+j]= copy (empty);
      if (i> i0) M[i*n+j]= t[(i-1)*n+j];
    }
  assign (et, p, M);
  path q= path_down (path_down (p, i0*n+j0), 0);
  if (is_func (t, MOSAIC)) q= path_down (q, 0);
  set_cursor (q);
}

void
edit_modify::column_insert (bool d) {
  int i, j, i0, j0, n, m;
  path p= get_matrix (i0, j0, n, m);
  if (nil (p)) return;
  tree t= subtree (et, p);
  tree empty= get_default (t);
  if (d) j0++;
  if (!column_safe (i0, j0, FALSE)) return;
  tree M (t->label, (n+1)*m+2);
  M[N(M)-2]= n+1;
  M[N(M)-1]= m;
  for (i=0; i<m; i++)
    for (j=0; j<n+1; j++) {
      if (j< j0) M[i*(n+1)+j]= t[i*n+j];
      if (j==j0) M[i*(n+1)+j]= copy (empty);
      if (j> j0) M[i*(n+1)+j]= t[i*n+j-1];
    }
  assign (et, p, M);
  path q= path_down (path_down (p, i0*(n+1)+j0), 0);
  if (is_func (t, MOSAIC)) q= path_down (q, 0);
  set_cursor (q);
}

/******************************************************************************
* Deletions
******************************************************************************/

void
edit_modify::row_delete () {
  int i, j, i0, j0, n, m;
  path p= get_matrix (i0, j0, n, m);
  if (nil (p)) return;
  if (!row_safe (i0, j0, TRUE)) return;
  tree t= subtree (et, p);
  if (m==1) {
    assign (et, p, "");
    correct (path_up (p));
    return;
  }

  tree M (t->label, n*(m-1)+2);
  M[N(M)-2]= n;
  M[N(M)-1]= m-1;
  for (i=0; i<m; i++)
    for (j=0; j<n; j++) {
      if (i<i0) M[i*n+j]= t[i*n+j];
      if (i>i0) M[(i-1)*n+j]= t[i*n+j];
    }
  assign (et, p, M);
  if (i0>0) i0--;
  path q= path_down (p, i0*n+j0);
  if (is_func (t, MOSAIC)) q= path_down (q, 0);
  set_cursor (end (et, q));
}

void
edit_modify::column_delete () {
  int i, j, i0, j0, n, m;
  path p= get_matrix (i0, j0, n, m);
  if (nil (p)) return;
  if (!column_safe (i0, j0, TRUE)) return;
  tree t= subtree (et, p);
  if (n==1) {
    assign (et, p, "");
    correct (path_up (p));
    return;
  }

  tree M (t->label, (n-1)*m+2);
  M[N(M)-2]= n-1;
  M[N(M)-1]= m;
  for (i=0; i<m; i++)
    for (j=0; j<n; j++) {
      if (j<j0) M[i*(n-1)+j]= t[i*n+j];
      if (j>j0) M[i*(n-1)+j-1]= t[i*n+j];
    }
  assign (et, p, M);
  if (j0>0) j0--;
  path q= path_down (p, i0*(n-1)+j0);
  if (is_func (t, MOSAIC)) q= path_down (q, 0);
  set_cursor (end (et, q));
}

void
edit_modify::back_in_matrix (tree t) {
  int i, j, i0, j0, n, m;
  path p= get_matrix (i0, j0, n, m);
  if (nil (p)) return;
  bool flag=TRUE;
  for (j=0; j<n; j++) flag= flag && is_func (t[i0*n+j], "", 0);
  if (flag) { row_delete (); return; }
  flag= TRUE;
  for (i=0; i<m; i++) flag= flag && is_func (t[i*n+j0], "", 0);
  if (flag) { column_delete (); return; }
  if (j0>0) { set_cursor (end (et, path_down (p, i0*n+j0-1))); return; }
  if (i0>0) { set_cursor (end (et, path_down (p, i0*n-1))); return; }
  set_cursor (start (et, p));
}

void
edit_modify::back_in_mosaic () {
  int i, j, i0, j0, n, m;
  path p= get_matrix (i0, j0, n, m);
  tree t= subtree (et, p);
  if (nil (p)) return;
  bool flag=TRUE;
  for (j=0; j<n; j++) flag= flag && is_func (t[i0*n+j][0], "", 0);
  if (flag) { row_delete (); return; }
  flag= TRUE;
  for (i=0; i<m; i++) flag= flag && is_func (t[i*n+j0][0], "", 0);
  if (flag) { column_delete (); return; }
  if (j0>0) { mosaic_set_cursor (path_down (p, i0*n+j0-1)); return; }
  if (i0>0) { mosaic_set_cursor (path_down (p, i0*n-1)); return; }
  set_cursor (start (et, p));
}

/******************************************************************************
* Specific routines for mosaics
******************************************************************************/

void
edit_modify::mosaic_glue (string s) {
  int i1, j1, i2, j2, n, m;
  path p= get_matrix (i1, j1, n, m);
  if (nil (p)) return;

  int  k1 = i1*n+j1;
  tree t1 = subtree (et, path_down (p, k1));
  int  dx1= as_int (t1[2]->label);
  int  dy1= as_int (t1[3]->label);

  if (s == "d") { j2= j1; i2= i1+dy1; }
  else if (s == "r") { j2= j1+ dx1; i2= i1; }
  else return;
  if ((j1>=n) || (i1>=m)) return;

  int  k2 = i2*n+j2;
  tree t2 = subtree (et, path_down (p, k2));
  int  dx2= as_int (t2[2]->label);
  int  dy2= as_int (t2[3]->label);

  if (s == "d") {
    if (dx2 != dx1) return;
    assign (et, path_down (path_down (p, k1), 3), as_string (dy1+dy2));
    assign (et, path_down (path_down (p, k2), 3), as_string (0));
  }
  if (s == "r") {
    if (dy2 != dy1) return;
    assign (et, path_down (path_down (p, k1), 2), as_string (dx1+dx2));
    assign (et, path_down (path_down (p, k2), 2), as_string (0));
  }
}

void
edit_modify::mosaic_cut () {
  int i0, j0, i, j, n, m;
  path p= get_matrix (i0, j0, n, m);
  if (nil (p)) return;
  int  k0= i0*n+j0;
  tree t0= subtree (et, path_down (p, k0));
  int  dx= as_int (t0[2]->label);
  int  dy= as_int (t0[3]->label);

  for (i=0; i<dy; i++)
    for (j=0; j<dx; j++) {
      int k= (i0+i)*n+(j0+j);
      assign (et, path_down (path_down (p, k), 2), as_string (1));
      assign (et, path_down (path_down (p, k), 3), as_string (1));
    }
}

void
edit_modify::mosaic_color (string s) {
  int i, j, n, m;
  path p= get_matrix (i, j, n, m);
  if (nil (p)) return;
  int k= i*n+j;
  assign (et, path_down (path_down (p, k), 4), s);
}

void
edit_modify::mosaic_horizontal (string pos) {
  int i, j, n, m;
  path p= get_matrix (i, j, n, m);
  if (nil (p)) return;
  int k= i*n+j;
  p= path_down (p, k);
  tree t= subtree (et, p);

  if ((pos == "<") && (t[1]->label[1] != 'e')) pos= "w";
  if ((pos == ">") && (t[1]->label[1] != 'w')) pos= "e";
  string s(2); s[0]= t[1]->label[0]; s[1]= pos[0];
  assign (et, path_down (p, 1), s);
}

void
edit_modify::mosaic_vertical (string pos) {
  int i, j, n, m;
  path p= get_matrix (i, j, n, m);
  if (nil (p)) return;
  int k= i*n+j;
  p= path_down (p, k);
  tree t= subtree (et, p);

  if ((pos == "<") && (t[1]->label[1] != 'n')) pos= "s";
  if ((pos == ">") && (t[1]->label[1] != 's')) pos= "n";
  string s(2); s[0]= pos[0]; s[1]= t[1]->label[1];
  assign (et, path_down (p, 1), s);
}

void
edit_modify::row_line (string height) {
  int i, j, k, n, m;
  path p= get_matrix (i, j, n, m);
  if (nil (p)) return;
  for (j=0; j<n; j++) {
    k= i*n+j;
    tree t= subtree (et, path_down (p, k));
    if (as_int (t[2]->label)* as_int (t[3]->label) != 1) continue;
    assign (et, path_down (path_down (p, k), 0),
	    tree (SPACE, "0cm", "0cm", height));
    assign (et, path_down (path_down (p, k), 4), "black");
  }
}

void
edit_modify::row_position (string pos) {
  int i, j, k, n, m;
  path p= get_matrix (i, j, n, m);
  if (nil (p)) return;
  for (j=0; j<n; j++) {
    k= i*n+j;
    tree t= subtree (et, path_down (p, k));
    if (as_int (t[3]->label) != 1) continue;
    string s= pos * t[1]->label(1,2);
    assign (et, path_down (path_down (p, k), 1), s);
  }
}

void
edit_modify::column_line (string width) {
  int i, j, k, n, m;
  path p= get_matrix (i, j, n, m);
  if (nil (p)) return;
  for (i=0; i<m; i++) {
    k= i*n+j;
    tree t= subtree (et, path_down (p, k));
    if (as_int (t[2]->label)* as_int (t[3]->label) != 1) continue;
    assign (et, path_down (path_down (p, k), 0),
	    tree (SPACE, width, "0cm", "0cm"));
    assign (et, path_down (path_down (p, k), 4), "black");
  }
}

void
edit_modify::column_position (string pos) {
  int i, j, k, n, m;
  path p= get_matrix (i, j, n, m);
  if (nil (p)) return;
  for (i=0; i<m; i++) {
    k= i*n+j;
    tree t= subtree (et, path_down (p, k));
    if (as_int (t[2]->label) != 1) continue;
    string s= t[1]->label(0,1) * pos;
    assign (et, path_down (path_down (p, k), 1), s);
  }
}

#endmodule // code_modify_mosaic
