
/******************************************************************************
* MODULE     : modify_select.gen.cc
* DESCRIPTION: selection of text
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#module code_modify_select

/******************************************************************************
* Traversal of the selection
******************************************************************************/

path
start_forward (tree t) {
  if (is_concat (t)) return path (0);
  if (is_document (t)) {
    if (!is_concat (t[0])) return path (0);
    return path (0, 0);
  }
  return path ();
}

path
increment (tree t, path p) {
  if (is_concat (t)) {
    path q= path_inc (p);
    if (q->item == N(t)) return path ();
    return q;
  }

  if (is_document (t)) {
    path q;
    if (N(p)==2) {
      q= path_inc (p);
      if (q->next->item == N(t[q->item])) p= path_up (p);
    }
    if (N(p)==1) {
      if (p->item+1 == N(t)) return path ();
      q= path (p->item+1);
      if (is_concat (subtree (t, q))) q= path_down (q, 0);
    }
    return q;
  }
  
  return path ();
}

path
start_backward (tree t) {
  if (is_concat (t)) return path (N(t)-1);
  if (is_document (t)) {
    if (!is_concat (t[N(t)-1])) return path (N(t)-1);
    return path (N(t)-1, N(t[N(t)-1])-1);
  }
  return path ();
}

path
decrement (tree t, path p) {
  if (is_concat (t)) {
    if (p->item == 0) return path ();
    return path_dec (p);
  }
  
  if (is_document (t)) {
    path q;
    if (N(p)==2) {
      if (p->next->item == 0) p= path_up (p);
      else return path_dec (p);
    }
    if (N(p)==1) {
      if (p->item == 0) return path ();
      q= path (p->item-1);
      if (is_concat (subtree (t, q)))
	q= path_down (q, N (subtree (t, q)) - 1);
    }
    return q;
  }

  return path ();
}

/******************************************************************************
* Find corresponding matches
******************************************************************************/

path
right_match (tree t, path p, tree which, int level) {
  if (nil (p)) return p;
  path q= increment (t, p);
  if (nil (q)) return q;
  tree u= subtree (t, q);
  if (is_func (u, SET) || is_func (u, BEGIN))
    if (u[0] == which[0]) return right_match (t, q, which, level+1);
  if (is_func (u, RESET) || is_func (u, END))
    if (u[0] == which[0]) {
      if (level==0) return q;
      return right_match (t, q, which, level-1);
    }
  return right_match (t, q, which, level);
}

path
left_match (tree t, path p, tree which, int level) {
  if (nil (p)) return p;
  path q= decrement (t, p);
  if (nil (q)) return q;
  tree u= subtree (t, q);
  if (is_func (u, RESET) || is_func (u, END))
    if (u[0] == which[0]) return left_match (t, q, which, level+1);
  if (is_func (u, SET) || is_func (u, BEGIN))
    if (u[0] == which[0]) {
      if (level==0) return q;
      return left_match (t, q, which, level-1);
    }
  return left_match (t, q, which, level);
}

/******************************************************************************
* Treat unclosed and unopened environments
******************************************************************************/

tree
left_unmatched (tree t) {
  path p= start_forward (t);
  tree r (CONCAT);
  while (!nil (p)) {
    tree u= subtree (t, p);
    if (is_func (u, RESET) || is_func (u, END))
      if (nil (left_match (t, p, u))) r << copy (u);
    p= increment (t, p);
  }
  return r;
}

tree
right_unmatched (tree t) {
  path p= start_forward (t);
  tree r (CONCAT);
  while (!nil (p)) {
    tree u= subtree (t, p);
    if (is_func (u, SET) || is_func (u, BEGIN))
      if (nil (right_match (t, p, u))) r << copy (u);
    p= increment (t, p);
  }
  return r;
}

bool
is_well_matched (tree t, path p, tree which) {
  if (is_func (which, SET) || is_func (which, BEGIN))
    return !nil (right_match (t, p, which));
  if (is_func (which, RESET) || is_func (which, END))
    return !nil (left_match (t, p, which));
  return TRUE;
}

tree
well_matched (tree t) {
  if (is_concat (t)) {
    int i;
    tree r (CONCAT);
    for (i=0; i<N(t); i++)
      if (is_well_matched (t, path (i), t[i]))
	if (!is_func (t[i], "", 0)) r << t[i];
    if (N(r)==0) return "";
    if (N(r)==1) return r[0];
    return r;
  }

  if (is_document (t)) {
    int i;
    tree r (DOCUMENT);
    for (i=0; i<N(t); i++) {
      if (is_concat (t[i])) {
	int ii;
	tree rr (CONCAT);
	for (ii=0; ii<N(t[i]); ii++)
	  if (is_well_matched (t, path (i, ii), t[i][ii]))
	    if (!is_func (t[i][ii], "", 0))
	      rr << t[i][ii];
	if (N(rr)==0) rr= "";
	else if (N(rr)==1) rr= rr[0];
	r << rr;
      }
      else {
	if (is_well_matched (t, path (i), t[i])) r << t[i];
	else r << "";
      }
    }
    return r;
  }

  return t;
}

/******************************************************************************
* Remove particular environment changes from a tree
******************************************************************************/

bool
no_changes_in (tree t, path p, string which) {
  tree st= subtree (t, p);
  if (is_func (st, SET, 2) ||
      is_func (st, ASSIGN, 2) ||
      is_func (st, RESET, 1))
    return st[0]->label != which;
  return TRUE;
}

tree
remove_changes_in (tree t, string which) {
  if (is_concat (t)) {
    int i;
    tree r (CONCAT);
    for (i=0; i<N(t); i++)
      if (no_changes_in (t, path (i), which))
	if (!is_func (t[i], "", 0)) r << t[i];
    if (N(r)==0) return "";
    if (N(r)==1) return r[0];
    return r;
  }

  if (is_document (t)) {
    int i;
    tree r (DOCUMENT);
    for (i=0; i<N(t); i++) {
      if (is_concat (t[i])) {
	int ii;
	tree rr (CONCAT);
	for (ii=0; ii<N(t[i]); ii++)
	  if (no_changes_in (t, path (i, ii), which))
	    if (!is_func (t[i][ii], "", 0))
	      rr << t[i][ii];
	if (N(rr)==0) rr= "";
	else if (N(rr)==1) rr= rr[0];
	r << rr;
      }
      else {
	if (no_changes_in (t, path (i), which)) r << t[i];
	else r << "";
      }
    }
    return r;
  }

  return t;
}

/******************************************************************************
* Get the selection
******************************************************************************/

tree
compute_selection (tree t, path start, path end) {
  int  i1= start->item;
  int  i2= end->item;
  path p1= start->next;
  path p2= end->next;

  if (nil (p1) || nil (p2)) {
    if (start == path (right_index (t))) return "";
    if (end == path (0)) return "";
    if (start == end) return "";
    if (nil (p1) && nil (p2)) {
      if (N(t)>0) return copy (t);
      if (i1>=i2) return "";
      return t->label (i1, i2);
    }
    if ((N(t)!=0) && (!is_format (t))) return copy (t);
    if (nil (p1)) {
      i1= 0;
      p1= (start->item==0? 0: right_index (t[i1]));
    }
    if (nil (p2)) {
      i2= N(t)-1;
      p2= (end->item==0? 0: right_index (t[i2]));
    }
  }

  if (i1==i2) return compute_selection (t[i1], p1, p2);
  if ((N(t)!=0) && (!is_format (t))) return copy (t);

  int i;
  tree r (t->label, i2-i1+1);
  r[0]     = compute_selection (t[i1], p1, path (right_index (t[i1])));
  r[N(r)-1]= compute_selection (t[i2], path (0), p2);
  for (i=1; i<N(r)-1; i++) r[i]= copy (t[i+i1]);
  return r;
}

tree
edit_modify::the_selection () {
  if (!selection_active ()) return "";
  path start, end;
  // cout << "Selecting...\n";
  selection_get (start, end);
  // cout << "Between paths: " << start << " and " << end << "\n";
  tree t= ::compute_selection (et, start, end);
  // cout << "Preselection : " << t << "\n";
  tree u= well_matched (t);
  // cout << "Selection    : " << u << "\n";
  return u;
}

/******************************************************************************
* Cutting the selection
******************************************************************************/

path
common (path start, path end) {
  if (nil (start) || nil (end)) return path ();
  if (start->item != end->item) return path ();
  return path (start->item, common (start->next, end->next));
}

void
edit_modify::cut (path p1, path p2) {
  if (p2 == p1) return;
  path p = common (p1, p2);
  tree t = subtree (et, p);
  int  n = N(p);
  int  i1= p1[n];
  int  i2= p2[n];

  if (is_document (t) || is_concat (t)) {
    path q1= copy (p); q1 << path (i1, end (t[i1]));
    path q2= copy (p); q2 << path (i2, start (t[i2]));
    cut (q2, p2);
    if (i2>i1+1) remove (et, path_down (p, i1+1), i2-i1-1);
    cut (p1, q1);
    if (is_concat (t)) correct_concat (p);
    else remove_return (path_down (p, i1));
    return;
  }

  if ((N(t)!=0) && (!is_format (t))) {
    assign (et, p, "");
    return;
  }

  if ((N(p1) != (N(p)+1)) || (N(p2) != (N(p)+1))) {
    cerr << "t = " << t << "\n";
    cerr << "p = " << p << "\n";
    cerr << "p1= " << p1 << "\n";
    cerr << "p2= " << p2 << "\n";
    fatal_error ("invalid cut", "edit_modify::cut");
  }

  if (N(t)==0) {
    int pos= last_item (p1);
    int nr = last_item (p2)-pos;
    if (nr>0) remove (et, p1, nr);
  }
  else {
    if ((last_item (p1) != 0) || (last_item (p2) != 1)) {
      cerr << "t = " << t << "\n";
      cerr << "p = " << p << "\n";
      cerr << "p1= " << p1 << "\n";
      cerr << "p2= " << p2 << "\n";
      fatal_error ("invalid object cut", "edit_modify::cut");
    }
    assign (et, p, "");
  }
}

void
edit_modify::selection_cut () {
  if (!selection_active ()) return;
  path p1, p2;
  selection_get (p1, p2);
  set_cursor (p2);
  if (p2 == p1) return;

  tree sel= compute_selection (et, p1, p2);
  // cout << "Selection " << sel << "\n";
  selection_set (well_matched (sel));
  // cout << "Selected  " << well_matched (sel) << "\n";
  cut (p1, p2);
  // cout << "Cut text\n";
  tree t1 = left_unmatched (sel);
  // cout << "Left unmatched = " << t1 << "\n";
  tree t2 = right_unmatched (sel);
  // cout << "Right unmatched= " << t2 << "\n";
  tree ins (CONCAT, N(t1)+N(t2));
  // cout << "Finishing (" << N(ins) << ")\n";
  if (N(ins)==0) return;
  if (N(ins)==1) {
    if (N(t1)==1) insert_tree (t1[0], end (t1[0]));
    else insert_tree (t2[0], start (t2[0]));
    return;
  }

  int i;
  for (i=0; i<N(t1); i++) ins[i]= t1[i];
  for (i=0; i<N(t2); i++) ins[i+N(t1)]= t2[i];
  if (N(t1)==0) insert_tree (ins, start (ins));
  else insert_tree (ins, end (ins, path (N(t1)-1)));
}

/******************************************************************************
* Cutting selections for mathematical constructions
******************************************************************************/

bool
edit_modify::math_selection_active () {
  if (!selection_active ()) return FALSE;
  path p1, p2;
  selection_get (p1, p2);
  if (p2 == p1) return FALSE;
  if (is_document (subtree (et, common (p1, p2)))) return FALSE;
  return TRUE;
}

tree
edit_modify::math_selection () {
  tree t= the_selection ();
  selection_cut ();
  return t;
}

#endmodule // code_modify_select
