
/******************************************************************************
* MODULE     : modify_session.gen.cc
* DESCRIPTION: input and output handling for programming sessions
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <list.gen.h>

#module code_modify_session
#import list (path)

/******************************************************************************
* Compute the context of a path in the edit tree
******************************************************************************/

void
compute_context (tree t, path where, list<path>& l, list<path>& r) {
  list<path> e;
  l= list<path>();
  r= list<path>();
  path p= start_forward (t);
  while (!nil (p)) {
    tree u= subtree (t, p);
    if (is_func (u, BEGIN))
      e= list<path> (p, e);
    if (is_func (u, END))
      if (!nil (e)) {
	if (path_inf (e->item, where) && path_inf (where, p)) {
	  l= list<path> (e->item, l);
	  r= list<path> (p, r);
	}
	e= e->next;
      }
    p= increment (t, p);
  }
}

string
compute_context (tree t, path where) {
  string s;
  list<path> l, r;
  compute_context (t, where, l, r);
  while (!nil (l)) {
    tree u= subtree (t, l->item);
    s << (u[0]->label * "#");
    l= l->next;
  }
  return s;
}

/******************************************************************************
* Start session
******************************************************************************/

void
edit_modify::make_session (string lan, string session) {
  make_active_begin (tree ("session", lan, session));
  tree bi (BEGIN, "input"), ei (END, "input");
  insert_tree (tree (CONCAT, bi, ei), path (0, 1));
}

bool
edit_modify::in_input_mode () {
  list<path> l, r;
  compute_context (et, tp, l, r);
  while ((!nil (l)) && (!nil (r))) {
    tree u= subtree (et, l->item);
    if (u == tree (BEGIN, "input")) return TRUE;
    l= l->next; r= r->next;
  }
  return FALSE;
}

/******************************************************************************
* Get input
******************************************************************************/

tree
edit_modify::get_input () {
  list<path> l, r;
  compute_context (et, tp, l, r);
  while ((!nil (l)) && (!nil (r))) {
    tree u= subtree (et, l->item);
    if (u == tree (BEGIN, "input")) {
      path p1= path_down (l->item, 1);
      path p2= path_down (r->item, 0);
      tree back= compute_selection (et, p1, p2);
      // cout << "Input : " << back << "\n";
      return back;
    }
    l= l->next; r= r->next;
  }
  return "";
}

/******************************************************************************
* Set output
******************************************************************************/

void
edit_modify::set_output (tree t) {
  // cout << "Output: " << t << "\n";
  list<path> l, r;
  compute_context (et, tp, l, r);
  while ((!nil (l)) && (!nil (r))) {
    tree u= subtree (et, l->item);
    if (u == tree (BEGIN, "input")) {
      set_cursor (path_down (r->item, 1));
      break;
    }
    l= l->next; r= r->next;
  }

  insert_return ();
  if (t != "") {
    if (is_document (t));
    else t= tree (DOCUMENT, t);
    int l= N(t)-1;
    if (!is_concat (t[0])) t[0]= tree (CONCAT, t[0]);
    if (!is_concat (t[l])) t[l]= tree (CONCAT, t[l]);
    tree u (CONCAT, tree (BEGIN, "output"));
    u << t[0]->a; t[0]= u;
    t[l] << tree (END, "output");
    insert_document (t);
    insert_return ();
  }
  tree bi (BEGIN, "input"), ei (END, "input");
  insert_tree (tree (CONCAT, bi, ei), path (0, 1));
}

#endmodule // code_modify_session
