
/******************************************************************************
* MODULE     : modify_text.gen.cc
* DESCRIPTION: modification of text
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#module code_modify_text

/******************************************************************************
* Inserting line breaks (text-mode only)
******************************************************************************/

bool
edit_modify::insert_return () {
  if (is_document (subtree (et, path_up (tp, 2))))
    ins_unary (et, path_up (tp), CONCAT);
  path p= path_up (tp, 2);
  if (!is_concat (subtree (et, p))) return TRUE;
  if (!is_document (subtree (et, path_up (p)))) return TRUE;

  if (N (subtree (et, path_up (tp)))==0) split (et, tp);
  if (last_item (tp)==right_index (subtree (et, path_up (tp))))
    split (et, path_inc (path_up (tp)));
  else split (et, path_up (tp));
  set_cursor (correct_cursor (et, path_down (path_inc (p), 0)));
  correct_concat (p);
  correct_concat (path_inc (p));
  return FALSE;
}

void
edit_modify::remove_return (path p) {
  if (!is_document (subtree (et, path_up (p))))
    fatal_error ("Parent not a document", "edit_modify::glue_concat");

  if (!is_concat (subtree (et, p)))
    ins_unary (et, p, CONCAT);
  if (!is_concat (subtree (et, path_inc (p))))
    ins_unary (et, path_inc (p), CONCAT);
  join (et, p);
  correct_concat (p);
}

/******************************************************************************
* Inserting an entire document (text-mode only)
******************************************************************************/

path
inner_paragraph (tree t, path p) {
  while (!nil (p)) {
    if (is_document (subtree (t, path_up (p)))) return p;
    p= path_up (p);
  }
  fatal_error ("couldn't find paragraph", "inner_paragraph");
}

void
edit_modify::insert_document (tree t) {
  t= math_correct (t);
  if (!is_document (t))
    fatal_error ("document expected", "edit_modify::insert_document");
  if (insert_return ()) return;
  path p= inner_paragraph (et, tp);
  insert (et, p, t);
  remove_return (path_dec (p));
  remove_return (path_add (p, N(t)-2));
}

/******************************************************************************
* Inserting a simple formula or tree
******************************************************************************/

path
edit_modify::prepare_for_insert () {
  path p= path_up (tp);
  tree t= subtree (et, p);
  int  l= last_item (tp);

  if ((!nil(p)) && is_concat (subtree (et, path_up (p)))) {
    if (l==0) return p;
    if ((N(t)!=0) || (l==N(t->label))) return path_inc (p);
    split (et, tp);
    return path_inc (p);
  }
  
  ins_unary (et, p, CONCAT);
  if ((N(t)==0) && (l!=0) && (l!=N(t->label))) {
    split (et, path_down (path_down (p, 0), l));
    return path_down (p, 1);
  }
  return path_down (p, (l==0)? 0: 1);
}

void
edit_modify::insert_tree (tree t, path p_in_t) {
  path p= prepare_for_insert ();
  if (!is_concat (t)) { t= tree (CONCAT, t); p_in_t= path (0, p_in_t); }
  insert (et, p, t);
  set_cursor (merge (path_add (p, p_in_t->item), p_in_t->next));
  set_cursor (correct_cursor (et, tp));
  correct_concat (path_up (p));
}

void
edit_modify::insert_tree (tree t) {
  t= math_correct (copy (t));
  insert_tree (t, end (t));
}

void
edit_modify::insert_char (char c) {
  insert_tree (tree (c));
}

void
edit_modify::insert (tree t) {
  if (is_document (t)) insert_document (t);
  else insert_tree (t);
}

/******************************************************************************
* Spaces
******************************************************************************/

static string
get_unit (string s) {
  int i;
  for (i=0; i<N(s); i++)
    if ((s[i]>='a') && (s[i]<='z')) break;
  return s (i, N(s));
}

static string
get_quantity (string s) {
  int i;
  for (i=0; i<N(s); i++)
    if ((s[i]>='a') && (s[i]<='z')) break;
  return s (0, i);
}

void
edit_modify::make_space (tree u) {
  tree t= subtree (et, path_up (tp));

  int i, n=N(u);
  bool flag= is_func (t, u->label, n);
  for (i=0; i<n; i++) {
    if (!flag) break;
    string u1= get_unit (t[i]->label);
    string u2= get_unit (t[0]->label);
    double x1= as_double (get_quantity (t[i]->label));
    double x2= as_double (get_quantity (u[i]->label));
    flag= flag && (u1==u2) && ((x1*x2)>0);
  }
  
  if (flag) {
    for (i=0; i<n; i++) {
      string q1= get_quantity (t[i]->label);
      string q2= get_quantity (u[i]->label);
      double w1= as_double (q1);
      double w2= as_double (q2);
      string sum= as_string (w1+w2) * get_unit (t[i]->label);
      assign (et, path_down (path_up (tp), i), sum);
    }
  }
  else insert_tree (u);
}

void
edit_modify::make_space (string w) {
  make_space (tree (SPACE, w));
}

void
edit_modify::make_space (string w, string y1, string y2) {
  insert_tree (tree (SPACE, w, y1, y2));
}

void
edit_modify::make_hspace (string s) {
  make_space (tree (HSPACE, s));
}

void
edit_modify::make_hspace (string smin, string sdef, string smax) {
  make_space (tree (HSPACE, smin, sdef, smax));
}

void
edit_modify::make_vspace_before (string s) {
  make_space (tree (VSPACE_BEFORE, s));
}

void
edit_modify::make_vspace_before (string smin, string sdef, string smax) {
  make_space (tree (VSPACE_BEFORE, smin, sdef, smax));
}

void
edit_modify::make_vspace_after (string s) {
  make_space (tree (VSPACE_AFTER, s));
}

void
edit_modify::make_vspace_after (string smin, string sdef, string smax) {
  make_space (tree (VSPACE_AFTER, smin, sdef, smax));
}

/******************************************************************************
* Insert formatting objects and miscellaneous
******************************************************************************/

void
edit_modify::make_htab (string spc) {
  insert_tree (tree (HTAB, spc));
}

void
edit_modify::make_format (tree t) {
  insert_tree (tree (FORMAT, t));
}

void
edit_modify::make_split () {
  make_deactivated (tree (SPLIT, ""), path (0, 0));
  set_message ("<tab>: insert argument, <return>: activate", "begin");
}

void
edit_modify::make_move (string x, string y) {
  insert_tree (tree (MOVE, "", x, y), path (0, 0));
}

void
edit_modify::make_resize (string mode,
			  string x1, string y1,
			  string x2, string y2)
{
  tree t (RESIZE, 6);
  t[0]= ""; t[1]= mode;
  t[2]= x1; t[3]= y1;
  t[4]= x2; t[5]= y2;
  insert_tree (t, path (0, 0));
}

void
edit_modify::make_postscript (
  string file_name, string w, string h,
  string x1, string y1, string x2, string y2)
{
  tree t (POSTSCRIPT, file_name, w, h);
  t << tree (x1) << tree (y1) << tree (x2) << tree (y2);
  insert_tree (t);
}

#endmodule // code_modify_text
