
/******************************************************************************
* MODULE     : base.gen.h
* DESCRIPTION: the common base for most of the editor components
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <tree.gen.h>
#include <path.gen.h>
#include <Boxes/boxes.gen.h>
#include <program.gen.h>

#module edit_base
#import tree
#import path
#import boxes
#import program

#define TEXMACS_VERSION (string("0.2.4c"))
#define TEXMACS_COPYRIGHT (string("(c) 1999 by Joris van der Hoeven"))

#define THE_CURSOR 1
#define THE_FOCUS 2
#define THE_TREE 4
#define THE_ENVIRONMENT 8
#define THE_SELECTION 16
#define THE_FOOTER 32
#define THE_EXTENTS 64

class tm_buffer_rep;
typedef tm_buffer_rep* tm_buffer;

class edit_base {
protected:
  tm_buffer buf;  // the underlying buffer
  tree&     et;   // the tree being edited
  box       eb;   // box translation of tree
  path      tp;   // path of cursor in tree

public:
  edit_base (tm_buffer buf);
  inline virtual ~edit_base () {}

  virtual void notify_change (int changed) = 0;
  virtual void set_message (string l, string r= "") = 0;
  virtual void set_cursor (path p) = 0;
  virtual bool selection_active () = 0;
  virtual void selection_get (path& start, path& end) = 0;
  virtual void selection_set (tree t) = 0;
  virtual void check_data_integrety () = 0;
  virtual program compile (program p) = 0;
  virtual string get_name () = 0;
  virtual bool get_latex_command (string which, string& help, program& p) = 0;
  virtual void focus_on_this_editor () = 0;

  virtual tree get_env_value (string var_name) = 0;
  virtual tree get_init_value (string var_name) = 0;
  virtual void init_env (string var_name, tree with) = 0;
  virtual void init_style (string style_file) = 0;
  virtual void typeset_notify_modify (int pos) = 0;
  virtual void typeset_notify_assign (int nr) = 0;
  virtual void typeset_notify_insert (int pos, int nr) = 0;
  virtual void typeset_notify_remove (int pos, int nr) = 0;
  virtual void typeset (SI& x1, SI& y1, SI& x2, SI& y2) = 0;

  virtual void assign (tree& t, path p, tree u) = 0;
  virtual void remove (tree& t, path p, int nr) = 0;
  virtual void insert_tree (tree t, path path_in_t) = 0;
  virtual void insert (tree t) = 0;
  virtual tree the_selection () = 0;

  friend class tm_window_rep;
  friend class tm_data_rep;
  friend class tm_server_rep;
};

#endmodule // edit_base
