
/******************************************************************************
* MODULE     : cursor.gen.h
* DESCRIPTION: for cursor handling in TeXmacs
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <edit_base.gen.h>

#module edit_cursor
#import edit_base

#define DIRECT 0
#define CENTER 1
#define HORIZONTAL 2
#define VERTICAL 3

class edit_cursor: virtual public edit_base {
protected:
  cursor cu;         // the cursor
  cursor mv;         // "ghost cursor" position when moving cursor
  cursor oc;         // the previous cursor
  int    mv_status;  // cursor status during movements
  
  bool       start_drag;
  bool       dragging;
  SI         start_x, start_y;
  SI         end_x, end_y;
  bool       made_selection;
  path       start_p, end_p;
  rectangles selection_rects;

protected:
  path tree_path (SI x, SI y, SI delta);
  bool cursor_move_sub (SI& x0, SI& y0, SI& delta, SI dx, SI dy);
  void cursor_move (SI dx, SI dy);
  void adjust_ghost_cursor (int status);
  void notify_cursor_moved (int status);

public:
  edit_cursor (tm_buffer buf);
  virtual ~edit_cursor ();

  void set_cursor (SI x, SI y);
  void go_left ();
  void go_right ();
  void go_up ();
  void go_down ();
  void go_page_up ();
  void go_page_down ();

  void set_cursor (path p);
  void go_home ();
  void go_end ();
  void go_home_line ();
  void go_end_line ();

  virtual bool has_changed (int question) = 0;
  virtual SI   get_window_height () = 0;
};

#endmodule // edit_cursor
