
/******************************************************************************
* MODULE     : edit_main.gen.h
* DESCRIPTION: the main structure for the mathematical editor
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <edit_modify.gen.h>
#include <edit_interface.gen.h>

#module edit_main
#import edit_modify
#import edit_interface

class edit_main: public edit_modify, public edit_interface {
public:
  edit_main (server_rep* sv, display dis, tm_buffer buf);
  ~edit_main ();
  
  void clear_buffer ();
  void new_window ();
  void clone_window ();
  void tex_buffer ();
  string get_name ();
  void focus_on_this_editor ();

  void set_page_parameters ();
  void set_page_medium (string medium);
  void set_page_type (string type);
  void set_page_orientation (string orientation);

  void set_printing_command (string s);
  void set_printer_page_type (string s);
  void set_printer_dpi (string dpi);
  void print (string ps_name, bool to_file, int first, int last);
  void print_to_file (string ps_name, string first="1", string last="1000000");
  void print_buffer (string first="1", string last="1000000");
  void export_ps (string ps_name, string first="1", string last="1000000");

  program   compile (program p);
  void      execute_edit_lisp (string s);
  tree      the_line ();
  tree      the_buffer ();
  tree      prog_eval (tree t);
  tree      shell_eval (tree t);
  tree      scheme_eval (tree t);
  tree      edit_lisp_compile (tree t);
  tree      edit_lisp_eval (tree t);

  void show_tree ();
  void show_env ();
  void show_path ();
  void show_cursor ();
  void show_selection ();
  void show_meminfo ();
  void edit_special ();
  void edit_test ();

  friend class editor;
};

class editor {
#import extend_null (widget, editor, edit_main)
public:
  inline bool operator == (editor w) { return rep == w.rep; }
  inline bool operator != (editor w) { return rep != w.rep; }
};
#import code_extend_null (widget, editor, edit_main)

#endmodule // edit_main
