
/******************************************************************************
* MODULE     : modifier.gen.h
* DESCRIPTION: Routines for the modification of the edit tree
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <edit_base.gen.h>

#module edit_modify
#import edit_base

class edit_modify: virtual public edit_base {
public:
  edit_modify (tm_buffer buf);
  ~edit_modify ();

  /********* main modification handling and notification of changes *********/ 
  void assign           (tree& t, path p, tree u);
  void insert           (tree& t, path p, tree u);
  void remove           (tree& t, path p, int nr);
  void split            (tree& t, path p);
  void join             (tree& t, path p);
  void ins_unary        (tree& t, path p, string op);
  void rem_unary        (tree& t, path p);
  void finished         (tree& t);
  void notify_assign    (tree& t, path p, tree u);
  void notify_insert    (tree& t, path p, tree u);
  void notify_remove    (tree& t, path p, int nr);
  void notify_split     (tree& t, path p);
  void notify_join      (tree& t, path p);
  void notify_ins_unary (tree& t, path p, string op);
  void notify_rem_unary (tree& t, path p);
  void post_notify      (tree& t);

  /******************* maintain correctness of edit tree *********************/
  void correct_concat (path p, int done=0);
  void correct (path p);
  void check_data_integrety ();

  /********************************* text ************************************/
  bool insert_return ();
  void remove_return (path p);
  void insert_document (tree t);
  path prepare_for_insert ();
  void insert_char (char c);
  void insert_tree (tree t, path p_in_t);
  void insert_tree (tree t);
  void insert (tree t);

  void make_space (tree t);
  void make_space (string w);
  void make_space (string w, string y1, string y2);
  void make_hspace (string s);
  void make_hspace (string smin, string sdef, string smax);
  void make_vspace_before (string s);
  void make_vspace_before (string smin, string sdef, string smax);
  void make_vspace_after (string s);
  void make_vspace_after (string smin, string sdef, string smax);
  void make_format (tree t);
  void make_htab (string spc);
  void make_split ();
  void make_move (string x, string y);
  void make_resize (string mode, string x1, string y1, string x2, string y2);
  void make_postscript (string file_name, string w, string h,
			string x1, string y1, string y2, string y2);

  /***************************** mathematics *********************************/
  void make_script (bool sup, bool right);
  void make_fraction ();
  void make_sqrt ();
  void make_var_sqrt ();
  void make_wide (string wide);
  void make_neg ();
  void make_tree ();
  void make_matrix ();
  void make_table ();
  void make_mosaic ();
  void back_in_math (tree t, path p);
  void back_in_math_accent (tree t, path p);

  bool inside_tree ();
  path get_tree (int& i);
  void branch_insert (bool at_right);
  void branch_delete ();
  void back_in_tree (tree t, path p);

  path get_matrix (int& i, int& j, int& n, int& m);
  void mosaic_set_cursor (path p);
  void back_in_matrix (tree t);
  void back_in_mosaic ();
  bool row_safe (int i0, int j0, bool flag);
  void row_insert (bool downwards);
  void row_delete ();
  void row_line (string height);
  void row_position (string pos);
  bool column_safe (int i0, int j0, bool flag);
  void column_insert (bool at_right);
  void column_delete ();
  void column_line (string width);
  void column_position (string pos);
  void mosaic_glue (string s);
  void mosaic_cut ();
  void mosaic_color (string s);
  void mosaic_horizontal (string pos);
  void mosaic_vertical (string pos);

  /***************************** environment *********************************/
  path find_dynamic (path p);
  path find_deactivated (path p);
  bool in_preamble_mode ();
  bool is_deactivated ();
  void activate ();
  void make_deactivated (tree t, path p);
  void make_symbol ();
  void make_latex ();
  void make_hybrid ();
  void make_dynamic ();
  void make_quote ();
  void make_assign ();
  void make_set (string s);
  void make_apply ();
  void make_function ();
  void make_environment ();
  void make_label ();
  void make_reference ();
  void make_write ();
  void make_plus ();
  void make_minus ();
  void make_times ();
  void make_merge ();
  void make_number ();
  void make_translate ();
  bool make_return_before ();
  bool make_return_after ();
  void make_active_assign (tree var, tree by);
  void make_active_set (tree var, tree by);
  void make_active_begin_from_apply ();
  bool make_active_begin (tree t);
  void make_active_apply (tree t);
  void make_active_list (string s, string first="");
  void make_active_item (tree t);
  void make_active_equation ();
  void insert_argument ();
  void back_env (tree t, path p, string close);
  void back_dynamic (path p);
  void back_in_dynamic (tree t, path p, int min_args);
  void back_in_environment (tree t, path p);
  void rem_env (tree t, path p);

  /***************************** special actions *****************************/
  void backspace ();
  void remove_object ();
  tree the_selection ();
  void cut (path start, path end);
  void selection_cut ();
  bool math_selection_active ();
  tree math_selection ();
  void generate_bibliography (string bib, string style, string fname);
  void generate_aux ();
  void make_session (string lan, string session= "default");
  bool in_input_mode ();
  tree get_input ();
  void set_output (tree t);
};

typedef edit_modify* edit_modify_ptr;

path inner_paragraph (tree t, path p);

#endmodule // edit_modify
