
/******************************************************************************
* MODULE     : function.gen.cc
* DESCRIPTION: Concrete function classes
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <program.gen.h>

#module code_function
#import program

/******************************************************************************
* System functions
******************************************************************************/

class system_function_rep: public function_rep {
protected:
  pointer (*r)(program);
  string  s;

public:
  system_function_rep (pointer (*r2) (program), string s2): r (r2), s (s2) {}
  pointer execute (program p) { return r(p); }
  operator tree () { return tree ("System$" * s); }
  void print (ostream& out) { out << "System$" << s; }
};

/******************************************************************************
* Lambda functions
******************************************************************************/

class lambda_function_rep: public function_rep {
protected:
  program p;

public:
  lambda_function_rep (program p2): p(p2) {}
  pointer execute (program);
  operator tree () { return (tree) p; }
  void print (ostream& out) { out << p; }
};

pointer
lambda_function_rep::execute (program q) {
  int i;
  program arg (p[1]);
  pointer old [N(arg)];
  for (i=0; i<N(arg); i++) {
    old[i]= arg[i]->ins->value();
    arg[i]->ins->value()= ::execute (q[i+1]);
  }
  pointer ret= ::execute (p[2]);
  for (i=0; i<N(arg); i++) {
    // delete p[0][1][i]->ins->value()
    arg[i]->ins->value()= old[i];
  }
  return ret;
}

/******************************************************************************
* The function class
******************************************************************************/

function_rep::function_rep () { DEBUG(function_count++); }

function_rep::~function_rep () { DEBUG(function_count--); }

function::function (pointer (*routine) (program), string name):
  rep (new system_function_rep (routine, name)) { rep->ref_count++; }

function::function (program compiled_lambda_expr):
  rep (new lambda_function_rep (compiled_lambda_expr)) { rep->ref_count++; }

pointer
function::operator () (program p) {
  return rep->execute (p);
}
  
function::operator tree () {
  return (tree) (*rep);
}

ostream&
operator << (ostream& out, function f) {
  f->print (out);
  return out;
}

#endmodule // code_function
