
/******************************************************************************
* MODULE     : instance.gen.cc
* DESCRIPTION: Implementation of the different kinds of instances
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <program.gen.h>

#module code_instance
#import program

/******************************************************************************
* The instance class
******************************************************************************/

instance_rep::instance_rep (string name2, type t2):
  name (name2), t (t2) { DEBUG(instance_count++); }

instance_rep::~instance_rep () { DEBUG(instance_count--); }

pointer&
instance_rep::value () {
  fatal_error ("applies only to variable instances", "instance::value");
}

instance::operator tree () {
  return tree (":", (tree) rep->name, (tree) rep->t);
}

ostream&
operator << (ostream& out, instance ins) {
  return out << ins->name << " : " << ins->t;
}

bool
operator == (instance ins1, instance ins2) {
  return ins1.rep==ins2.rep;
}

bool
operator != (instance ins1, instance ins2) {
  return ins1.rep!=ins2.rep;
}

/******************************************************************************
* System instances
******************************************************************************/

class system_instance_rep: public instance_rep {
public:
  string  s;
  pointer (*f) (program);
  system_instance_rep (string name, type t, pointer (*f2) (program)):
    instance_rep (name, t), s (name), f (f2) {}
  pointer execute () { return (pointer) new function (f, s); }
};

instance
system_instance (string name, type t, pointer (*f) (program)) {
  return new system_instance_rep (name, t, f);
}

/******************************************************************************
* Literal instances
******************************************************************************/

class integer_instance_rep: public instance_rep {
public:
  int i;
  integer_instance_rep (string name, type t, int i2):
    instance_rep (name, t), i (i2) {}
  pointer execute () { return (pointer) new int (i); }
};

instance
integer_instance (string name, type t, int i) {
  return new integer_instance_rep (name, t, i);
}

class string_instance_rep: public instance_rep {
public:
  string s;
  string_instance_rep (string name, type t, string s2):
    instance_rep (name, t), s (s2) {}
  pointer execute () { return (pointer) new string (s); }
};

instance
string_instance (string name, type t, string s) {
  return new string_instance_rep (name, t, s);
}

class program_instance_rep: public instance_rep {
public:
  program p;
  program_instance_rep (string name, type t, program p2):
    instance_rep (name, t), p (p2) {}
  pointer execute () { return (pointer) new program (p); }
};

instance
program_instance (string name, type t, program s) {
  return new program_instance_rep (name, t, s);
}

/******************************************************************************
* Variable instances
******************************************************************************/

class variable_instance_rep: public instance_rep {
public:
  pointer ptr;
  variable_instance_rep (string name, type t):
    instance_rep (name, t), ptr (NULL) {}
  pointer  execute () { return new pointer (ptr); }
  pointer& value ();
};

instance
variable_instance (string name, type t) {
  return new variable_instance_rep (name, t);
}

pointer&
variable_instance_rep::value () {
  return ptr;
}

/******************************************************************************
* Constant instances
******************************************************************************/

class define_instance_rep: public instance_rep {
public:
  program action;
  define_instance_rep (string name, type t, program act):
    instance_rep (name, t), action (act) {}
  pointer  execute () { return ::execute (action); }
  pointer& value ();
};

pointer&
define_instance_rep::value () {
  return action[1]->ins->value ();
}

instance
define_instance (string name, type t, program action) {
  return new define_instance_rep (name, t, action);
}

#endmodule // program
