
/******************************************************************************
* MODULE     : coerce.gen.cc
* DESCRIPTION: Automatic coercion between types
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <compiler.gen.h>

#module code_coerce
#import compiler

#define TP  typed_program
#define LTP list<typed_program>

program
compiler_rep::coerce (program p, type source, type dest, int& count) {
  // Straigtforward coercion
  if (source==dest) { count=0; return p; }

  // Coercion via the coerce primitive
  count=1;
  type ct= composite ("->", source, dest);
  if (exists ("coerce!", ct)) {
    program r= find ("coerce!", composite ("->", source, dest));
    return composite (r, p, p->start, p->end);
  }

  // coercion of primitive types to trees
  if (is_string (dest, "Tree")) {
    if (is_func (source, "->", 2))
      return composite (system ("Function_to_tree!"), p, p->start, p->end);
    /*
    if (is_func (source, "Cross", 2))
      return composite (system ("Cross_to_tree!"), p, p->start, p->end);
    if (is_func (source, "Tuple", 1))
      return composite (system ("Tuple_to_tree!"), p, p->start, p->end);
      */
  }

  // source=Reference(dest) and dest is a primitive type
  if (is_func (source, "&", 1)) {
    if (is_func (dest, "->", 2))
      return composite (system ("Function_coerce!"), p, p->start, p->end);
    /*
    if (is_func (dest, "Cross", 2))
      return composite (system ("Cross_coerce!"), p, p->start, p->end);
    if (is_func (dest, "Tuple", 1))
      return composite (system ("Tuple_coerce!"), p, p->start, p->end);
      */
  }

  // source=Reference(X) and X coerces to dest
  if (is_func (source, "&", 1)) {
    program q= coerce (p, source, source[1], count);
    if (!is_string (q, "FAILED!")) {
      int count2;
      program r= coerce (q, source[1], dest, count2);
      if (!is_string (r, "FAILED!")) {
	count += count2;
	return r;
      }
    }
  }

  count= 1998;
  return program ("FAILED!"); // FAILURE
}

program
compiler_rep::coerce (program p, type source, type dest) {
  int dummy;
  return coerce (p, source, dest, dummy);
}

bool
compiler_rep::match (type source, type dest, int max_coerce) {
  int count;
  program p= coerce (program ("dummy"), source, dest, count);
  return (!is_string (p, "FAILED!")) && (count<=max_coerce);
}

#undef TP
#undef LTP

#endmodule // code_coerce
