
/******************************************************************************
* MODULE     : prg_env.gen.cc
* DESCRIPTION: The programming environment for interactive compilation
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <prg_env.gen.h>

#module code_prg_env
#import prg_env
#define AI array<instance>
#define HM hashmap<string,array<instance>>
#define LHM list<hashmap<string,array<instance>>>

/******************************************************************************
* Changing the level
******************************************************************************/

void
prg_env_rep::level_up () {
  local    = LHM (HM (AI ()), local);
  import   = LHM (HM (AI ()), import);
  prev_used= LHM (HM (AI ()), prev_used);
}

void
prg_env_rep::level_down () {
  local    = local->next;
  import   = import->next;
  prev_used= prev_used->next;
}

/******************************************************************************
* Definition of new variables and importation of variables
******************************************************************************/

static bool
contains (HM h, string var, type t) {
  int i;
  AI a= h[var];
  for (i=0; i<N(a); i++)
    if (a[i]->t==t) return TRUE;
  return FALSE;
}

static instance&
find (HM h, string var, type t) {
  int i;
  AI a= h[var];
  for (i=0; i<N(a); i++)
    if (a[i]->t==t) return a[i];
  fatal_error ("couldn't find instance of right type",
	       "find", "prg_env.gen.cc");
}

bool
prg_env_rep::define_var (instance ins)
// returns TRUE (not yet implemented) if redefining a type
{
  string& var (ins->name);
  type&   t   (ins->t);

  if (nil (local)) {
    if (contains (global->item, var, t)) find (global->item, var, t)= ins;
    else {
      if (!global->item->contains (var)) global->item (var)= AI ();
      global->item (var) << ins;
    }
  }
  else {
    if (contains (local->item, var, t)) find (local->item, var, t)= ins;
    else {
      if (!local->item->contains (var)) local->item (var)= AI ();
      local->item (var) << ins;
    }
  }
  return FALSE;
}

bool
prg_env_rep::import_var (instance ins)
// returns TRUE if variable exists as a local variable,
// or (not implemented) if redefining a type
{
  string& var (ins->name);
  type&   t   (ins->t);

  if (nil (local)) {
    if (contains (global->item, var, t)) find (global->item, var, t)= ins;
    else {
      if (!global->item->contains (var)) global->item (var)= AI ();
      global->item (var) << ins;
    }
  }
  else {
    if (contains (local->item, var, t)) return TRUE;
    if (contains (import->item, var, t)) find (import->item, var, t)= ins;
    else {
      if (!import->item->contains (var)) import->item (var)= AI ();
      import->item (var) << ins;
    }
  }
  return FALSE;
}

/******************************************************************************
* Retrieve all possible values of a variable
******************************************************************************/

static void
join (AI& a, AI b) {
  int i, j;
  for (i=0; i<N(b); i++) {
    bool flag=TRUE;
    for (j=0; j<N(a); j++)
      if (a[j]->t==b[i]->t) flag= FALSE;
    if (flag) a << b[i];
  }
}

AI
prg_env::operator [] (string var) {
  AI  ret;
  LHM global= rep->global;
  LHM local = rep->local;
  LHM import= rep->import;

  if (!nil (local)) {
    join (ret, local->item [var]);
    join (ret, import->item [var]);
    local = local->next;
    import= import->next;
  }

  while (!nil (local)) {
    if (local->item->contains (var)) {
      join (ret, local->item [var]);
      if (!rep->prev_used->item->contains (var))
	fatal_error ("Nested lambda's not yet implemented",
		     "prg_env::operator []");
      /*
	rep->prev_used->item (var)= AI ();
	join (rep->prev_used->item (var), local->item [var]);
      */
    }
    join (ret, import->item [var]);
    local = local->next;
    import= import->next;
  }
  
  while (!nil (global)) {
    join (ret, global->item [var]);
    global= global->next;
  }

  return ret;
}

/******************************************************************************
* Output
******************************************************************************/

ostream&
operator << (ostream& out, prg_env env) {
  out << "---------------------------------------------------------------\n";
  out << "Global variables\n";
  out << "---------------------------------------------------------------\n";
  out << env->global << "\n";
  if (!nil (env->local)) {
    out << "Local variables\n";
    out << "---------------------------------------------------------------\n";
    out << env->local << "\n";
    out << "---------------------------------------------------------------\n";
    out << "Imported variables\n";
    out << "---------------------------------------------------------------\n";
    out << env->import << "\n";
    out << "---------------------------------------------------------------\n";
  }
  return out << "\n";
}

#undef LHM
#undef HM
#undef AI
#endmodule // code_prg_env
