
/******************************************************************************
* MODULE     : executor.gen.cc
* DESCRIPTION: Interactive execution of programs
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <program.gen.h>

#module code_executor
#import program

pointer
execute (program p) {
  // cout << "Executing " << p << "\n";
  switch (p->status) {
  case NONCOMPILED:
    cout << "Executing " << p << "\n";
    fatal_error ("not yet compiled", "execute");
  case COMPILED:
    return p->ins->execute ();
  case COMPOSITE:
    function* cmd= (function*) execute (p[0]);
    pointer res= (*cmd)(p);
    delete cmd;
    return res;
  }
}

pointer
prg_skip (program p) { (void) p;
  return NULL;
}

pointer
prg_define (program p) {
  p[1]->ins->value()= execute (p[2]);
  return NULL;
}

pointer
prg_var (program p) {
  p[1]->ins->value()= execute (p[2]);
  return NULL;
}

pointer
prg_function_coerce (program p) {
  function** arg0 = (function**) execute (p[1]);
  function* res = new function (**arg0);
  delete arg0;
  return res;
}

pointer
prg_function_to_tree (program p) {
  function* arg0 = (function*) execute (p[1]);
  tree t= (tree) (*arg0);
  tree* res = new tree (t);
  delete arg0;
  return res;
}

pointer
prg_lambda (program p) {
  return (pointer) new function (p);
}

pointer
prg_seq (program p) {
  int i;
  pointer ptr=NULL;
  for (i=1; i<N(p); i++) ptr= execute (p[i]);
  return ptr;
}

pointer
prg_if (program p) {
  bool* cond = (bool*) execute (p[1]);
  if (*cond) (void) execute (p[2]);
  delete cond;
  return NULL;
}

pointer
prg_if_else (program p) {
  bool* cond = (bool*) execute (p[1]);
  pointer res;
  if (*cond) res= execute (p[2]);
  else res= execute (p[3]);
  delete cond;
  return res;
}

#endmodule // code_executor
