
/******************************************************************************
* MODULE     : gen_init.gen.cc
* DESCRIPTION: generate initialization program for Prg package
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <program.gen.h>
#include <file.gen.h>
#include <list.gen.h>

#module code_gen_init
#import program
#import file
#import list (tree)
#import hashmap (string,tree)

program lisp_to_program (string s, int& i, location root);

/******************************************************************************
* The make_init structure
******************************************************************************/

struct make_init {
  hashmap<string,tree> ttran;
  list<tree>           vtype;
  list<tree>           vtran;
  string               bpref;
  string               btype;
  string               bpath;

  make_init (string bpref, string btype, string bpath);
  void insert (program p);
  void generate_prologue ();
  void generate_routines ();
  void generate_init (program p);
  string cpp_type (tree t);
  string cpp_func (tree t, tree arg);
};

make_init::make_init (string s1, string s2, string s3):
  ttran ("?"), bpref (s1), btype (s2), bpath (s3) {}

/******************************************************************************
* Definition of types and routines
******************************************************************************/

tree
simplify_apply (tree t) {
  int i;
  if (N(t)==0) return t;
  if ((N(t)>0) && (t->label == "()") && (N(t[0])==0)) {
    tree r (t[0]->label, N(t)-1);
    for (i=1; i<N(t); i++) r[i-1]= simplify_apply (t[i]);
    return r;
  }
  if ((N(t)>0) && (t->label== "()") && (N(t[0])>0)) {
    tree r ("apply", N(t));
    for (i=0; i<N(t); i++) r[i]= simplify_apply (t[i]);
    return r;
  }
  tree r (t->label, N(t));
  for (i=0; i<N(t); i++) r[i]= simplify_apply (t[i]);
  return r;
}

void
make_init::insert (program p) {
  if ((p->status!=COMPOSITE) || (N(p)!=3) ||
      (p[0]->status!=NONCOMPILED)) {
    cerr << "Fatal error: bad definition: " << p << "\n";
    exit (1);
    return;
  }
  if ((p[1]->status==NONCOMPILED) && (p[1]->s == "Type")) {
    tree t (simplify_apply (p[2]));
    ttran (p[0]->s) = t;
    return;
  }
  tree t1 (simplify_apply (p[1]));
  tree t2 (simplify_apply (p[2]));
  vtype << tree (":", p[0]->s, t1);
  vtran << tree ("->", p[0]->s, t2);
}

/******************************************************************************
* Code generation subroutines
******************************************************************************/

string
make_init::cpp_type (tree t) {
  if (N(t)==0) {
    tree u= ttran [t->label];
    if ((N(u)>0) || (u->label == "?"))
      fatal_error ("unknown type encountered", "make_init::cpp_type");
    return copy (u->label);
  }
  if (is_func (t, "*")) return cpp_type (t[0]) * "*";
  fatal_error ("unknown type encountered", "make_init::cpp_type");
}

string
make_init::cpp_func (tree t, tree arg) {
  if (N(t)!=0)
    fatal_error ("unknown function type encountered", "make_init::cpp_func");

  int i, n=N(arg);
  string s= copy (t->label);

  if (((N(s)>1) && (s[0]=='.')) ||
      ((N(s)>2) && (s[0]=='-') && (s[1]=='>'))) {
    s= "(" * copy (arg[0]->label) * ")" * s;
    if (n==1) {
      if (s[N(s)-1]=='*') s= s (0, N(s)-1);
      else return s;
    }
    s= s * " (";
    for (i=1; i<n; i++) {
      if (i>0) s << ", ";
      s << copy (arg[i]->label);
    }
    s << ")";
    return s;
  }

  if (n==1) {
    if ((s == "&") ||
	(s == "*") ||
	(s == "~") ||
	(s == "-"))
      return s * "(" * copy (arg[0]->label) * ")";
    if (s == "print!")
      return "cout << " * copy (arg[0]->label);
  }
  
  if (n==2) {
    if ((s == "==") ||
	(s == "!=") ||
	(s == "<=") ||
	(s == ">=") ||
	(s == "<") ||
	(s == ">") ||
	(s == "<") ||
	(s == ">") ||
	(s == "=") ||
	(s == "|") ||
	(s == "&") ||
	(s == "||") ||
	(s == "&&") ||
	(s == "+") ||
	(s == "-") ||
	(s == "*") ||
	(s == "/") ||
	(s == "%") ||
	(s == "<<") ||
	(s == ">>") ||
	(s == "+=") ||
	(s == "-=") ||
	(s == "*=") ||
	(s == "/=")) {
      return "(" * copy (arg[0]->label) * ") " * s *
	" (" * copy (arg[1]->label) * ")";
    }
    if (s == "[]")
      return "(" * copy (arg[0]->label) * ") [" * copy (arg[1]->label) * "]";
  }
  
  if (s == "new!") {
    s= string ("");
    for (i=0; i<n; i++) {
      if (i>0) s << ", ";
      s << copy (arg[i]->label);
    }
    return s;
  }
  
  if (s == "apply!") {
    s= "(" * copy (arg[0]->label) * ") (";
    for (i=1; i<n; i++) {
      if (i>1) s << ", ";
      s << copy (arg[i]->label);
    }
    s << ")";
    return s;
  }
  
  s << " (";
  for (i=0; i<n; i++) {
    if (i>0) s << ", ";
    s << copy (arg[i]->label);
  }
  s << ")";
  return s;
}

/******************************************************************************
* Code generation
******************************************************************************/

void
make_init::generate_prologue () {
  if (N(btype)==0) return;
  
  cout << "#ifndef __FUNCTION_" << btype << "\n";
  cout << "#define __FUNCTION_" << btype << "\n\n";

  cout << "class " << btype << "_function_rep: public function_rep {\n";
  cout << "protected:\n";
  cout << "  " << btype << " x;\n";
  cout << "  pointer (*r)(" << btype << ", program);\n";
  cout << "  string  s;\n\n";
  cout << "public:\n";
  cout << "  " << btype << "_function_rep (" << btype
       << " x2, pointer (*r2) (" << btype << ", program), string s2):\n"
       << "    x (x2), r (r2), s (s2) {}\n";
  cout << "  pointer execute (program p) { return r (x, p); }\n";
  cout << "  operator tree () { return tree (\"" << btype << "$\" * s); }\n";
  cout << "  void print (ostream& out) { out << \"" <<btype<< "$\" << s; }\n";
  cout << "};\n\n";

  cout << "class " << btype << "_instance_rep: public instance_rep {\n";
  cout << "public:\n";
  cout << "  " << btype << " x;\n";
  cout << "  pointer (*f) (" << btype << ", program);\n";
  cout << "  string  s;\n\n";
  cout << "  " << btype << "_instance_rep (string name, type t, "
       << btype << " x2, pointer (*f2) (" << btype
       << ", program)):\n";
  cout << "    instance_rep (name, t), x (x2), f (f2), s (name) {}\n";
  cout << "  pointer execute () { return (pointer) new function (new "
       << btype << "_function_rep (x, f, s)); }\n";
  cout << "};\n\n";

  cout << "extern program lisp_to_program (string, int&, int);\n\n";

  cout << "void\n";
  cout << "primitive (compiler cpl, string name, "
       << btype << " x, string t, pointer (*r)("
       << btype << ", program)) {\n";
  cout << "  int i=0;\n";
  cout << "  program p (lisp_to_program (t, i, 0));\n";
  cout << "  instance ins (new " << btype
       << "_instance_rep (name, p, x, r));\n";
  cout << "  cpl->env->define_var (ins);\n";
  cout << "}\n\n";

  cout << "#endif\n\n";

  cout << "/****************************************************************************\n";
  cout << "* Main routines\n";
  cout << "****************************************************************************/\n\n";  
}

void
make_init::generate_routines () {
  int count=0, i;
  list<tree> l1= vtype;
  list<tree> l2= vtran;
  while (!nil (l1)) {
    string var = l1->item[0]->label;
    tree   type= l1->item[1];
    tree   tran= l2->item[1];
    if (is_func (type, "->", 2)) {
      tree dom= type[0];
      tree im = type[1];
      if (dom->label == "Void") dom= tree ("Cross");
      else if (!is_func (dom, "Cross")) dom= tree ("Cross", dom);

      tree arg ("", N(dom));
      for (i=0; i<N(dom); i++) {
	arg[i]= tree ("*arg" * as_string(i));
	if (is_func (dom[i], "&", 1)) {
	  arg[i]= tree ("**arg" * as_string(i));
	  dom[i]= tree ("*", dom[i][0]);
	}
      }
      bool flag= is_func (im, "&", 1);
      if (flag) im= tree ("*", im[0]);
      
      cout << "pointer\n";
      if (N(btype)==0) cout << "prg_" << bpref << "_" << count
			   << " (program p) {";
      else cout << "prg_" << bpref << "_" << count << " ("
		<< btype << " x, program p) {";
      if (N(dom)>0) cout << "\n";
      else cout << " (void) p;\n";
      for (i=0; i<N(dom); i++) {
	cout << "  " << cpp_type (dom[i]) << "* arg" << i << " = ("
	     << cpp_type (dom[i]) << "*) execute (p[" << i+1 << "]);\n";
      }
      if (im->label == "Void") {
	if (N(btype)==0) cout << "  " << cpp_func (tran, arg) << ";\n";
	else cout << "  " << bpath << "->" << cpp_func (tran, arg) << ";\n";
      }
      else {
	if (N(btype)==0) {
	  if (N (cpp_func (tran, arg))==0)
	    cout << "  " << cpp_type (im) << " res;\n";
	  else {
	    cout << "  " << cpp_type (im) << " res (";
	    if (flag) cout << "& (" << cpp_func (tran, arg) << "));\n";
	    else cout << cpp_func (tran, arg) << ");\n";
	  }
	}
	else {
	  if ((N (cpp_func (tran, arg))==0) || flag)
	    fatal_error ("invalid method name", "make_init::generate_routines");
	  cout << "  " << cpp_type (im) << " res ("
	       << bpath << "->" << cpp_func (tran, arg) << ");\n";
	}
      }
      for (i=0; i<N(dom); i++) cout << "  delete arg" << i << ";\n";
      if (im->label == "Void") cout << "  return NULL;\n";
      else cout << "  return (pointer) new " << cpp_type (im) << " (res);\n";
      cout << "}\n\n";

    }
    count++;
    l1=l1->next;
    l2=l2->next;
  }
}

void
make_init::generate_init (program q) {
  int i,j;
  cout << "/****************************************************************************\n";
  cout << "* Initialize\n";
  cout << "****************************************************************************/\n\n";  
  cout << "void\n";
  cout << "initialize_" << bpref << " (compiler cpl";
  if (N(btype)==0) cout << ") {\n";
  else cout << ", " << btype << " x) {\n";
  for (i=1,j=0; i<N(q); i++) {
    program& p(q[i]);
    if ((p[1]->status==NONCOMPILED) && (p[1]->s == "Type")) {
      cout << "  // cpl->define (\42" << p[0]->s << "\42, program (\42Type\42));\n";
      continue;
    }
  if (N(btype)==0)
    cout << "  cpl->primitive (\42" << p[0]->s << "\42, \42"
	 << p[1] << "\42, prg_" << bpref << "_" << j << ");\n";
  else 
    cout << "  primitive (cpl, \42" << p[0]->s << "\42, x, \42"
	 << p[1] << "\42, prg_" << bpref << "_" << j << ");\n";
    j++;
  }
  cout << "}\n";
}

int
main (int argc, char** argv) {
  if ((argc!=3) && (argc!=5)) {
    cerr << "Usage: gen_init <file.ext> <prefix>\n";
    cerr << "  or   gen_init <file.ext> <prefix> <class> x\n";
    cerr << "  or   gen_init <file.ext> <prefix> <class> \"x-><item>\"\n";
    return 0;
  }

  string s, bpref, btype, bpath;
  load_string (argv[1], s, TRUE);
  int i=0;
  program p (lisp_to_program (s, i, 0));
  if ((p->status!=COMPOSITE) ||
      (p[0]->status!=NONCOMPILED) ||
      (!(p[0]->s == "extern")))
    fatal_error ("invalid file", "gen_init");
  bpref= argv[2];
  if (argc==5) { btype= argv[3]; bpath= argv[4]; }

  make_init mi (bpref, btype, bpath);
  for (i=1; i<N(p); i++)
    mi.insert (p[i]);

  cout << "\n/****************************************************************************\n";
  cout << "* Initialization file for interactive compiler and executor\n";
  cout << "* This file has been generated automatically by the command\n";
  cout << "*   gen_init " << argv[1] << " " << bpref;
  if (N(btype)>0) cout << " " << btype << " \"" << bpath << "\"";
  cout << "\n****************************************************************************/\n\n";
  if (argc==2) cout << "\43module code_prg_" << bpref << "\n\n";
  else cout << "\43module code_prg_" << bpref << "\n\n";
  mi.generate_prologue ();
  mi.generate_routines ();
  mi.generate_init (p);
  if (argc==2) cout << "\n\43endmodule // code_prg_" << bpref << "\n";
  else cout << "\n\43endmodule // code_prg_" << bpref << "\n\n";
  return 0;
}

#endmodule // code_gen_init

