
/******************************************************************************
* MODULE     : compile.gen.cc
* DESCRIPTION: Interactive compilation of programs
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <program.gen.h>
#include <prg_env.gen.h>

#module compiler
#import program
#import prg_env

class compiler_rep: concrete_struct {
public:
  prg_env env;

  compiler_rep ();
  void primitive (string name, string type, void* (*routine) (program));
  list<typed_program> compile (program p);
  list<typed_program> compile_composite (typed_program tp, program ty, bool f);
  list<typed_program> compile_integer (program p);
  list<typed_program> compile_string (program p);
  list<typed_program> compile_define (program p, bool dynamic);
  list<typed_program> compile_quote (program p);
  list<typed_program> compile_seq (program p);
  list<typed_program> compile_if (program p);
  list<typed_program> compile_if_else (program p);
  list<typed_program> compile_lambda (program p);
  bool match (program t1, program t2, int max_coerce);
  program coerce (program p, program source, program dest, int& count);
  program coerce (program p, program source, program dest);
  void error (program in, string message);
  bool ok (list<typed_program> ltp, program p);
  bool of_type (list<typed_program> ltp, program type, program original);
  program filter (list<typed_program> ltp, program type, program original);
  bool exists (string name, program type);
  instance find (string name, program type);
  instance system (string name);

  friend class compiler;
};

class compiler {
#import concrete (compiler, compiler_rep)
  compiler ();
};
#import code_concrete (compiler, compiler_rep)

#endmodule // compiler
