
/******************************************************************************
* MODULE     : prg_env.gen.h
* DESCRIPTION: The programming environment for interactive compilation
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <program.gen.h>
#include <hashmap.gen.h>
#include <list.gen.h>

#module prg_env
#import program
#import hashmap (string,array<instance>)
#import list (hashmap<string,array<instance>>)
#define AI array<instance>
#define HM hashmap<string,array<instance>>
#define LHM list<hashmap<string,array<instance>>>

class prg_env_rep: concrete_struct {
  LHM global;     // stack of global variables for recursive environments
  LHM local;      // stack of local variables at different nesting levels
  LHM import;     // stack of imported variables at different nesting levels

public:
  LHM  prev_used; // variables from previous nesting levels used at
		  // current (e.g. (lambda (a) (lambda (b) (+ a b))))

  inline prg_env_rep ():
    global (HM (array<instance> ())) {}
  inline prg_env_rep (LHM previous):
    global (HM (array<instance> ()), previous) {}

  void level_up ();
  void level_down ();
  bool define_var (instance c);
  bool import_var (instance c);
  
  friend class prg_env;
  friend ostream& operator << (ostream& out, prg_env env);
};

class prg_env {
#import concrete (prg_env, prg_env_rep)
  inline prg_env (LHM previous): rep (new prg_env_rep (previous)) {}
  AI operator [] (string var);
};
#import code_concrete (prg_env, prg_env_rep)

#undef LHM
#undef HM
#undef AI
#endmodule // prg_env
