
/******************************************************************************
* MODULE     : program.gen.h
* DESCRIPTION: Intern representation of uncompiled and compiled programs
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <tree.gen.h>
#include <list.gen.h>
#include <hashmap.gen.h>

#module program
#import tree

/********************* Some macros and forward definitions *******************/

#define NONCOMPILED    1
#define COMPILED       2
#define COMPOSITE      3

typedef int location;
class program_rep;
class typed_program_rep;
class instance_rep;
class function_rep;
class program;
class instance;
class array<program>;

/**************************** The main classes *******************************/

class program {
#import concrete (program, program_rep)
  program (char* s="", location start=-1, location end=-1);
  program (string s, location start=-1, location end=-1);
  program (instance ins, location start=-1, location end=-1);
  program (array<program> a, location start=-1, location end=-1);
  program& operator [] (int i);
  operator tree ();
};
typedef program type;

class typed_program {
#import concrete (typed_program, typed_program_rep)
  typed_program (program p, type t);
  operator tree ();
};

class instance {
#import abstract_null (instance, instance_rep)
  operator tree ();
  friend bool operator == (instance, instance);
  friend bool operator != (instance, instance);
  friend pointer& get_define_pointer (instance ins);
  friend pointer& get_variable_pointer (instance ins);
};

class function {
#import abstract_null (function, function_rep)
  function (pointer (*routine) (program), string name);
  function (program compiled_lambda_expr);
  pointer operator () (program);
  operator tree ();
};

#import array(program)
#import list(typed_program)
#import array(instance)

/********************************* Programs **********************************/

class program_rep: public concrete_struct {
  array<program> a;

public:
  int            status;
  string         s;
  instance       ins;
  location       start, end;

  program_rep (string s, location start, location end);
  program_rep (instance ins, location start, location end);
  program_rep (array<program> a, location start, location end);

  friend class program;
  friend int N (program p);
  friend bool operator == (program p1, program p2);
  friend bool operator != (program p1, program p2);
};

ostream& operator << (ostream& out, program p);
program composite (program cmd, program arg1,
		   location start=-1, location end=-1);
program composite (program cmd, program arg1, program arg2,
		   location start=-1, location end=-1);
bool is_string (program p);
bool is_instance (program p);
bool is_composite (program p);
bool is_compiled (program p);
bool is_string (program p, char* s);
bool is_string (program p, string s);
bool is_func (program p, char* s, int n=-1);
bool is_func (program p, string s, int n=-1);
program lisp_expr_to_program (string s, location root);
program lisp_prg_to_program (string s, location root);
string program_to_lisp_expr (program p);

/**************************** Typed Programs *********************************/

class typed_program_rep: public concrete_struct {
public:
  program p;
  type    t;

  typed_program_rep (program p, type t);
};

ostream& operator << (ostream& out, typed_program tp);
bool operator == (typed_program tp1, typed_program tp2);
bool operator != (typed_program tp1, typed_program tp2);

/******************* Instances of programming environments *******************/

extern int instance_count;
class instance_rep: public abstract_struct {
public:
  string name;
  type   t;

  instance_rep (string name, type t);
  virtual ~instance_rep ();
  virtual pointer  execute () = 0;
  virtual pointer& value ();        // for initialization of variable instances
};

ostream& operator << (ostream& out, instance ins);
bool operator == (instance ins1, instance ins2);
bool operator != (instance ins1, instance ins2);

instance system_instance (string name, type t, pointer (*f) (program));
instance integer_instance (string name, type t, int i);
instance string_instance (string name, type t, string s);
instance program_instance (string name, type t, program p);
instance define_instance (string name, type t, program constructor);
instance variable_instance (string name, type t);

/************************** Abstract functions *******************************/

extern int function_count;
class function_rep: public abstract_struct {
public:
          function_rep();
  virtual ~function_rep();
  virtual pointer execute (program) = 0;
  virtual operator tree () = 0;
  virtual void print (ostream& out) = 0;
};

ostream& operator << (ostream& out, function f);

/**************************** Final inlining *********************************/

#import code_concrete (program, program_rep)
#import code_concrete (typed_program, typed_program_rep)
#import code_abstract_null (instance, instance_rep)
#import code_abstract_null (function, function_rep)

void* execute (program p);

#endmodule // program
