
/******************************************************************************
* MODULE     : math_language.gen.cc
* DESCRIPTION: the language for typesetting mathematics
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <analyze.gen.h>
#include <Languages/hyphenate.gen.h>
#include <Languages/Mathematical/math_language.gen.h>

#module code_math_language
#import analyze
#import hyphenate
#import math_language

/******************************************************************************
* Load a mathematical language
******************************************************************************/

string
math_language_rep::get_string (string s, int& i) {
  while ((i<N(s)) && (s[i]!=' ') && (s[i]!='\t') && (s[i]!='\n')) i++;
  while ((i<N(s)) && ((s[i]==' ') || (s[i]=='\t') || (s[i]=='='))) i++;
  int start= i;
  while ((i<N(s)) && (s[i]!='\n')) i++;
  int end= i;
  while ((end>start) && ((s[end-1]==' ') || (s[end-1]=='\t'))) end--;
  // cout << "String= " << s (start, end) << "\n";
  return s (start, end);
}

void
math_language_rep::get_class (string s, int& i) {
  class_name= get_string (s, i);
  class_def = TRUE;
}

void
math_language_rep::get_members (string s, int& i) {
  class_name= get_string (s, i);
  class_def = FALSE;
  // cout << "Members " << class_name << "\n";
}

void
math_language_rep::get_type (string s, int& i) {
  if (class_def) {
    string r= get_string (s, i);
    if (r == "Symbol") op_type (class_name)= SYMBOL;
    else if (r == "Prefix") op_type (class_name)= PREFIX;
    else if (r == "Postfix") op_type (class_name)= POSTFIX;
    else if (r == "Infix") op_type (class_name)= INFIX;
    else if (r == "Left Associative Infix")
      op_type (class_name)= LEFT_ASS_INFIX;
    else if (r == "Right Associative Infix")
      op_type (class_name)= RIGHT_ASS_INFIX;
    else if (r == "Associative Infix") op_type (class_name)= ASS_INFIX;
    else if (r == "Opening Bracket") op_type (class_name)= OPENING_BRACKET;
    else if (r == "Separator") op_type (class_name)= SEPARATOR;
    else if (r == "Closing Bracket") op_type (class_name)= CLOSING_BRACKET;
    else {
      cerr << "Attempt to associate type " << r
	   << " to " << class_name << "\n";
      fatal_error ("Unknown type", "math_language_rep::get_type");
    }
  }
  else fatal_error ("Type declaration outside class definition",
		    "math_language_rep::get_type");
}

void
math_language_rep::get_precedence (string s, int& i) {
  if (class_def) priority (class_name)= as_int (get_string (s, i));
  else fatal_error ("Precedence declaration outside class definition",
		    "math_language_rep::get_precedence");
}

void
math_language_rep::get_lpenalty (string s, int& i) {
  if (class_def) {
    string r= get_string (s, i);
    if (r == "Panic") pen1 (class_name)= HYPH_PANIC;
    else pen1 (class_name)= as_int (r);
  }
  else fatal_error ("Left Penalty declaration outside class definition",
		    "math_language_rep::get_lpenalty");
}

void
math_language_rep::get_rpenalty (string s, int& i) {
  if (class_def) {
    string r= get_string (s, i);
    if (r == "Panic") pen2 (class_name)= HYPH_PANIC;
    else pen2 (class_name)= as_int (r);
  }
  else fatal_error ("Right Penalty declaration outside class definition",
		    "math_language_rep::get_rpenalty");
}

void
math_language_rep::get_spacing (string s, int& i) {
  if (class_def) {
    int j;
    string both= get_string (s, i);
    for (j=0; j<N(both); j++)
      if (both[j]==',') {
	string l= both (0, j);
	string r= both (j+1, N(both));
	while ((N(l)>0) && ((l[N(l)-1] == ' ') || (l[N(l)-1] == '\t')))
	  l= l(0, N(l)-1);
	while ((N(r)>0) && ((r[0]==' ') || (r[0]=='\t'))) r= r(1, N(r));
	if (l == "None") spc1 (class_name)= NO_SPACE;
	else if (l == "Default") spc1 (class_name)= OP_SPACE;
	else if (l == "Big") spc1 (class_name)= BIGOP_SPACE;
	else {
	  cerr << "Attempt to associate space " << l
	       << " to " << class_name << "\n";
	  fatal_error ("Unknown space", "math_language_rep::get_type");
	}
	if (r == "None") spc2 (class_name)= NO_SPACE;
	else if (r == "Default") spc2 (class_name)= OP_SPACE;
	else if (r == "Big") spc2 (class_name)= BIGOP_SPACE;
	else {
	  cerr << "Attempt to associate space " << r
	       << " to " << class_name << "\n";
	  fatal_error ("Unknown space", "math_language_rep::get_type");
	}
	return;
      }
    fatal_error ("Missing comma in Spacing declaration",
		 "math_language_rep::get_spacing");
  }
  else fatal_error ("Spacing declaration outside class definition",
		    "math_language_rep::get_spacing");
}

void
math_language_rep::get_limits (string s, int& i) {
  if (class_def) has_limits (class_name)= (get_string (s, i) != "False");
  else fatal_error ("Limits declaration outside class definition",
		    "math_language_rep::get_limits");
}

math_language_rep::math_language_rep (string name, string s):
  language_rep (name, math_enc), op_type (UNKNOWN), priority (0),
  pen1 (HYPH_INVALID), pen2 (HYPH_INVALID),
  spc1 (NO_SPACE), spc2 (NO_SPACE), has_limits (FALSE),
  type ("symbol"), class_name ("symbol"), class_def (TRUE)
{
  resource<language> (name)= (int) this;
  op_type ("symbol") = SYMBOL;
  priority ("symbol")= 1000;

  int i;
  for (i=0; i<N(s); i++) {
    while ((i<N(s)) && ((s[i]==' ') || (s[i]=='\t'))) i++;
    if ((i+5<N(s)) && (s(i,i+5)=="Class")) get_class (s, i);
    else if ((i+7<N(s)) && (s(i,i+7)=="Members")) get_members (s, i);
    else if ((i+4<N(s)) && (s(i,i+4)=="Type")) get_type (s, i);
    else if ((i+10<N(s)) && (s(i,i+10)=="Precedence")) get_precedence (s, i);
    else if ((i+12<N(s)) && (s(i,i+12)=="Left-Penalty")) get_lpenalty (s, i);
    else if ((i+7<N(s)) && (s(i,i+7)=="Penalty")) get_rpenalty (s, i);
    else if ((i+13<N(s)) && (s(i,i+13)=="Right-Penalty")) get_rpenalty (s, i);
    else if ((i+7<N(s)) && (s(i,i+7)=="Spacing")) get_spacing (s, i);
    else if ((i+6<N(s)) && (s(i,i+6)=="Limits")) get_limits (s, i);
    else if ((i+2<N(s)) && (s(i,i+2)=="##")) (void) get_string (s, i);
    else {
      while ((i<N(s)) && (s[i]!='\n')) {
	while ((i<N(s)) && ((s[i]==' ') || (s[i]=='\t'))) i++;
	int start= i;
	while ((i<N(s)) && (s[i]!=' ') && (s[i]!='\t') && (s[i]!='\n')) i++;
	string symbol= s (start, i);
	if ((N(symbol)>1) &&
	    (class_def || (class_name != "operator-with-limits")))
	  symbol= "<" * symbol * ">";
	// cout << "  Member: " << symbol << "\n";
	if ((!class_def) && (op_type->contains (class_name)))
	  type (symbol)= class_name;
	else {
	  cerr << "Attempt to insert " << symbol
	       << " to class " << class_name << "\n";
	  if (class_def)
	    fatal_error ("Member declaration inside class definition",
			 "math_language_rep::get_members");
	  else
	    fatal_error ("Unknown class",
			 "math_language_rep::get_members");
	}
      }
    }
  }
}

/******************************************************************************
* Spacing
******************************************************************************/

space
get_space (int i, space fn_spc) {
  switch (i) {
  case NO_SPACE:
    return space (0);
  case TINY_SPACE:
    return space (0, fn_spc->min>>3, 5*(fn_spc->min>>5));
  case OP_SPACE:
    return space (fn_spc->min>>1, fn_spc->def>>1, fn_spc->max);
  case BIGOP_SPACE:
    return fn_spc;
  default:
    fatal_error ("unknown space", "get_space", "math_language.gen.cc");
  }
}

/******************************************************************************
* Advancing a word
******************************************************************************/

string
math_language_rep::next_word (string s, int& pos) {
  int start= pos;

  if (pos>=N(s)) return string ("");

  if ((s[pos]>='0') && (s[pos]<='9')) {
    while ((pos<N(s)) && is_numeric (s[pos])) pos++;
    while (s[pos-1]=='.') pos--;
    return s (start, pos);
  }

  if (is_alpha (s[pos])) {
    while ((pos<N(s)) && (is_alpha (s[pos]))) pos++;
    return s (start, pos);
  }

  if (s[pos]=='<') {
    while ((pos<N(s)) && (s[pos]!='>')) pos++;
    if (pos<N(s)) pos++;
    return s (start, pos);
  }

  if ((s[pos]=='`') || (s[pos]=='\'')) {
    pos++;
    while ((pos<N(s)) && (s[pos]==s[pos-1])) pos++;
    return s (start, pos);
  }

  pos++;
  return s (start, pos);
}

void
math_language_rep::advance (string s, int& pos, space fn_spc, space fn_extra,
			    space& Spc1, space& Spc2, int& Pen1, int& Pen2)
{
  (void) fn_extra;
  bool op_flag1=
    (pos==0) ||
    ((pos>=2) && is_alpha (s[pos-2]) && is_alpha (s[pos-1]));
  string r= next_word (s, pos);
  string Type (type [r]);
  Spc1= get_space (spc1 [Type], fn_spc);
  Spc2= get_space (spc2 [Type], fn_spc);
  Pen1= pen1 [Type];
  Pen2= pen2 [Type];

  if (r == " ") {
    bool op_flag2=
      (pos==N(s)) ||
      (((pos+2)<N(s)) && is_alpha (s[pos]) && is_alpha (s[pos+1]));
    if (op_flag1 || op_flag2) Spc2= get_space (OP_SPACE, fn_spc);
  }
  if (op_type[Type] == CLOSING_BRACKET) {
    int end= pos;
    string next (next_word (s, pos));
    pos= end;
    if (op_type [type [next]] == CLOSING_BRACKET) Pen2= HYPH_INVALID;
  }
}

/******************************************************************************
* Hyphenation
******************************************************************************/

array<int>
math_language_rep::get_hyphens (string s) {
  if (N(s)==0)
    fatal_error ("hyphenation of empty string",
		 "math_language_rep::get_hyphens");

  int i;
  bool flag= is_numeric (s);
  array<int> penalty (N(s)-1);
  for (i=0; i<N(s)-1; i++) penalty[i]= (flag? HYPH_PANIC: HYPH_INVALID);
  return penalty;
}

void
math_language_rep::hyphenate (string s, int after, string& left, string& right)
{
  left = s (0, after+1) * string ("-");
  right= s (after+1, N(s));
}

#endmodule // code_math_language
