
/******************************************************************************
* MODULE     : math_language.gen.h
* DESCRIPTION: Mathematical languages
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <language.gen.h>

#module math_language
#import language
#import hashmap (string,int)
#import hashmap (string,string)

typedef hashmap<string,int> hashmap<string,bool>;

/******************************************************************************
* Operator and space types
******************************************************************************/

#define UNKNOWN           0
#define SYMBOL            1
#define PREFIX            2
#define POSTFIX           3
#define INFIX             4
#define LEFT_ASS_INFIX    5
#define RIGHT_ASS_INFIX   6
#define ASS_INFIX         7
#define OPENING_BRACKET  10
#define SEPARATOR        11
#define CLOSING_BRACKET  12

#define NO_SPACE     0
#define TINY_SPACE   1
#define OP_SPACE     2
#define BIGOP_SPACE  3

/******************************************************************************
* Mathematical languages
******************************************************************************/

struct math_language_rep: language_rep {
  hashmap<string,int>  op_type;    // the operator type (see above)
  hashmap<string,int>  priority;   // the operator priority
  hashmap<string,int>  pen1;       // left linebreak penalty
  hashmap<string,int>  pen2;       // right linebreak penalty
  hashmap<string,int>  spc1;       // spacing before the operator
  hashmap<string,int>  spc2;       // spacing after the operator
  hashmap<string,bool> has_limits; // operator has limits

  hashmap<string,string> type;     // associates a class to a symbol
  string class_name;               // current type name
  bool   class_def;                // in class definition phase

  math_language_rep (string name, string s);
  string get_string (string s, int& i);
  void get_class (string s, int& i);
  void get_members (string s, int& i);
  void get_type (string s, int& i);
  void get_precedence (string s, int& i);
  void get_lpenalty (string s, int& i);
  void get_rpenalty (string s, int& i);
  void get_spacing (string s, int& i);
  void get_limits (string s, int& i);

  void skip_spaces (string s, int& pos, space fn_spc, space& spc);
  string next_word (string s, int& pos);
  void advance (string s, int& pos, space fn_spc, space fn_extra,
		space& spc1, space& spc2, int& pen1, int& pen2);
  array<int> get_hyphens (string s);
  void hyphenate (string s, int after, string& left, string& right);
};

#endmodule // math_language
