
/******************************************************************************
* MODULE     : dutch.gen.cc
* DESCRIPTION: the module for the dutch language
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <analyze.gen.h>
#include <language.gen.h>
#include <Languages/hyphenate.gen.h>

#module code_dutch
#import analyze
#import language
#import hyphenate

struct dutch_rep: language_rep {
  hashmap<string,string> H;

  dutch_rep ();
  void advance (string s, int& pos, space fn_spc, space fn_extra,
		space& spc1, space& spc2, int& pen1, int& pen2);
  array<int> get_hyphens (string s);
  void hyphenate (string s, int after, string& left, string& right);
};

dutch_rep::dutch_rep ():
  language_rep ("dutch"), H (load_hyphen_table ("dutch")) {}

void
dutch_rep::advance (string s, int& pos, space fn_spc, space fn_extra,
		      space& spc1, space& spc2, int& pen1, int& pen2)
     // This routine scans the next logical entity in a phrase;
     // the entity initially starts at 'pos' and ends at 'pos' at exit.
     // 'spc1' and 'spc2' are ponctuation spaces needed before and after
     // the word, which may be a ponctuation character or sequence of them.
     // 'pen1' resp. 'pen2' are the penalties for a line-break
     // before resp. after the word.
{
  spc1= space (0);
  spc2= space (0);
  pen1= HYPH_INVALID;
  pen2= HYPH_INVALID;
  if (pos==N(s)) return;

  if (s[pos]==' ') {
    pos++;
    // while ((pos<N(s)) && (s[pos]==' ')) pos++;
    spc2= fn_spc;
    pen2= 0;
    return;
  }
  
  if (is_ponctuation (s[pos])) {
    while ((pos<N(s)) && is_ponctuation (s[pos])) pos++;
    if ((pos==N(s)) || (s[pos]!=' ')) return;
    switch (s[pos-1]) {
    case ',': case ':': case ';':
      spc2= fn_spc;
      break;
    case '.': case '!': case '?':
      spc2= fn_spc+ fn_extra;
      break;
    }
    pen2= 0;
    return;
  }

  if (s[pos]=='-') {
    pos++;
    pen2= 0;
    return;
  }

  if (is_iso_alpha (s[pos])) {
    while ((pos<N(s)) && is_iso_alpha (s[pos])) pos++;
    return;
  }

  if (is_numeric (s[pos])) { // can not be a '.'
    while ((pos<N(s)) && is_numeric (s[pos])) pos++;
    while (s[pos-1]=='.') pos--;
    return;
  }

  pos++;
}

array<int>
dutch_rep::get_hyphens (string s) {
  return ::get_hyphens (s, H);
}

void
dutch_rep::hyphenate (string s, int after, string& left, string& right) { 
  array<int> penalty= get_hyphens (s);
  std_hyphenate (s, after, left, right, penalty[after]);
}

#endmodule // code_dutch
