
/******************************************************************************
* MODULE     : verbatim.gen.cc
* DESCRIPTION: the "verbatim" language
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <analyze.gen.h>
#include <language.gen.h>

#module code_verbatim
#import analyze
#import language

struct verbatim_rep: language_rep {
  verbatim_rep ();
  void advance (string s, int& pos, space fn_spc, space fn_extra,
		space& spc1, space& spc2, int& pen1, int& pen2);
  array<int> get_hyphens (string s);
  void hyphenate (string s, int after, string& left, string& right);
};

verbatim_rep::verbatim_rep ():
  language_rep ("verbatim") {}

void
verbatim_rep::advance (string s, int& pos, space fn_spc, space fn_extra,
		       space& spc1, space& spc2, int& pen1, int& pen2)
{ (void) fn_extra;

  spc1= space (0);
  spc2= space (0);
  pen1= HYPH_INVALID;
  pen2= HYPH_INVALID;
  if (pos==N(s)) return;
  
  if (s[pos]==' ') {
    pos++;
    spc2= fn_spc;
    pen2= 0;
  }
  else while ((pos<N(s)) && (s[pos]!=' ')) pos++;
}

array<int>
verbatim_rep::get_hyphens (string s) {
  int i;
  array<int> penalty (N(s)+1);
  for (i=0; i<N(penalty); i++) penalty[i]= HYPH_INVALID;
  return penalty;
}

void
verbatim_rep::hyphenate (string s, int after, string& left, string& right) { 
  left = s(0, after);
  right= s(after, N(s));
}

#endmodule // code_verbatim
