
/******************************************************************************
* MODULE     : pk_char.gen.cc
* DESCRIPTION: manipulation of the bitmaps associated to characters
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <basic.gen.h>
#include <tex.gen.h>

#module code_pk_char
#import tex
#import code_resource (pk_char_set_rep,pk_char_set)

#define PIXEL 256

/*****************************************************************************/
// Constructors and destructors for pk_char
/*****************************************************************************/

pk_char_rep::pk_char_rep (int w2, int h2, int xoff2, int yoff2,
			  int depth2, int status2)
{
  width   = w2;
  height  = h2;
  xoff    = xoff2;
  yoff    = yoff2;
  depth   = depth2;
  dpy_xoff= xoff*PIXEL;
  dpy_yoff= yoff*PIXEL;
  status  = status2;

  int i, n= (depth==1? (width*height+7)/8: width*height);
  raster= new QN [n];
  for (i=0; i<n; i++) raster[i]=0;
}

pk_char_rep::~pk_char_rep () {
  delete[] raster;
}

pk_char::pk_char (int w2, int h2, int xoff2, int yoff2,
		  int depth2, int status2) {
  rep= new pk_char_rep (w2, h2, xoff2, yoff2, depth2, status2);
}

/*****************************************************************************/
// Getting and setting pixels
/*****************************************************************************/

inline int
pk_char_rep::Get_1 (int i, int j) {
  int bit= j*width+i;
  return (raster[bit>>3] >> (bit&7)) & 1;
}

inline void
pk_char_rep::Set_1 (int i, int j, int with) {
  int bit= j*width+i;
  if (with==0) raster[bit>>3] &= ~(1 << (bit&7));
  else raster[bit>>3] |= (1 << (bit&7));
}

int
pk_char_rep::Get (int i, int j) {
  if ((i<0) || (i>=width))  return 0;
  if ((j<0) || (j>=height)) return 0;
  if (depth==1) {
    int bit= j*width+i;
    return (raster[bit>>3] >> (bit&7)) & 1;
  }
  else return raster[j*width+i];
}

void
pk_char_rep::Set (int i, int j, int with) {
  if ((i<0) || (i>=width))
    fatal_error ("bad x-index", "pk_char_rep::Set");
  if ((j<0) || (j>=height))
    fatal_error ("bad y-index", "pk_char_rep::Set");
  if (depth==1) {
    int bit= j*width+i;
    if (with==0) raster[bit>>3] &= ~(1 << (bit&7));
    else raster[bit>>3] |= (1 << (bit&7));
  }
  else raster [j*width+ i]= with;
}

int
pk_char_rep::get (int i, int j) {
  return Get (i+xoff, yoff-j);
}

void
pk_char_rep::set (int i, int j, int with) {
  Set (i+xoff, yoff-j, with);
}

/******************************************************************************
* Adjusting top and bottom lines for extensible characters
******************************************************************************/

void
pk_char_rep::adjust_bot () {
  int i;
  if (height<=2) return;
  for (i=0; i<width; i++) Set (i, height-1, Get (i, height-2));
}

void
pk_char_rep::adjust_top () {
  int i;
  if (height<=2) return;
  for (i=0; i<width; i++) Set (i, 0, Get (i, 1));
}

/*****************************************************************************/
// Text output
/*****************************************************************************/

ostream&
operator << (ostream& out, pk_char pkc) {
  int i, j;
  out << "Size  : (" << pkc->width << ", " << pkc->height << ")\n";
  out << "Offset: (" << pkc->xoff << ", " << pkc->yoff << ")\n";
  for (i=0; i<pkc->width+2; i++) out << "-";
  out << "\n";
  for (j=0; j<pkc->height; j++) {
    out << "|";
    for (i=0; i<pkc->width; i++) {
      int k= pkc->Get (i, j);
      if (k==0) out << " ";
      else if (pkc->depth==1) out << "*";
      else if (k <= 9) out << k;
      else out << ((char) (55+k));
    }
    out << "|\n";
  }
  for (i=0; i<pkc->width+2; i++) out << "-";
  out << "\n";
  return out;
}

/*****************************************************************************/
// Thickening and shrinking
/*****************************************************************************/

static int
log2 (int i) {
  if (i==1) return 0;
  if (i<=2) return 1;
  if (i<=4) return 2;
  if (i<=8) return 3;
  if (i<=16) return 4;
  if (i<=32) return 5;
  if (i<=64) return 6;
  if (i<=128) return 7;
  fatal_error ("too large shrinking factor", "shrink", "pk_char.gen.cc");
}

static int
my_div (int a, int b) {
  if (a>=0) return a/b;
  else return -((b-1-a)/b);
}

static int
my_mod (int a, int b) {
  if (a>=0) return a%b;
  else return b-1-((-1-a)%b);
}

void
hor_shifts (pk_char pkc, int xfactor, int* dx, int tx) {
  int x, first_off=-1, last_on=-1;
  for (x=0; x<pkc->width; x++) {
    int count=0, y;
    for (y=0; y<pkc->height; y++)
      if (pkc->Get_1 (x,y)) count++;
    dx[x]= (count>(pkc->height>>1));
    if ((!dx[x]) && (first_off==-1)) first_off= x;
    if (dx[x] && ((x==0) || (!dx[x-1]))) last_on= x;
  }
  for (x=first_off; x<last_on; x++) dx[x]= FALSE;

  int sh, prev_x, prev_sh, i, x2;
  prev_x = 0;
  prev_sh= sh= -(tx>>1);
  for (x=0; x<pkc->width; x++)
    if (dx[x]) {
      for (x2=x; x2<pkc->width; x2++)
	if (!dx[x2]) break;
      sh= (xfactor- 1+ tx)- my_mod (x2- pkc->xoff- 1, xfactor);
      if (prev_x==0) for (i=prev_x; i<x; i++) dx[i]=sh;
      else for (i=prev_x; i<x; i++)
	dx[i]= prev_sh+ ((((sh-prev_sh)<<1)+1)*(i-prev_x)) / ((x-prev_x)<<1);
      for (i=x; i<x2; i++) dx[i]=sh;
      prev_x = x2;
      prev_sh= sh;
      x      = x2-1;
    }
  for (i=prev_x; i<x; i++) dx[i]=sh;
}

void
ver_shifts (pk_char pkc, int yfactor, int* dy, int ty) {
  int y;
  for (y=0; y<pkc->height; y++) {
    int count=0, x;
    for (x=0; x<pkc->width; x++)
      if (pkc->Get_1 (x,y)) count++;
    dy[y]= (count>(pkc->width>>1));
  }

  int sh, prev_y, prev_sh, j, y2;
  prev_y = pkc->height-1;
  prev_sh= sh= -(ty>>1);
  for (y=pkc->height-1; y>=0; y--)
    if (dy[y]) {
      for (y2=y; y2>=0; y2--) if (!dy[y2]) break;
      sh= (yfactor- 1+ ty)- my_mod (pkc->yoff- y2- 1, yfactor);
      if (prev_y==(pkc->height-1)) for (j=prev_y; j>y; j--) dy[j]=sh;
      else for (j=prev_y; j>y; j--)
	dy[j]= prev_sh+ ((((sh-prev_sh)<<1)+1)*(prev_y-j)) / ((prev_y-y)<<1);
      for (j=y; j>y2; j--) dy[j]=sh;
      prev_y = y2;
      prev_sh= sh;
      y      = y2+1;
    }
  for (j=prev_y; j>y; j--) dy[j]=sh;
}

pk_char
shrink (pk_char pkc, int xfactor, int yfactor,
	int* dx, int* dy, int tx, int ty) {
  int gx1= dx[0];                 // left x glue
  int gx2= dx[pkc->width-1]+ tx;  // right x glue
  int gy1= dy[pkc->height-1];     // under y glue
  int gy2= dy[0]+ ty;             // upper y glue

  int x1= gx1- pkc->xoff;
  int x2= gx2- pkc->xoff+ pkc->width;
  int X1= my_div (x1, xfactor);
  int X2= my_div (x2+xfactor-1, xfactor);

  int y1= gy1+ pkc->yoff+ 1- pkc->height;
  int y2= gy2+ pkc->yoff+ 1;
  int Y1= my_div (y1, yfactor);
  int Y2= my_div (y2+yfactor-1, yfactor);

  int i, j, x, y;
  SI mean_dx=0, mean_dy=0;
  for (x=0; x<pkc->width; x++)  mean_dx += dx[x]*PIXEL;
  for (y=0; y<pkc->height; y++) mean_dy += dy[y]*PIXEL;
  mean_dx= (mean_dx/pkc->width) + (tx*(PIXEL/2));
  mean_dy= (mean_dy/pkc->height)+ (ty*(PIXEL/2));
  SI off_x= (mean_dx- X1*xfactor*PIXEL)/xfactor;
  SI off_y= (mean_dy+ (Y2-1)*yfactor*PIXEL)/yfactor;

  int pre_index, index;
  int ww=(X2-X1)*xfactor, hh=(Y2-Y1)*yfactor;
  int bitmap[ww*hh];
  for (i=0; i<ww*hh; i++) bitmap[i]=0;
  for (y=0; y<pkc->height; y++)
    for (x=0; x<pkc->width; x++)
      if (pkc->Get_1(x,y)) {
	pre_index=
	  (dy[y]+ pkc->yoff- y- Y1*yfactor)*ww+
	  (dx[x]+ x- pkc->xoff- X1*xfactor);
	for (j=0, index=pre_index; j<ty+1; j++, index+=ww)
	  for (i=0; i<tx+1; i++)
	    bitmap[index+i]= 1;
      }

  int X, Y, sum;
  pk_char CB (X2-X1, Y2-Y1, -X1, Y2-1,
	      pkc->depth+ log2 (xfactor*yfactor), pkc->status);
  for (Y=Y1; Y<Y2; Y++)
    for (X=X1; X<X2; X++) {
      sum=0;
      pre_index= ((Y-Y1)*ww+ (X-X1))*xfactor;
      for (j=0, index=pre_index; j<yfactor; j++, index+=ww)
	for (i=0; i<xfactor; i++)
	  sum += bitmap[index+ i];
      CB->set (X, Y, sum);
    }
  CB->dpy_xoff= off_x;
  CB->dpy_yoff= off_y;

  return CB;
}

pk_char
shrink (pk_char pkc, int xfactor, int yfactor) {
  int i;
  int tx= xfactor>>1;
  int ty= yfactor>>1;
  int dx[pkc->width];
  int dy[pkc->height];
  if ((pkc->status==0) && (xfactor>1)) hor_shifts (pkc, xfactor, dx, tx);
  else for (i=0; i<pkc->width; i++)  dx[i]= -(tx>>1);  
  if ((pkc->status==0) && (yfactor<1)) ver_shifts (pkc, yfactor, dy, ty);
  else for (i=0; i<pkc->height; i++) dy[i]= -(ty>>1);
  if ((pkc->width==0) || (pkc->height==0))
    fatal_error ("zero size character", "shrink", "pk_char.gen.cc");
  pk_char ret= shrink (pkc, xfactor, yfactor, dx, dy, tx, ty);
  if (ret->status != 0) {
    if (ret->status&1) ret->adjust_top ();
    if (ret->status&2) ret->adjust_bot ();
    ret->yoff= ret->dpy_yoff= 0;
  }
  return ret;
}

/******************************************************************************
* pk_char_sets
******************************************************************************/

pk_char_set_rep::pk_char_set_rep (string name, pk_char* pkf2):
  rep<pk_char_set> (name), pkf (pkf2) {}

pk_char_set_rep::~pk_char_set_rep () {
  fatal_error ("not yet implemented",
	       "pk_char_set_rep::~pk_char_set_rep",
	       "pk_char_set.gen.cc");
}

#endmodule // code_pk_char
