
/******************************************************************************
* MODULE     : language.gen.h
* DESCRIPTION: language specific features, which include
*              - ponctuation rules
*              - hyphenation
*              - (future) dictionary and grammar rumes for spell-checker
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <space.gen.h>
#include <array.gen.h>
#include <encoding.gen.h>

#module language
#import space
#import array (int)
#import encoding
#import resource (language_rep,language)

#define HYPH_STD     10000
#define HYPH_PANIC   1000000
#define HYPH_INVALID 100000000

/******************************************************************************
* The language structure
******************************************************************************/

struct language_rep: rep<language> {
  string   lan_name;  // name of the language
  encoding enc;       // the underlying encoding of the language

  inline language_rep (string s, encoding enc= always_enc);
  virtual void advance (string s, int& pos, space fn_spc, space fn_extra,
			space& spc1, space& spc2, int& pen1, int& pen2) = 0;
  virtual array<int> get_hyphens (string s) = 0;
  virtual void hyphenate (string s, int after, string& l, string& r) = 0;
};

inline language_rep::language_rep (string s, encoding enc2):
  rep<language> (s), enc(enc2) {}

language text_language (string s);
language math_language (string s);
language prog_language (string s);

#endmodule // language
