
/******************************************************************************
* MODULE     : tex.gen.h
* DESCRIPTION: tex fonts: .tfm and .pk files
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <resource.gen.h>

#module tex
#import resource (tex_font_metric_rep,tex_font_metric)
#import resource (pk_char_set_rep,pk_char_set)

typedef int SI;
typedef unsigned short HN;
typedef unsigned char QN;
typedef int wm_object;
typedef wm_object bitmap;
typedef wm_object pixmap;

/******************************************************************************
* The tex_font_metric data type
******************************************************************************/

struct tex_font_metric_rep: rep<tex_font_metric> {
  HN      lf, lh, bc, ec;
  HN      nw, nh, nd, ni;
  HN      nl, nk, ne, np;

  SI*     header;
  SI*     char_info;
  SI*     width;
  SI*     height;
  SI*     depth;
  SI*     italic;
  SI*     lig_kern;
  SI*     kern;
  SI*     exten;
  SI*     param;

  SI      left     , right;
  SI      left_prog, right_prog;

  /***************************************************************************/

  tex_font_metric_rep (string name);
  ~tex_font_metric_rep ();

  SI      w (QN c);
  SI      h (QN c);
  SI      d (QN c);
  SI      i (QN c);
  SI      tag (QN c);
  SI      rem (QN c);
  int     list_len (QN c);
  QN      nth_in_list (QN c, int n);
  QN      top (QN c);
  QN      mid (QN c);
  QN      bot (QN c);
  QN      rep (QN c);

  SI      design_size ();
  double  slope ();
  SI      spc ();
  SI      spc_stretch ();
  SI      spc_shrink ();
  SI      x_height ();
  SI      spc_quad ();
  SI      spc_extra ();
  SI      parameter (int i);

  void    execute (SI* s, int n, SI* buf, SI* ker, int& m);
};

tex_font_metric load_tfm (string family, int size);

/******************************************************************************
* The pk_char structure and the pk_char_set resource
******************************************************************************/

struct pk_char_rep: concrete_struct {
  int width, height;       // width and height in pixels
  int xoff, yoff;          // offset of origin
  int depth;               // number of bits per pixel >= depth
  SI  dpy_xoff, dpy_yoff;  // fractional offset for shrinked characters
  int status;              // status for extensible characters
  QN* raster;              // character definition

  pk_char_rep (int w, int h, int xoff, int yoff, int depth, int status=0);
  ~pk_char_rep ();
  inline int  Get_1 (int i, int j);
  inline void Set_1 (int i, int j, int with);
  int  Get (int i, int j);
  void Set (int i, int j, int with);
  int  get (int i, int j);
  void set (int i, int j, int with);
  void adjust_bot ();
  void adjust_top ();
};

struct pk_char {
#import concrete_null (pk_char, pk_char_rep)
  pk_char (int w2, int h2, int xoff2, int yoff2, int depth2=1, int status2= 0);
};
#import code_concrete_null (pk_char, pk_char_rep)

pk_char shrink (pk_char pkc, int xfactor, int yfactor);

struct pk_char_set_rep:rep<pk_char_set> {
  pk_char*   pkf; // definitions of the characters

  pk_char_set_rep (string name, pk_char* pkf);
  ~pk_char_set_rep ();
};

/******************************************************************************
* Loading a TeX font
******************************************************************************/

void load_tex (string family, int size, int dpi, int dsize,
	       tex_font_metric& tfm, pk_char_set& pk);

#endmodule // tex
