
/******************************************************************************
* MODULE     : tm_config.gen.cc
* DESCRIPTION: Configuration routines for TeXmacs server
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <tm_config.gen.h>

#module code_tm_config
#import tm_config

/******************************************************************************
* Constructor and destructor
******************************************************************************/

tm_config_rep::tm_config_rep ():
  in_lan ("english"),
  out_lan ("english"),
  latex_cmds (program ("undefined latex command")),
  latex_help (""),
  keymap (program ("undefined keyboard action")),
  shorthands (""),
  help_message ("") {}

tm_config_rep::~tm_config_rep () {}

/******************************************************************************
* Set input and output language
******************************************************************************/

void
tm_config_rep::set_input_language (string s) {
  in_lan= s;
}

void
tm_config_rep::set_output_language (string s) {
  out_lan= s;
  get_display()->set_output_language (s);
}

string
tm_config_rep::get_input_language () {
  return in_lan;
}

string
tm_config_rep::get_output_language () {
  return out_lan;
}

/******************************************************************************
* Setup TeXmacs fonts
******************************************************************************/

void
tm_config_rep::set_font_rules (program rules) {
  int i;
  for (i=0; i<N(rules); i++)
    if (N(rules[i])==2) {
      tree l= (tree) rules[i][0];
      tree r= (tree) rules[i][1];
      font_rule (l, r);
    }  
}

/******************************************************************************
* Latex (or user) commands
******************************************************************************/

void
tm_config_rep::set_latex_commands (program cmds) {
  int i;
  for (i=0; i<N(cmds); i++)
    if ((N(cmds[i])==3) && is_string (cmds[i][0]) && is_string (cmds[i][1])) {
      string  l     = as_string (cmds[i][0]->s);
      latex_help (l)= as_string (cmds[i][1]->s);
      latex_cmds (l)= cmds[i][2];
    }
}

void
tm_config_rep::set_latex_symbols (program syms) {
  int i;
  for (i=0; i<N(syms); i++)
    if (is_string (syms[i])) {
      string  l= as_string (syms[i]->s);
      if ((N(l)>=2) && (l[0]=='\42') && (l[N(l)-1]=='\42'))
	l= l (1, N(l)-1);
      latex_help (l)= "insert#<" * l * ">";
      latex_cmds (l)= convert ("(insert_tree \"<" * l * ">\"");
    }
}

bool
tm_config_rep::get_latex_command (string which, string& help, program& p) {
  if (!latex_help->contains (which)) return FALSE;
  help= latex_help [which];
  p   = latex_cmds [which];
  return TRUE;
}

/******************************************************************************
* Server keyboard mappings and shorthands
******************************************************************************/

static string
un_angular (string s, string less, string gtr) {
  int i;
  string r ("");
  for (i=0; i<N(s); i++)
    switch (s[i]) {
    case '<': r << less; break;
    case '>': r << gtr; break;
    default: r << s[i];
    }
  return r;
}

static void
msh (hashmap<string,string> H, string ml,
     string sh, string s, string less, string gtr) {
  int i=0, j;
  sh= un_angular (sh, less, gtr);
  for (j=i; i<N(sh); ) {
    if (sh[i]=='<')
      while ((i<N(sh)) && (sh[i]!='>')) i++;
    i++;
    if (i>=N(sh)) break;
    string r= sh (0, i);
    if (!H->contains (ml * "|" * r))
      H (ml * "|" * r)= H [ml * "|" * sh (0, j)]* sh (j, i);
  }
  H (ml * "|" * sh)= s;
}

void
tm_config_rep::set_keymap (program mls, program cmds) {
  int i, j;
  for (j=0; j<N(mls); j++) {
    if (!is_string (mls[j])) continue;
    string ml= mls[j]->s;
    for (i=0; i<N(cmds); i++)
      if ((N(cmds[i])==2) && is_string (cmds[i][0])) {
	string  l= cmds[i][0]->s; l= l (1, N(l) -1);
	program r= cmds[i][1];
	msh (shorthands, ml, copy (l), copy (l), "<", ">");
	keymap (ml * "|" * l)= r;
      }
  }
}

void
tm_config_rep::set_shorthands (program mls,
			       string less, string gtr, program combs)
{
  int i, j;
  for (j=0; j<N(mls); j++) {
    if (!is_string (mls[j])) continue;
    string ml= mls[j]->s;
    for (i=0; i<N(combs); i++)
      if ((N(combs[i])>=2) &&
	  is_string (combs[i][0]) &&
	  is_string (combs[i][1])) {
	string l= combs[i][0]->s; l= l (1, N(l)-1);
	string r= combs[i][1]->s; r= r (1, N(r)-1);
	msh (shorthands, ml, l, r, less, gtr);
	if ((N(combs[i])==3) && is_string (combs[i][2])) {
	  string h= combs[i][2]->s; h= h (1, N(h)-1);
	  help_message (ml * "|" * l)= h;
	}
      }
  }
}

void
tm_config_rep::get_keycomb (
  string mode, string lan, string which,
  int& status, program& keym, string& shorth, string& help)
{
  string s= lan  * "|" * which;
  string r= mode * "|" * which;
  status= 0;
  if (keymap->contains (s)) {
    status |= 1;
    keym= keymap [s];
  }
  else {
    if (keymap->contains (r)) status |= 1;
    keym= keymap [r];
  }
  if (shorthands->contains (s)) {
    status |= 2;
    shorth= shorthands [s];
    help  = help_message [s];
  }
  else {
    if (shorthands->contains (r)) status |= 2;
    shorth= shorthands [r];
    help  = help_message [r];
  }
}

#endmodule // code_tm_config
