
/******************************************************************************
* MODULE     : tm_data.gen.cc
* DESCRIPTION: Buffer management for TeXmacs server
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <tm_data.gen.h>
#include <file.gen.h>
#include <array.gen.cc>

#module code_tm_data
#import tm_data
#import file
#define no_tree_converter<pointer>
#import code_array (pointer)

static string file_format (string file_name);

/******************************************************************************
* Constructor and destructor
******************************************************************************/

tm_data_rep::tm_data_rep () {}
tm_data_rep::~tm_data_rep () {}

/******************************************************************************
* Low level functions for maintaining the buffer menu
******************************************************************************/

int
tm_data_rep::find_buffer (string name) {
  int i;
  for (i=0; i<N(bufs); i++)
    if (((tm_buffer) bufs[i])->name == name)
      return i;
  return -1;
}

string
tm_data_rep::new_menu_name (string name) {
  int i, j;
  for (i=N(name)-1; i>=0; i--)
    if (name[i]=='/') break;
  name= name (i+1, N(name));

  for (j=1; TRUE; j++) {
    bool flag= TRUE;
    string ret (name);
    if (j>1) ret= name * " (" * as_string (j) * ")";
    for (i=0; i<N(bufs); i++)
      if (((tm_buffer) bufs[i])->abbr==ret) flag= FALSE;
    if (flag) return ret;
  }
}

void
tm_data_rep::update_menu () {
  int i;
  string s ("(menu_set buffer_menu '(");
  for (i=0; i<N(bufs); i++) {
    if (i>0) s << " ";
    s << "(\"" << ((tm_buffer) bufs[i])->abbr << "\" ";
    s << "(switch_to_buffer \"" * ((tm_buffer) bufs[i])->name * "\"))";
  }
  s << "))";
  execute (s);
}

void
tm_data_rep::menu_insert_buffer (tm_buffer buf) {
  bufs << ((pointer) buf);
  update_menu ();
}

void
tm_data_rep::menu_delete_buffer (tm_buffer buf) {
  int i, nr, n=N(bufs);
  for (nr=0; nr<n; nr++)
    if (bufs[nr] == ((pointer) buf)) break;
  if (nr==N(bufs)) return;

  for (i=nr; i<(n-1); i++) bufs[i]= bufs[i+1];
  bufs.resize (n-1);
  update_menu();
}

void
tm_data_rep::menu_focus_buffer (tm_buffer buf) {
  int i, nr;
  for (nr=0; nr<N(bufs); nr++)
    if (bufs[nr] == ((pointer) buf)) break;
  if (nr==N(bufs)) return;

  for (i=nr; i>=1; i--) bufs[i]= bufs[i-1];
  bufs[0]= buf;
  update_menu ();
}

/******************************************************************************
* Low level buffer routines
******************************************************************************/

tm_buffer
tm_data_rep::new_buffer (string name) {
  int nr= find_buffer (name);
  if (nr != -1) return (tm_buffer) bufs[nr];
  tm_buffer buf= new tm_buffer_rep (name);
  buf->abbr= new_menu_name (name);
  menu_insert_buffer (buf);
  return buf;
}

static tree
reformat (tree doc) {
  tree t      = (N(doc)>=1? doc[0]: tree (DOCUMENT, ""));
  tree project= (N(doc)>=2? doc[1]: tree (""));
  tree style  = (N(doc)>=3? doc[2]: tree ("none"));
  tree init   = (N(doc)>=4? doc[3]: tree ("{,}"));
  tree fin    = (N(doc)>=5? doc[4]: tree ("{,}"));
  tree ref    = (N(doc)>=6? doc[5]: tree ("{,}"));
  tree aux    = (N(doc)>=7? doc[6]: tree ("{,}"));
  if ((style == "none") || (style == "")) style= "style";
  else if (style->label != "style") style= tree ("style", style);
  tree ret ("", t);
  ret << project << style << init << fin << ref << aux;
  return ret;
}

tm_buffer
tm_data_rep::new_buffer (string name, tree doc) {
  int nr= find_buffer (name);
  if (nr != -1) return (tm_buffer) bufs[nr];
  doc= reformat (doc);
  tm_buffer buf= new_buffer (name);
  buf->t      = doc[0];
  buf->project= doc[1];
  buf->style  = doc[2];
  buf->init   = hashmap<string,tree> ("?", doc[3]);
  buf->fin    = hashmap<string,tree> ("?", doc[4]);
  buf->ref    = hashmap<string,tree> ("?", doc[5]);
  buf->aux    = hashmap<string,tree> ("?", doc[6]);
  if (buf->project->label != "") {
    string prj_name= get_relative_file_name (name, buf->project->label);
    buf->prj= load_passive_buffer (prj_name);
  }
  return buf;
}

void
tm_data_rep::revert_buffer (string name, tree doc) {
  int i, nr= find_buffer (name);
  if (nr == -1) return;
  tm_buffer buf= (tm_buffer) bufs[nr];
  doc= reformat (doc);
  buf->project= doc[1];
  buf->style  = doc[2];
  buf->init   = hashmap<string,tree> ("?", doc[3]);
  buf->fin    = hashmap<string,tree> ("?", doc[4]);
  buf->ref    = hashmap<string,tree> ("?", doc[5]);
  buf->aux    = hashmap<string,tree> ("?", doc[6]);
  if (N(buf->vws)==0) buf->t= doc[0];
  else for (i=0; i<N(buf->vws); i++) {
    tm_view vw= (tm_view) buf->vws[i];
    if (i==0) vw->ed->assign (buf->t, path(), doc[0]);
    vw->ed->the_style= copy (buf->style);
    vw->ed->init_env (buf->init);
    vw->ed->fin  = buf->fin;
    vw->ed->notify_change (THE_FOOTER);
  }
  buf->modification_flag= FALSE;
  buf->not_saved_flag= FALSE;
}

void
tm_data_rep::delete_buffer (tm_buffer buf) {
  int i;
  menu_delete_buffer (buf);
  for (i=0; i<N(buf->vws); i++)
    delete_view ((tm_view) buf->vws[i]);
  delete buf;
}

void
tm_data_rep::set_name_buffer (string s) {
  int i;
  tm_buffer buf= get_buffer ();
  if (buf->name == s) return;
  buf->name= s;
  buf->abbr= new_menu_name (s);
  update_menu ();
  for (i=0; i<N(buf->vws); i++) {
    tm_view vw2= (tm_view) buf->vws[i];
    if (vw2->win != NULL)
      vw2->win->win->set_name (buf->abbr);
  }
}

string
tm_data_rep::get_name_buffer () {
  tm_buffer buf= get_buffer ();
  return buf->name;
}

/******************************************************************************
* Low level view routines
******************************************************************************/

tm_view
tm_data_rep::new_view (string name) {
  // cout << "Creating new view\n";

  tm_buffer buf= new_buffer (name);
  editor    ed = new edit_main (get_server (), get_display (), buf);
  tm_view   vw = new tm_view_rep (buf, ed);
  buf->vws << vw;

  ed->the_style= copy (buf->style);
  ed->init_env (buf->init);
  ed->fin  = buf->fin;
  ed->notify_change (THE_FOOTER);

  tm_view temp_vw= get_view (FALSE);
  set_view (vw);
  if (file_exists ("$TEXMACS_HOME_PATH/progs", "Init-buffer.tl"))
    lisp_eval_file ("$TEXMACS_HOME_PATH/progs", "Init-buffer.tl");
  else lisp_eval_file ("$TEXMACS_PATH/progs", "Init-buffer.tl");
  set_view (temp_vw);

  // cout << "View created\n";
  return vw;
}

tm_view
tm_data_rep::get_passive_view (tm_buffer buf) {
  int i;
  for (i=0; i<N(buf->vws); i++)
    if (((tm_view) buf->vws[i])->win == NULL)
      return (tm_view) buf->vws[i];
  return new_view (buf->name);
}

void
tm_data_rep::delete_view (tm_view vw) {
  tm_buffer buf= vw->buf;
  int i, j, n= N(buf->vws);
  for (i=0; i<n; i++)
    if (buf->vws[i] == vw) {
      array<tm_view> a (n-1);
      for (j=0; j<n-1; j++)
	if (j<i) a[j]= buf->vws[j];
	else a[j]= buf->vws[j+1];
      buf->vws= a;
    }
  delete vw;
}

void
tm_data_rep::attach_view (tm_window win, tm_view vw) {
  // cout << "Attach view " << vw->buf->name << "\n";
  vw->win= win;
  win->wid->a[0]->a[2]->a[0]->a[0]= vw->ed;
  if (win->wid->attached ()) {
    vw->ed->resume ();
    win->wid->set_window_name (vw->buf->abbr);
    win->wid->a[0] << emit_update ();
  }
  // cout << "View attached\n";
}

void
tm_data_rep::detach_view (tm_view vw) {
  // cout << "Detach view " << vw->buf->name << "\n";
  tm_window win= vw->win;
  if (win == NULL) return;
  vw->win= NULL;
  win->wid->a[0]->a[2]->a[0]->a[0]= glue_widget (get_display ());
  if (win->wid->attached ()) {
    vw->ed->suspend ();
    vw->ed << emit_attach_window (NULL);
    win->wid->set_window_name ("TeXmacs");
    win->wid->a[0] << emit_update ();
  }
  // cout << "View detached\n";
}

/******************************************************************************
* Low level window routines
******************************************************************************/

tm_window
tm_data_rep::new_window (display dis, bool map_flag) {
  tm_window win= new tm_window_rep (new tm_widget_rep (this, dis));
  if (map_flag) win->win->map ();
  return win;
}

bool
tm_data_rep::delete_view_from_window (tm_window win) {
  int i, j;
  for (i=0; i<N(bufs); i++) {
    tm_buffer buf= (tm_buffer) bufs[i];
    for (j=0; j<N(buf->vws); j++) {
      tm_view vw= (tm_view) buf->vws[j];
      if (vw->win == win) {
	detach_view (vw);
	delete_view (vw);
	return TRUE;
      }
    }
  }
  return FALSE;
}

void
tm_data_rep::delete_window (tm_window win) {
  while (delete_view_from_window (win));
  win->win->unmap ();
  delete win->win;
  delete win;
}

/******************************************************************************
* Other subroutines
******************************************************************************/

void
tm_data_rep::new_buffer_in_this_window (string name, tree doc) {
  int nr= find_buffer (name);
  if (nr != -1) switch_to_buffer (nr);
  else {
    (void) new_buffer (name, doc);
    switch_to_buffer (name);
  }
}

void
tm_data_rep::new_buffer_in_new_window (string name, tree doc) {
  tm_window win= new_window (get_display ());
  tm_buffer buf= new_buffer (name, doc);
  tm_view   vw = get_passive_view (buf);
  attach_view (win, vw);
  set_view (vw);
  menu_focus_buffer (buf);
}

/******************************************************************************
* Exported routines
******************************************************************************/

void
tm_data_rep::new_buffer () {
  int i;
  for (i=1; TRUE; i++) {
    string name= "no name " * as_string (i);
    if (i==1) name= "no name";
    if (find_buffer (name) != -1) continue;
    new_buffer_in_this_window (name, "");
    return;
  }
}

void
tm_data_rep::switch_to_buffer (int nr) {
  // cout << "Switching to buffer " << nr << "\n";
  tm_window win    = get_window ();
  tm_buffer buf    = (tm_buffer) (bufs[nr]);
  tm_view   old_vw = get_view ();
  tm_view   new_vw = get_passive_view (buf);
  detach_view (old_vw);
  attach_view (win, new_vw);
  set_view (new_vw);
  menu_focus_buffer (buf);
  project_update_menu ();
  // cout << "Switched to buffer " << nr << "\n";
}

void
tm_data_rep::switch_to_buffer (string name) {
  int nr= find_buffer (name);
  if (nr == -1) {
    load_passive_buffer (name);
    nr= find_buffer (name);
  }
  if (nr != -1) switch_to_buffer (nr);
}

void
tm_data_rep::revert_buffer () {
  tm_buffer buf= get_buffer ();
  tree doc= file_to_tree ("", buf->name, buf->format);
  if (doc == "error") set_message ("Error: file not found", "revert buffer");
  else revert_buffer (buf->name, doc);
}

void
tm_data_rep::kill_buffer () {
  int i, nr;
  if (N(bufs) <= 1) quit();
  tm_buffer buf= get_buffer();
  for (nr=0; nr<N(bufs); nr++) if (buf == bufs[nr]) break;
  if (nr == N(bufs))
    fatal_error ("Invalid situation", "tm_data_rep::kill_buffer");

  for (i=0; i<N(buf->vws); i++) {
    tm_view old_vw= (tm_view) buf->vws[i];
    if (old_vw->win != NULL) {
      tm_window win = old_vw->win;
      tm_view new_vw= get_passive_view ((tm_buffer) bufs[1]);
      detach_view (old_vw);
      attach_view (win, new_vw);
      if (get_view () == old_vw) set_view (new_vw);
    }
  }

  delete_buffer (buf);
}

void
tm_data_rep::open_window () {
  int i;
  for (i=1; TRUE; i++) {
    string name= "no name " * as_string (i);
    if (i==1) name= "no name";
    if (find_buffer (name) != -1) continue;
    new_buffer_in_new_window (name, "");
    return;
  }
}

void
tm_data_rep::clone_window () {
  tm_window win= new_window (get_display ());
  tm_buffer buf= get_buffer ();
  tm_view   vw = get_passive_view (buf);
  attach_view (win, vw);
  set_view (vw);
  menu_focus_buffer (buf);
}

void
tm_data_rep::kill_window () {
  int i, j;
  tm_window win    = get_window ();
  for (i=0; i<N(bufs); i++) {
    tm_buffer buf= (tm_buffer) bufs[i];
    for (j=0; j<N(buf->vws); j++) {
      tm_view vw= (tm_view) buf->vws[j];
      if ((vw->win != NULL) && (vw->win != win)) {
	set_view (vw);
	menu_focus_buffer (vw->buf);
	delete_window (win);
	return;
      }
    }
  }
  quit ();
}

#endmodule // code_tm_data
