
/******************************************************************************
* MODULE     : tm_file.gen.h
* DESCRIPTION: Loading and saving TeXmacs files
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <tm_buffer.gen.h>
#include <file.gen.h>
#include <dir.gen.h>
#include <convert.gen.h>

#module code_tm_file
#import tm_buffer
#import file
#import dir
#import convert

/******************************************************************************
* Subroutines for loading and saving files
******************************************************************************/

static string
file_type (string file_name) {
  int i;
  for (i=N(file_name)-1; i>=0; i--)
    if (file_name[i]=='.') break;
  if ((i>0) && (i<N(file_name)-1)) {
    string r= file_name (i+1, N(file_name));
    while ((N(r)>0) && (r[N(r)-1]=='~')) r= r(0, N(r)-1);
    return r;
  }
  return "";
}

static string
file_format (string file_name) {
  string tp= file_type (file_name);
  if (tp == "tex") return "tex";
  else return "TeXmacs";
}

static bool
has_no_name (string s) {
  return (N(s)>=7) && (s(0,7)=="no name");
}

static tree
un_paragraph (tree t) {
  if (N(t) == 0) return t;
  if (is_func (t, PARAGRAPH, 1)) return t[0];
  else {
    int i;
    tree r (t->label, N(t));
    for (i=0; i<N(t); i++) r[i]= un_paragraph (t[i]);
    return r;
  }
}

/******************************************************************************
* Loading files
******************************************************************************/

tree
tm_data_rep::file_to_tree (string dir, string name, string format) {
  string fnf   = "Error: file#" * name * "#not found";
  string action= "load " * format * " file#" * name;
  if (!file_exists (dir, name)) { set_message (fnf, action); return "error"; }
  string unique_name= get_unique_file_name (dir, name);

  if (format == "TeXmacs") {
    tree t;
    if (load_tree (dir, name, t)) set_message (fnf, action);
    else if ((N(t)<3) ||
	     ((t->label != "edit-0.2") &&
	      ((N(t->label) < 11) || (t->label (0, 11) != "TeXmacs-0.2"))))
      set_message ("Error: bad data format", action);
    else {
      if (N(t)<4) return tree ("", un_paragraph (t[2]), "", t[0], t[1]);
      else if (N(t)<7) {
	tree doc ("", un_paragraph (t[0]));
	doc << "" << t[1] << t[2] << "{,}" << t[3];
	return doc;
      }
      else {
	tree doc ("", un_paragraph (t[0]));
	doc << t[1] << t[2] << t[3] << t[4] << t[5] << t[6];
	return doc;
      }
    }
  }

  else if ((format == "tex") || (format == "latex") || (format == "help")) {
    string s, style;
    if (load_string (dir, name, s)) set_message (fnf, action);
    else {
      tree t= math_correct (latex_document_to_edit (string (s), style));
      if (format == "help") style= string ("help");
      return tree ("", t, "", style);
    }
  }

  else if (format == "verbatim") {
    string s;
    if (load_string (dir, name, s)) set_message (fnf, action);
    else {
      tree t    = verbatim_to_edit (s * "\n");
      tree init = tree ("{,}",
			tree ("->", TEXT_LANGUAGE, "verbatim"),
			tree ("->", TEXT_FAMILY, "tt"),
			tree ("->", PAR_FIRST, "0cm"));
      return tree ("", t, "", "", init);
    }
  }

  else set_message ("Error: unknown format", action);
  return "error";
}

void
tm_data_rep::load_buffer (
  string dir, string name, string format, int where, bool test_newer)
{
  if (!file_exists (dir, name)) {
    set_message ("Error: file#" * name * "#not found",
		 "load " * format * " file");
    return;
  }
  if (test_newer && file_exists (dir, name * "~")) {
    string unique_name= get_unique_file_name (dir, name);
    string backup_name= get_unique_file_name (dir, name * "~");
    if (is_newer (backup_name, unique_name)) {
      string args  =
	"\"" * dir * "\" " *
	"\"" * name * "\" " *
	"\"" * format * "\" " *
	"\"" * as_string (where) * "\"";
      string lambda= "'(lambda (s:String) (recover_auto_save " * args * " s))";
      execute ("(interactive '(\"Load more recent autosave file?\")" * lambda);
      return;
    }
  }

  string unique_name= get_unique_file_name (dir, name);
  if ((!test_newer) && (unique_name [N(unique_name)-1] == '~'))
    unique_name= unique_name (0, N(unique_name)-1);
  int nr= find_buffer (unique_name);
  tree doc= ((nr == -1)? file_to_tree (dir, name, format): tree (""));
  if (is_func (doc, "error", 0)) return;
  switch (where) {
  case 0: new_buffer_in_this_window (unique_name, doc); break;
  case 1: new_buffer_in_new_window (unique_name, doc); break;
  case 2: new_buffer (unique_name, doc); break;
  default: fatal_error ("Bad value for 'where'", "load_buffer");
  }
  nr= find_buffer (unique_name);
  if (nr != -1) {
    tm_buffer buf= (tm_buffer) bufs[nr];
    buf->format= format;
    if (format == "help") {
      tm_buffer buf= get_buffer ();
      buf->read_only= TRUE;
    }
  }
}

void
tm_data_rep::load_buffer (string name, string format, int where) {
  if (format == "help")
    load_buffer ("$TEXMACS_HELP_PATH", name, format, where);
  else load_buffer ("$TEXMACS_FILE_PATH", name, format, where);
}

void
tm_data_rep::load_generic_buffer (string dir, string name, int where) {
  if (has_no_name (name)) return;
  load_buffer (dir, name, file_format (name), where);
}

void
tm_data_rep::load_generic_buffer (string name, int where) {
  load_generic_buffer ("$TEXMACS_FILE_PATH", name, where);
}

tm_buffer
tm_data_rep::load_passive_buffer (string name) {
  int nr= find_buffer (name);
  if (nr != -1) return (tm_buffer) bufs[nr];
  load_buffer (".", name, "TeXmacs", 2, FALSE);
  nr= find_buffer (name);
  if (nr != -1) return (tm_buffer) bufs[nr];
  return NULL;
}

/******************************************************************************
* Saving files
******************************************************************************/

bool
tm_data_rep::save (string name, tm_view vw) {
  tree doc ("TeXmacs-" * TEXMACS_VERSION);
  doc << vw->buf->t;
  doc << vw->buf->project;
  doc << vw->ed->the_style;
  doc << ((tree) vw->ed->init);
  doc << ((tree) vw->ed->fin);
  doc << ((tree) vw->buf->ref);
  doc << ((tree) vw->buf->aux);

  bool flag= save_tree (name, doc);
  if (name == "~/.TeXmacs-save.tm") return flag;
  if (flag) set_message ("Error: file " * name * " did not open",
			 "save TeXmacs file");
  else set_message ("saved " * name, "save TeXmacs file");
  return flag;
}

void
tm_data_rep::save_buffer (string name, string format) {
  string action= "save " * format * " file";

  tm_buffer buf= get_buffer ();

  if (buf->read_only && ((format == "TeXmacs") || (format == "help"))) {
    set_message ("Error: file is read only", action);
    return;
  }

  if ((format == "TeXmacs") &&
      (!buf->modification_flag) && (!buf->not_saved_flag))
    if (buf->name == name) {
      set_message ("No changes need to be saved", action);
      return;
    }

  if (format == "TeXmacs") {
    tm_view vw= get_view ();
    if (!save (name, vw)) {
      set_name_buffer (get_unique_file_name ("", name));
      vw->buf->modification_flag= FALSE;
      vw->buf->not_saved_flag= FALSE;
    }
  }

  else if ((format == "tex") || (format == "latex") || (format == "help")) {
    editor ed   = get_editor ();
    tree   t    = ed->the_style;
    string style= (N(t)==0? t->label: t[0]->label);
    string lan  = ed->init [TEXT_LANGUAGE]->label;
    if (save_string (name, edit_to_latex_document (ed->et, style, lan)))
      set_message ("Error: file did not open", action);
    else set_message ("saved " * name, action);
  }

  else if (format == "verbatim") {  
    if (save_string (name, edit_to_string (get_editor()->et)))
      set_message ("Error: file did not open", action);
    else set_message ("saved " * name, action);
  }

  else set_message ("Error: unknown format", action);
}

void
tm_data_rep::save_buffer (string dir, string name, string format) {
  save_buffer (get_unique_file_name (dir, name), format);
}

void
tm_data_rep::save_generic_buffer (string name) {
  if (has_no_name (name)) return;
  save_buffer (name, file_format (name));
}

void
tm_data_rep::save_generic_buffer (string dir, string name) {
  save_generic_buffer (get_unique_file_name (dir, name));
}

/******************************************************************************
* Other routines concerning loading and saving buffers
******************************************************************************/

bool
tm_data_rep::no_name () {
  tm_buffer buf= get_buffer ();
  return has_no_name (buf->name);
}

bool
tm_data_rep::exists_unsaved_buffer () {
  bool flag= FALSE;
  int i, n= N(bufs);
  for (i=0; i<n; i++) {
    tm_buffer buf= (tm_buffer) bufs[i];
    flag = flag || buf->modification_flag || buf->not_saved_flag;
  }
  return flag;
}

void
tm_data_rep::pretend_save_buffer () {
  tm_buffer buf= get_buffer ();
  buf->modification_flag= FALSE;
  buf->not_saved_flag= FALSE;
}

void
tm_data_rep::auto_save () {
  int i, n= N(bufs);
  for (i=0; i<n; i++) {
    tm_buffer buf= (tm_buffer) bufs[i];
    if (buf->modification_flag && (!buf->read_only)) {
      string name= buf->name * "~";
      if (has_no_name (buf->name))
	name= "$TEXMACS_HOME_PATH/system/autosave.tm";
      if (N(buf->vws)!=0) {
	if (!save (name, (tm_view) buf->vws[0])) {
	  buf->modification_flag = FALSE;
	  buf->not_saved_flag    = TRUE;
	}
      }
    }
  }
  get_display()->delayed_message (get_meta()->get_this(), "auto save", 120000);
}

void
tm_data_rep::recover_auto_save (
  string dir, string name, string format, string where, string answer)
{
  if (is_yes (answer))
    load_buffer (dir, name * "~", format, as_int (where), FALSE);
  else load_buffer (dir, name, format, as_int (where), FALSE);
}

#endmodule // code_tm_buffer
