
/******************************************************************************
* MODULE     : tm_menus.gen.cc
* DESCRIPTION: Dynamical menus
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <tm_layout.gen.h>
#include <Widget/make_widget.gen.h>

#module code_tm_menus
#import tm_layout
#import make_widget

widget make_menu_widget (server sv, display dis, program p);
program menu_merge (program p, program q);

/******************************************************************************
* Constructor and destructor
******************************************************************************/

tm_layout_rep::tm_layout_rep () {}
tm_layout_rep::~tm_layout_rep () {}

/******************************************************************************
* Compilation routines
******************************************************************************/

class server_command_rep: public command_rep {
  server  sv;
  program p;
public:
  server_command_rep (server sv2, program p2): sv (sv2), p(p2) {}
  void apply () {
    sv->execute_delayed (p);
    sv->get_editor()->notify_change (THE_FOOTER);
  }
  ostream& print (ostream& out) { return out << p; }
};

/******************************************************************************
* Abstract function for creating menu widgets
******************************************************************************/

class make_menu_widget_rep: public make_widget_rep {
  server  sv;
  program p;
public:
  make_menu_widget_rep (server sv2, program p2): sv(sv2), p(p2) {}
  ostream& print (ostream& out) { return out << p; }
  widget get_widget (display dis);
};

widget
make_menu_widget_rep::get_widget (display dis) {
  return make_menu_widget (sv, dis, p);
}

make_widget
make_menu_widget (server sv, program p) {
  return new make_menu_widget_rep (sv, p);
}

/******************************************************************************
* Menu creation routines
******************************************************************************/

void
insert_menu (server sv, display dis, array<widget>& a, program p) {
  int i;

  if ((is_composite (p) && (N(p)==0)) ||
      ((N(p)>0) && is_composite (p[0]))) {
    for (i=0; i<N(p); i++)
      insert_menu (sv, dis, a, p[i]);
    return;
  }

  if (is_func (p, "if") && (N(p)>=2)) {
    compiler& cpl (sv->get_compiler());
    program q= cpl->filter (cpl->compile (p[1]), type ("Bool"), p[1]);
    bool flag= FALSE;
    if (is_compiled (q)) {
      void* ret= ::execute (q);
      flag= *((bool*) ret);
      delete ((bool*) ret);
    }
    if (flag)
      for (i=2; i<N(p); i++)
	insert_menu (sv, dis, a, p[i]);
    return;
  }

  if (is_func (p, "=>") && (N(p)>=2) && is_string (p[1])) {
    array<program> b (N(p)-1);
    b[0]= program ("ver_menu", p[0]->start, p[0]->end);
    for (i=0; i<N(p)-2; i++) b[i+1]= p[i+2];
    program q (b, p->start, p->end);
    make_widget mw= make_menu_widget (sv, q);

    string s  = as_string (p[1]->s);
    widget tw = text_widget (dis, s, TRUE, sv->get_input_language());
    widget but= pulldown_button (tw, mw);
    a << but;
    return;
  }

  if (is_func (p, "->") && (N(p)>=2) && is_string (p[1])) {
    array<program> b (N(p)-1);
    b[0]= program ("ver_menu", p[0]->start, p[0]->end);
    for (i=0; i<N(p)-2; i++) b[i+1]= p[i+2];
    program q (b, p->start, p->end);
    make_widget mw= make_menu_widget (sv, q);

    string s  = as_string (p[1]->s);
    widget tw = text_widget (dis, s, TRUE, sv->get_input_language());
    widget but= pullright_button (tw, mw);
    a << but;
    return;
  }

  if (is_func (p, "tile") && (N(p)>=2) && is_string (p[1])) {
    array<widget> b (0);
    for (i=2; i<N(p); i++)
      insert_menu (sv, dis, b, p[i]);
    a << tile (dis, b, as_int (p[1]->s));
    return;
  }

  if (is_composite (p) && (N(p)==2) &&
      is_string (p[0]) && is_string (p[0]->s))
  {
    string s  = as_string (p[0]->s);
    widget tw = text_widget (dis, s, TRUE, sv->get_input_language());
    widget but= command_button (tw, new server_command_rep (sv, p[1]));
    a << but;
    return;
  }

  if (is_composite (p) && (N(p)==3) &&
      is_string (p[0]) && is_string (p[0]->s))
  {
    string ls = as_string (p[0]->s);
    string rs = as_string (p[1]->s);
    widget lw = text_widget (dis, ls, TRUE, sv->get_input_language());
    widget rw = text_widget (dis, rs, TRUE, sv->get_input_language());
    widget but= command_button (lw, rw, new server_command_rep (sv, p[2]));
    a << but;
    return;
  }

  if ((is_func (p, "symbol", 2) ||
       (is_func (p, "symbol", 3) && is_string (p[3]))) &&
      is_composite (p[1]) && is_string (p[2])) {
    string  s  = "(insert_tree \"" * p[2]->s * "\")";
    if (N(p)==4) s= "(emulate_keyboard \"" *p[3]->s* "\" \"" *p[2]->s* "\")";
    program pp = lisp_expr_to_program (s, 0);
    widget  bw = box_widget (dis, p[1], p[2]->s, TRUE);
    widget  but= command_button (bw, new server_command_rep (sv, pp));
      a << but;
      return;
  }

  if (is_string (p, "---")) {
    widget sep= separator_widget (dis, 2*PIXEL, 2*PIXEL);
    a << sep;
    return;
  }

  if (is_func (p, "link", 1)) {
    // CHANGE: coercion should be avoided in next version of compiler
    array<program> b (2);
    b[0]= program ("coerce!");
    b[1]= p[1];
    program c (b, p[1]->start, p[1]->end);

    compiler& cpl (sv->get_compiler());
    program q= cpl->filter (cpl->compile (c), type ("Program"), p[1]);
    if (is_compiled (q)) {
      void* ret= ::execute (q);
      program r= *((program*) ret);
      delete ((program*) ret);
      insert_menu (sv, dis, a, r);
    }
    return;
  }

  cerr << "The menu was " << p << "\n";
  fatal_error ("bad format", "insert_menu");
}

widget
make_menu_widget (server sv, display dis, program p) {
  int i;

  if ((is_composite (p) && (N(p)==0)) ||
      ((N(p)>0) && is_composite (p[0]))) {
    array<widget> a (0);
    for (i=0; i<N(p); i++)
      insert_menu (sv, dis, a, p[i]);
    return horizontal_array (dis, a);
  }

  if (is_func (p, "ver_menu")) {
    array<widget> a (0);
    for (i=1; i<N(p); i++)
      insert_menu (sv, dis, a, p[i]);
    return vertical_list (dis, a);
  }

  if (is_func (p, "link", 1)) {
    // CHANGE: coercion should be avoided in next version of compiler
    array<program> a (2);
    a[0]= program ("coerce!");
    a[1]= p[1];
    program c (a, p[1]->start, p[1]->end);

    compiler& cpl (sv->get_compiler());
    program q= cpl->filter (cpl->compile (c), type ("Program"), p[1]);
    if (is_compiled (q)) {
      void* ret= ::execute (q);
      program r= *((program*) ret);
      delete ((program*) ret);
      return make_menu_widget (sv, dis, r);
    }
    else return text_widget (dis, "Error");
  }

  cerr << "The menu was " << p << "\n";
  fatal_error ("bad format", "make_menu_widget");
}

/******************************************************************************
* Operations on menus
******************************************************************************/

void
menu_merge (array<program>& a, program p) {
  if (is_composite (p) && ((N(p)==2) || (N(p)==3)) &&
      is_string (p[0]) && is_string (p[0]->s))
  {
    int i;
    for (i=0; i<N(a); i++)
      if (is_composite (a[i]) &&
	  ((N(a[i])==2) || (N(a[i])==3)) &&
	  (a[i][0]==p[0]))
	{
	  a[i]= p;
	  return;
	}
  }

  if ((is_func (p, "=>") || is_func (p, "->")) &&
      (N(p)>=2) && is_string (p[1]))
  {
    int i;
    for (i=0; i<N(a); i++)
      if (is_func (a[i], p[0]->s) && (N(a[i])>=2) && (a[i][1]==p[1])) {
	int j;
	array<program> b (N(p)-2);
	for (j=0; j<N(p)-2; j++) b[j]= p[j+2];
	a[i]= menu_merge (a[i], program (b, p->start, p->end));
	return;
      }
  }

  a << p;
}

program
menu_merge (program p, program q) {
  int i;
  array<program> a (N(p));
  for (i=0; i<N(p); i++) a[i]= p[i];

  if ((N(q)>0) && (is_composite (q[0])))
    for (i=0; i<N(q); i++) menu_merge (a, q[i]);
  else menu_merge (a, q);

  return  program (a, p->start, p->end);
}

/******************************************************************************
* Interface
******************************************************************************/

void
tm_layout_rep::menu_widget (program p, widget& w) {
  tm_widget meta= get_meta ();
  w= make_menu_widget (server (this), meta->dis, p);
}

void
tm_layout_rep::menu_main (program p) {
  tm_widget meta= get_meta ();
  meta->set_main_menu (make_menu_widget (server (this), meta->dis, p));
}

void
tm_layout_rep::menu_set (program& m, program p) {
  m= p;
}

void
tm_layout_rep::menu_merge (program& m, program p) {
  m= ::menu_merge (m, p);
}

#endmodule // code_tm_menus
