
/******************************************************************************
* MODULE     : tm_widget.gen.cc
* DESCRIPTION: Main current graphical interface for user applications
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <window.gen.h>
#include <widget.gen.h>
#include <tm_widget.gen.h>

#module code_tm_widget
#import window
#import widget
#import tm_widget

#define THIS widget (this)

/******************************************************************************
* Meta editor constructor and destructor
******************************************************************************/

tm_widget_rep::tm_widget_rep (server_rep* sv2, display dis):
  basic_widget_rep (dis, 1), sv (sv2),
  cur_menu (horizontal_menu (dis)),
  cmd_mode (FALSE), text_ptr (NULL)
{
  array<widget> M (2);
  array<string> M_name (2);
  M[0]= cur_menu;
  M[1]= glue_widget (dis, TRUE, FALSE);
  M_name[0]= "menu";
  widget menu_bar= horizontal_array (dis, M, M_name);

  array<widget> F (3);
  array<string> F_name (3);
  F[0]= text_widget (dis, "Welcome to TeXmacs!", FALSE, "english");
  F[1]= glue_widget (dis, TRUE, FALSE);
  F[2]= text_widget (dis, "beta release " * TEXMACS_VERSION, FALSE, "english");
  F_name[0]= "left";
  F_name[1]= "middle";
  F_name[2]= "right";
  widget footer= horizontal_array (dis, F, F_name, 1);
  left  = text_widget (dis, "");
  middle= text_widget (dis, "");
  right = text_widget (dis, "");

  array<widget> V (4);
  array<string> V_name (4);
  V[0]= menu_bar;
  V[1]= glue_widget (dis, TRUE, FALSE, 0, 2*PIXEL);
  V[2]= canvas_widget (glue_widget (dis));
  V[3]= footer;
  V_name[0]= "header";
  V_name[2]= "canvas";
  V_name[3]= "footer";
  a[0]= vertical_list (dis, V, V_name);
}

tm_widget_rep::~tm_widget_rep () {}

widget_rep*
tm_widget_rep::get_this () {
  return this;
}

tm_widget_rep::operator tree () {
  return tree ("TeXmacs window", (tree) a[0]);
}

void
tm_widget_rep::set_window_name (string s) {
  win->set_name (s);
}

void
tm_widget_rep::set_main_menu (widget w) {
  THIS ["header"] << set_widget ("menu", w);
  if (attached ()) THIS ["header"] << emit_update ();
}

/******************************************************************************
* The footer and executing commands on the bottom line
******************************************************************************/

void
tm_widget_rep::set_left_footer (string s) {
  if (!cmd_mode) {
    widget tw= text_widget (dis, s, FALSE, "english");
    THIS ["footer"] << set_widget ("left", tw);
    THIS ["footer"] << emit_update ();
  }
}

void
tm_widget_rep::set_right_footer (string s) {
  if (!cmd_mode) {
    widget tw= text_widget (dis, s, FALSE, "english");
    THIS ["footer"] << set_widget ("right", tw);
    THIS ["footer"] << emit_update ();
  }
}

class ia_command_rep: public command_rep {
  tm_widget_rep* man;
public:
  ia_command_rep (tm_widget_rep* man2): man (man2) {}
  void apply () { man->interactive_return (); }
  ostream& print (ostream& out) { return out << "tm_widget_rep command"; }
};

void
tm_widget_rep::swap_mode () {
  widget temp;
  temp= left;
  THIS ["footer"] << get_widget ("left", left);
  THIS ["footer"] << set_widget ("left", temp);
  temp= middle;
  THIS ["footer"] << get_widget ("middle", middle);
  THIS ["footer"] << set_widget ("middle", temp);
  temp= right;
  THIS ["footer"] << get_widget ("right", right);
  THIS ["footer"] << set_widget ("right", temp);
  cmd_mode= !cmd_mode;
}

void
tm_widget_rep::interactive (string name, string& s, command cmd) {
  if (cmd_mode) { s= "cancel"; return; }
  call_back= cmd;
  swap_mode ();
  widget tw= text_widget (dis, name, FALSE, "english");
  THIS ["footer"] << set_widget ("left", tw);
  THIS ["footer"] << set_widget ("middle",
		       input_text_widget (dis, new ia_command_rep (this)));
  THIS ["footer"] << emit_update ();
  THIS ["canvas"] << emit_keyboard_focus (FALSE);
  THIS ["footer|middle"] << emit_keyboard_focus (TRUE);
  text_ptr= &s;
}

void
tm_widget_rep::interactive_return () {
  THIS ["footer|middle"] << get_input_string (*text_ptr);
  THIS ["footer|middle"] << emit_keyboard_focus (FALSE);
  text_ptr= NULL;
  THIS ["canvas"] << emit_keyboard_focus (TRUE);
  swap_mode ();
  THIS ["footer"] << emit_update ();
  call_back ();
}

/******************************************************************************
* Handling other events
******************************************************************************/

void
tm_widget_rep::handle_get_size (get_size_event ev) {
  if (ev->mode == 0) { ev->w= 788*PIXEL /* 776*PIXEL */; ev->h=572*PIXEL; }
  else basic_widget_rep::handle_get_size (ev);
}

void
tm_widget_rep::handle_get_widget (get_widget_event ev) {
  a[0] << ev;
}

void
tm_widget_rep::handle_set_widget (set_widget_event ev) {
  a[0] << ev;
}

void
tm_widget_rep::handle_set_string (set_string_event ev) {
  if (ev->which == "left footer") { set_left_footer (ev->s); return; }
  if (ev->which == "right footer") { set_right_footer (ev->s); return; }
  fatal_error ("Could not set string attribute " * ev->which);
}

void
tm_widget_rep::handle_keypress (keypress_event ev) {
  if (cmd_mode) THIS ["footer|middle"] << ev;
  else THIS ["canvas"] << ev;
}

void
tm_widget_rep::handle_mouse (mouse_event ev) {
  basic_widget_rep::handle_mouse (ev);
}

void
tm_widget_rep::handle_keyboard_focus (keyboard_focus_event ev) {
  if (cmd_mode) THIS ["footer|middle"] << ev;
  else THIS ["canvas"] << ev;
}

void
tm_widget_rep::handle_resize (resize_event ev) {
  THIS ["footer|middle"] << ev;
  THIS ["canvas"] << ev;
}

void
tm_widget_rep::handle_alarm (alarm_event ev) {
  if (ev->message != "auto save") return;
  sv->auto_save ();
}

/******************************************************************************
* Handling standard events
******************************************************************************/

bool
tm_widget_rep::handle (event ev) {
  switch (ev->type) {
  case SET_STRING_EVENT:
    handle_set_string (ev);
    return TRUE;
  default:
    return basic_widget_rep::handle (ev);
  }
}

#endmodule // code_tm_widget
