
/******************************************************************************
* MODULE     : tm_lisp.gen.cc
* DESCRIPTION: The TeXmacs-lisp motor
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <tm_lisp.gen.h>
#include <file.gen.h>

#module code_tm_lisp
#import tm_lisp
#import file

extern void initialize_system (compiler cpl);
extern void initialize_editor (compiler cpl, server ed);
extern void initialize_server (compiler cpl, server ed);

/******************************************************************************
* Constructor and destructor
******************************************************************************/

tm_lisp_rep::tm_lisp_rep () {
  initialize_system (cpl);
  initialize_editor (cpl, this);
  initialize_server (cpl, this);
}

tm_lisp_rep::~tm_lisp_rep () {
}

/******************************************************************************
* TeXmacs-lisp functions
******************************************************************************/

program
tm_lisp_rep::convert (string s) {
  return lisp_expr_to_program (s, 0);
}

program
tm_lisp_rep::compile (program p) {
  return cpl->filter (cpl->compile (p), type ("Void"), p);
}

void
tm_lisp_rep::execute (program p) {
  program q= compile (p);
  if (is_compiled (q)) ::execute (q);
  else set_message ("Error: TeXmacs-lisp compilation error",
		    "execute TeXmacs-lisp expression");
}

void
tm_lisp_rep::execute (string s) {
  program p= lisp_expr_to_program (s, 0);
  execute (p);
}

void
tm_lisp_rep::lisp_eval_file (string dir, string name) {
  file f (dir, name, "r");
  f->check_open ("tm_lisp_rep::lisp_eval_file", "tm_lisp_rep.gen.cc");
  string s;
  f >> s;
  execute (string ("(seq " * s * ")"));
}

/******************************************************************************
* Delayed execution of commands
******************************************************************************/

void
tm_lisp_rep::execute_delayed (program p) {
  cmds << p;
}

void
tm_lisp_rep::execute_pending_commands () {
  while (N(cmds)!=0) {
    int i;
    program p= cmds[0];
    array<program> a (N(cmds)-1);
    for (i=1; i<N(cmds); i++) a[i-1]= cmds[i];
    cmds= a;
    execute (p);
  }
}

#endmodule // code_tm_lisp
