
/******************************************************************************
* MODULE     : tm_project.gen.cc
* DESCRIPTION: Buffer management for TeXmacs server
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <tm_project.gen.h>
#include <file.gen.h>
#include <iterator.gen.cc>

#module code_tm_project
#import tm_project
#import file
#import code_hashmap_iterator (string, tree)

/******************************************************************************
* Constructor and destructor
******************************************************************************/

tm_project_rep::tm_project_rep (): global ("?") {}
tm_project_rep::~tm_project_rep () {}

/******************************************************************************
* Process commands
******************************************************************************/

void
tm_project_rep::globalize_variable (string var) {
  global (var) = "?";
}

void
tm_project_rep::localize_variable (string var) {
  global->reset (var);
}

void
tm_project_rep::assign_variable (string var, tree t) {
  global (var) = t;
}

void
tm_project_rep::include_document (string s) {
  (void) s;
}

void
tm_project_rep::include_project (string s) {
  (void) s;
}

/******************************************************************************
* Project compilation motor
******************************************************************************/

bool
tm_project_rep::start (tm_buffer buf) {
  if (buf->prj == NULL) return TRUE;
  prj= buf->prj;
  et = prj->t;
  tp = path (0);
  global ("pagenr")= "0";
  global ("chapternr")= "0";
  return FALSE;
}

tree
tm_project_rep::next () {
  if (tp->item >= N(et)) return "";
  while (is_concat (subtree (et, tp)) && (N (subtree (et, tp)) > 0))
    tp= path_down (tp, 0);

  tree st= subtree (et, tp);
  
  while (TRUE) {
    tp= path_inc (tp);
    if ((atom(tp)) || (last_item (tp) < N (subtree (et, path_up (tp))))) break;
    else tp= path_up (tp);
  }
  
  if (is_func (st, APPLY)) return st;
  else return next ();
}

void
tm_project_rep::process (tree t) {
  if ((N(t)==2) && (t[0] == "globalize-variable") && (N(t[1])==0))
    globalize_variable (t[1]->label);
  if ((N(t)==2) && (t[0] == "localize-variable") && (N(t[1])==0))
    localize_variable (t[1]->label);
  if ((N(t)==3) && (t[0] == "assign-variable") && (N(t[1])==0))
    assign_variable (t[1]->label, t[2]);
  if ((N(t)==2) && (t[0] == "include-project") && (N(t[1])==0))
    include_project (t[1]->label);
  if ((N(t)==2) && (t[0] == "include-document") && (N(t[1])==0))
    include_document (t[1]->label);
}

/******************************************************************************
* Updating menus, buffers and views
******************************************************************************/

void
tm_project_rep::project_update_menu () {
  tm_buffer buf= get_buffer ();
  if (start (buf)) return;
  string s ("(menu_set project_buffer_menu '(");
  s << "(\"" << prj->abbr << "\" ";
  s << "(switch_to_buffer \"" * prj->name * "\"))";

  int i;
  tree t;
  while ((t=next()) != "") {
    if ((N(t)==2) && (t[0] == "include-document") && (N(t[1])==0)) {
      string name= get_relative_file_name (prj->name, t[1]->label, FALSE);
      for (i=N(name)-1; i>=0; i--)
	if (name[i]=='/') break;
      s << " (\"" << name (i+1, N(name)) << "\" ";
      s << "(switch_to_buffer \"" * name * "\"))";
    }
  }

  s << "))";
  execute (s);
}

void
tm_project_rep::project_update_view (tm_view vw) {
  (void) vw;
}

void
tm_project_rep::project_update_buffer (tm_buffer buf) {
  if (debug (0)) cout << "TeXmacs]   Updating " << buf->name << "\n";
  int i;
  for (i=0; i<N(buf->vws); i++)
    project_update_view ((tm_view) buf->vws[i]);
}

void
tm_project_rep::project_process_buffer (tm_buffer buf) {
  if (debug (0)) cout << "TeXmacs]   Processing " << buf->name << "\n";
  prj->ref->join (buf->ref);
  iterator<string> it= iterate (buf->aux);
  while (it->busy ()) {
    string to= it->next ();
    if (!prj->aux->contains (to))
      prj->aux (to)= tree (DOCUMENT);
    prj->aux (to) << buf->aux[to]->a;
  }
}

/******************************************************************************
* Main compilation routines
******************************************************************************/

void
tm_project_rep::project_attach (string prj_name) {
  int i;
  tm_buffer buf= get_buffer ();
  buf->project= prj_name;
  buf->modification_flag= TRUE;
  for (i=0; i<N(buf->vws); i++) {
    tm_view vw= (tm_view) buf->vws[i];
    vw->ed->set_main_menu (vw->ed->main_menu);
  }
  if (prj_name == "") buf->prj= NULL;
  else {
    string full_name= get_relative_file_name (buf->name, prj_name);
    buf->prj= load_passive_buffer (full_name);
    project_update_menu ();
  }
}

bool
tm_project_rep::project_attached () {
  tm_buffer buf= get_buffer ();
  return buf->project != "";
}

void
tm_project_rep::project_compile_all () {
  tm_buffer buf= get_buffer ();
  if (start (buf)) return;
  prj->aux= hashmap<string,tree> ("?");

  if (debug (0)) cout << "TeXmacs] Compiling project " << prj->name << "\n";
  tree t;
  while ((t=next()) != "") {
    // cout << "  Processing " << t << "\n";
    process (t);
    if ((N(t)==2) && (t[0] == "include-document") && (N(t[1])==0)) {
      string name= get_relative_file_name (prj->name, t[1]->label, FALSE);
      tm_buffer cur_buf= load_passive_buffer (name);
      if (cur_buf == NULL) continue;
      project_process_buffer (cur_buf);
      project_update_buffer (cur_buf);
    }
  }
  if (debug (0)) cout << "TeXmacs] Compilation done\n";
}

void
tm_project_rep::project_compile_buffer () {
  tm_buffer buf= get_buffer ();
  if (start (buf)) return;
  prj->aux= hashmap<string,tree> ("?");

  if (debug (0))
    cout << "TeXmacs] Compiling " << buf->name << " in " << prj->name << "\n";
  tree t;
  while ((t=next()) != "") {
    // cout << "  Processing " << t << "\n";
    process (t);
    if ((N(t)==2) && (t[0] == "include-document") && (N(t[1])==0)) {
      string name= get_relative_file_name (prj->name, t[1]->label, FALSE);
      tm_buffer cur_buf= load_passive_buffer (name);
      if (cur_buf == NULL) continue;
      project_process_buffer (cur_buf);
      if (name == buf->name) project_update_buffer (buf);
    }
  }
  if (debug (0)) cout << "TeXmacs] Compilation done\n";
}

#endmodule // code_tm_project
