
/******************************************************************************
* MODULE     : tm_server.gen.cc
* DESCRIPTION: The TeXmacs server
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <tm_server.gen.h>
#include <analyze.gen.h>
#include <connect.gen.h>

#module code_tm_server
#import tm_server
#import analyze
#import connect

/******************************************************************************
* Texmacs server constructor and destructor
******************************************************************************/

static tm_server_rep* the_server= NULL;
static void texmacs_interpose_handler () {
  if (the_server != NULL) the_server->interpose_handler (); }

server_rep::server_rep () {}
server_rep::~server_rep () {}

tm_server_rep::tm_server_rep (display dis2): dis (dis2), vw (NULL) {
  the_server= this;
  set_interpose_handler (texmacs_interpose_handler);
  tm_lisp_rep::execute (string ("(var buffer_menu '())"));
  tm_lisp_rep::execute (string ("(var project_buffer_menu '())"));

  if (file_exists ("$TEXMACS_HOME_PATH/progs", "Init.tl"))
    lisp_eval_file ("$TEXMACS_HOME_PATH/progs", "Init.tl");
  else lisp_eval_file ("$TEXMACS_PATH/progs", "Init.tl");
#os linux
  return; // in order to avoid segmentation faults
#os all
}

tm_server_rep::~tm_server_rep () {}
server::server (display dis): rep (new tm_server_rep (dis)) {}

/******************************************************************************
* Get and set objects associated to server
******************************************************************************/

server_rep*
tm_server_rep::get_server () {
  return this;
}

compiler&
tm_server_rep::get_compiler () {
  return cpl;
}

display
tm_server_rep::get_display () {
  return dis;
}

tm_view
tm_server_rep::get_view (bool must_be_valid) {
  if (must_be_valid && (vw==NULL))
    fatal_error ("No active view", "tm_server_rep::get_meta");
  return vw;
}

void
tm_server_rep::set_view (tm_view vw2) {
  vw= vw2;
}

tm_buffer
tm_server_rep::get_buffer () {
  tm_view vw= get_view ();
  return vw->buf;
}

editor
tm_server_rep::get_editor () {
  tm_view vw= get_view ();
  // cout << "Get editor" << vw->ed << "\n";
  return vw->ed;
}

tm_window
tm_server_rep::get_window () {
  tm_view vw= get_view ();
  if (vw->win==NULL)
    fatal_error ("No window attached to view", "tm_server_rep::get_meta");
  return vw->win;
}

tm_widget
tm_server_rep::get_meta () {
  tm_window win= get_window ();
  return win->wid;
}

/******************************************************************************
* Miscellaneous routines
******************************************************************************/

void
tm_server_rep::interpose_handler () {
  execute_pending_commands ();
  int i,j;
  for (i=0; i<N(bufs); i++) {
    tm_buffer buf= (tm_buffer) bufs[i];
    for (j=0; j<N(buf->vws); j++) {
      tm_view vw= (tm_view) buf->vws[j];
      if (vw->win != NULL) vw->ed->apply_changes ();
    }
  }
}

void
tm_server_rep::focus_on_editor (editor ed) {
  int i,j;
  for (i=0; i<N(bufs); i++) {
    tm_buffer buf= (tm_buffer) bufs[i];
    for (j=0; j<N(buf->vws); j++) {
      tm_view vw= (tm_view) buf->vws[j];
      if (vw->ed == ed) {
	set_view (vw);
	return;
      }
    }
  }
  fatal_error ("Invalid situation", "tm_server_rep::focus_on_editor");
}

bool
tm_server_rep::is_yes (string s) {
  s= locase_all (s);
  return
    (s == "yes") || (s == "y") ||
    (s == "oui") || (s == "o") ||
    (s == "ja") || (s == "j");
}

void
tm_server_rep::quit (string confirm) {
  if (is_yes (confirm)) exit (0);
}

void
tm_server_rep::get_visible (SI& x1, SI& y1, SI& x2, SI& y2) {
  widget meta= (widget) get_meta ();
  meta["canvas"] << ::get_visible (x1, y1, x2, y2);
}

void
tm_server_rep::scroll_to (SI x, SI y) {
  widget meta= (widget) get_meta ();
  meta["canvas"] << set_scroll_pos (x, y);
}

void
tm_server_rep::set_extents (SI x1, SI y1, SI x2, SI y2) {
  widget meta= (widget) get_meta ();
  meta["canvas"] << ::set_extents (x1, y1, x2, y2);
}

void
tm_server_rep::set_left_footer (string s) {
  if ((vw == NULL) || (vw->win == NULL)) return;
  get_meta()->set_left_footer (s);
}

void
tm_server_rep::set_right_footer (string s) {
  if ((vw == NULL) || (vw->win == NULL)) return;
  get_meta()->set_right_footer (s);
}

void
tm_server_rep::set_message (string left, string right) {
  get_editor()->set_message (left, right);
}

void
tm_server_rep::interactive (string name, string& s, command call_back) {
  get_meta()->interactive (name, s, call_back);
}

/******************************************************************************
* Extern packages
******************************************************************************/

void
tm_server_rep::declare (string name, string lib, string symb, string init) {
  package_declare (name, lib, symb, init);
}

string
tm_server_rep::evaluate (string name, string session, string expr) {
  if (!package_declared (name)) {
    set_message ("Package#'" * name * "'#not declared",
		 "Evaluate#'" * name * "'#expression");
    return "";
  }
  if (!package_installed (name)) {
    string r= package_install (name);
    set_message (r, "Install#'" * name * "'#package");
    if (!package_installed (name)) return "";
  }
  return package_evaluate (name, session, expr);
}

string
tm_server_rep::execute (string name, string session, string expr) {
  if (!package_declared (name)) {
    set_message ("Package#'" * name * "'#not declared",
		 "Evaluate#'" * name * "'#expression");
    return "";
  }
  if (!package_installed (name)) {
    string r= package_install (name);
    set_message (r, "Install#'" * name * "'#package");
    if (!package_installed (name)) return "";
  }
  return package_execute (name, session, expr);
}

/******************************************************************************
* System commands
******************************************************************************/

void
tm_server_rep::shell (string s) {
  system (s);
}

#endmodule // code_tm_server
