
/******************************************************************************
* MODULE     : texmacs.gen.cc
* DESCRIPTION: main program
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <dir.gen.h>
#include <server.gen.h>

#module code_texmacs
#import dir
#import server

extern bool char_clip;
extern bool active_envs;
extern bool install_guile (int, char**, void (*) (int,char**));
extern bool initialize_guile (pointer ptr_sv);

bool use_guile;

/******************************************************************************
* Main program
******************************************************************************/

void
TeXmacs_main (int argc, char** argv) {
  int i;
  bool flag= TRUE;
  cout << "\n";
  for (i=1; i<argc; i++)
    if (((argv[i][0] == '-') || (argv[i][0] == '+')) && (argv[i][1] != '\0')) {
      string s= argv[i];
      if (s == "-s") flag= FALSE;
      else if ((s == "-g") || (s == "--guile"));
      else if ((s == "-e") || (s == "--events")) debug (1, TRUE);
      else if ((s == "-d") || (s == "--debug")) debug (2, TRUE);
      else if ((s == "-v") || (s == "--version")) {
	cout << "TeXmacs version " << TEXMACS_VERSION << "\n";
	cout << TEXMACS_COPYRIGHT << "\n";
	cout << "\n";
	exit (0);
      }
      else if ((s == "-x") || (s == "manual")) {
	system ("xdvi $TEXMACS_PATH/doc/dvi/texmacs.dvi &");
        exit (0);
      }
      else if ((s == "-Oc") || (s == "--no-char-clipping")) char_clip= FALSE;
      else if ((s == "+Oc") || (s == "--char-clipping")) char_clip= TRUE;
      else if ((s == "-Oe") || (s == "--no-active-envs")) active_envs= FALSE;
      else if ((s == "+Oe") || (s == "--active_envs")) active_envs= TRUE;
      else {
	cout << "Options for TeXmacs:\n\n";
	cout << "  -d  For debugging purposes\n";
	cout << "  -e  Display all widget events\n";
	cout << "  -g  Dynamically link guile\n";
	cout << "  -h  Display this help message\n";
	cout << "  -s  Do not inform about loaded files\n";
	cout << "  -v  Display current TeXmacs version\n";
	cout << "  -x  Consult the TeXmacs manual using xdvi\n";
	cout << "  -Oc TeX characters bitmap clipping off\n";
	cout << "  +Oc TeX characters bitmap clipping on (default)\n";
	cout << "  -Oe Don't show active environments (default)\n";
	cout << "  +Oe Show active environments\n";
	cout << "\n";
	exit (0);
      }
    }
  if (flag) debug (0, TRUE);

  install_texmacs ();
  install_tex ();
  display dis= open_display (argc, argv);
  server sv (dis);
  if (use_guile) initialize_guile ((pointer) &sv);

  flag= TRUE;
  for (i=1; i<argc; i++)
    if ((argv[i][0] != '-') && (argv[i][0] != '+')) {
      sv->load_generic_buffer (argv[i], 1);
      flag= FALSE;
    }
  if (flag) sv->open_window ();

  dis->delayed_message (sv->get_meta(), "auto save", 120000);
  dis->event_loop ();
  close_display (dis);
}

/******************************************************************************
* Real main program for encaptulation of guile
******************************************************************************/

int
main (int argc, char** argv) {
  int i;
  use_guile= FALSE;
  for (i=1; i<argc; i++) {
    char* s= argv[i];
    if ((s[0]=='-') && (s[1]=='g')) use_guile= TRUE;
    if ((s[0]=='-') && (s[1]=='-') && (s[1]=='g') && (s[1]=='u') &&
	(s[1]=='i') && (s[1]=='l') && (s[1]=='e')) use_guile= TRUE;
  }

  if (use_guile) install_guile (argc, argv, TeXmacs_main);
  else TeXmacs_main (argc, argv);
  return 0;
}

#endmodule // code_texmacs
