
/******************************************************************************
* MODULE     : server.gen.h
* DESCRIPTION: Main current graphical interface for user applications
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <data.gen.h>
#include <edit_main.gen.h>
#include <compiler.gen.h>
#include <tm_widget.gen.h>

#module server
#import data
#import edit_main
#import compiler
#import array (editor)
#import tm_widget

class tm_buffer_rep;
class tm_view_rep;
class tm_window_rep;
typedef tm_buffer_rep* tm_buffer;
typedef tm_view_rep*   tm_view;
typedef tm_window_rep* tm_window;

class server_rep: public abstract_struct {
public:
  server_rep ();
  virtual ~server_rep ();

  /* Control global server parameters */
  virtual void   set_input_language (string s) = 0;
  virtual void   set_output_language (string s) = 0;
  virtual string get_input_language () = 0;
  virtual string get_output_language () = 0;
  virtual void   set_font_rules (program rules) = 0;
  virtual void   set_latex_commands (program cmds) = 0;
  virtual void   set_latex_symbols (program syms) = 0;
  virtual bool   get_latex_command (string s, string& help, program& p) = 0;
  virtual void   set_keymap (program m, program cmds) = 0;
  virtual void   set_shorthands (program m, string l ,string g, program p) = 0;
  virtual void   get_keycomb (string mode, string lan, string s,
			      int& status, program& keym, string& shorth,
			      string& help) = 0;

  /* TeXmacs-lisp motor */
  virtual program convert (string s) = 0;
  virtual program compile (program p) = 0;
  virtual void execute (program p) = 0;
  virtual void execute (string s) = 0;
  virtual void lisp_eval_file (string dir, string name) = 0;
  virtual void execute_delayed (program p) = 0;

  /* TeXmacs layout */
  virtual void menu_widget (program p, widget& w) = 0;
  virtual void menu_main (program p) = 0;
  virtual void menu_set (program& m, program p) = 0;
  virtual void menu_merge (program& m, program p) = 0;

  /* Buffer management */
  virtual bool no_name () = 0;
  virtual string get_name_buffer () = 0;
  virtual void set_name_buffer (string name) = 0;
  virtual void new_buffer () = 0;
  virtual void switch_to_buffer (string name) = 0;
  virtual void revert_buffer () = 0;
  virtual void kill_buffer () = 0;
  virtual void open_window () = 0;
  virtual void clone_window () = 0;
  virtual void kill_window () = 0;

  /* Loading and saving files */
  virtual void load_buffer (string name, string format, int where=0) = 0;
  virtual void load_buffer (string d,string n,string f,int w=0,bool t=TRUE)=0;
  virtual void load_generic_buffer (string name, int where=0) = 0;
  virtual void load_generic_buffer (string dir, string name, int where=0) = 0;
  virtual void save_buffer (string name, string format) = 0;
  virtual void save_buffer (string dir, string name, string format) = 0;
  virtual void save_generic_buffer (string name) = 0;
  virtual void save_generic_buffer (string dir, string name) = 0;
  virtual bool exists_unsaved_buffer () = 0;
  virtual void pretend_save_buffer () = 0;
  virtual void auto_save () = 0;
  virtual void recover_auto_save
    (string dir, string name, string format, string where, string answer) = 0;

  /* Projects */
  virtual void project_update_menu () = 0;
  virtual void project_attach (string prj_name= "") = 0;
  virtual bool project_attached () = 0;
  virtual void project_compile_all () = 0;
  virtual void project_compile_buffer () = 0;

  /* Get and set objects associated to server */
  virtual server_rep* get_server () = 0;
  virtual compiler&   get_compiler () = 0;
  virtual display     get_display () = 0;
  virtual tm_view     get_view (bool must_be_valid= TRUE) = 0;
  virtual void        set_view (tm_view vw) = 0;
  virtual tm_buffer   get_buffer () = 0;
  virtual editor      get_editor () = 0;
  virtual tm_window   get_window () = 0;
  virtual tm_widget   get_meta () = 0;

  /* Misscelaneous routines */
  virtual void   focus_on_editor (editor ed) = 0;
  virtual bool   is_yes (string s) = 0;
  virtual void   quit (string confirm= "yes") = 0;
  virtual void   declare (string name, string lib, string symb, string init)=0;
  virtual string evaluate (string name, string session, string expr) = 0;
  virtual string execute  (string name, string session, string expr) = 0;
  virtual void   shell    (string s) = 0;

  virtual void get_visible (SI& x1, SI& y1, SI& x2, SI& y2) = 0;
  virtual void scroll_to (SI x, SI y) = 0;
  virtual void set_extents (SI x1, SI y1, SI x2, SI y2) = 0;
  virtual void set_left_footer (string s) = 0;
  virtual void set_right_footer (string s) = 0;
  virtual void set_message (string left, string right) = 0;
  virtual void interactive (string name, string& s, command call_back) = 0;
};

class server {
#import abstract (server, server_rep)
  server (display dis);
};
#import code_abstract (server, server_rep)

#endmodule // server
