
/******************************************************************************
* MODULE     : tm_buffer.gen.h
* DESCRIPTION: TeXmacs main data structures (buffers, views and windows)
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <server.gen.h>
#include <tm_widget.gen.h>

#module tm_buffer
#import server
#import tm_widget
#import array (pointer)

#define array<tm_buffer> array<pointer>
#define array<tm_view> array<pointer>

class tm_buffer_rep {
public:
  string name;            // full name
  string abbr;            // abbreviated name
  string format;          // buffer format
  array<tm_view> vws;     // views attached to buffer
  bool modification_flag; // modification since last (auto)save?
  bool not_saved_flag;    // buffer autosaved, but not saved?
  bool read_only;         // buffer is read only?

  tree t;                     // the edit tree
  tree project;               // a project the document belongs to
  tree style;                 // the style of the buffer
  hashmap<string,tree> init;  // initial values of environment variables
  hashmap<string,tree> fin;   // final values of environment variables
  hashmap<string,tree> ref;   // all labels with references
  hashmap<string,tree> aux;   // auxiliary output: toc, bib, etc.

  tm_buffer prj;          // buffer which corresponds to the project

  inline tm_buffer_rep (string name2):
    name (name2), abbr (name2), format ("TeXmacs"), vws (0),
    modification_flag (FALSE), not_saved_flag (FALSE), read_only (FALSE),
    t (DOCUMENT, ""), project (""), style ("style"),
    init ("?"), fin ("?"), ref ("?"), aux ("?"),
    prj (NULL) {}
};

class tm_view_rep {
public:
  tm_buffer buf;
  editor    ed;
  tm_window win;
  inline tm_view_rep (tm_buffer buf2, editor ed2):
    buf (buf2), ed (ed2), win (NULL) {}
};

class tm_window_rep {
public:
  window    win;
  tm_widget wid;
  inline tm_window_rep (tm_widget wid2):
    win (plain_window (wid2, "TeXmacs")), wid (wid2) {}
};

typedef tm_buffer_rep* tm_buffer;
typedef tm_view_rep*   tm_view;
typedef tm_window_rep* tm_window;

#endmodule // tm_buffer
