
/******************************************************************************
* MODULE     : tm_server.gen.h
* DESCRIPTION: Main current graphical interface for user applications
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <tm_config.gen.h>
#include <tm_buffer.gen.h>
#include <tm_layout.gen.h>
#include <tm_lisp.gen.h>

#module tm_server
#import tm_config
#import tm_buffer
#import tm_layout
#import tm_lisp

class tm_server_rep:
  public tm_config_rep,
  // public tm_data_rep,
  public tm_project_rep,
  public tm_layout_rep,
  public tm_lisp_rep
{
protected:
  display dis; // current display
  tm_view vw;  // current editor

public:
  tm_server_rep (display dis);
  ~tm_server_rep ();

  /* Get and set objects associated to server */
  server_rep* get_server ();
  compiler&   get_compiler ();
  display     get_display ();
  tm_view     get_view (bool must_be_valid= TRUE);
  void        set_view (tm_view vw);
  tm_buffer   get_buffer ();
  editor      get_editor ();
  tm_window   get_window ();
  tm_widget   get_meta ();

  /* Miscellaneous routines */
  void   interpose_handler ();
  void   focus_on_editor (editor ed);
  bool   is_yes (string s);
  void   quit (string confirm);
  void   declare (string name, string lib, string symb, string init);
  string evaluate (string name, string session, string expr);
  string execute  (string name, string session, string expr);
  void   shell    (string s);

  void get_visible (SI& x1, SI& y1, SI& x2, SI& y2);
  void scroll_to (SI x, SI y);
  void set_extents (SI x1, SI y1, SI x2, SI y2);
  void set_left_footer (string s);
  void set_right_footer (string s);
  void set_message (string left, string right);
  void interactive (string name, string& s, command call_back);
};

#endmodule // tm_server
