
/******************************************************************************
* MODULE     : typesetter.gen.cc
* DESCRIPTION: conversion of TeX_tree's into boxes
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <analyze.gen.h>
#include <file.gen.h>
#include <Atom/typesetter.gen.h>
#include <Boxes/construct.gen.h>

#module code_typesetter
#import analyze
#import file
#import typesetter
#import construct_boxes

typesetter_rep::typesetter_rep (edit_env env2): env (env2) {}

/******************************************************************************
* text
******************************************************************************/

box
typesetter_rep::typeset_string (string s) {
  return env->fn->convert (s, env->col);
}

box
typesetter_rep::typeset_space (tree t) {
  SI w = env->decode_length (t[0]->label);
  SI y1= env->fn->y1;
  SI y2= env->fn->y2;
  if (N(t)==3) {
    y1= env->decode_length (t[1]->label);
    y2= env->decode_length (t[2]->label);
  }
  return empty_box (0, y1, w, y2);
}

box
typesetter_rep::typeset_move (tree t) {
  box  b= typeset (t[0]);
  SI   x= env->decode_length (t[1]->label);
  SI   y= env->decode_length (t[2]->label);
  return move_box (b, x, y, TRUE);
}

static SI
resize (edit_env env, SI old, SI ink, string new_size) {
  if (new_size == "same") return old;
  if (new_size == "ink") return ink;
  return env->decode_length (new_size);
}

box
typesetter_rep::typeset_resize (tree t) {
  box  b   = typeset (t[0]);
  bool flag= t[1]->label == "extend"; 
  SI   x1  = resize (env, b->x1, b->x3, t[2]->label);
  SI   y1  = resize (env, b->y1, b->y3, t[3]->label);
  SI   x2  = resize (env, b->x2, b->x4, t[4]->label);
  SI   y2  = resize (env, b->y2, b->y4, t[5]->label);
  if (flag) {
    x1= min (x1, b->x1);
    y1= min (y1, b->y1);
    x2= max (x2, b->x2);
    y2= max (y2, b->y2);
  }
  return resize_box (b, x1, y1, x2, y2, TRUE);
}

/******************************************************************************
* mathematics
******************************************************************************/

box
typesetter_rep::typeset_frac (tree t) {
  bool disp= env->display_style;
  if (disp) env->set (DISPLAY_STYLE, "false");
  else env->script_size ();
  env->set (VERTICAL_POS, "1");
  box nom= typeset (t[0]);
  env->assign (VERTICAL_POS, "-1");
  box den= typeset (t[1]);
  env->reset (VERTICAL_POS);
  if (disp) env->reset (DISPLAY_STYLE);
  else env->normal_size ();
  return frac_box (nom, den, env->fn->wfrac, 3*env->fn->wfrac,
		   env->fn->yfrac, env->col);
}

box
typesetter_rep::typeset_sqrt (tree t) {
  box b= typeset (t[0]);
  box ind;
  if (N(t)==2) {
    bool disp= env->display_style;
    if (disp) env->set (DISPLAY_STYLE, "false");
    env->script_size ();
    ind= typeset (t[1]);
    env->normal_size ();
    if (disp) env->reset (DISPLAY_STYLE);
  }
  box sqrtb= env->fn->convert ("<sqrt>", env->col, b->y1, b->y2);
  return sqrt_box (b, ind, sqrtb, env->fn->wsqrt, env->col);
}

box
typesetter_rep::typeset_wide (tree t) {
  box b= typeset (t[0]);
  string s= t[1]->label;
  box wideb= env->fn->convert (s, env->col, b->x1, b->x2);
  return wide_box (b, wideb, env->fn->ysup);
}

box
typesetter_rep::typeset_neg (tree t) {
  return neg_box (typeset (t[0]), env->fn->wneg, env->fn->wquad/6, env->col);
}

box
typesetter_rep::typeset_tree (tree t) {
  int i, n= N(t);
  array<box> bs(n);
  for (i=0; i<n; i++) bs[i]= typeset (t[i]);
  SI spc= env->fn->spc->def;
  return tree_box (bs, 2*spc, 4*spc, env->fn->wneg, env->col);
}

box
typesetter_rep::typeset_matrix (tree t) {
  int n= N(t)-2;
  int x= as_int (t[n]->label);
  int y= as_int (t[n+1]->label);
  int i;

  array<box> bs(n);
  for (i=0; i<n; i++) bs[i]= typeset (t[i]);
  return matrix_box (x, y, bs, 2*env->fn->spc->def, 0, env->fn->yfrac);
}

box
typesetter_rep::typeset_table (tree t) {
  int n= N(t)-2;
  int x= as_int (t[n]->label);
  int y= as_int (t[n+1]->label);
  int i;

  array<box> bs (n);
  for (i=0; i<n; i++) bs[i]= typeset (t[i]);
  SI x_spc= 2*env->fn->spc->def;
  SI y_spc= 2*env->fn->wpt;
  return table_box (x,y, bs, x_spc, y_spc, env->fn->yfrac, env->col);
}

box
typesetter_rep::typeset_mosaic (tree t) {
  int   i;
  int   n= N(t)-2;
  int   x= as_int (t[n]->label);
  int   y= as_int (t[n+1]->label);

  array<box>    bs (n);
  array<string> pos (n);
  array<int>    nr_x (n);
  array<int>    nr_y (n);
  array<color>  col (n);

  for (i=0; i<n; i++) {
    bs[i]  = mosaic_item_box (typeset (t[i][0]));
    pos[i] = t[i][1]->label;
    nr_x[i]= as_int (t[i][2]->label);
    nr_y[i]= as_int (t[i][3]->label);
    col[i] = (t[i][4]->label == "none"? -1: env->col);
  }

  return mosaic_box (x, y, bs, pos, nr_x, nr_y, env->fn->yfrac, col);
}

/******************************************************************************
* Graphics
******************************************************************************/

box
typesetter_rep::typeset_graphics (tree t) {
  (void) t;
  return empty_box ();
}

box
typesetter_rep::typeset_point (tree t) {
  (void) t;
  return empty_box ();
}

box
typesetter_rep::typeset_line (tree t) {
  (void) t;
  return empty_box ();
}

box
typesetter_rep::typeset_arc (tree t) {
  (void) t;
  return empty_box ();
}

box
typesetter_rep::typeset_bezier (tree t) {
  (void) t;
  return empty_box ();
}

static double
get_magnification (string s) {
  int i=0;
  double magn= 1.0, result;
  while (i<N(s)) {
    if (s[i]=='*') { i++; read_double (s, i, result); magn *= result; }
    else if (s[i]=='/') { i++; read_double (s, i, result); magn /= result; }
    else return magn;
  }
  return magn;
}

box
typesetter_rep::typeset_postscript (tree t) {
  if (N(t)<7) return empty_box ();
  string file_name = t[0]->label;
  SI bx1, by1, bx2, by2;
  ps_bounding_box (file_name, bx1, by1, bx2, by2);
  double pt= ((double) env->dpi*PIXEL)/72.0;

  int x1, y1, x2, y2;
  if (N(t[3]->label)==0) x1= bx1;
  else x1= bx1+ ((SI) (((double) env->decode_length (t[3]->label)) / pt));
  if (N(t[4]->label)==0) y1= by1;
  else y1= by1+ ((SI) (((double) env->decode_length (t[4]->label)) / pt));
  if (N(t[5]->label)==0) x2= bx2;
  else x2= bx1+ ((SI) (((double) env->decode_length (t[5]->label)) / pt));
  if (N(t[6]->label)==0) y2= by2;
  else y2= by1+ ((SI) (((double) env->decode_length (t[6]->label)) / pt));
  if ((x1>=x2) || (y1>=y2)) return empty_box ();

  SI w, h;
  string ws= t[1]->label;
  string hs= t[2]->label;
  bool wflag= (N(ws)!=0) && (ws[0]!='*') && (ws[0]!='/');
  bool hflag= (N(hs)!=0) && (hs[0]!='*') && (hs[0]!='/');
  if (wflag) {
    w = env->decode_length (ws);
    if (hflag) h= env->decode_length (hs);
    else h= (w*(y2-y1)) / (x2-x1);
  }
  else {
    if (hflag) {
      h= env->decode_length (hs);
      w= (h*(x2-x1)) / (y2-y1);
    }
    else {
      w= (SI) (((double) (x2-x1)) * pt);
      h= (SI) (((double) (y2-y1)) * pt);
    }
  }
  if ((!wflag) && (N(ws)!=0)) w= (SI) (((double) w) * get_magnification (ws));
  if ((!hflag) && (N(hs)!=0)) h= (SI) (((double) h) * get_magnification (hs));

  return postscript_box (file_name, w, h, x1, y1, x2, y2);
}

/******************************************************************************
* Dispatcher
******************************************************************************/

box
typesetter_rep::typeset (tree t) {
  string s(t->label);
  
  if (N(t)==0) return typeset_concat (s);

  switch (math_primary [s]) {
  case Document:         return typeset_document (t);
  case Paragraph:        return typeset_paragraph (t);
  case Concat:
  case Format:
  case Hspace:
  case Vspace_before:
  case Vspace_after:     return typeset_concat (t);
  case Space:            return typeset_space (t);
  case Htab:             return typeset_concat (t);
  case Move:             return typeset_move (t);
  case Resize:           return typeset_resize (t);

  case Left_sub:
  case Left_sup:
  case Right_sub:
  case Right_sup:        return typeset_concat (t);
  case Frac:             return typeset_frac (t);
  case Sqrt:             return typeset_sqrt (t);
  case Wide:             return typeset_wide (t);
  case Neg:              return typeset_neg (t);
  case Tree:             return typeset_tree (t);
  case Matrix:           return typeset_matrix (t);
  case Table:            return typeset_table (t);
  case Mosaic:           return typeset_mosaic (t);

  case Inactive:
  case Symbol:
  case Backup:
  case Quote:
  case Assign:
  case Set:
  case Reset:
  case Apply:
  case Begin:
  case End:
  case Function:
  case Environment:
  case Label:
  case Reference:
  case Plus:
  case Minus:
  case Times:
  case Merge:            return typeset_concat (t);
  case Number:           return test_box ();

  case Graphics:         return typeset_graphics (t);
  case Point:            return typeset_point (t);
  case Line:             return typeset_line (t);
  case Arc:              return typeset_arc (t);
  case Bezier:           return typeset_bezier (t);
  case Postscript:       return typeset_postscript (t);

  default:               return test_box ();
  }
}

#endmodule // code_typesetter
