
/******************************************************************************
* MODULE     : basic.gen.h
* DESCRIPTION: Basic boxes are the most elementary boxes.
*              We have implemented the following ones:
*                - a test box
*                - line boxes
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <Boxes/boxes.gen.h>

#module code_basic_boxes
#import boxes

/******************************************************************************
* The test box
******************************************************************************/

struct test_box_rep: public box_rep {
  test_box_rep () { x1=x3=0; x2=x4=50<<8; y1=y3=0; y2=y4= 25<<8; }
  operator tree () { return "test"; }
  void display (ps_device dev);
};

void
test_box_rep::display (ps_device dev) {
  dev->set_color (dev->green);
  dev->set_line_style (PIXEL);
  dev->line (x1, y1, x2, y2);
  dev->line (x1, y2, x2, y1);
  // dev->arc (x1, y1, x2, y2, 45*64, 90*64);
}

/******************************************************************************
* Line boxes
******************************************************************************/

struct line_box_rep: public box_rep {
  SI    X1, Y1, X2, Y2;
  SI    width;
  color col;

  line_box_rep (SI X1b, SI Y1b, SI X2b, SI Y2b, SI w, color c);
  operator tree () { return "line"; }
  void display (ps_device dev);
};

line_box_rep::line_box_rep (SI X1b, SI Y1b, SI X2b, SI Y2b, SI w, color c) {
  X1     = X1b;
  Y1     = Y1b;
  X2     = X2b;
  Y2     = Y2b;
  width  = w;
  col    = c;
  x1= min (X1, X2); y1= min (Y1, Y2);
  x2= max (X1, X2); y2= max (Y1, Y2);
  x3= x1-(w>>1);    y3= y1-(w>>1); 
  x4= x2+(w>>1);    y4= y2+(w>>1);
}

void
line_box_rep::display (ps_device dev) {
  dev->set_line_style (width);
  dev->set_color (col);
  dev->line (X1, Y1, X2, Y2);
}

/******************************************************************************
* Postscript boxes
******************************************************************************/

struct postscript_box_rep: public box_rep {
  string file_name;
  int X1, Y1, X2, Y2;

  postscript_box_rep (string file_name2, SI w, SI h,
		      int X1, int Y1, int X2, int Y2);
  operator tree () { return "postscript"; }
  void display (ps_device dev);
};

postscript_box_rep::postscript_box_rep (
  string file_name2, SI w, SI h, int X1b, int Y1b, int X2b, int Y2b)
{
  file_name= file_name2;
  X1= X1b; Y1= Y1b;
  X2= X2b; Y2= Y2b;
  x1= x3= 0; y1= y3= 0;
  x2= x4= w; y2= y4= h;
}

void
postscript_box_rep::display (ps_device dev) {
  dev->postscript (file_name, x2, y2, 0, 0, X1, Y1, X2, Y2);
}

/******************************************************************************
* box construction routines
******************************************************************************/

box
test_box () {
  return new test_box_rep ();
}

box
line_box (SI x1, SI y1, SI x2, SI y2, SI w, SI c) {
  return new line_box_rep (x1, y1, x2, y2, w, c);
}

box
postscript_box (string file_name, SI w, SI h, int x1, int y1, int x2, int y2) {
  return new postscript_box_rep (file_name, w, h, x1, y1, x2, y2);
}

#endmodule // code_basic_boxes
