
/******************************************************************************
* MODULE     : composite.gen.cc
* DESCRIPTION: Boxes with a list of child boxes
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <Boxes/composite.gen.h>

#module code_composite_boxes
#import composite_boxes

/******************************************************************************
* Composite boxes
******************************************************************************/

composite_box_rep::composite_box_rep () { }

composite_box_rep::composite_box_rep (array<box> B) {
  bs= B;
  position ();
}

composite_box_rep::composite_box_rep (array<box> B, array<SI> x, array<SI> y) {
  int i;
  bs= B;
  for (i=0; i<N(B); i++) {
    B[i]->x0= x[i];
    B[i]->y0= y[i];
  }
  position ();
}

composite_box_rep::~composite_box_rep () {}

void
composite_box_rep::insert (box b, SI x, SI y) {
  bs << b;
  b->x0= x;
  b->y0= y;
}

void
composite_box_rep::position () {
  int i;
  if (N(bs)==0) fatal_error ("empty composite box", "composite_box::position");
  else {
    x1= y1= x3= y3= MAX_SI;
    x2= y2= x4= y4= -MAX_SI;
  }
  for (i=0; i<N(bs); i++) {
    box& b (bs[i]);
    x1= min (x1, b->x0+ b->x1);
    y1= min (y1, b->y0+ b->y1);
    x2= max (x2, b->x0+ b->x2);
    y2= max (y2, b->y0+ b->y2);
    x3= min (x3, b->x0+ b->x3);
    y3= min (y3, b->y0+ b->y3);
    x4= max (x4, b->x0+ b->x4);
    y4= max (y4, b->y0+ b->y4);
  }
}

void
composite_box_rep::left_justify () {
  int i;
  SI d= x1;
  x1-=d; x2-=d; x3-=d; x4-=d;
  for (i=0; i<N(bs); i++) bs[i]->x0-=d;
}

void
composite_box_rep::display (ps_device dev) { (void) dev; }

int
composite_box_rep::subnr () {
  return N(bs);
}

box
composite_box_rep::subbox (int i) {
  return bs[i];
}

cursor
composite_box_rep::get_cursor (path p) {
  if (nil (p->next)) return std_cursor (p);
  else {
    int i=p->item;
    cursor cu (bs[i]->var_get_cursor (p->next));
    cu->x0 += bs[i]->x0;
    cu->y0 += bs[i]->y0;
    return cu;
  }
}

path
composite_box_rep::get_path (SI x, SI y, SI delta) {
  int i, d=MAX_SI, m=-1;

  for (i=0; i<N(bs); i++)
    if (distance (bs[i], x, y, delta)< d) {
      d= distance (bs[i], x, y, delta);
      m= i;
    }
  if ((m==-1) || outside (x, delta, x1, x2))
    return std_path (x, y, delta);
  else {
    path p= bs[m]->var_get_path (x- bs[m]->x0, y- bs[m]->y0, delta);
    return path (m, p);
  }
}

SI
composite_box_rep::sub_base () {
  int i, m=MAX_SI;
  for (i=0; i<N(bs); i++)
    m= min (m, bs[i]->y0+ bs[i]->sub_base ());
  return m;
}

SI
composite_box_rep::sup_base () {
  int i, m=-MAX_SI;
  for (i=0; i<N(bs); i++)
    m= max (m, bs[i]->y0+ bs[i]->sup_base ());
  return m;
}

/******************************************************************************
* User interface
******************************************************************************/

struct concrete_composite_box_rep: public composite_box_rep {
  concrete_composite_box_rep (array<box> bs, array<SI> x, array<SI> y):
    composite_box_rep (bs, x, y) {}
  operator tree () { return tree ("composite"); }
};

box
composite_box (array<box> bs, array<SI> x, array<SI> y) {
  return new concrete_composite_box_rep (bs, x, y);
}

#endmodule // code_composite_boxes
