
/******************************************************************************
* MODULE     : stack.gen.cc
* DESCRIPTION: Boxes stacked one above another
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <Boxes/composite.gen.h>

#module code_stack_boxes
#import composite_boxes

typedef array<int> array<SI>;

/*****************************************************************************/
// The stack_box representation
/*****************************************************************************/

struct stack_box_rep: public composite_box_rep {
  stack_box_rep (array<box> bs, array<SI> spc);
  operator tree () { return tree ("stack"); }

  void      position (array<SI> spc);

  virtual path      get_path (SI x, SI y, SI delta);
  virtual path      left_path ();
  virtual path      right_path ();
  virtual selection select (path start, path end);
};

/*****************************************************************************/
// Stack boxes
/*****************************************************************************/

void
stack_box_rep::position (array<SI> spc) {
  int i;
  if (N(bs)==0)
    fatal_error ("stack of zero boxes", "stack_box_rep::position");
  y1 = y2 = 0;
  for (i=0; i<N(bs); i++) {
    bs[i]->x0 = 0;
    bs[i]->y0 = y1- bs[i]->y2;
    y1 -= (bs[i]->h()+ spc[i]);
  }
  composite_box_rep::position ();
}

stack_box_rep::stack_box_rep (array<box> bs2, array<SI> spc) {
  bs = bs2;
  position (spc);
}

path
stack_box_rep::get_path (SI x, SI y, SI delta) {
  int i, h, n=N(bs);
  if (n<4) i=0;
  else {
    i= h= n>>1;
    while (h != 0) {
      box sb1= bs[i];
      box sb2= bs[i+1];
      SI yhi = sb1->y1+ sb1->y0;
      SI ylo = sb2->y2+ sb2->y0;
      if (h==1) h= 0;
      else h= (h+1) >> 1;
      if ((y < yhi) && (y < ylo)) i += h;
      else i -= max (h, 1);
      if (i < 0  ) i= 0;
      if (i > n-2) i= n-2;
    }
  }

  for (; i<n; i++) {
    if (i<n-1) {
      box sb1= bs[i];
      box sb2= bs[i+1];
      SI yhi = sb1->y1+ sb1->y0;
      SI ylo = sb2->y2+ sb2->y0;
      if ((y < yhi) && (y < ylo)) continue;
      if ((ylo <= yhi) ||
	  (N (sb1) != 1) || (N (sb2) != 1) ||
	  (N (sb1[0]) == 0) || (N (sb2[0]) == 0)) {
	if (y < ((ylo+yhi) >> 1)) continue;
      }
      else {
	int j;
	SI  m= sb2->y4+ sb2->y0, M= sb1->y3+ sb1->y0;
	box ssb1= sb1[0], ssb2= sb2[0];
	SI  ox1= sb1->x0+ ssb1->x0, ox2= sb2->x0+ ssb2->x0;
	SI  oy1= sb1->y0+ ssb1->y0, oy2= sb2->y0+ ssb2->y0;
	for (j=0; j<N(ssb1); j++)
	  if (!outside (x- ox1- ssb1[j]->x0, delta, ssb1[j]->x1, ssb1[j]->x2))
	    m= min (m, oy1+ ssb1[j]->y0+ ssb1[j]->y3);
	for (j=0; j<N(ssb2); j++)
	  if (!outside (x- ox2- ssb2[j]->x0, delta, ssb2[j]->x1, ssb2[j]->x2))
	    M= max (M, oy2+ ssb2[j]->y0+ ssb2[j]->y4);
	if (y < ((m+M) >> 1)) continue;
      }
    }
    path p= bs[i]->var_get_path (x-bs[i]->x0, y-bs[i]->y0, delta);
    return path (i, p);
  }

  return std_path (x, y, delta);
}

path
stack_box_rep::left_path () {
  if (N(bs)==0) return path (0);
  return path (0, bs[0]->left_path());
}

path
stack_box_rep::right_path () {
  if (N(bs)==0) return path (1);
  return path (N(bs)-1, bs[N(bs)-1]->right_path());
}

/******************************************************************************
* Making selections
******************************************************************************/

static rectangles
descend (rectangles l, SI y) {
  if (nil (l)) return l;
  rectangle& r (l->item);
  return rectangles (rectangle (r->x1, min (r->y1, y), r->x2, r->y2),
		     descend (l->next, y));
}

static rectangles
ascend (rectangles l, SI y) {
  if (nil (l)) return l;
  rectangle& r (l->item);
  return rectangles (rectangle (r->x1, r->y1, r->x2, max (r->y2, y)),
		     ascend (l->next, y));
}

static rectangles
extend_left (rectangles l, SI x) {
  if (nil (l)) return l;
  rectangle& r (l->item);
  return rectangles (rectangle (min (r->x1, x), r->y1, r->x2, r->y2),
		     extend_left (l->next, x));
}

static rectangles
extend_right (rectangles l, SI x) {
  if (nil (l)) return l;
  rectangle& r (l->item);
  return rectangles (rectangle (r->x1, r->y1, max (r->x2, x), r->y2),
		     extend_right (l->next, x));
}

selection
stack_box_rep::select (path start, path end) {
  if (N(bs)==0) return selection (rectangles (), start, end);
  if (atom (start) && (start == end))
    return selection (rectangles (), start, end);
  if ((!atom (start)) && (!atom (end)) && (start->item==end->item)) {
    selection sel= subbox (start->item)->select (start->next, end->next);
    return selection (translate (sel->rs, x0, y0),
		      path (start->item, sel->start),
		      path (end->item, sel->end));
  }
  else {
    int  i1 = atom (start)? 0: start->item;
    int  i2 = atom (end)? N(bs)-1: end->item;
    box  sb1= bs[i1];
    box  sb2= bs[i2];
    path sp1= atom (start)? sb1->left_path (): start->next;
    path sp2= atom (end)? sb2->right_path (): end->next;
    selection sel1= sb1->select (sp1, sb1->right_path ());
    selection sel2= sb2->select (sb2->left_path (), sp2);
    rectangles top= extend_right (descend (sel1->rs, sb1->y0+ sb1->y1), x2);
    rectangles bot= extend_left (ascend  (sel2->rs, sb2->y0+ sb2->y2), x1);
    rectangles rs = rectangle (x1, sb2->y0+sb2->y2, x2, sb1->y0+sb1->y1);
    rs << top << bot;
    path p1 (i1, sel1->start), p2 (i2, sel2->end);
    return selection (translate (rs, x0, y0), p1, p2);
  }
}

/*****************************************************************************/
// box construction routines
/*****************************************************************************/

box
stack_box (array<box> bs, array<SI> spc) {
  return new stack_box_rep (bs, spc);
}

#endmodule // code_stack_boxes
