
/******************************************************************************
* MODULE     : accented.gen.cc
* DESCRIPTION: accented boxes
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <Boxes/boxes.gen.h>
#include <server_font.gen.h>

#module code_accented_boxes
#import boxes
#import server_font

/*****************************************************************************/
// Text boxes
/*****************************************************************************/

struct accented_box_rep: public box_rep {
  string       str;
  string       acc;
  server_font  fn;
  color        col;

  accented_box_rep (string s, string acc, server_font fn, color col);
  operator tree () { return str; }

  void      display (ps_device dev);
  path      get_path (SI x, SI y, SI delta);
  cursor    get_cursor (path p);
  path      right_path ();
  selection select (path start, path end);
  double    left_slope ();
  double    right_slope ();
  SI        sub_base ();
  SI        sup_base ();
  SI        right_correction ();
  SI        right_sub_correction (double level);
  SI        right_sup_correction (double level);
};

/*****************************************************************************/
// Routines for accented boxes
/*****************************************************************************/

accented_box_rep::accented_box_rep (
  string s, string r, server_font fn2, color col2):
    str (s), acc (r), fn (fn2), col (col2)
{
  int i;
  fn->get_extents (str, x1, y1, x2, y2, x3, y3, x4, y4);
  for (i=0; i<N(acc); i++)
    if (acc[i] != ' ') {
      SI x , y ;
      SI a1, b1, a2, b2, a3, b3, a4, b4;
      SI c1, d1, c2, d2, c3, d3, c4, d4;
      fn->get_extents (str(0,i+1), a1, b1, x , b2, a3, b3, a4, b4);
      fn->get_extents (str[i]    , a1, b1, a2, b2, a3, b3, a4, b4);
      fn->get_extents (acc[i]    , c1, d1, c2, d2, c3, d3, c4, d4);
      x -= (((a2-a1)+ (c2-c1)) >> 1);
      y  = b4- fn->ysup;
      if (acc[i] == 24) y= PIXEL;
      else x += (SI) (((double) y) * fn->slope);
      x3 = min (x3, x + c3);
      y3 = min (y3, y + d3);
      x4 = max (x4, x + c4);
      y4 = max (y4, y + d4);
    }
}

path
accented_box_rep::right_path () {
  return path (N(str));
}

void
accented_box_rep::display (ps_device dev) {
  int i;
  dev->set_color (col);
  dev->text (str, fn, 0, 0);
  for (i=0; i<N(acc); i++)
    if (acc[i] != ' ') {
      SI x , y ;
      SI a1, b1, a2, b2, a3, b3, a4, b4;
      SI c1, d1, c2, d2, c3, d3, c4, d4;
      fn->get_extents (str(0,i+1), a1, b1, x , b2, a3, b3, a4, b4);
      fn->get_extents (str[i]    , a1, b1, a2, b2, a3, b3, a4, b4);
      fn->get_extents (acc[i]    , c1, d1, c2, d2, c3, d3, c4, d4);
      x -= (((a2-a1)+ (c2-c1)) >> 1);
      y  = b4- fn->ysup;
      if (acc[i] == 24) y= PIXEL;
      else x += (SI) (((double) y) * fn->slope);
      dev->text (acc[i], fn, x, y);
    }
}

path
accented_box_rep::get_path (SI x, SI y, SI delta) { (void) y;
  int i, minx, maxx, d1, d2, d3, d4, d5, d6, d7;
  minx=0;
  for (i=1; i<=N(str); i++) {
    fn->get_extents (str (0, i), d1, d2, maxx, d3, d4, d5, d6, d7);
    int m= (minx+maxx)>>1;
    if ((x<m) || ((x==m) && (delta<0))) break;
    minx=maxx;
  }
  return path (i-1);
}

cursor
accented_box_rep::get_cursor (path p) {
  SI d1, d2, d3, d4, d5;
  cursor cu (0, 0);
  fn->get_extents (str(0,p->item), d1, cu->y1, cu->x0, cu->y2, d2, d3, d4, d5);
  cu->x1= (SI) (cu->y1* fn->slope);
  cu->x2= (SI) (cu->y2* fn->slope);
  return cu;
}

selection
accented_box_rep::select (path start, path end) {
  SI x1, y1, x2, y2, d1, d2, d3, d4, d5;
  fn->get_extents (str(0,start->item), d1, y1, x1, y2, d2, d3, d4, d5);
  fn->get_extents (str(0,end->item), d1, y1, x2, y2, d2, d3, d4, d5);
  return selection (rectangle (x0+x1, y0+y1, x0+x2, y0+y2), start, end);
}

double accented_box_rep::left_slope () { return fn->slope; }
double accented_box_rep::right_slope () { return fn->slope; }
SI accented_box_rep::sub_base () { return fn->ysub; }
SI accented_box_rep::sup_base () { return fn->ysup; }
SI accented_box_rep::right_sub_correction (double level) { (void) level;
  return - box_rep::left_correction (); }
SI accented_box_rep::right_sup_correction (double level) { (void) level;
  return right_correction ()- box_rep::right_correction (); }
SI accented_box_rep::right_correction () { return fn->right_correction (str); }

/******************************************************************************
* Exported routines
******************************************************************************/

box
accented_box (string s, string acc, server_font fn, color col) {
  return new accented_box_rep (s, acc, fn, col);
}

#endmodule // code_accented_boxes
