
/******************************************************************************
* MODULE     : bigop.gen.cc
* DESCRIPTION: Big operators for TeX fonts
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#module code_bigop_boxes

/******************************************************************************
* Big operator boxes
******************************************************************************/

struct bigop_box_rep: public box_rep {
  unsigned char c;
  int           n;
  server_font   fn;
  color         col;
  double        slope;

  bigop_box_rep (unsigned char c, int n, server_font fn, color col);
  operator tree () { return tree ("bigop", string (c)); }
  void display (ps_device dev);
  SI right_correction ();
  SI right_sub_correction (double level);
  SI right_sup_correction (double level);
  double left_slope ();
  double right_slope ();
  cursor get_cursor (path p);
};

/******************************************************************************
* Routines for big operator boxes
******************************************************************************/

bigop_box_rep::bigop_box_rep (unsigned char C, int N, server_font F, color CL):
  c (C), n (N), fn (F), col (CL)
{
  fn->get_extents (c, n, x1, y1, x2, y2, x3, y3, x4, y4);
  if (right_correction() == 0) slope= 0.0;
  else {
    slope= 0.25;
    SI extra= 2*fn->wpt;
    x1+=extra; x2+=extra;
  }
}

void
bigop_box_rep::display (ps_device dev) {
  dev->set_color (col);
  dev->text (c, n, fn, 0, y2);
}

cursor
bigop_box_rep::get_cursor (path p) {
  cursor cu (0, 0);
  cu->x0= (p->item==0)? x1: x2;
  cu->y1= y1; cu->y2= y2;
  cu->x1= (SI) (cu->y1* slope);
  cu->x2= (SI) (cu->y2* slope);
  return cu;
}

SI bigop_box_rep::right_correction () {
  return fn->right_correction (c, n); }
SI bigop_box_rep::right_sub_correction (double level) { (void) level;
  return 0; } // - box_rep::left_correction (); }
SI bigop_box_rep::right_sup_correction (double level) { (void) level;
  return right_correction ()- box_rep::right_correction (); }
double bigop_box_rep::left_slope () {
  return slope; }
double bigop_box_rep::right_slope () {
  return slope; }

/******************************************************************************
* Exported routine
******************************************************************************/

box
bigop_box (unsigned char c, int n, server_font fn, color col) {
  return new bigop_box_rep (c, n, fn, col);
}

#endmodule // code_bigop_boxes
