
/******************************************************************************
* MODULE     : delim.gen.cc
* DESCRIPTION: Large delimiter boxes
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#module code_delim_boxes

/******************************************************************************
* Large delimiter boxes
******************************************************************************/

struct delim_box_rep: public box_rep {
  unsigned char c;
  int           n;
  server_font   fn;
  color         col;

  delim_box_rep (unsigned char c, int n, server_font fn, SI dy, color col);
  operator tree () { return tree ("delim", string (c)); }
  void display (ps_device dev);
  SI   sub_base ();
  SI   sup_base ();
  SI   left_sub_correction (double level);
  SI   left_sup_correction (double level);
  SI   right_sub_correction (double level);
  SI   right_sup_correction (double level);
};

/******************************************************************************
* Routines for large delimiter boxes
******************************************************************************/

delim_box_rep::delim_box_rep (unsigned char C, int N, server_font FN,
			      SI dy, color col2):
  c (C), n (N), fn (FN), col (col2)
{
  fn->get_extents (c, n, x1, y1, x2, y2, x3, y3, x4, y4);
  y1 += dy; y2 += dy; y3 += dy; y4 += dy;
}

void
delim_box_rep::display (ps_device dev) {
  dev->set_color (col);
  dev->text (c, n, fn, 0, y2);
}

SI delim_box_rep::sub_base () { return y3+ fn->wpt; }
SI delim_box_rep::sup_base () { return y4- fn->wpt; }
SI delim_box_rep::left_sub_correction (double level) { (void) level;
  return (3*(x1-x3)) >> 2; }
SI delim_box_rep::left_sup_correction (double level) { (void) level;
  return (3*(x1-x3)) >> 2; }
SI delim_box_rep::right_sub_correction (double level) { (void) level;
  return (3*(x4-x2)) >> 2; }
SI delim_box_rep::right_sup_correction (double level) { (void) level;
  return (3*(x4-x2)) >> 2; }

/******************************************************************************
* Exported routine
******************************************************************************/

box
delim_box (unsigned char c, int n, server_font fn, SI dy, color col) {
  return new delim_box_rep (c, n, fn, dy, col);
}

#endmodule // code_delim_boxes
