
/******************************************************************************
* MODULE     : boxes.gen.h
* DESCRIPTION: the low level box structure
*              and formatting routines
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <basic.gen.h>
#include <rectangles.gen.h>
#include <path.gen.h>
#include <ps_device.gen.h>

#module boxes
#import basic
#import path
#import ps_device
#import rectangles

#define MAX_SI 0x7fffffff
#define MIN_SI 0x80000000

/******************************************************************************
* The cursor class
******************************************************************************/

struct cursor_rep: concrete_struct {
  SI x0, y0; // main cursor position
  SI delta;  // infinitesimal shift to the right
  SI x1, y1; // under left corner relative to (x0, y0)
  SI x2, y2; // upper right corner relative to (x0, y0)
};

struct cursor {
#import concrete (cursor, cursor_rep)
  cursor (SI x=0, SI y=0, SI delta=0, SI x1=0, SI y1=0, SI x2=0, SI y2=0);
};
#import code_concrete (cursor, cursor_rep)

cursor copy (cursor cu);

/******************************************************************************
* The selection class
******************************************************************************/

struct selection_rep: concrete_struct {
  rectangles rs;
  path start;
  path end;
};

struct selection {
#import concrete (selection, selection_rep)
  selection (rectangles rs= rectangles(),
	     path start= path(), path end= path ());
};
#import code_concrete (selection, selection_rep)

/******************************************************************************
* The box class
******************************************************************************/

class box_rep;

class box {
#import abstract_null (box, box_rep)
  inline box operator [] (int i);
  operator tree ();
  bool operator == (box b2);
  bool operator != (box b2);
  friend inline int N (box b);
};

class box_rep: public abstract_struct {
public:
  SI x0, y0;  // offset w.r.t. parent box
  SI x1, y1;  // under left corner (logical)
  SI x2, y2;  // upper right corner (logical)
  SI x3, y3;  // under left corner (ink)
  SI x4, y4;  // upper right corner (ink)

  inline            box_rep ();
  inline            virtual ~box_rep ();
  virtual operator  tree () = 0;

  virtual void      display (ps_device dev) = 0;
  virtual int       subnr ();
  virtual box       subbox (int i);
  virtual path      encode (path p);
  virtual path      decode (path p);
  virtual path      get_path (SI x, SI y, SI delta);
  virtual cursor    get_cursor (path p);
  virtual path      left_path ();
  virtual path      right_path ();
  virtual selection select (path start, path end);

  virtual double    left_slope ();
  virtual double    right_slope ();
  virtual SI        left_correction ();
  virtual SI        right_correction ();
  virtual SI        left_sub_correction (double level);
  virtual SI        left_sup_correction (double level);
  virtual SI        right_sub_correction (double level);
  virtual SI        right_sup_correction (double level);
  virtual SI        sub_base ();
  virtual SI        sup_base ();

  inline SI         w () { return x2-x1; }
  inline SI         h () { return y2-y1; }

  inline bool       test_in (SI x, SI y);
         path       std_path (SI x, SI y, SI delta);
         cursor     std_cursor (path p);
         path       var_get_path (SI x, SI y, SI delta);
         cursor     var_get_cursor (path p);

         void       redraw (ps_device dev, path p, rectangles& l);
};

#import code_abstract_null (box, box_rep)
extern int box_count;
inline box_rep::box_rep ():
  x0(0), y0(0), x1(0), y1(0), x2(0), y2(0), x3(0), y3(0), x4(0), y4(0) {
    DEBUG(box_count++); }
inline box_rep::~box_rep () { DEBUG(box_count--); }
inline box box::operator [] (int i) { return rep->subbox(i); }
inline bool box_rep::test_in (SI x, SI y) {
  return (x>=x1) && (x<x2) && (y>=y1) && (y<y2); }
inline int N (box b) { return b.rep->subnr(); }
ostream& operator << (ostream& out, box b);

SI   get_delta (SI x, SI x1, SI x2);
SI   distance (box b, SI x, SI y, SI delta);
bool outside (SI x, SI delta, SI x1, SI x2);

#endmodule // boxes
