
/******************************************************************************
* MODULE     : construct.gen.h
* DESCRIPTION: the exported box construction routines
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <Boxes/boxes.gen.h>
#include <array.gen.h>

#module construct_boxes
#import boxes
#import array (box)
#import array (int)
#import array (string)

typedef array<int> array<SI>;
typedef array<int> array<color>;

/******************************************************************************
* Construction routines for boxes
******************************************************************************/

box empty_box (int x1=0, int y1=0, int x2=0, int y2=0);
box test_box ();
box line_box (SI x1, SI y1, SI x2, SI y2, SI w, color c);
box postscript_box (string fname, SI w, SI h, int x1, int y1, int x2, int y2);
box text_box (string s, server_font fn, color col);
box accented_box (string s, string acc, server_font fn, color col);
box delim_box (unsigned char c, int n, server_font fn, SI y, color);
box bigop_box (unsigned char c, int n, server_font fn, color col);
box bracket_box (int br_type, SI penw, color col, SI y1, SI y2);
box image_box (string fname, SI w, SI h, double mw, double mh,
	       int x1, int y1, int x2, int y2);

box concat_box (array<box> bs, array<SI> spc);
box stack_box (array<box> bs, array<SI> spc);
box composite_box (array<box> bs, array<SI> x, array<SI> y);
box matrix_box (int col, int row, array<box> bs, SI hspace, SI vspace, SI h);
box table_box (int col, int row, array<box> bs,
	       SI hspace, SI vspace, SI h, color c);
box mosaic_item_box (box b);
box mosaic_box (int cols, int rows, array<box> b, array<string> pos,
		array<int> nr_x, array<int> nr_y, SI h, array<color> col);

box frac_box (box b1, box b2, SI bar_w, SI bar_sep, SI bar_h, color c);
box sqrt_box (box b1, box b2, box sqrtb, SI wsqrt, color c);
box neg_box (box b, SI wneg, SI delta, color c);
box tree_box (array<box> bs, SI hsep, SI vsep, SI line_w, color line_c);
box limit_box (box ref, box lo, box hi);
box wide_box (box ref, box hi, SI default_y);
box script_box (box b1, box b2);
box prime_box (box b);
box left_script_box (box ref, box b1, box b2, double level);
box right_script_box (box ref, box b1, box b2, double level);
box side_box (box ref, box l1, box l2, box r1, box r2, double level);

box macro_box (box b);
box move_box (box b, SI x, SI y, bool child_flag= FALSE);
box resize_box (box b, SI x1, SI y1, SI x2, SI y2, bool child_flag= FALSE);

box group_box (box b);
box symbol_box (box b, int n);
box error_box (box b, int n);
box shorter_box (box b, int n);

box graphics_box (array<box> bs, SI x1, SI y1, SI x2, SI y2);

#endmodule // construct_boxes
