
/******************************************************************************
* MODULE     : env.gen.cc
* DESCRIPTION: the environment of the math_editor
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <analyze.gen.h>
#include <hashmap.gen.h>
#include <env.gen.h>
#include <PsDevice/page_type.gen.h>

#module code_env
#import analyze
#import env
#import page_type

#define TER edit_env_rep

/******************************************************************************
* Important environment variables and actions
******************************************************************************/

string DPI ("dpi");
string SFACTOR ("shrinking factor");
string MAGNIFICATION ("magnification");

string PREAMBLE ("preamble");
string MODE ("mode");
string TEXT_LANGUAGE ("language");
string TEXT_FONT ("font");
string TEXT_FAMILY ("font family");
string TEXT_SHAPE ("font shape");
string TEXT_SERIES ("font series");
string MATH_LANGUAGE ("math language");
string MATH_FONT ("math font");
string MATH_FAMILY ("math font family");
string MATH_SHAPE ("math font shape");
string MATH_SERIES ("math font series");
string PROG_LANGUAGE ("prog language");
string PROG_FONT ("prog font");
string PROG_FAMILY ("prog font family");
string PROG_SHAPE ("prog font shape");
string PROG_SERIES ("prog font series");
string FONT_BASE_SIZE ("font base size");
string FONT_SIZE ("font size");
string INDEX_LEVEL ("index level");
string DISPLAY_STYLE ("formula style");
string MATH_CONDENSED ("math condensed");
string VERTICAL_POS ("vertical position");
string COLOR ("color");

/******************************************************************************
* Environment variables for paragraphs
******************************************************************************/

string PAR_MODE ("paragraph mode");
string PAR_HYPHEN ("paragraph hyphenation");
string PAR_WIDTH ("paragraph width");
string PAR_LEFT ("left margin");
string PAR_RIGHT ("right margin");
string PAR_FIRST ("first indentation");
string PAR_LAST ("last indentation");
string PAR_NO_FIRST ("no first indentation");
string PAR_NO_LAST ("no last indentation");
string PAR_SEP ("interline space");
string PAR_HOR_SEP ("horizontal ink separation");
string PAR_LINE_SPC ("line stretch");
string PAR_PAR_SPC ("interparagraph space");

/******************************************************************************
* Environment variables for pages
******************************************************************************/

string PAGE_MEDIUM ("page medium");
string PAGE_TYPE ("page type");
string PAGE_ORIENTATION ("page orientation");
string PAGE_NR ("page number");
string PAGE_WIDTH ("page width");
string PAGE_HEIGHT ("page height");
string PAGE_ODD ("odd page margin");
string PAGE_EVEN ("even page margin");
string PAGE_RIGHT ("page right margin");
string PAGE_TOP ("page top margin");
string PAGE_BOT ("page bottom margin");
string PAGE_HEAD_SEP ("page header separation");
string PAGE_FOOT_SEP ("page footer separation");
string PAGE_ODD_HEADER ("odd page header");
string PAGE_ODD_FOOTER ("odd page footer");
string PAGE_EVEN_HEADER ("even page header");
string PAGE_EVEN_FOOTER ("even page footer");
string PAGE_THIS_HEADER ("this page header");
string PAGE_THIS_FOOTER ("this page footer");
string PAGE_REDUCE_LEFT ("reduction page left margin");
string PAGE_REDUCE_RIGHT ("reduction page right margin");
string PAGE_REDUCE_TOP ("reduction page top margin");
string PAGE_REDUCE_BOT ("reduction page bottom margin");
string PAGE_SHOW_HF ("show header and footer");

/******************************************************************************
* Initialization
******************************************************************************/

TER::TER (hashmap<string,tree>& local_ref2,
	  hashmap<string,tree>& global_ref2,
	  hashmap<string,tree>& local_aux2,
	  hashmap<string,tree>& global_aux2):
  dis (NULL), env (hashmap<string,tree> (tree ("?"))), var_type (Env_User),
  local_ref (local_ref2), global_ref (global_ref2),
  local_aux (local_aux2), global_aux (global_aux2)
{
  env (DPI)              = "600";       // resolution in dots per inch
  env (SFACTOR)          = "5";         // shrinking factor on screen
  env (MAGNIFICATION)    = "1";         // magnification (slides for instance)

  env (PREAMBLE)         = "false";     // preamble mode ?
  env (MODE)             = "text";      // typesetting mode
  env (TEXT_LANGUAGE)    = "english";   // the language
  env (TEXT_FONT)        = "roman";     // the font name in text mode
  env (TEXT_FAMILY)      = "rm";        // the font family in text mode
  env (TEXT_SERIES)      = "medium";    // the font series in text mode
  env (TEXT_SHAPE)       = "right";     // the font shape in text mode
  env (MATH_LANGUAGE)    = "texmath";   // the default mathematical language
  env (MATH_FONT)        = "roman";     // the font name in math mode
  env (MATH_FAMILY)      = "mr";        // the font family in math mode
  env (MATH_SERIES)      = "medium";    // the font series in math mode
  env (MATH_SHAPE)       = "normal";    // the font shape in math mode
  env (PROG_LANGUAGE)    = "edit-lisp"; // the default programming language
  env (PROG_FONT)        = "roman";     // the font name in prog mode
  env (PROG_FAMILY)      = "tt";        // the font family in prog mode
  env (PROG_SERIES)      = "medium";    // the font series in prog mode
  env (PROG_SHAPE)       = "right";     // the font shape in prog mode
  env (FONT_BASE_SIZE)   = "10";        // the font base size
  env (FONT_SIZE)        = "1";         // the font size multiplier
  env (INDEX_LEVEL)      = "0";         // the index level (0, 1 or 2)
  env (DISPLAY_STYLE)    = "false";     // true if we are in display style
  env (MATH_CONDENSED)   = "false";     // ignore spaces between operators ?
  env (VERTICAL_POS)     = "0";         // used in fractions (-1, 0 or 1)
  env (COLOR)            = "black";     // the color

  env (PAR_MODE)         = "justify";   // outline method
  env (PAR_HYPHEN)       = "normal";    // quality of hyphenation
  env (PAR_WIDTH)        = "15cm";      // width of paragraph
  env (PAR_LEFT)         = "0cm";       // left indentation
  env (PAR_RIGHT)        = "0cm";       // right indentation
  env (PAR_FIRST)        = "5mm";       // extra first indentation
  env (PAR_LAST)         = "0mm";       // extra last indentation
  env (PAR_NO_FIRST)     = "false";     // no extra first indent. on next line
  env (PAR_NO_LAST)      = "false";     // no extra last indent. on next line
  env (PAR_SEP)          = "0.2fn";     // min. vert. spc. between ink
  env (PAR_HOR_SEP)      = "0.5fn";     // min. hor. spc. between ink
  env (PAR_LINE_SPC)     = "0.025fn*";  // extra (small) space between lines
  env (PAR_PAR_SPC)      = "0.5fn*";    // extra space between paragraphs

  env (PAGE_MEDIUM)      = "papyrus";   // paper medium: paper, papyrus, auto
  env (PAGE_TYPE)        = "a4";        // paper type (-> width & height)
  env (PAGE_ORIENTATION) = "portrait";  // paper orientation
  env (PAGE_NR)          = "0";         // the page number
  env (PAGE_ODD)         = "30mm";      // left margin on odd pages
  env (PAGE_EVEN)        = "30mm";      // left margin on even pages
  env (PAGE_RIGHT)       = "30mm";      // right margin in auto mode
  env (PAGE_TOP)         = "30mm";      // top margin
  env (PAGE_BOT)         = "30mm";      // bottom margin
  env (PAGE_HEAD_SEP)    = "8mm";       // separation between header and text
  env (PAGE_FOOT_SEP)    = "8mm";       // separation between footer and text
  env (PAGE_ODD_HEADER)  = "";          // header on odd pages
  env (PAGE_ODD_FOOTER)  = "";          // footer on odd pages
  env (PAGE_EVEN_HEADER) = "";          // header on even pages
  env (PAGE_EVEN_FOOTER) = "";          // footer on even pages
  env (PAGE_REDUCE_LEFT) = "25mm";      // screen reduction of left margin
  env (PAGE_REDUCE_RIGHT)= "25mm";      // screen reduction of right margin
  env (PAGE_REDUCE_TOP)  = "15mm";      // screen reduction of top margin
  env (PAGE_REDUCE_BOT)  = "15mm";      // screen reduction of bottom margin
  env (PAGE_SHOW_HF)     = "false";     // show header and footer

  var_type (DPI)             = Env_Fixed;
  var_type (SFACTOR)         = Env_Fixed;
  var_type (MAGNIFICATION)   = Env_Magnification;
  var_type (PAGE_TYPE)       = Env_Fixed;
  var_type (PAGE_WIDTH)      = Env_Fixed;
  var_type (PAGE_HEIGHT)     = Env_Fixed;

  var_type (MODE)            = Env_Mode;
  var_type (TEXT_LANGUAGE)   = Env_Language;
  var_type (TEXT_FONT)       = Env_Font;
  var_type (TEXT_FAMILY)     = Env_Font;
  var_type (TEXT_SERIES)     = Env_Font;
  var_type (TEXT_SHAPE)      = Env_Font;
  var_type (MATH_LANGUAGE)   = Env_Language;
  var_type (MATH_FONT)       = Env_Font;
  var_type (MATH_FAMILY)     = Env_Font;
  var_type (MATH_SERIES)     = Env_Font;
  var_type (MATH_SHAPE)      = Env_Font;
  var_type (PROG_LANGUAGE)   = Env_Language;
  var_type (PROG_FONT)       = Env_Font;
  var_type (PROG_FAMILY)     = Env_Font;
  var_type (PROG_SERIES)     = Env_Font;
  var_type (PROG_SHAPE)      = Env_Font;
  var_type (FONT_BASE_SIZE)  = Env_Font_Size;
  var_type (FONT_SIZE)       = Env_Font_Size;
  var_type (INDEX_LEVEL)     = Env_Index_Level;
  var_type (DISPLAY_STYLE)   = Env_Display_Style;
  var_type (MATH_CONDENSED)  = Env_Math_Condensed;
  var_type (VERTICAL_POS)    = Env_Vertical_Pos;
  var_type (COLOR)           = Env_Color;

  var_type (PAR_MODE)        = Env_Paragraph;
  var_type (PAR_HYPHEN)      = Env_Paragraph;
  var_type (PAR_WIDTH)       = Env_Paragraph;
  var_type (PAR_LEFT)        = Env_Paragraph;
  var_type (PAR_RIGHT)       = Env_Paragraph;
  var_type (PAR_FIRST)       = Env_Paragraph;
  var_type (PAR_LAST)        = Env_Paragraph;
  var_type (PAR_NO_FIRST)    = Env_Paragraph;
  var_type (PAR_NO_LAST)     = Env_Paragraph;
  var_type (PAR_SEP)         = Env_Paragraph;
  var_type (PAR_HOR_SEP)     = Env_Paragraph;
  var_type (PAR_LINE_SPC)    = Env_Paragraph;
  var_type (PAR_PAR_SPC)     = Env_Paragraph;

  var_type (PAGE_ODD)        = Env_Page;
  var_type (PAGE_EVEN)       = Env_Page;
  var_type (PAGE_RIGHT)      = Env_Page;
  var_type (PAGE_TOP)        = Env_Page;
  var_type (PAGE_BOT)        = Env_Page;
  var_type (PAGE_HEAD_SEP)   = Env_Page;
  var_type (PAGE_FOOT_SEP)   = Env_Page;
  var_type (PAGE_ODD_HEADER) = Env_Page;
  var_type (PAGE_ODD_FOOTER) = Env_Page;
  var_type (PAGE_EVEN_HEADER)= Env_Page;
  var_type (PAGE_EVEN_FOOTER)= Env_Page;
  var_type (PAGE_THIS_HEADER)= Env_Page;
  var_type (PAGE_THIS_FOOTER)= Env_Page;

  complete= FALSE;
}

void
TER::set_display (display dis2) {
  dis= dis2;
  style_init_env ();
  update ();
}

void
TER::style_init_env () {
  dpi= get_int (DPI);
  string medium= get_string (PAGE_MEDIUM);
  string type  = get_string (PAGE_TYPE);
  bool   landsc= (get_string (PAGE_ORIENTATION) == "landscape");
  if ((medium != "automatic") && (type != "user")) {
    assign (PAGE_WIDTH , copy (page_get_feature (type, PAGE_WIDTH , landsc)));
    assign (PAGE_HEIGHT, copy (page_get_feature (type, PAGE_HEIGHT, landsc)));
  }
}

edit_env::edit_env (hashmap<string,tree>& local_ref,
		    hashmap<string,tree>& global_ref,
		    hashmap<string,tree>& local_aux,
		    hashmap<string,tree>& global_aux):
  rep (new TER (local_ref, global_ref, local_aux, global_aux)) {}

/******************************************************************************
* Evaluation of trees
******************************************************************************/

bool is_length (string s);

static tree
substitute (tree t, tree macro, tree args) {
  if (N(t) == 0) return t;
  if (is_func (t, APPLY, 1)) {
    int i, n= N(args)-1;
    for (i=0; i<n; i++)
      if (macro[i]==t[0])
	return copy (args[i+1]);
  }

  int i;
  tree r (t->label, N(t));
  for (i=0; i<N(t); i++)
    r[i]= substitute (t[i], macro, args);
  return r;
}

tree
TER::exec (tree t) {
  // cout << "Execute: " << t << "\n";
  if (get_bool (PREAMBLE) &&
      (((!is_func (t, SET)) &&
	(!is_func (t, RESET)) &&
	(!is_func (t, ASSIGN))) || (t[0] != MODE)))
    {
      int i;
      tree r (t->label, N(t));
      for (i=0; i<N(t); i++) r[i]= exec (t[i]);
      return r;
    }

  if (N(t)==0) return t;
  switch (math_primary[t->label]) {
  case Inactive:
    {
      int i;
      tree u= t[0];
      tree r (u->label, N(u));
      for (i=0; i<N(u); i++) r[i]= exec (u[i]);
      return tree (INACTIVE, u);
    }
  case Quote:
    return copy (t[0]);
  case Assign:
    {
      if (N(t)!=2) return "<error>";
      tree r= exec(t[0]);
      assign (r->label, t[1]);
      return tree (ASSIGN, r, tree (QUOTE, get (r->label)));
    }
  case Set:
    {
      if (N(t)!=2) return "<error>";
      // cout << "  Set " << t << "\n";
      tree r= exec(t[0]);
      // cout << "    Variable " << r << "\n";
      set (r->label, t[1]);
      // cout << "    Set " << r << " with " << get (r->label) << "\n";
      // cout << "    New value of " << r << " is " << env [r->label] << "\n";
      return tree (SET, r, tree (QUOTE, get (r->label)));
    }
  case Reset:
    {
      if (N(t)!=1) return "<error>";
      // cout << "  Reset " << t << "\n";
      tree r= exec(t[0]);
      // cout << "    Variable " << r << "\n";
      reset (r->label);
      // cout << "    New value of " << r << " is " << env [r->label] << "\n";
      return tree (RESET, r);
    }
  case Apply:
    {
      // cout << "  Apply " << t << "\n";
      tree x= exec (t[0]);
      tree f= is_func (x, FUNCTION)? x: get (x->label);
      // cout << "    Function " << f << "\n";
      if (is_func (f, FUNCTION)) {
	if (N(f) != N(t)) return "<error>";
	tree f_t= substitute (f[N(f)-1], f, t);
	// cout << "    Execute " << f_t << "\n";
	return exec (f_t);
      }
      else {
	if (N(t)==1) return f;
	else return "<error>";
      }
    }
  case Begin:
    {
      // cout << "  Begin " << t << "\n";
      tree x= exec (t[0]);
      tree e= is_func (x, ENVIRONMENT)? x: get (x->label);
      // cout << "    Environment " << e << "\n";
      if (is_func (e, ENVIRONMENT)) {
	if ((N(e)-1) != N(t)) return "<error>";
	tree e_t= substitute (e[N(e)-2], e, t);
	// cout << "    Execute " << e_t << "\n";
	return exec (e_t);
      }
      else return "<error>";
    }
  case End:
    {
      // cout << "  End " << t << "\n";
      tree x= exec (t[0]);
      tree e= is_func (x, ENVIRONMENT)? x: get (x->label);
      // cout << "    Environment " << e << "\n";
      if (is_func (e, ENVIRONMENT)) {
	if (N(t) != 1) return "<error>";
	return exec (e[N(e)-1]);
      }
      else return "<error>";
    }
  case Function:
  case Environment:
    return copy (t);
  case Plus:
    {
      if (N(t)!=2) return "<error>";
      string s1= exec (t[0])->label;
      string s2= exec (t[1])->label;
      if (is_double (s1) && is_double (s2))
	return as_string (as_double (s1)+ as_double (s2));
      if (is_length (s1) && is_length (s2))
	return add_lengths (s1, s2);
      return "<error>";
    }
  case Minus:
    {
      if (N(t)!=2) return "<error>";
      string s1= exec (t[0])->label;
      string s2= exec (t[1])->label;
      if (is_double (s1) && is_double (s2))
	return as_string (as_double (s1)- as_double (s2));
      if (is_length (s1) && is_length (s2))
	return add_lengths (s1, "-" * s2);
      return "<error>";
    }
  case Times:
    {
      if (N(t)!=2) return "<error>";
      string s1= exec (t[0])->label;
      string s2= exec (t[1])->label;
      if (is_double (s1) && (is_double (s2)))
	return as_string (as_double (s1)* as_double (s2));
      return "<error>";
    }
  case Merge:
    {
      if (N(t)!=2) return "<error>";
      tree t1= exec (t[0]);
      tree t2= exec (t[1]);
      if ((N(t1)==0) && (N(t2)==0)) return t1->label * t2->label;
      return "<error>";
    }
  case Number:
    {
      if (N(t)!=2) return "<error>";
      string s1= exec (t[0])->label;
      string s2= exec (t[1])->label;
      int nr= as_int (s1);
      if (s2 == "roman") return roman_nr (nr);
      if (s2 == "Roman") return Roman_nr (nr);
      if (s2 == "alpha") return alpha_nr (nr);
      if (s2 == "Alpha") return Alpha_nr (nr);
      return "<error>";
    }
  case Translate:
    {
      if (N(t)!=3) return "<error>";
      string s   = exec (t[0])->label;
      string from= exec (t[1])->label;
      string to  = exec (t[2])->label;
      return dis->translate (s, from, to);
    }
  default:
    {
      int i;
      // cout << "Executing " << t << "\n";
      tree r (t->label, N(t));
      for (i=0; i<N(t); i++) r[i]= exec (t[i]);
      // cout << "Executed " << t << " -> " << r << "\n";
      return r;
    }
  }
}

/******************************************************************************
* Modification of environment variables
******************************************************************************/

void
TER::set (string s, tree t) {
  // cout << "Set " << s << " := " << t << "\n";
  tree r= exec (t); // Replacing r by exec(t) below yields bug when optimizing
  // cout << "Do " << s << " := " << r << "\n";
  env(s)= tree (BACKUP, r, env[s]);
  update (s);
}

void
TER::reset (string s) {
  // cout << "Reset " << s << "\n";
  if (is_func (env[s], BACKUP, 2)) env(s)=env[s][1];
  update (s);
}

bool
TER::assign (string s, tree t) {
  // cout << "Assign " << s << " := " << t << "\n";
  t= exec (t);
  if (is_func (env[s], BACKUP, 2)) {
    if (env[s][0] == t) return FALSE;
    env(s)= tree (BACKUP, t, env[s][1]);
  }
  else {
    if (env[s] == t) return FALSE;
    env(s)= t;
  }
  update (s);
  return TRUE;
}

void TER::save () { env->extend (); }
void TER::recover () { env= env->next; update (); }
void TER::confirm () { env->merge (); }

void TER::script_size () { set (INDEX_LEVEL, as_string (index_level+1)); }
void TER::normal_size () { reset (INDEX_LEVEL); }

/******************************************************************************
* Retrieve environment variables
******************************************************************************/

tree
TER::get (string name) {
  tree t= env [name];
  if (is_func (t, BACKUP)) return t[0];
  return t;
}

bool
TER::get_bool (string name) {
  return as_bool (get (name)->label);
}

int
TER::get_int (string name) {
  return as_int (get (name)->label);
}

double
TER::get_double (string name) {
  return as_double (get (name)->label);
}

string
TER::get_string (string name) {
  return get (name)->label;
}

SI
TER::get_length (string name) {
  string s= get (name)->label;
  return decode_length (s);
}

/******************************************************************************
* Decoding and adding lengths
******************************************************************************/

SI
TER::decode_length (string s) {
  while ((N(s) >= 2) && (s[0]=='-') && (s[1]=='-')) s= s (2, N(s));

  int i;
  for (i=0; (i<N(s)) && ((s[i]<'a') || (s[i]>'z')); i++);
  string s1 = s(0,i);
  string s2 = s(i,N(s));
  double x  = as_double (s1);
  double in = ((double) dpi*PIXEL);
  double cm = in/2.54;
  double f  = (get_int(FONT_BASE_SIZE)*magn*in*get_double(FONT_SIZE))/72.0;

  if (s2 == "cm") { return (SI) (x*cm); }
  if (s2 == "mm") { return (SI) (x*cm/10.0); }
  if (s2 == "in") { return (SI) (x*in); }
  if (s2 == "pt") { return (SI) (x*in/72.0); }
  if (s2 == "spc") { return (SI) (x*fn->spc->def); }
  if (s2 == "spc-") { return (SI) (x*fn->spc->min); }
  if (s2 == "spc+") { return (SI) (x*fn->spc->max); }
  if (s2 == "fn") { return (SI) (x*f); }
  if (s2 == "fn-") { return 0; }
  if (s2 == "fn+") { return (SI) (2*x*f); }
  if (s2 == "fn*") { return 0; }
  if (s2 == "fn*-") { return 0; }
  if (s2 == "fn*+") { return (SI) (x*f); }
  if (s2 == "px") { return (SI) (x*PIXEL); }
  if (s2 == "unit") { return (SI) (x); }
  return 0;
}

string
TER::add_lengths (string s1, string s2) {
  SI l1= decode_length (s1);
  SI l2= decode_length (s2);
  int i;
  for (i=0; i<N(s1); i++)
    if ((s1[i]>='a') && (s1[i]<='z')) break;
  SI un= decode_length ("1" * s1 (i, N(s1)));
  if (un==0) return "0cm";
  double x= ((double) (l1+l2)) / ((double) un);
  return as_string (x) * s1 (i, N(s1));
}

bool
is_length (string s) {
  int i;
  for (i=0; (i<N(s)) && ((s[i]<'a') || (s[i]>'z')); i++);
  if (!is_double (s (0, i))) return FALSE;
  int j=N(s);
  while ((j>i) && ((s[j-1]=='+') || (s[j-1]=='-') || (s[j-1]=='*'))) j--;
  return is_alpha (s (i, j));
}

/******************************************************************************
* Retrieving the page size
******************************************************************************/

void
TER::get_page_pars (SI& w, SI& h, SI& width, SI& height,
		    SI& odd, SI& even, SI& top, SI& bot)
{
  SI right;
  SI rl= get_length (PAGE_REDUCE_LEFT);
  SI rr= get_length (PAGE_REDUCE_RIGHT);
  SI rt= get_length (PAGE_REDUCE_TOP);
  SI rb= get_length (PAGE_REDUCE_BOT);

  width = get_length (PAGE_WIDTH);
  height= get_length (PAGE_HEIGHT);
  odd   = get_length (PAGE_ODD);
  even  = get_length (PAGE_EVEN);
  right = get_length (PAGE_RIGHT);
  top   = get_length (PAGE_TOP);
  bot   = get_length (PAGE_BOT);

  string medium= get_string (PAGE_MEDIUM);
  if (medium != "automatic") {
    width  -= rl+rr;
    height -= rt+rb;
    odd    -= rl;
    even   -= rl;
    top    -= rt;
    bot    -= rb;

    w= get_length (PAR_WIDTH);
    h= height- top- bot;
  }
  else {
    w= width- odd- right;
    h= height- top- bot;
  }

  /*
  cout << "w     = " << (w/PIXEL) << "\n";
  cout << "h     = " << (h/PIXEL) << "\n";
  cout << "width = " << (width/PIXEL) << "\n";
  cout << "height= " << (height/PIXEL) << "\n";
  cout << "odd   = " << (odd/PIXEL) << "\n";
  cout << "even  = " << (even/PIXEL) << "\n";
  cout << "right = " << (right/PIXEL) << "\n";
  cout << "top   = " << (top/PIXEL) << "\n";
  cout << "bot   = " << (bot/PIXEL) << "\n";
  */
}

/******************************************************************************
* Updating the environment from the variables
******************************************************************************/

void
TER::update_font () {
  if (dis==NULL) fatal_error ("no display yet", "env_rep::update_font");
  fn_size= (int) ((((double) get_int (FONT_BASE_SIZE))+0.5)*
		  get_double (FONT_SIZE));
  switch (mode) {
  case 0:
    fn= find_font (dis,
		   get_string (TEXT_FONT), get_string (TEXT_FAMILY),
		   get_string (TEXT_SERIES), get_string (TEXT_SHAPE),
		   script (fn_size, index_level), (int) (magn*dpi));
    break;
  case 1:
    fn= find_font (dis,
		   get_string (MATH_FONT), get_string (MATH_FAMILY),
		   get_string (MATH_SERIES), get_string (MATH_SHAPE),
		   script (fn_size, index_level), (int) (magn*dpi));
    break;
  case 2:
    fn= find_font (dis,
		   get_string (PROG_FONT), get_string (PROG_FAMILY),
		   get_string (PROG_SERIES), get_string (PROG_SHAPE),
		   script (fn_size, index_level), (int) (magn*dpi));
    break;
  }
}

void
TER::update_color () {
  string s= get_string (COLOR);
  if (s == "black")      col= dis->black;
  if (s == "white")      col= dis->white;
  if (s == "grey")       col= dis->grey;
  if (s == "red")        col= dis->red;
  if (s == "blue")       col= dis->blue;
  if (s == "yellow")     col= dis->yellow;
  if (s == "green")      col= dis->green;
  if (s == "magenta")    col= dis->magenta;
  if (s == "orange")     col= dis->orange;
  if (s == "brown")      col= dis->brown;
  if (s == "pink")       col= dis->pink;
  if (s == "dark green") col= dis->rgb (0, 128, 0);
}

void
TER::update_mode () {
  string s= get_string (MODE);
  if (s == "text") mode=0;
  if (s == "math") mode=1;
  if (s == "prog") mode=2;
}

void
TER::update_language () {
  switch (mode) {
  case 0:
    lan= text_language (get_string (TEXT_LANGUAGE));
    break;
  case 1:
    lan= math_language (get_string (MATH_LANGUAGE));
    break;
  case 2:
    lan= prog_language (get_string (PROG_LANGUAGE));
    break;
  }
}

void
TER::update () {
  magn           = get_double (MAGNIFICATION);
  index_level    = get_int (INDEX_LEVEL);
  display_style  = get_bool (DISPLAY_STYLE);
  math_condensed = get_bool (MATH_CONDENSED);
  vert_pos       = get_int (VERTICAL_POS);

  update_color ();
  update_mode ();
  update_language ();
  update_font ();
}

/******************************************************************************
* Update a particular changed variable
******************************************************************************/

void
TER::update (string s) {
  switch (var_type[s]) {
  case Env_User:
    break;
  case Env_Fixed:
    break;
  case Env_Magnification:
    magn= get_double (MAGNIFICATION);
    update_font ();
    break;
  case Env_Language:
    update_language ();
    break;
  case Env_Mode:
    update_mode ();
    update_language ();
    update_font ();
    break;
  case Env_Font:
    update_font ();
    break;
  case Env_Font_Size:
    update_font ();
    break;
  case Env_Index_Level:
    index_level= get_int (INDEX_LEVEL);
    update_font ();
    break;
  case Env_Display_Style:
    display_style= get_bool (DISPLAY_STYLE);
    break;
  case Env_Math_Condensed:
    math_condensed= get_bool (MATH_CONDENSED);
    break;
  case Env_Vertical_Pos:
    vert_pos= get_int (VERTICAL_POS);
    break;
  case Env_Color:
    update_color ();
    break;
  case Env_Paragraph:
    break;
  case Env_Page:
    break;
  }
}

#undef TER

#endmodule // code_env
