
/******************************************************************************
* MODULE     : document.gen.cc
* DESCRIPTION: The result of typesetting a paragraph is
*              an instance of the paragraph class
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <Document/document.gen.h>

#module code_document
#import document

static rectangle zr (0, 0, 0, 0);

/******************************************************************************
* Initialization and notification of changes in the tree
******************************************************************************/

document_rep::document_rep (tree& et2, edit_env& env2):
  et (et2), env (env2), p (N(et)), r (0, 0, 0, 0) {}

void
document_rep::notify_modify (int pos) {
  p[pos]->status= 1;
}

void
document_rep::notify_assign (int nr) {
  int i;
  for (i=0; i<N(et); i++) {
    rectangles rs;
    if (r != zr) rs= rectangles (r);
    if (p[i]->r != zr) rs= rectangles (p[i]->r, rs);
    if (!nil (rs)) r= least_upper_bound (rs);
  }
  p= array<paragraph> (nr);
}

void
document_rep::notify_insert (int pos, int nr) {
  int i;
  array<paragraph> p2 (N(p)+nr);
  for (i=0; i<pos; i++) p2[i]= p[i];
  for (; i<N(p); i++) p2[i+nr]= p[i];
  p= p2;
}

void
document_rep::notify_remove (int pos, int nr) {
  int i;
  for (i=pos; i<pos+nr; i++) {
    rectangles rs;
    if (r != zr) rs= rectangles (r);
    if (p[i]->r != zr) rs= rectangles (p[i]->r, rs);
    if (!nil (rs)) r= least_upper_bound (rs);
  }

  array<paragraph> p2 (N(p)-nr);
  for (i=0; i<pos; i++) p2[i]= p[i];
  for (; i<N(p2); i++) p2[i]= p[i+nr];
  p= p2;
}

/******************************************************************************
* Getting environment variables
******************************************************************************/

void
document_rep::change_env (tree t, path tp) {
  if (nil (tp)) return;
  if (atom (tp)) {
    if (tp->item!=0)
      (void) env->exec (t);
  }
  else {
    int i;
    for (i=0; i<tp->item; i++)
      (void) env->exec (t[i]);
    change_env (t[tp->item], tp->next);
  }
}

tree
document_rep::get_env (string var, path tp) {
  int i;
  if (tp == path (0)) tp= path (0, tp);
  if (tp == path (1)) tp= path (N(et)-1, tp);
  env->save ();
  for (i=0; i<tp->item; i++)
    if (p[i]->status < 2) env->exec (et[i]);
    else env->env->change (p[i]->change);
  change_env (et[i], tp->next);
  tree t= env->get (var);
  env->recover ();
  return t;
}

/******************************************************************************
* Typesetting paragraphs
******************************************************************************/

void
document_rep::typeset_paragraph (int nr) {
  liner L= new liner_rep (env);
  L->typeset_3_passes (et[nr]);
  env->env->next->find_changes (env->env->item);
  p[nr]= L->par;
  p[nr]->status= 2;
  delete L;
}

box
document_rep::typeset (SI& x1, SI& y1, SI& x2, SI& y2) {
  if (N(p) != N(et)) {
    int i;
    cerr << "\nLines of et:\n";
    for (i=0; i<N(et); i++) cout << "  " << i << ":\t" << et[i] << "\n";
    cerr << "Does not match number " << N(p) << " of paragraphs\n";
    fatal_error ("mismatch in number of paragraphs", "document_rep::typeset");
  }

  int i, n= N(p);
  rectangle old_r[n];
  int old_status[n];
  env->complete= TRUE;
  for (i=0; i<n; i++) {
    old_r[i]     = p[i]->r;
    old_status[i]= p[i]->status;
    env->complete= env->complete && (p[i]->status==1);
  }
  if (env->complete) env->local_aux= hashmap<string,tree> ("?");

  pager P= new pager_rep (env);
  hashmap<string,tree> old ("?");
  rel_hashmap<string,tree> rel_old (old);
  env->save ();
  for (i=0; i<n; i++) {
    // cout << "---------------------------- " << i << " -----------------------------------\n";
    // cout << "env: " << env->env->item << "\n";
    if ((p[i]->status == 2) && (!old->empty ())) {
      old_status[i]= 1;
      p[i]->status = 1;
    }
    if (p[i]->status < 2) {
      rel_old->change (p[i]->change);
      env->save ();
      typeset_paragraph (i);
      p[i]->change= env->env->item;
      env->env->find_differences (old);
      env->confirm ();
    }
    else {
      env->env->change (p[i]->change);
      env->update ();
    }
    P->insert_paragraph (p[i], i, FALSE);
    // P->insert_paragraph (p[i], i, (i<n-1) && is_func (et[i][0], "", 0));
    // cout << "change: " << p[i]->change << "\n";
    // cout << "old: " << old << "\n";
  }
  env->recover ();
  env->save ();
  box b= P->make_pages ();
  env->recover ();
  delete P;
  
  env->complete= FALSE;
  for (i=0; i<n; i++)
    if ((old_status[i]<2) || (old_r[i] != p[i]->r)) {
      rectangles rs;
      if (r != zr) rs= rectangles (r);
      if (p[i]->r != zr) rs= rectangles (p[i]->r, rs);
      if ((old_status[i]>=1) && (old_r[i]!=zr)) rs= rectangles (old_r[i], rs);
      if (!nil (rs)) r= least_upper_bound (rs);
    }
  x1= r->x1; y1= r->y1; x2= r->x2; y2= r->y2;
  r= rectangle (0, 0, 0, 0);
  return b;
}

#endmodule // document
