
/******************************************************************************
* MODULE     : page_box.gen.cc
* DESCRIPTION: boxes for typesetting lines into paragraphs
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <Boxes/construct.gen.h>
#include <Boxes/composite.gen.h>
#include <Boxes/modifier.gen.h>

#module code_page_boxes
#import construct_boxes
#import composite_boxes
#import modifier_boxes
#import array (path)
#import array (int)

int find_index (path& p, array<path> P, array<path> Q, array<int> F);

/******************************************************************************
* Once a paragraph has been typeset, a lines_box is returned
******************************************************************************/

struct lines_box_rep: modifier_box_rep {
  array<path> P; // start path for each line in paragraph
  array<path> Q; // end path for each line in paragraph
  array<int>  F; // border flags

  lines_box_rep (array<box> bs, array<SI> spc, 
		 array<path> P, array<path> Q, array<int> F);
  path encode (path p);
  path decode (path p);
};

lines_box_rep::lines_box_rep (array<box> B, array<SI> S,
  array<path> P2, array<path> Q2, array<int> F2):
  modifier_box_rep (stack_box (B, S)), P (P2), Q (Q2), F (F2) {}

path
lines_box_rep::encode (path p) {
  // cout << "Paragraph encode " << p << "\n";
  if (atom (p) || atom (p->next))
    fatal_error ("invalid path", "lines_box_rep::encode");
  int par_num= P[p->next->item]->item;
  // cout << "---> " << path (par_num, p->next->next) << "\n";
  return path (par_num, p->next->next);
}

path
lines_box_rep::decode (path p) {
  // cout << "Paragraph decode " << p << "\n";
  // cout << "  P= " << P << "\n";
  // cout << "  Q= " << Q << "\n";
  // cout << "  F= " << F << "\n";
  int i= find_index (p, P, Q, F);
  if (i!=-1) {
    // cout << "---> " << path (0, path (i, p->next->next)) << "\n";
    return path (0, path (i, p->next));
  }

  // handle selections starting or ending at invalid borders
  if (p==P[0]) return path (0, 0);
  if (p==Q[N(Q)-1]) return path (0, 1);

  cerr << "\n";
  cerr << "Error] Searching for " << p << "\n";
  cerr << "Error]   start paths " << P << "\n";
  cerr << "Error]   end paths " << Q << "\n";
  cerr << "Error]   border flags " << F << "\n";
  fatal_error ("invalid situation", "paragraph_rep_box::decode");
}

/******************************************************************************
* A page box contains a main lines_box and decorations
******************************************************************************/

struct page_box_rep: composite_box_rep {
  SI  width, height, left, top, bot;
  box header, footer;
  SI  head_sep, foot_sep;

  page_box_rep (box b, SI width, SI height, SI left, SI top, SI bot,
		box header, box footer, SI head_sep, SI foot_sep);
  operator tree ();
  path get_path (SI x, SI y, SI delta);
  path left_path ();
  path right_path ();
  void display (ps_device dev);
};

page_box_rep::page_box_rep (box b, SI width2, SI height2,
			    SI left2, SI top2, SI bot2,
			    box header2, box footer2, SI head2, SI foot2):
  width (width2), height (height2), left (left2), top (top2), bot (bot2),
  header (header2), footer (footer2), head_sep (head2), foot_sep (foot2)
{
  insert (b, left, -top);
  position ();
  header->x0= left;
  header->y0= -top- header->y1+ head_sep;
  footer->x0= left;
  footer->y0= -bot- footer->y2- foot_sep;

  x1= 0;
  x2= width;
  y1= -height;
  y2= 0;
  x3= min (x3, min (header->x0+ header->x3, footer->x0+ footer->x3));
  x4= max (x4, max (header->x0+ header->x4, footer->x0+ footer->x4));
  y3= min (y3, min (header->y0+ header->y3, footer->y0+ footer->y3));
  y4= max (y4, max (header->y0+ header->y4, footer->y0+ footer->y4));
}

page_box_rep::operator tree () { return tree ("page", (tree) bs[0]); }
path page_box_rep::get_path (SI x, SI y, SI delta) {
  return path (0, bs[0]->var_get_path (x- bs[0]->x0, y- bs[0]->y0, delta)); }
path page_box_rep::left_path () { return path (0, bs[0]->left_path ()); }
path page_box_rep::right_path () { return path (0, bs[0]->right_path ()); }

void
page_box_rep::display (ps_device dev) {
  rectangles rs;
  header->redraw (dev, path (), rs);
  footer->redraw (dev, path (), rs);
}

/******************************************************************************
* Once all pages have been typeset, a pages_box is returned
******************************************************************************/

struct pages_box_rep: modifier_box_rep {
  array<path> P; // start path for each page in document
  array<path> Q; // end path for each page in document
  array<int>  F; // border flags

  pages_box_rep (array<box> bs, array<SI> spc,
		 array<path> P, array<path> Q, array<int> F);
  path encode (path p);
  path decode (path p);
};

pages_box_rep::pages_box_rep (array<box> B, array<SI> S,
  array<path> P2, array<path> Q2, array<int> F2):
  modifier_box_rep (stack_box (B, S)), P (P2), Q (Q2), F (F2) {}

path
pages_box_rep::encode (path p) {
  // cout << "Pages encode " << p << "\n";
  if (N(p) <= 3) fatal_error ("invalid path", "pages_box_rep::encode");
  // cout << "  -> " << p->next->next << "\n";
  return p->next->next->next;
}

path
pages_box_rep::decode (path p) {
  // cout << "Pages decode " << p << "\n";
  int i= find_index (p, P, Q, F);
  if (i!=-1) {
    // cout << "  -> " << path (0, path (0, path (i, p))) << "\n";
    return path (0, path (i, path (0, p)));
  }

  // handle selections starting or ending at invalid borders
  if (p==P[0]) return path (0, 0);
  if (p==Q[N(Q)-1]) return path (0, 1);

  cerr << "\n";
  cerr << "Error] Searching for " << p << "\n";
  cerr << "Error]   start paths " << P << "\n";
  cerr << "Error]   end paths " << Q << "\n";
  cerr << "Error]   border flags " << F << "\n";
  fatal_error ("invalid situation", "pages_rep_box::decode");
}

#endmodule // code_page_boxes
