
/******************************************************************************
* MODULE     : page_item.gen.cc
* DESCRIPTION: A typesetted document consists of an array of page_items.
*              Each page item contains spacing and page breaking information.
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <Document/page_item.gen.h>
#include <array.gen.cc>

#module code_page_item
#import page_item
#import code_array (page_item)

/******************************************************************************
* Routines for the page item class
******************************************************************************/

page_item_rep::page_item_rep (box b2, path p2, path q2, int f2):
  type (PAGE_LINE_ITEM), b (b2), p (p2), q (q2), f (f2),
  spc (0), penalty (0) {}
page_item_rep::page_item_rep (tree t2):
  type (PAGE_CONTROL_ITEM), b (empty_box ()), spc (0),
  penalty (HYPH_INVALID), t (t2) {}
page_item_rep::page_item_rep (page_item old, int par_nr):
  type (old->type), b (old->b),
  p (path (par_nr, old->p)),
  q (path (par_nr, old->q)),
  f (old->f), spc (old->spc), penalty (old->penalty), t (old->t) {}

page_item::page_item (box b, path p, path q, int f):
  rep (new page_item_rep (b, p, q, f)) {}
page_item::page_item (tree t):
  rep (new page_item_rep (t)) {}
page_item::page_item (page_item old, int par_nr):
  rep (new page_item_rep (old, par_nr)) {}
bool page_item::operator == (page_item item2) { return rep==item2.rep; }
bool page_item::operator != (page_item item2) { return rep!=item2.rep; }

ostream&
operator << (ostream& out, page_item item) {
  switch (item->type) {
  case PAGE_LINE_ITEM:
    return out << "line (" << item->b << ", " << item->p << ")";
  case PAGE_CONTROL_ITEM:
    return out << "control (" << item->t << ")";
  }
  return out << "unknown";
}

#endmodule // code_page_item
