
/******************************************************************************
* MODULE     : paragraph.gen.cc
* DESCRIPTION: The result of typesetting a paragraph is
*              an instance of the paragraph class
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <Document/paragraph.gen.h>

#module code_paragraph
#import paragraph

/******************************************************************************
* Constructors and basic routines for paragraphs
******************************************************************************/

paragraph_rep::paragraph_rep ():
  status (0), change ("?"), r (0, 0, 0, 0), l (0), new_par_flag (FALSE),
  vspc1 (0), vspc2 (0), vspc_before (0), vspc_after (0),
  nobr1 (FALSE), nobr2 (FALSE), nobr_before (FALSE), nobr_after (FALSE),
  last_break (0) {}
paragraph::paragraph (): rep (new paragraph_rep ()) {}
bool paragraph::operator == (paragraph par) { return rep==par.rep; }
bool paragraph::operator != (paragraph par) { return rep!=par.rep; }
ostream& operator << (ostream& out, paragraph par) {
  return out << "paragraph (" << par->status << ", " << par->l << ")"; }
#import code_array(paragraph)

void
paragraph_rep::set_env_vars (
  SI height2, SI sep2, SI hor_sep2, SI bot2, SI top2)
{
  height = height2;
  sep    = sep2;
  hor_sep= hor_sep2;
  bot    = bot2;
  top    = top2;
}

/******************************************************************************
* Compute the degree of inshoveability between two successive lines
******************************************************************************/

void merge_sort (array<SI>&);

int
get_pos (array<SI> a, SI which) {
  int step, test;
  step= test= N(a)>>1;
  while (a[test] != which) {
    step= step >> 1;
    if (step==0) {
      if (which < a[test]) return test-1;
      else return test+1;
    }
    else {
      if (which < a[test]) test= max (0, test- step);
      else test= min (N(a)-1, test+ step);
    }
  }
  return test;
}

SI
paragraph_rep::shove_in (box b1, box b2) {
  if ((N(b1)==0) || (N(b2)==0)) return top-bot;

  int i, j;
  array<SI> hpos;
  for (i=0; i<N(b1); i++)
    hpos << (b1[i]->x0+b1[i]->x3-hor_sep)
	 << (b1[i]->x0+b1[i]->x4+hor_sep);
  for (i=0; i<N(b2); i++)
    hpos << (b2[i]->x0+b2[i]->x3-hor_sep)
	 << (b2[i]->x0+b2[i]->x4+hor_sep);
  merge_sort (hpos);

  int n= N(hpos)-1;
  array<SI> vpos1 (n);
  array<SI> vpos2 (n);
  for (i=0; i<n; i++) {
    vpos1[i]= bot;
    vpos2[i]= top;
  }

  for (i=0; i<N(b1); i++) {
    SI  y    = b1[i]->y0+ b1[i]->y3;
    int start= get_pos (hpos, b1[i]->x0+b1[i]->x3-hor_sep);
    int end  = get_pos (hpos, b1[i]->x0+b1[i]->x4+hor_sep);
    for (j=start; j<end; j++)
      vpos1[j]= min (vpos1[j], y);
  }
  for (i=0; i<N(b2); i++) {
    SI  y    = b2[i]->y0+ b2[i]->y4;
    int start= get_pos (hpos, b2[i]->x0+b2[i]->x3-hor_sep);
    int end  = get_pos (hpos, b2[i]->x0+b2[i]->x4+hor_sep);
    for (j=start; j<end; j++)
      vpos2[j]= max (vpos2[j], y);
  }

  SI m= vpos2[0]-vpos1[0];
  for (i=1; i<n; i++)
    m= max (m, vpos2[i]-vpos1[i]);
  return m;
}

/******************************************************************************
* Printing to a paragraph
******************************************************************************/

void
paragraph_rep::print (box b, path p, path q, int f) {
  int i= N(l)-1;
  while ((i>=0) && (l[i]->type == PAGE_CONTROL_ITEM)) i--;
  l << page_item (b, p, q, f);
  if ((!new_par_flag) && (i>=0)) {
    page_item& item1 (l [i]);
    page_item& item2 (l [N(l)-1]);
    if ((item2->b->y2- item1->b->y1) < height)
      item1->spc= item1->spc+ space (height- (item2->b->y2- item1->b->y1));
    else {
      SI h= max (height, shove_in (item1->b[0], item2->b[0])+ sep);
      item1->spc= item1->spc+ space (h- (item2->b->y2- item1->b->y1));
    }
  }
  new_par_flag= FALSE;
}

void
paragraph_rep::print (tree t) {
  ctrl << page_item (t);
}

void
paragraph_rep::print (space spc) {
  int i= N(l)-1;
  while ((i>=0) && (l[i]->type == PAGE_CONTROL_ITEM)) i--;
  if (i<0) return;
  l[i]->spc = l[i]->spc + spc;
}

/******************************************************************************
* Other routines
******************************************************************************/

void
paragraph_rep::new_paragraph () {
  if (last_break == 0) {
    vspc_before= vspc1;
    vspc_after = vspc2;
    nobr_before= nobr1;
    nobr_after = nobr2;
  }
  else {
    int i= last_break-1;
    while ((i>=0) && (l[i]->type == PAGE_CONTROL_ITEM)) i--;
    if (i<0) return;
    l[i]->spc = l[i]->spc + vspc1;
    vspc_after= vspc2;
    if (nobr1) l[i]->penalty= HYPH_INVALID;
    nobr_after= nobr2;
  }

  int i1= last_break, i2= N(l)-1, mul= 100;
  while ((i1<=i2) && (mul>1)) {
    while ((i1<i2) && (l[i1]->type == PAGE_CONTROL_ITEM)) i1++;
    while ((i1<i2) && (l[i2]->type == PAGE_CONTROL_ITEM)) i2--;
    if (l[i1]->type != PAGE_CONTROL_ITEM)
      if (l[i1]->penalty < HYPH_INVALID)
	l[i1]->penalty *= mul;
    if (l[i2]->type != PAGE_CONTROL_ITEM)
      if (l[i2]->penalty < HYPH_INVALID)
	if (i1<i2) l[i2]->penalty *= mul;
    mul /= 10; i1++; i2--;
  }

  last_break  = N(l);
  vspc1       = vspc2;
  vspc2       = space (0);
  nobr1       = nobr2;
  nobr2       = FALSE;
  new_par_flag= TRUE;
}

void
paragraph_rep::vspace_before (space spc) {
  vspc1= max (vspc1, spc);
}

void
paragraph_rep::vspace_after (space spc) {
  vspc2= max (vspc2, spc);
}

void
paragraph_rep::no_page_break_before () {
  nobr1= TRUE;
}

void
paragraph_rep::no_page_break_after () {
  nobr2= TRUE;
}

void
paragraph_rep::penalty (int pen) {
  int i= N(l)-1;
  while ((i>=0) && (l[i]->type == PAGE_CONTROL_ITEM)) i--;
  if (i<0) return;
  l[i]->penalty = pen;  
}

void
paragraph_rep::flush () {
  l << ctrl;
  ctrl= array<page_item> (0);
}

#endmodule // code_paragraph
