
/******************************************************************************
* MODULE     : find_font.gen.cc
* DESCRIPTION: decoding font names
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <tree.gen.h>
#include <font.gen.h>
#include <hashmap.gen.h>

#module code_find_font
#import tree
#import font
#import hashmap (string,tree)

hashmap<string,tree> font_conversion ("rule");

/******************************************************************************
* Declare a new rule
******************************************************************************/

static tree
font_tree_to_tree (tree t) {
  if (N(t)==0) return t;
  else {
    int i;
    tree r (t[0]->label, N(t)-1);
    for (i=1; i<N(t); i++) r[i-1]= font_tree_to_tree (t[i]);
    return r;
  }
}

void
font_rule (tree l, tree r) {
  if (N(l)*N(r)==0) return;
  tree which= font_tree_to_tree (l);
  tree by   = font_tree_to_tree (r);
  if (!font_conversion->contains (which->label))
    font_conversion (which->label)= tree ("rule", tree ("->", which, by));
  else font_conversion (which->label) << tree ("->", which, by);
}

/******************************************************************************
* Find a font
******************************************************************************/

static bool
matches (tree t, tree which, hashmap<string,tree>& H) {
  int i, n=N(which);
  if (N(t)!=n) return FALSE;
  for (i=0; i<n; i++) {
    if (which[i]->label[0]=='$') H (which[i]->label)= t[i];
    else if (t[i]!=which[i]) return FALSE;
  }
  return TRUE;
}

static tree
substitute (tree by, hashmap<string,tree>& H) {
  int i, n=N(by);
  tree r (by->label, n);
  for (i=0; i<n; i++) {
    if ((N(by[i]->label)>0) && (by[i]->label[0]=='$')) r[i]= H [by[i]->label];
    else r[i]= substitute (by[i], H);
  }
  return r;
}

font
find_font (display dis, tree t) {
  // cout << "Find " << t << "\n";

  if (is_func (t, "tex", 3))
    return tex_text_font (dis, t[0]->label,
			  as_int (t[1]->label),
			  as_int (t[2]->label));

  if (is_func (t, "tex", 4))
    return tex_text_font (dis, t[0]->label,
			  as_int (t[1]->label),
			  as_int (t[2]->label),
			  as_int (t[3]->label));

  if (is_func (t, "ps", 3))
    return ps_font (dis, t[0]->label,
		    as_int (t[1]->label),
		    as_int (t[2]->label));

  if (is_func (t, "error", 2)) {
    font fn1= find_font (dis, t[0]);
    if (nil (fn1)) return fn1;
    font fn2= find_font (dis, t[1]);
    if (nil (fn2)) return fn2;
    return error_font (fn1, fn2);
  }

  if (is_func (t, "numeric", 1)) {
    font fn= find_font (dis, t[0]);
    if (nil (fn)) return fn;
    return subfont (fn, num_enc ());
  }

  if (is_func (t, "capital", 1)) {
    font fn= find_font (dis, t[0]);
    if (nil (fn)) return fn;
    return subfont (fn, capital_enc ());
  }

  if (is_func (t, "alpha", 1)) {
    font fn= find_font (dis, t[0]);
    if (nil (fn)) return fn;
    return subfont (fn, alpha_enc ());
  }

  if (is_func (t, "alpha-numeric", 1)) {
    font fn= find_font (dis, t[0]);
    if (nil (fn)) return fn;
    return subfont (fn, alpha_num_enc ());
  }

  if (is_func (t, "join", 2)) {
    font fn1= find_font (dis, t[0]);
    font fn2= find_font (dis, t[1]);
    if (nil (fn1) || nil (fn2)) return fn1;
    return join (fn1, fn2);
  }

  if (is_func (t, "math", 4)) {
    font fn= find_font (dis, t[3]);
    if (nil (fn)) return fn;
    return math_font (t[2], as_int (t[0]->label), as_int (t[1]->label), fn);
  }

  if (!font_conversion->contains (t->label)) return font ();

  int i;
  tree rule= font_conversion [t->label];
  for (i=0; i<N(rule); i++) {
    hashmap<string,tree> H ("?");
    if (matches (t, rule[i][0], H))
      return find_font (dis, substitute (rule[i][1], H));
  }

  return font ();
}

/******************************************************************************
* User interface
******************************************************************************/

font
find_font (display dis, string family, string fn_class,
	   string series, string shape, int sz, int dpi)
{
  string s=
    family * "-" * fn_class * "-" *
    series * "-" * shape * "-" *
    as_string (sz) * "-" * as_string (dpi);
  if (resource<font>->contains (s)) return font (s);

  tree t1 (family, 5);
  t1[0]= fn_class;
  t1[1]= series; t1[2]= shape;
  t1[3]= as_string (sz); t1[4]= as_string (dpi);
  font fn= find_font (dis, t1);
  if (!nil (fn)) {
    resource<font> (s)= (int) fn.rep;
    return fn;
  }

  tree t2 (family, 4);
  t2[0]= fn_class; t2[1]= series;
  t2[2]= as_string (sz); t2[3]= as_string (dpi);
  fn= find_font (dis, t2);
  if (!nil (fn)) {
    resource<font> (s)= (int) fn.rep;
    return fn;
  }

  tree t3 (family, 3);
  t3[0]= fn_class; t3[1]= as_string (sz); t3[2]= as_string (dpi);
  fn= find_font (dis, t3);
  if (!nil (fn)) {
    resource<font> (s)= (int) fn.rep;
    return fn;
  }

  tree panic ("tex", "cmr", as_string (sz), as_string (dpi));
  fn= find_font (dis, panic);
  resource<font> (s)= (int) fn.rep;
  return fn;
}

#endmodule // code_find_font
