
/******************************************************************************
* MODULE     : ps_rubber_font.gen.cc
* DESCRIPTION: a font for producing variable size symbols such as
*                - empty and plain boxes
*                - parenthesis boxes
*                - overline and underline like boxes
*                - negation boxes (\not\something in TeX)
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <font.gen.h>

#module code_ps_rubber_font
#import font
#include <math.h>

/*****************************************************************************/
// Rubber fonts
/*****************************************************************************/

struct ps_rubber_font_rep: font_rep {
  SI     penw;  // the penwidth to be used
  ps_rubber_font_rep (display dis, string name, int size);
  box convert (string s, color col, SI len1, SI len2);
};

/*****************************************************************************/
// Initialization of ps_rubber fonts
/*****************************************************************************/

ps_rubber_font_rep::ps_rubber_font_rep (display dis, string name, int size):
  font_rep (dis, name), penw ((size*PIXEL)/10) {}

font
ps_rubber_font (font fn, int size, int dpi) {
  int ss= dpi*size/72;
  string fname= "ps_rubber" * as_string (ss);
  return make<font> (fname, new ps_rubber_font_rep (fn->dis, fname, ss));
}

/*****************************************************************************/
// Creating a ps_rubber box
/*****************************************************************************/

#define Lbracket        1011
#define Lcrochet        1012
#define Langular        1013
#define Laccolade       1014
#define Rbracket        1021
#define Rcrochet        1022
#define Rangular        1023
#define Raccolade       1024
#define Voidbr          1030
#define Absolute        1031

box symbol_box (box b, int n);

box
ps_rubber_font_rep::convert (string s, color col, SI len1, SI len2) {
  if (s == "<left-(>")
    return symbol_box (bracket_box (Lbracket, penw, col, len1, len2), N(s));
  if (s == "<right-)>")
    return symbol_box (bracket_box (Rbracket, penw, col, len1, len2), N(s));
  if (s == "<left-[>")
    return symbol_box (bracket_box (Lcrochet, penw, col, len1, len2), N(s));
  if (s == "<right-]>")
    return symbol_box (bracket_box (Rcrochet, penw, col, len1, len2), N(s));
  if (s == "<left-{>")
    return symbol_box (bracket_box (Laccolade, penw, col, len1, len2), N(s));
  if (s == "<right-}>")
    return symbol_box (bracket_box (Raccolade, penw, col, len1, len2), N(s));
  return empty_box ();
}

#endmodule // code_ps_rubber_font
