
/******************************************************************************
* MODULE     : tex_font.gen.cc
* DESCRIPTION: TeX text fonts
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <font.gen.h>
#include <Boxes/construct.gen.h>

#module code_tex_font
#import font
#import construct_boxes

/*****************************************************************************/
// TeX text fonts
/*****************************************************************************/

struct tex_font_rep: font_rep {
  server_font fn;   // the underlying server font
  tex_font_rep (display dis, string name,
		string family, int size, int dpi, int dsize);
  box convert (string s, color col, SI len1, SI len2);
};

/******************************************************************************
* Handle accents
******************************************************************************/

static char Cork_unaccented[128]= {
  'A', ' ', 'C', 'C', 'D', 'E', ' ', 'G',
  'L', 'L', ' ', 'N', 'N', ' ', 'O', 'R',
  'R', 'S', 'S', 'S', 'T', 'T', 'U', 'U',
  'Y', 'Z', 'Z', 'Z', ' ', 'I', 'd', ' ',
  'a', ' ', 'c', 'c', 'd', 'e', ' ', 'g',
  'l', 'l', ' ', 'n', 'n', ' ', 'o', 'r',
  'r', 's', 's', 's', 't', 't', 'u', 'u',
  'y', 'z', 'z', 'z', ' ', ' ', ' ', ' ',
  'A', 'A', 'A', 'A', 'A', 'A', ' ', 'C',
  'E', 'E', 'E', 'E', 'I', 'I', 'I', 'I',
  'D', 'N', 'O', 'O', 'O', 'O', 'O', ' ',
  ' ', 'U', 'U', 'U', 'U', 'Y', ' ', ' ',
  'a', 'a', 'a', 'a', 'a', 'a', ' ', 'c',
  'e', 'e', 'e', 'e', 16 , 16 , 16 , 16 ,
  'd', 'n', 'o', 'o', 'o', 'o', 'o', ' ',
  ' ', 'u', 'u', 'u', 'u', 'y', ' ', ' '
};

static char Cork_accents[128]= {
  21 , ' ', 19 , 20 , 20 , 20 , ' ', 21 ,
  19 , 39 , ' ', 19 , 20 , ' ', 125, 19 ,
  20 , 19 , 20 , 24 , 20 , 24 , 125, 23 ,
  127, 19 , 20 , 95 , ' ', 95 , 22 , ' ',
  21 , ' ', 19 , 20 , 20 , 20 , ' ', 21 ,
  19 , 39 , ' ', 19 , 20 , ' ', 125, 19 ,
  20 , 19 , 20 , 24 , 20 , 24 , 125, 23 ,
  127, 19 , 20 , 95 , ' ', ' ', ' ', ' ',
  18 , 19 , 94 , 126, 127, 23 , ' ', 24 ,
  18 , 19 , 94 , 127, 18 , 19 , 94 , 127,
  22 , 126, 18 , 19 , 94 , 126, 127, ' ',
  ' ', 18 , 19 , 94 , 127, 19 , ' ', ' ',
  18 , 19 , 94 , 126, 127, 23 , ' ', 24 ,
  18 , 19 , 94 , 127, 18 , 19 , 94 , 127,
  22 , 126, 18 , 19 , 94 , 126, 127, ' ',
  ' ', 18 , 19 , 94 , 127, 19 , ' ', ' '
};

static string
get_unaccented (string s) {
  int i;
  string r(N(s));
  for (i=0; i<N(s); i++)
    if ((s[i] & 128) == 0) r[i]= s[i];
    else {
      char c= Cork_unaccented[s[i] & 127];
      if (c==' ') r[i]= s[i];
      else r[i]= Cork_unaccented[s[i] & 127];
    }
  return r;
}

static string
get_accents (string s) {
  int i;
  string r(N(s));
  for (i=0; i<N(s); i++)
    if ((s[i] & 128) == 0) r[i]= ' ';
    else r[i]= Cork_accents[s[i] & 127];
  return r;
}

/******************************************************************************
* The implementation of tex_fonts
******************************************************************************/

tex_font_rep::tex_font_rep (display dis, string name,
  string family, int size, int dpi, int dsize):
  font_rep (dis, name)
{
  fn= dis->server_tex_font (family, size, dpi, dsize);
  copy_parameters (fn);
}

box
tex_font_rep::convert (string s, color col, SI len1, SI len2) {
  (void) len1; (void) len2;
  int i;
  for (i=0; i<N(s); i++)
    if ((s[i] & 128) != 0)
      return accented_box (get_unaccented (s), get_accents (s), fn, col);
  return text_box (s, fn, col);
}

font
tex_text_font (display dis, string family, int size, int dpi, int dsize) {
  string name= "tex:" * family * as_string (size) * "@" * as_string(dpi);
  return make<font> (name,
    new tex_font_rep (dis, name, family, size, dpi, dsize));
}

#endmodule // code_tex_font
