
/******************************************************************************
* MODULE     : par_line.gen.cc
* DESCRIPTION: boxes for typesetting lines into paragraphs
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <Boxes/construct.gen.h>
#include <Boxes/composite.gen.h>
#include <Boxes/modifier.gen.h>

#module code_line_boxes
#import construct_boxes
#import composite_boxes
#import modifier_boxes
#import array (path)
#import array (int)

#define GLUE_LEFT_SUBS_ITEM   12
#define GLUE_RIGHT_SUBS_ITEM  13
#define GLUE_LEFT_ITEM        14
#define GLUE_RIGHT_ITEM       15
#define GLUE_BOTH_ITEM        16

/******************************************************************************
* Once a line has been typeset, a par_line_box is returned.
* Such boxes contain information to recover the path in
* the typesetted tree into a physical cursor and vice versa.
******************************************************************************/

struct par_line_box_rep: modifier_box_rep {
  array<path> P; // start path for each box in par_line
  array<path> Q; // end path for each box in par_line
  array<int>  F; // border flags
  array<path> S; // extra information for handling scripts

  par_line_box_rep (array<box> bs, array<SI> spc,
		    array<path> P, array<path> Q, array<int> F, array<path> S);
  
  path encode_script (path p, path ctrl, int pos, box b);
  path encode_script (path p);
  path encode (path p);
  path decode_script (path p, path ctrl, int pos, box b);
  path decode_script (path p);
  path decode (path p);
};

par_line_box_rep::par_line_box_rep (array<box> bs, array<SI> spc,
  array<path> P2, array<path> Q2, array<int> F2, array<path> S2):
  modifier_box_rep (concat_box (bs, spc)), P (P2), Q (Q2), F (F2), S (S2) {}

/******************************************************************************
* 'script_gap' Computes the number of scripts
* which have been glued together from the control path
******************************************************************************/

static int
script_gap (path ctrl) {
  if (nil (ctrl)) return 0;

  int type = ctrl->item;
  int data = ctrl->next->item;
  int ltype= type==GLUE_LEFT_ITEM? data: 0;
  int rtype= type==GLUE_LEFT_ITEM? 0: data;
  if (ctrl->item==GLUE_BOTH_ITEM) {
    ltype= data>>3;
    rtype= data&7;
  }
  int lnr= ((ltype&6)!=0)? 2: 1;
  int rnr= ((rtype&6)!=0)? 2: 1;

  switch (type) {
  case GLUE_LEFT_SUBS_ITEM:
  case GLUE_RIGHT_SUBS_ITEM:
    return 1;
  case GLUE_LEFT_ITEM:
    return script_gap (ctrl->next->next)+lnr;
  case GLUE_RIGHT_ITEM:
    return script_gap (ctrl->next->next)+rnr;
  case GLUE_BOTH_ITEM:
    return script_gap (ctrl->next->next)+lnr+rnr;
  }

  fatal_error ("unexpected value", "script_gap");
}

/******************************************************************************
* Encoding glued scripts
******************************************************************************/

path
par_line_box_rep::encode_script (path p, path ctrl, int pos, box b) {
  // cout << "      encode {" << p << ", " << ctrl << ", " << pos << "}\n";
  if (nil (ctrl)) return path (pos, p);

  int n   = script_gap (ctrl);
  int type= ctrl->item;
  int data= ctrl->next->item;
  ctrl= ctrl->next->next;
  
  int ltype= type==GLUE_LEFT_ITEM? data: 0;
  int rtype= type==GLUE_LEFT_ITEM? 0: data;
  if (type==GLUE_BOTH_ITEM) {
    ltype= data>>3;
    rtype= data&7;
  }
  int lnr  = ((ltype&6)!=0)? 2: 1;
  int arity= (type==GLUE_LEFT_ITEM) || (type==GLUE_BOTH_ITEM)? lnr: 0;

  if ((type==GLUE_LEFT_SUBS_ITEM) || (type==GLUE_RIGHT_SUBS_ITEM)) {
    if (p == path (0)) return path (pos, 0);
    if (p == path (1)) {
      if (((data&1)==0) || ((data&6)==4)) return path (pos+1, 1);
      else return path (pos+1, b[1]->right_path ());
    }
    int plus= ((data&6)==4)? 1-p->item: p->item;
    int sup = ((data&6)==2)? 1: 0;
    if (((data&1)==0) || (sup!=plus))
      return path (pos+plus, path (0, p->next));
    else return path (pos+plus, p->next);
  }

  if ((type==GLUE_LEFT_ITEM) || (type==GLUE_BOTH_ITEM))
    if (p == path (0)) return path (pos, 0);
  if (type==GLUE_LEFT_ITEM)
    if (p == path (1)) {
      if (nil (ctrl)) return path (pos+lnr, b[0]->right_path());
      else return encode_script (p, ctrl, pos+lnr, b[0]);
    }
  if (type==GLUE_RIGHT_ITEM)
    if (p == path (0)) return path (pos, b[0]->left_path());
  if ((type==GLUE_RIGHT_ITEM) || (type==GLUE_BOTH_ITEM))
    if (p == path (1)) {
      if (((rtype&1)==0) || ((rtype&6)==4)) return path (pos+n, 1);
      else return path (pos+n, b[N(b)-1]->right_path ());
    }

  if (p->item==0) {
    if (type==GLUE_RIGHT_ITEM) return encode_script (p->next, ctrl, pos, b[0]);
    else return encode_script (p->next, ctrl, pos+lnr, b[0]);
  }

  if ((type==GLUE_LEFT_ITEM) || ((type==GLUE_BOTH_ITEM) && (p->item<=arity))) {
    int plus= ((ltype&6)==4)? 2-p->item: p->item-1;
    int sup = ((ltype&6)==2)? 1: 0;
    if (((ltype&1)==0) || (sup!=plus))
      return path (pos+plus, path (0, p->next));
    else return path (pos+plus, p->next);
  }
  if ((type==GLUE_RIGHT_ITEM) || (type==GLUE_BOTH_ITEM)) {
    int minus= ((rtype&6)==4)? 2+p->item-N(b): N(b)-1-p->item;
    int sup  = ((rtype&6)==4)? 1: 0;
    if (((rtype&1)==0) || (sup!=minus))
      return path (pos+n-minus, path (0, p->next));
    else return path (pos+n-minus, p->next);
  }

  fatal_error ("unexpected value", "par_line_box_rep::encode_script");
}

path
par_line_box_rep::encode_script (path p) {
  // cout << "   encode script " << p << "\n";
  int i;
  box sb= b[p->item];
  for (i=0; i<N(S); i++) {
    if (p->item < S[i]->item) break;
    if (p->item > S[i]->item) { p->item += script_gap (S[i]->next); continue; }
    path pp= encode_script (p->next, S[i]->next, p->item, sb);
    // cout << "   ---> " << pp << "\n";
    return pp;
  }
  // cout << "   ---> " << p << "\n";
  return p;
}

/******************************************************************************
* Decoding glued scripts
******************************************************************************/

path
par_line_box_rep::decode_script (path p, path ctrl, int pos, box b) {
  // cout << "      decode {" << p << ", " << ctrl << ", " << pos << "}\n";
  if (nil (ctrl)) return p;

  int n   = script_gap (ctrl);
  int type= ctrl->item;
  int data= ctrl->next->item;
  ctrl= ctrl->next->next;
  
  int ltype= type==GLUE_LEFT_ITEM? data: 0;
  int rtype= type==GLUE_LEFT_ITEM? 0: data;
  if (type==GLUE_BOTH_ITEM) {
    ltype= data>>3;
    rtype= data&7;
  }
  int lnr  = ((ltype&6)!=0)? 2: 1;
  int rnr  = ((rtype&6)!=0)? 2: 1;
  int arity= (type==GLUE_LEFT_ITEM) || (type==GLUE_BOTH_ITEM)? lnr: 0;

  if ((type==GLUE_LEFT_SUBS_ITEM) || (type==GLUE_RIGHT_SUBS_ITEM)) {
    int cur    = ((data&6)==4)? 1-pos: pos;
    int sup    = ((data&6)==2)? 1: 0;
    bool prime = ((data&1)!=0) && (sup==pos);
    bool vprime= ((data&1)!=0) && (sup!=pos);

    if ((pos == 0) && (p == path (0)))
      return path (0);
    if (((!prime) && (pos == 1) && (p == path (1))) ||
	(prime && (pos == 1) && (p == b[cur]->right_path ())))
      return path (1);
    if ((pos == 1) && (p == path (0))) {
      if (!prime) return path (cur, b[cur]->left_path ());
      else return path (cur, p);
    }
    if (((!prime) && (pos == 0) && (p == path (1))) ||
	(prime && (pos == 0) && (p == b[cur]->right_path ()))) {
      if (!vprime) return path (cur, b[cur]->right_path ());
      else return path (cur+1, path (0));
    }
    
    if (!prime) return path (cur, p->next);
    else return path (cur, p);
  }

  if ((type==GLUE_LEFT_ITEM) || (type==GLUE_BOTH_ITEM))
    if (pos < lnr) {
      int cur= ((ltype&6)==4)? 2-pos: 1+pos;
      int sup= ((ltype&6)==2)? 1: 0;
      bool prime = ((ltype&1)!=0) && (sup==pos);
      bool vprime= ((ltype&1)!=0) && (sup!=pos);
      
      if ((pos == 0) && (p == path (0)))
	return path (0);
      if (((!prime) && (pos == lnr-1) && (p == path (1))) ||
	  (prime && (pos == lnr-1) && (p == b[cur]->right_path ())))
	return path (0, b[0]->left_path ());
      if ((pos == lnr-1) && (p == path (0))) {
	if (!prime) return path (cur, b[cur]->left_path ());
	else return path (cur, p);
      }
      if (((!prime) && (pos == 0) && (p == path (1))) ||
	  (prime && (pos == 0) && (p == b[cur]->right_path ()))) {
	if (!vprime) return path (cur, b[cur]->right_path ());
	else return path (cur+1, path (0));
      }

      if (!prime) return path (cur, p->next);
      else return path (cur, p);
    }

  if ((type==GLUE_RIGHT_ITEM) || (type==GLUE_BOTH_ITEM))
    if (pos > n-rnr) {
      int  cur   = ((rtype&6)==2)? arity+2-(n-pos): arity+1+(n-pos);
      int  sup   = ((rtype&6)==4)? n-1: n;
      bool prime = ((rtype&1)!=0) && (sup==pos);
      bool vprime= ((rtype&1)!=0) && (sup!=pos);
      
      if ((pos == n+1-rnr) && (p == path (0)))
	return path (0, b[0]->right_path ());
      if (((!prime) && (pos == n) && (p == path (1))) ||
	  (prime && (pos == n) && (p == b[cur]->right_path ())))
	return path (1);
      if ((pos == n) && (p == path (0))) {
	if (!prime) return path (cur, b[cur]->left_path ());
	else return path (cur, p);
      }
      if (((!prime) && (pos == n+1-rnr) && (p == path (1))) ||
	  (prime && (pos == n+1-rnr) && (p == b[cur]->right_path ()))) {
	if (!vprime) return path (cur, b[cur]->right_path ());
	else return path (cur+1, path (0));
      }

      if (!prime) return path (cur, p->next);
      else return path (cur, p);
    }

  if ((type==GLUE_LEFT_ITEM) || (type==GLUE_BOTH_ITEM))
    return path (0, decode_script (p, ctrl, pos-lnr, b[0]));
  if (type==GLUE_RIGHT_ITEM)
    return path (0, decode_script (p, ctrl, pos, b[0]));
  fatal_error ("unexpected value", "par_line_box_rep::decode_script");
}

path
par_line_box_rep::decode_script (path p) {
  // cout << "   decode script " << p << "\n";
  int i, minus=0;
  for (i=0; i<N(S); i++) {
    if (p->item < S[i]->item) break;
    int n= script_gap (S[i]->next);
    if (p->item > S[i]->item+ n) { minus += n; continue; }
    int ref (S[i]->item- minus), rel (p->item- S[i]->item);
    path pp= path (ref, decode_script (p->next, S[i]->next, rel, b[ref]));
    // cout << "   ---> " << pp << "\n";
    return pp;
  }
  // cout << "   ---> " << path (p->item- minus, p->next) << "\n";
  return path (p->item- minus, p->next);
}

/******************************************************************************
* Main encoding and decoding
******************************************************************************/

path
par_line_box_rep::encode (path p) {
  // cout << "------------------------------------------------------------------------------\n";
  // cout << "P= " << P << "\n";
  // cout << "Q= " << Q << "\n";
  // cout << "encode " << p << "\n";
  if (atom (p) || atom (p->next))
    fatal_error ("invalid path", "par_line_box_rep::encode");
  p= encode_script (p->next);
  // cout << "1 -> " << p << "\n";
  path q;
  if (((F[p->item]==1) || (F[p->item]==2)) && (P[p->item]!=Q[p->item])) {
    cout << "F= " << F[p->item] << "\n";
    cout << "P= " << P[p->item] << "\n";
    cout << "Q= " << Q[p->item] << "\n";
    fatal_error ("Unexpected situation", "par_line_box_rep::encode");
  }
  switch (F[p->item]) {
  case 0:
    q= path_add (P[p->item], p->next->item);
    q << p->next->next;
    break;
  case 1:
  case 2:
    q= P[p->item];
    break;
  case 3:
    if (zero_path (p->next)) q= P[p->item];
    else q= Q[p->item];
    break;
  }
  // cout << "2 -> " << q << "\n";
  return q;
}

bool
test_found (path& p, int i, array<path> P, array<path> Q, array<int> F) {
  if (((i>0) && path_var_inf_eq (p, P[i])) ||
      (path_var_inf_eq (P[i], p) && path_var_inf_eq (p, Q[i]))) {
    if ((p == Q[i]) && ((F[i]&2) == 2)) return FALSE;
    if ((i>0) && path_var_inf_eq (p, P[i])) p= P[i];
    return TRUE;
  }
  return FALSE;
}

int
find_index (path& p, array<path> P, array<path> Q, array<int> F) {
  int i, h, n=N(P);
  if (n<4) i=0;
  else {
    i= h= n>>1;
    while (h != 0) {
      path test_p= p;
      if (h==1) h= 0;
      else h= (h+1) >> 1;
      if (test_found (test_p, i, P, Q, F)) i -= max (h, 1);
      else i += h;
      if (i < 0  ) i= 0;
      if (i > n-2) i= n-2;
    }
  }

  for (; i<N(P); i++) 
    if (test_found (p, i, P, Q, F))
      return i;
  return -1;
}

path
par_line_box_rep::decode (path p) {
  // cout << "------------------------------------------------------------------------------\n";
  // cout << "P= " << P << "\n";
  // cout << "Q= " << Q << "\n";
  // cout << "F= " << F << "\n";
  // cout << "decode " << p << "\n";

  int i= find_index (p, P, Q, F);
  if (i!=-1) {
    // cout << "---> found at " << i << "\n";
    int n= N(P[i]);
    path q= path (p[n-1]- P[i][n-1], tail (p, n));
    // cout << "1 -> " << path (i, q) << "\n";
    path r= path (0, decode_script (path (i, q)));
    // cout << "2 -> " << r << "\n";
    return r;
  }
  
  // handle selections starting or ending at invalid borders
  if (p==P[0]) return path (0, 0);
  if (p==Q[N(Q)-1]) return path (0, 1);

  cerr << "\n";
  cerr << "Error] Searching for " << p << "\n";
  cerr << "Error]   start paths " << P << "\n";
  cerr << "Error]   end paths " << Q << "\n";
  cerr << "Error]   border flags " << F << "\n";
  fatal_error ("invalid situation", "par_line_box_rep::decode");
}

#endmodule // code_line_boxes
