
/******************************************************************************
* MODULE     : line_item.gen.cc
* DESCRIPTION: Control routines for typesetting paragraphs
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <Paragraph/line_item.gen.h>
#include <array.gen.cc>

#module code_line_item
#import line_item
#import code_array (line_item)

/*****************************************************************************/
// Routines for the line item class
/*****************************************************************************/

line_item_rep::line_item_rep (int type2, box b2, path p2, path q2, int f2,
  int pen2):
  type (type2), b (b2), p (p2), q (q2), f (f2), spc (0), penalty (pen2)
    { DEBUG(line_item_count++); }
line_item_rep::line_item_rep (int type2, box b2, path p2, path q2, int f2,
  int pen2, string s2, font fn2, color col2, language lan2):
  type (type2), b (b2), p (p2), q (q2), f (f2), spc (0), penalty (pen2),
  s (s2), fn (fn2), col (col2), lan (lan2) { DEBUG(line_item_count++); }
line_item_rep::line_item_rep (int type2, box b2, path p2, path q2, int f2,
  int pen2, tree t2):
  type (type2), b (b2), p (p2), q (q2), f (f2), spc (0), penalty (pen2),
  t (t2) { DEBUG(line_item_count++); }
line_item_rep::line_item_rep (int type2, box b2, path p2, path q2, int f2,
  int pen2, int data2):
  type (type2), b (b2), p (p2), q (q2), f (f2), spc (0), penalty (pen2),
  data (data2) { DEBUG(line_item_count++); }
line_item_rep::line_item_rep (int type2, box b2, path p2, path q2, int f2,
  int pen2, line_item* glue2):
  type (type2), b (b2), p (p2), q (q2), f (f2), spc (0), penalty (pen2),
  glue (glue2) { DEBUG(line_item_count++); }
line_item_rep::~line_item_rep () {
  DEBUG(line_item_count--);
  if ((type==GLUE_LEFT_ITEM) ||
      (type==GLUE_RIGHT_ITEM) ||
      (type==GLUE_BOTH_ITEM)) delete [] glue;
}
line_item::line_item (int type, box b, path p, path q, int f, int penalty):
  rep (new line_item_rep (type, b, p, q, f, penalty)) {}
line_item::line_item (int type, box b, path p, path q, int f, int penalty,
  string s, font fn, color col, language lan):
  rep (new line_item_rep (type, b, p, q, f, penalty, s, fn, col, lan)) {}
line_item::line_item (int type, box b, path p, path q, int f, int penalty,
  tree t):
  rep (new line_item_rep (type, b, p, q, f, penalty, t)) {}
line_item::line_item (int type, box b, path p, path q, int f, int penalty,
  int data):
  rep (new line_item_rep (type, b, p, q, f, penalty, data)) {}
line_item::line_item (int type, box b, path p, path q, int f, int penalty,
  line_item* glue):
  rep (new line_item_rep (type, b, p, q, f, penalty, glue)) {}
bool line_item::operator == (line_item item2) { return rep==item2.rep; }
bool line_item::operator != (line_item item2) { return rep!=item2.rep; }

ostream&
operator << (ostream& out, line_item item) {
  switch (item->type) {
  case OBSOLETE_ITEM: return out << "obsolete";
  case CONTROL_ITEM: return out << "control (" << item->t << ")";
  case STD_ITEM: return out << "std";
  case STRING_ITEM: return out << item->s;
  case LEFT_BRACKET_ITEM: return out << "left-" << item->s;
  case MIDDLE_BRACKET_ITEM: return out << "mid-" << item->s;
  case RIGHT_BRACKET_ITEM: return out << "right-" << item->s;
  case LIMIT_ITEM: return out << "limit";
  case LEFT_SUB_ITEM: return out << "left-sub";
  case LEFT_SUP_ITEM: return out << "left-sup";
  case RIGHT_SUB_ITEM: return out << "right-sub";
  case RIGHT_SUP_ITEM: return out << "right-sup";
  case GLUE_LEFT_ITEM: return out << "glue-left (" << item->glue[0] << ", "
				  << item->glue[1] << ")";
  case GLUE_RIGHT_ITEM: return out << "glue-right (" << item->glue[0] << ", "
				   << item->glue[1] << ")";
  case GLUE_BOTH_ITEM: return out << "glue-both (" << item->glue[0] << ", "
				  << item->glue[1] << ", "
				  << item->glue[2] << ")";
  case GLUE_LEFT_SUBS_ITEM: return out << "glue-lsubs (" << item->glue[0]
				       << ", " << item->glue[1] << ")";
  case GLUE_RIGHT_SUBS_ITEM: return out << "glue-rsubs (" << item->glue[0]
					<< ", " << item->glue[1] << ")";
  }
  return out << "unknown";
}

#endmodule // code_line_item
