
/******************************************************************************
* MODULE     : line_pass1.gen.cc
* DESCRIPTION: First pass for typesetting paragraphs;
*              an array of line_items is created of the right types.
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <file.gen.h>

#module code_line_pass1
#import file

/******************************************************************************
* Printing items
******************************************************************************/

SI italic_correction (box left, box right, SI y);

void
liner_rep::print (int type, box b, path p, path q, int data) {
  if (b->w()>0) italic_correction (b);
  a << line_item (type, b, p, q, 0, HYPH_INVALID, data);
}

void
liner_rep::print (int type, box b, path p, string s, font fn) {
  if (b->w()>0) italic_correction (b);
  a << line_item (type, b, p, path_add (p, N(s)), 0, HYPH_INVALID,
		  s, fn, env->col, env->lan);
}

void
liner_rep::control (tree t, path p) {
  box b= empty_box (0, env->fn->y1, 0, env->fn->y2);
  a << line_item (CONTROL_ITEM, b, path_down (p, 0), path_down (p, 1), 0,
		  HYPH_INVALID, t);
}

void
liner_rep::ghost (string s, path p, int f) {
  ghost (s, p, f, env->dis->blue);
}

void
liner_rep::ghost (string s, path p, int f, color col) {
  if ((N(s)>2) && (s[0]=='<') && (s[N(s)-1]=='>')) {
    ghost ("<", p, f, col);
    ghost (s (1,N(s)-1), p, f, col);
    ghost (">", p, f, col);
    return;
  }
  
  string fn_name= "cmr";
  if (N(s)==1) {
    if (s[0] == '<') { fn_name= "cmsy"; s= "h"; }
    else if (s[0] == '>') { fn_name= "cmsy"; s= "i"; }
    else if (s[0] == '|') { fn_name= "cmsy"; s= "j"; }
    else if (s[0] == '\\') { fn_name= "cmsy"; s= "n"; }
    else if (s[0] == '{') { fn_name= "cmsy"; s= "f"; }
    else if (s[0] == '}') { fn_name= "cmsy"; s= "g"; }
  }
  int sz= script (env->fn_size, env->index_level);
  font gfn (tex_text_font (env->dis, fn_name, sz, (int) (env->magn*env->dpi)));
  box b= gfn->convert (s, col);
  if (b->w()>0) italic_correction (b);
  a << line_item (STD_ITEM, b, p, p, f, HYPH_INVALID, 0);
}

/******************************************************************************
* Printing spaces and setting penalties
******************************************************************************/

void
liner_rep::print (space spc) {
  int n= N(a);
  if (n==0) return;
  a[n-1]->spc= max (spc, a[n-1]->spc);
}

void
liner_rep::italic_correction (box b) {
  int i;
  for (i=N(a)-1; i>=0; i--)
    if (a[i]->b->w()>0) {
      if (b->right_slope()==0) print (space (a[i]->b->right_correction()));
      else print (space (::italic_correction (a[i]->b, b, 0)));
      break;
    }
}

void
liner_rep::penalty_min (int p) {
  if (N(a)>0) a[N(a)-1]->penalty = min (a[N(a)-1]->penalty, p);
}

void
liner_rep::penalty_max (int p) {
  if (N(a)>0) a[N(a)-1]->penalty = max (a[N(a)-1]->penalty, p);
}

/******************************************************************************
* Typesetting strings
******************************************************************************/

void
liner_rep::typeset_bigop (string s, path p) {
  box b= env->fn->convert (s, env->col, env->display_style? 2: 1);
  bool flag= (s != "<big-int>") && (s != "<big-oint>");
  path q= path_add (p, N(s));
  print ((flag && env->display_style)? LIMIT_ITEM: STD_ITEM, b, p, q);
}

void
liner_rep::typeset_bracket (tree t, path p) {
  string& s (t->label);
  box b= env->fn->convert (s, env->col, env->fn->y1, env->fn->y2);
  if (is_opening (t))
    print (LEFT_BRACKET_ITEM, b, p, s, env->fn);
  if (is_middle (t))
    print (MIDDLE_BRACKET_ITEM, b, p, s, env->fn);
  if (is_closing (t))
    print (RIGHT_BRACKET_ITEM, b, p, s, env->fn);
}

void
liner_rep::typeset_prime (string s, path p) {
  font fn;
  display dis= env->fn->dis;
  int sz= script (script (env->fn_size, 1), env->index_level);
  string name ((char*) ((s[0] == '`')? "msam": "cmsy"));
  fn= tex_text_font (dis, name, sz, (int) (env->magn*env->dpi));

  int i;
  string r (N(s));
  for (i=0; i<N(s); i++)
    if (s[0] == '`') r[i]= (char) 56;
    else r[i]= (char) 48;

  box sb= fn->convert (r, env->col);
  if (s[0] == '\'') sb= move_box (sb, env->decode_length ("0.04fn"), 0);
  if (s[0] == '`') sb= move_box (sb, env->decode_length ("-0.04fn"), 0);
  penalty_max (HYPH_INVALID);
  print ((s[0] == '`')? LEFT_SUP_ITEM: RIGHT_SUP_ITEM,
	 prime_box (sb), p, path_add (p, N(s)), 1);

  /*
  env->set (FONT_SIZE, as_string (script (env->get_int (FONT_SIZE), 1)));
  box b= env->fn->convert (s, env->col);
  if (s[0] == '\'') b= move_box (b, env->decode_length ("0.04fn"), 0);
  if (s[0] == '`') b= move_box (b, env->decode_length ("-0.04fn"), 0);
  penalty_max (HYPH_INVALID);
  print ((s[0] == '`')? LEFT_SUP_ITEM: RIGHT_SUP_ITEM,
	 prime_box (b), p, path_add (p, N(s)), 1);
  env->reset (FONT_SIZE);
  */
}

void
liner_rep::typeset_substring (string s, path p) {
  if (env->get (MODE) == "math") {
    if (is_bracket (s)) { typeset_bracket (s, p); return; }
    if (is_bigop (s)) { typeset_bigop (s, p); return; }
    if (is_prime (s)) { typeset_prime (s, p); return; }
  }
  print (STRING_ITEM, typesetter_rep::typeset_string (s), p, s, env->fn);
}

void
liner_rep::typeset_string (tree t, path p) {
  string s (t->label);
  int    start, pos=0;
  space  spc= env->fn->spc;
  space  extra= env->fn->extra;
  space  spc1, spc2;
  int    pen1, pen2;
  bool   not_condensed= !env->math_condensed;

  do {
    start= pos;
    env->lan->advance (s, pos, spc, extra, spc1, spc2, pen1, pen2);
    if ((pos>start) && (s[start]==' ')) {
      if (start==0)
	typeset_substring ("", path_down (p, 0));
      penalty_min (min (pen1, pen2));
      print (spc1+ spc2);
      if ((pos==N(s)) || (s[pos]==' '))
	typeset_substring ("", path_down (p, pos));
    }
    else {
      penalty_min (pen1);
      if (not_condensed) print (spc1);
      typeset_substring (s (start, pos), path_down (p, start));
      penalty_min (pen2);
      if (not_condensed) print (spc2);
    }
  } while (pos<N(s));
}

/******************************************************************************
* Typesetting other specific objects
******************************************************************************/

void
liner_rep::typeset_concat (tree t, path p) {
  int i;
  for (i=0; i<N(t); i++)
    typeset (t[i], path_down (p, i));
}

void
liner_rep::typeset_hspace (tree t, path p) {
  if (N(t)==1) {
    string s= t[0]->label;
    print (space (env->decode_length (s * "-"),
		  env->decode_length (s),
		  env->decode_length (s * "+")));
  }
  else print (space (env->decode_length (t[0]->label),
		     env->decode_length (t[1]->label),
		     env->decode_length (t[2]->label)));
  control (t, p);
}

void
liner_rep::typeset_script (tree t, path p, bool right) {
  int type= RIGHT_SUP_ITEM;
  box b1, b2;
  env->script_size ();
  env->set (DISPLAY_STYLE, "false");
  env->set (MATH_CONDENSED, "true");
  if (is_func (t, SUB (right))) {
    env->set (VERTICAL_POS, "-1");
    b1= typesetter_rep::typeset (t[0]);
    type= right? RIGHT_SUB_ITEM: LEFT_SUB_ITEM;
    env->reset (VERTICAL_POS);
  }
  if (is_func (t, SUP (right))) {
    env->set (VERTICAL_POS, "1");
    b2= typesetter_rep::typeset (t[0]);
    type= right? RIGHT_SUP_ITEM: LEFT_SUP_ITEM;
    env->reset (VERTICAL_POS);
  }
  env->reset (VERTICAL_POS);
  env->reset (MATH_CONDENSED);
  env->reset (DISPLAY_STYLE);
  env->normal_size ();
  if (right) penalty_max (HYPH_INVALID);
  print (type, script_box (b1, b2), path_down (p, 0), path_down (p, 1), 0);
  if (!right) penalty_max (HYPH_INVALID);
}

/******************************************************************************
* Typesetting deactivated trees
******************************************************************************/

void
liner_rep::typeset_inactive (tree t, path p) { 
  ghost ("", path_down (p, 0));
  typeset (t[0], path_down (p, 0), FALSE);
  ghost ("", path_down (p, 1));
}

void
liner_rep::typeset_inactive (string type, tree t, path p, int pos1, int pos2) {
  int i, j;
  penalty_min (0);
  ghost ("", path_down (p, 0));
  ghost ("<", path_down (p, 0), 1);
  ghost (type, path_down (p, 0), 1);
  for (i=0; i<N(t); i++) {
    print (space (0, 0, env->fn->spc->max));
    int start= N(a);
    print (space (0, 0, env->fn->spc->max));
    ghost ("|", path_down (path_down (p, i), 0), 2);
    print (space (0, 0, env->fn->spc->max));
    if (i<N(t)-1) penalty_min (0);
    if (i<pos1) env->set (COLOR, "dark green");
    typeset (t[i], path_down (p, i));
    if (i<pos1) env->reset (COLOR);
    // ghost ("}", path_down (path_down (p, i), right_index (t[i])), 1);
    int end= N(a);
    if (i<pos2) for (j=start; j<end; j++) {
      a[j]->p= a[j]->q= path_down (p, 0);
      a[j]->f= 1;
    }
  }
  if (i<=pos2) ghost (">", path_down (p, 1), 2);
  else ghost (">", path_down (path_down (p, i-1), right_index (t[i-1])), 1);
  ghost ("", path_down (p, 1));
  print (space (0, 0, env->fn->spc->max));
  penalty_min (0);
}

void
liner_rep::typeset_inactive_symbol (tree t, path p) {
  penalty_min (0);
  ghost ("", path_down (p, 0));
  ghost ("<", path_down (p, 0), 2);
  env->set (COLOR, "blue");
  typeset (t[0], path_down (p, 0));
  env->reset (COLOR);
  ghost (">", path_down (p, 1), 1);
  ghost ("", path_down (p, 1));
  print (space (0, 0, env->fn->spc->max));
  penalty_min (0);
}

void
liner_rep::typeset_inactive_latex (tree t, path p) {
  penalty_min (0);
  ghost ("", path_down (p, 0));
  ghost ("\\", path_down (p, 0), 2);
  env->set (COLOR, "dark green");
  typeset (t[0], path_down (p, 0));
  env->reset (COLOR);
  ghost ("", path_down (p, 1));
  penalty_min (0);
}

void
liner_rep::typeset_inactive_hybrid (tree t, path p) {
  penalty_min (0);
  ghost ("", path_down (p, 0));
  ghost ("<", path_down (p, 0), 2);
  ghost ("\\", path_down (p, 0), 2);
  env->set (COLOR, "dark green");
  typeset (t[0], path_down (p, 0));
  env->reset (COLOR);
  ghost (">", path_down (path_down (p, N(t)-1), right_index (t[N(t)-1])), 1);
  // ghost (">", path_down (p, 1), 1);
  ghost ("", path_down (p, 1));
  penalty_min (0);
}

void
liner_rep::typeset_inactive_apply (tree t, path p) {
  int i;
  penalty_min (0);
  ghost ("", path_down (p, 0));
  ghost ("<", path_down (path_down (p, 0), 0), 2);
  env->set (COLOR, "dark green");
  typeset (t[0], path_down (p, 0));
  env->reset (COLOR);
  for (i=1; i<N(t); i++) {
    print (space (0, 0, env->fn->spc->max));
    ghost ("|", path_down (path_down (p, i), 0), 2);
    print (space (0, 0, env->fn->spc->max));
    if (i<N(t)-1) penalty_min (0);
    typeset (t[i], path_down (p, i));
  }
  ghost (">", path_down (path_down (p, i-1), right_index (t[i-1])), 1);
  ghost ("", path_down (p, 1));
  print (space (0, 0, env->fn->spc->max));
  penalty_min (0);
}

void
liner_rep::typeset_inactive_string (string s, path p) {
  penalty_min (0);
  ghost ("", path_down (p, 0));
  ghost (s, path_down (p, 1), 2);
  ghost ("", path_down (p, 1));
  print (space (0, 0, env->fn->spc->max));
  penalty_min (0);
}

void
liner_rep::typeset_unknown (string which, tree t, path p, bool flag) {
  int i;
  penalty_min (0);
  ghost ("", path_down (p, 0));
  int start= N(a);
  ghost ("<", path_down (p, 0), 1);
  ghost (which, path_down (p, 0), 1, env->dis->red);
  for (i= (flag? 1: 0); i<N(t); i++) {
    print (space (0, 0, env->fn->spc->max));
    ghost ("|", path_down (path_down (p, i), 0), 2);
    print (space (0, 0, env->fn->spc->max));
    if (i<N(t)-1) penalty_min (0);
    if (i==0) env->set (COLOR, "red");
    typeset (t[i], path_down (p, i));
    if (i==0) env->reset (COLOR);
    // ghost ("}", path_down (path_down (p, i), right_index (t[i])), 1);
  }
  ghost (">", path_down (path_down (p, i-1), right_index (t[i-1])), 1);
  int end= N(a);
  for (i=start; i<end; i++) {
    a[i]->p= a[i]->q= path_down (p, 1);
    a[i]->f= 2;
  }
  ghost ("", path_down (p, 1));
  print (space (0, 0, env->fn->spc->max));
  penalty_min (0);
}

/******************************************************************************
* Typesetting environment changes and executable trees
******************************************************************************/

void
liner_rep::typeset_assign (tree t, path p) {
  string var= env->exec(t[0])->label;
  if (N(t)==2) {
    env->assign (var, t[1]);
    if (env->var_type [var] == Env_Paragraph)
      control (tree ("env_par", var, env->get (var)), p);
    else if (env->var_type [var] == Env_Page)
      control (tree ("env_page", var, env->get (var)), p);
    else control (t, p);
  }
}

void
liner_rep::typeset_set (tree t, path p) {
  string var= env->exec(t[0])->label;
  env->set (var, t[1]);
  if ((env->var_type [var] == Env_Paragraph) ||
      (env->var_type [var] == Env_Font_Size))
    control (tree ("env_par", var, env->get (var)), p);
  else if (env->var_type [var] == Env_Page)
    control (tree ("env_page", var, env->get (var)), p);
  else control (t, p);
}

void
liner_rep::typeset_reset (tree t, path p) {
  env->reset (env->exec(t[0])->label);
  control (t, p);
}

void
liner_rep::typeset_apply (tree t, path p) {
  string var= env->exec (t[0])->label;
  if (!env->env->contains (var))
    typeset_unknown (var, t, p, TRUE);
  else typeset_executable (t, p);
}

void
liner_rep::typeset_begin (tree t, path p) {
  string var= env->exec (t[0])->label;
  if (!env->env->contains (var))
    typeset_unknown ("begin", t, p);
  else typeset_executable (t, p);
}

void
liner_rep::typeset_end (tree t, path p) {
  string var= env->exec (t[0])->label;
  if (!env->env->contains (var))
    typeset_unknown ("end", t, p);
  else typeset_executable (t, p);
}

void
liner_rep::typeset_executable (tree t, path p) {
  ghost ("", path_down (p, 0));
  env->save ();
  tree r= env->exec (t);
  env->recover ();
  int start= N(a);
  typeset (r, path ());
  int end= N(a), i;

  SI total= 0, half= 0, mid;
  for (i=start; i<end; i++)
    total += a[i]->b->w();
  for (mid=start; mid<end; mid++) {
    half += a[mid]->b->w();
    if (half >= (total>>1)) break;
  }
  for (i=start; i<mid; i++) {
    a[i]->p= a[i]->q= path_down (p, 0);
    a[i]->f= 1;
  }
  if (i<end) {
    if ((a[i]->type==STD_ITEM) || (a[i]->type==STRING_ITEM))
      a[i]->b= macro_box (a[i]->b);
    a[i]->p= path_down (p, 0);
    a[i]->q= path_down (p, 1);
    a[i]->f= 3;
    i++;
  }
  for (; i<end; i++) {
    a[i]->p= a[i]->q= path_down (p, 1);
    a[i]->f= 2;
  }
  ghost ("", path_down (p, 1));
}

void
liner_rep::typeset_label (tree t, path p) {
  string key  = env->exec (t[0])->label;
  tree   value= env->get ("thelabel");
  env->local_ref (key)= copy (value);
  control ("label", p);
}

void
liner_rep::typeset_reference (tree t, path p) {
  ghost ("", path_down (p, 0));
  int start= N(a);
  string key= env->exec (t[0])->label;
  tree value=
    env->local_ref->contains (key)?
    env->local_ref [key]: env->global_ref [key];
  typeset (value, path ());
  int end= N(a), i;
  for (i=start; i<end; i++) {
    a[i]->p= a[i]->q= path_down (p, 1);
    a[i]->f= 2;
  }
  ghost ("", path_down (p, 1));  
}

void
liner_rep::typeset_write (tree t, path p) {
  if (N(t)==2) {
    string s= env->exec (t[0])->label;
    tree   r= env->exec (t[1]);
    if (env->complete) {
      if (!env->local_aux->contains (s))
	env->local_aux (s)= tree (DOCUMENT);
      env->local_aux (s) << r;
    }
  }
  control ("write", p);
}

/******************************************************************************
* Typesetting generic objects
******************************************************************************/

#define ACTIVATED (active_flag&&(!env->get_bool(PREAMBLE)))

void
liner_rep::typeset (tree t, path p, bool active_flag) {
  if (N(t)==0) {
    typeset_string (t, p);
    return;
  }
  
  switch (math_primary [t->label]) {
  case Concat:
    typeset_concat (t, p);
    break;
  case Format:
    if (ACTIVATED) {
      if (t[0]->label == NO_FIRST_INDENT)
	control (tree ("env_par", PAR_FIRST, "0cm"), p);
      else if (t[0]->label == NO_LAST_INDENT)
	control (tree ("env_par", PAR_LAST, "0cm"), p);
      else if (t[0]->label == YES_FIRST_INDENT)
	control (tree ("env_par", PAR_FIRST, env->get (PAR_FIRST)), p);
      else if (t[0]->label == YES_LAST_INDENT)
	control (tree ("env_par", PAR_LAST, env->get (PAR_LAST)), p);
      else control (t, p);
    }
    else typeset_inactive_string ("<" * t[0]->label * ">", p);
    break;
  case Hspace:
    if (ACTIVATED) {
      env->save (); t= env->exec (t); env->recover ();
      typeset_hspace (t, p);
    }
    else typeset_inactive ("hspace", t, p);
    break;
  case Vspace_before:
    if (ACTIVATED) {
      env->save (); t= env->exec (t); env->recover ();
      control (t, p);
    }
    else typeset_inactive ("vspace*", t, p);
    break;
  case Vspace_after:
    if (ACTIVATED) {
      env->save (); t= env->exec (t); env->recover ();
      control (t, p);
    }
    else typeset_inactive ("vspace", t, p);
    break;
  case Space:
    if (ACTIVATED) {
      env->save (); t= env->exec (t); env->recover ();
      box b= typesetter_rep::typeset (t);
      print (STD_ITEM, b, path_down (p, 0), path_down (p, 1));
    }
    else typeset_inactive ("space", t, p);
    break;
  case Htab:
    if (ACTIVATED) {
      print (space (env->decode_length (t[0]->label)));
      control (t, p);
    }
    else typeset_inactive_string ("<tab>", p);
    break;
  case Split:
    if (ACTIVATED) control (t, p);
    else typeset_inactive ("split", t, p);
    break;
  case Move:
    if (ACTIVATED) {
      box b= typesetter_rep::typeset (t);
      print (STD_ITEM, b, path_down (p, 0), path_down (p, 1));
    }
    else typeset_inactive ("move", t, p);
    break;
  case Resize:
    if (ACTIVATED) {
      box b= typesetter_rep::typeset (t);
      print (STD_ITEM, b, path_down (p, 0), path_down (p, 1));
    }
    else typeset_inactive ("resize", t, p);
    break;
  case Left_sub:
  case Left_sup:
    typeset_script (t, p, FALSE);
    break;
  case Right_sub:
  case Right_sup:
    typeset_script (t, p, TRUE);
    break;
  case Inactive:
    typeset_inactive (t, p);
    break;
  case Symbol:
    typeset_inactive_symbol (t, p);
    break;
  case Latex:
    typeset_inactive_latex (t, p);
    break;
  case Hybrid:
    typeset_inactive_hybrid (t, p);
    break;
  case Quote:
    typeset_inactive ("quote", t, p);
    break;
  case Assign:
    if (ACTIVATED) typeset_assign (t, p);
    else typeset_inactive ("assign", t, p, 1);
    break;
  case Set:
    if (ACTIVATED || (t[0]->label==MODE)) typeset_set (t, p);
    else typeset_inactive ("set", t, p, 1, 1);
    break;
  case Reset:
    if (ACTIVATED || (t[0]->label==MODE)) typeset_reset (t, p);
    else typeset_inactive ("reset", t, p, 1, 1);
    break;
  case Apply:
    if (ACTIVATED || (t[0]->label==MODE)) typeset_apply (t, p);
    else typeset_inactive_apply (t, p);
    break;
  case Begin:
    if (ACTIVATED) typeset_begin (t, p);
    else typeset_inactive ("begin", t, p, 1, 1);
    break;
  case End:
    if (ACTIVATED) typeset_end (t, p);
    else typeset_inactive ("end", t, p, 1, 1);
    break;
  case Function:
    typeset_inactive ("function", t, p, N(t)-1);
    break;
  case Environment:
    typeset_inactive ("environment", t, p, N(t)-2);
    break;
  case Label:
    if (ACTIVATED) typeset_label (t, p);
    else typeset_inactive ("label", t, p);
    break;
  case Reference:
    if (ACTIVATED) typeset_reference (t, p);
    else typeset_inactive ("reference", t, p);
    break;
  case Write:
    if (ACTIVATED) typeset_write (t, p);
    else typeset_inactive ("write", t, p);
    break;
  case Plus:
    if (ACTIVATED) typeset_executable (t, p);
    else typeset_inactive ("+", t, p);
    break;
  case Minus:
    if (ACTIVATED) typeset_executable (t, p);
    else typeset_inactive ("-", t, p);
    break;
  case Times:
    if (ACTIVATED) typeset_executable (t, p);
    else typeset_inactive ("x", t, p);
    break;
  case Merge:
    if (ACTIVATED) typeset_executable (t, p);
    else typeset_inactive ("*", t, p);
    break;
  case Number:
    if (ACTIVATED) typeset_executable (t, p);
    else typeset_inactive ("number", t, p);
    break;
  case Translate:
    if (ACTIVATED) typeset_executable (t, p);
    else typeset_inactive ("translate", t, p);
    break;
  case Graphics:
  case Point:
  case Line:
  case Arc:
  case Bezier:
    break;
  case Postscript:
    if (ACTIVATED) {
      box b= typesetter_rep::typeset (t);
      print (STD_ITEM, b, path_down (p, 0), path_down (p, 1));
    }
    else typeset_inactive ("postscript", t, p);
    break;
  default:
    {
      box b= typesetter_rep::typeset (t);
      print (STD_ITEM, b, path_down (p, 0), path_down (p, 1));
    }
  }
}

#undef ACTIVATED

#endmodule // code_line_pass1
