
/******************************************************************************
* MODULE     : line_pass2.gen.cc
* DESCRIPTION: Second pass for typesetting paragraphs
*                - The real heights of brackets are determined
*                - scripts are glued
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#module code_line_pass2

/******************************************************************************
* Miscellaneous routines
******************************************************************************/

int
liner_rep::prec (int i) {
  do i--; while ((i>=0) && (a[i]->type==OBSOLETE_ITEM));
  return i;
}

int
liner_rep::succ (int i) {
  do i++; while ((i<N(a)) && (a[i]->type==OBSOLETE_ITEM));
  return i;
}

/******************************************************************************
* Gluing scripts
******************************************************************************/

void
liner_rep::pre_glue () {
  int i;
  for (i=0; i<N(a)-1; i++) {
    line_item item1= a[i];
    line_item item2= a[i+1];
    int t1= item1->type;
    int t2= item2->type;
    if (((t1 == RIGHT_SUB_ITEM) && (t2 == RIGHT_SUP_ITEM)) ||
	((t1 == RIGHT_SUP_ITEM) && (t2 == RIGHT_SUB_ITEM)) ||
	((t1 == LEFT_SUB_ITEM) && (t2 == LEFT_SUP_ITEM)) ||
	((t1 == LEFT_SUP_ITEM) && (t2 == LEFT_SUB_ITEM)))
      {
	bool  flag1 = (t1 == LEFT_SUB_ITEM) || (t1 == RIGHT_SUB_ITEM);
	bool  flag2 = (t1 == LEFT_SUB_ITEM) || (t1 == LEFT_SUP_ITEM);
	int   type  = flag2? GLUE_LEFT_SUBS_ITEM: GLUE_RIGHT_SUBS_ITEM;
	box   b1    = flag1? item1->b[0]: item2->b[0];
	box   b2    = flag1? item2->b[0]: item1->b[0];
	box   b     = script_box (b1, b2);
	int   pen   = item2->penalty;
	space spc   = max (item1->spc, item2->spc);
	int   data  = (item1->data | item2->data)+ (flag1? 2: 4);

	line_item* items= new line_item[2];
	items[0]= item1;
	items[1]= item2;
	a[i]= line_item (type, b, item1->p, item2->q,
			 (item1->f&1) | (item2->f&2), pen, items);
	a[i]->spc = spc;
	a[i]->data= data;
	a[i+1]= line_item (OBSOLETE_ITEM, item2->b, item2->p, item2->q,
			   item2->f, pen);
      }
  }
}

bool
liner_rep::script_flag () {
  int i, j;
  for (i=0; i<N(a); i++) {
    if (a[i]->type==GLUE_LEFT_SUBS_ITEM) {
      j= succ (i);
      if ((j<N(a)) && (a[j]->type==STRING_ITEM)) return TRUE;
    }
    if (a[i]->type==GLUE_RIGHT_SUBS_ITEM) {
      j= prec (i);
      if ((j>=0) && (a[j]->type==STRING_ITEM)) return TRUE;
    }
  }
  return FALSE;
}

void
liner_rep::glue (box b, int ref, int arg) {
  space spc = max (a[ref]->spc, a[arg]->spc);
  int   data= a[arg]->data;

  line_item* items= new line_item[2];
  items[0]= a[ref];
  items[1]= a[arg];
  a[arg]  = line_item (OBSOLETE_ITEM, a[arg]->b,
		       a[arg]->p, a[arg]->q, a[arg]->f, a[arg]->penalty);
  a[ref]  = line_item (arg<ref? GLUE_LEFT_ITEM: GLUE_RIGHT_ITEM, b,
		       arg<ref? a[arg]->p: a[ref]->p,
		       arg<ref? a[ref]->q: a[arg]->q,
		       arg<ref? (a[arg]->f&1) | (a[ref]->f&2)
		              : (a[ref]->f&1) | (a[arg]->f&2),
		       min (a[ref]->penalty, a[arg]->penalty), items);
  a[ref]->spc = spc;
  a[ref]->data= data;
}

void
liner_rep::glue (box b, int ref, int arg1, int arg2) {
  space spc = max (a[ref]->spc, max (a[arg1]->spc, a[arg2]->spc));
  int   data= (a[arg1]->data<<3)+ a[arg2]->data;
  int   pen = min (a[ref]->penalty, min (a[arg1]->penalty, a[arg2]->penalty));

  space ref_spc= a[ref]->spc;
  line_item* items= new line_item[3];
  items[0]= a[ref];
  items[1]= a[arg1];
  items[2]= a[arg2];
  a[arg1]= line_item (OBSOLETE_ITEM, a[arg1]->b,
		      a[arg1]->p, a[arg1]->q, a[arg1]->f, a[arg1]->penalty);
  a[arg2]= line_item (OBSOLETE_ITEM, a[arg2]->b,
		      a[arg2]->p, a[arg2]->q, a[arg2]->f, a[arg2]->penalty);
  a[ref]= line_item (GLUE_BOTH_ITEM, b, a[arg1]->p, a[arg2]->q,
		     (a[arg1]->f&1) | (a[arg2]->f&2), pen, items);
  a[ref]->spc = spc;
  a[ref]->data= data;
}

void
liner_rep::handle_scripts (int start, int end) {
  int i;
  for (i=start; i<=end; ) {
    if ((a[i]->type == OBSOLETE_ITEM) ||
	(a[i]->type == LEFT_SUB_ITEM) ||
	(a[i]->type == LEFT_SUP_ITEM) ||
	(a[i]->type == GLUE_LEFT_SUBS_ITEM) ||
	(a[i]->type == RIGHT_SUB_ITEM) ||
	(a[i]->type == RIGHT_SUP_ITEM) ||
	(a[i]->type == GLUE_RIGHT_SUBS_ITEM)) { i++; continue; }

    int l= prec (i);
    box lb1, lb2;
    double ll= 0.0;
    if (l < start) l= -1;
    else switch (a[l]->type) {
    case LEFT_SUB_ITEM:
      lb1= a[l]->b[0];
      ll= level1;
      break;
    case LEFT_SUP_ITEM:
      lb2= a[l]->b[0];
      ll= level2;
      break;
    case GLUE_LEFT_SUBS_ITEM:
      lb1= a[l]->b[0];
      lb2= a[l]->b[1];
      if ((a[l]->data&1)==1) ll= level1;
      else ll= (level1+level2)/2;
      break;
    default:
      l = -1;
    }

    int r= succ (i);
    box rb1, rb2;
    double rl= 0.0;
    if (r > end) r= N(a);
    else switch (a[r]->type) {
    case RIGHT_SUB_ITEM:
      rb1= a[r]->b[0];
      rl= level1;
      break;
    case RIGHT_SUP_ITEM:
      rb2= a[r]->b[0];
      rl= level2;
      break;
    case GLUE_RIGHT_SUBS_ITEM:
      rb1= a[r]->b[0];
      rb2= a[r]->b[1];
      if ((a[r]->data&1)==1) rl= level1;
      else rl= (level1+level2)/2;
      break;
    default:
      r = N(a);
    }

    box b;
    if ((l==-1) && (r==N(a))) { i++; continue; }
    if ((l!=-1) && (r==N(a))) {
      b= left_script_box (a[i]->b, lb1, lb2, ll);
      glue (b, i, l);
    }
    if ((l==-1) && (r!=N(a))) {
      if (a[i]->type==LIMIT_ITEM) b= limit_box (a[i]->b, rb1, rb2);
      else b= right_script_box (a[i]->b, rb1, rb2, rl);
      glue (b, i, r);
    }
    if ((l!=-1) && (r!=N(a))) {
      double level= ((ll=rl)? ll : (level1+level2)/2);
      b= side_box (a[i]->b, lb1, lb2, rb1, rb2, level);
      glue (b, i, l, r);
    }
  }
}

void
liner_rep::eliminate_obsolete () {
  array<line_item> b;
  int i;
  for (i=0; i<N(a); i++)
    if (a[i]->type!=OBSOLETE_ITEM) b << a[i];
  a= b;
}

/******************************************************************************
* Resize brackets
******************************************************************************/

void
liner_rep::handle_matching (int start, int end) {
  int i;
  SI y1= min (a[start]->b->sub_base(), a[end]->b->sub_base());
  SI y2= max (a[start]->b->sup_base(), a[end]->b->sup_base());
  a[start]->penalty++;
  a[end]->penalty++;
  // cout << "Start: " << (y2-y1) << "\n";
  for (i=start+1; i<end; i++) {
    if (a[i]->type == OBSOLETE_ITEM) continue;
    // cout << "  " << a[i] << ": " << (a[i]->b->y2- a[i]->b->y1) << "\n";
    // y1= min (y1, a[i]->b->sub_base());
    // y2= max (y2, a[i]->b->sup_base());
    y1= min (y1, a[i]->b->y1);
    y2= max (y2, a[i]->b->y2);
    a[i]->penalty++;
  }
  for (i=start; i<=end; i++)
    if ((a[i]->type==LEFT_BRACKET_ITEM) ||
	(a[i]->type==MIDDLE_BRACKET_ITEM) ||
	(a[i]->type==RIGHT_BRACKET_ITEM))
      {
	a[i]->b   = a[i]->fn->convert (a[i]->s, a[i]->col, y1, y2);
	a[i]->type= STD_ITEM;
      }
}

void
liner_rep::handle_brackets () {
  int first=-1, start=0, i=0;
  while (i<N(a)) {
    if (a[i]->type==LEFT_BRACKET_ITEM) {
      if (first==-1) first= i;
      start= i;
    }
    if (a[i]->type==RIGHT_BRACKET_ITEM) {
      handle_scripts  (succ (start), prec (i));
      handle_matching (start, i);
      if (first!=-1) i=first-1;
      start= 0;
      first= -1;
    }
    i++;
  }
  if (N(a)>0) {
    handle_scripts  (0, N(a)-1);
    handle_matching (0, N(a)-1);
  }
}

/******************************************************************************
* Kill invalid spaces
******************************************************************************/

void
liner_rep::kill_spaces () {
  int i;
  for (i=0; i<N(a); i++)
    if (a[i]->type==CONTROL_ITEM) {
      if (is_func (a[i]->t, RESET) ||
	  is_func (a[i]->t, END))
	if (i>0) a[i-1]->spc= space (0);

      if (is_func (a[i]->t, FORMAT)) {
	string s= a[i]->t[0]->label;
	if ((s==NEXT_LINE) || (s==LINE_SEP) ||
	    (s==LINE_BREAK) || (s==NEW_LINE))
	  {
	    if (i>0) a[i-1]->spc= space (0);
	    a[i]->spc= space (0);
	  }
      }
      
      if (is_func (a[i]->t, SET) ||
	  is_func (a[i]->t, "env_par") ||
	  is_func (a[i]->t, "env_page") ||
	  is_func (a[i]->t, BEGIN))
	a[i]->spc= space (0);
    }
}

/******************************************************************************
* Main control
******************************************************************************/

void
liner_rep::pass2 () {
  kill_spaces ();
  pre_glue ();
  // bool flag= script_flag ();
  level1 = 0.9; level2 = 0.3;
  if (env->vert_pos==-1) level2 -= 0.2;
  if (env->vert_pos== 1) level1 += 0.1;
  // if (flag) level1= level2= (level1 + 2*level2)/3;
  handle_brackets ();
  eliminate_obsolete ();
}

#endmodule // code_line_pass2
