
/******************************************************************************
* MODULE     : line_pass3.gen.cc
* DESCRIPTION: Last pass for typesetting paragraphs;
*              hyphenation and creation of page items
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#module code_line_pass3

/******************************************************************************
* Filling lines of paragraphs
******************************************************************************/

path
liner_rep::treat_glue (line_item item) {
  switch (item->type) {
  case GLUE_LEFT_ITEM: {
    (void) treat_glue (item->glue[1]);
    path ctrl= treat_glue (item->glue[0]);
    return path (GLUE_LEFT_ITEM, path (item->data, ctrl));
  }
  case GLUE_RIGHT_ITEM: {
    path ctrl= treat_glue (item->glue[0]);
    (void) treat_glue (item->glue[1]);
    return path (GLUE_RIGHT_ITEM, path (item->data, ctrl));
  }
  case GLUE_BOTH_ITEM: {
    (void) treat_glue (item->glue[1]);
    path ctrl= treat_glue (item->glue[0]);
    (void) treat_glue (item->glue[2]);
    return path (GLUE_BOTH_ITEM, path (item->data, ctrl));
  }
  case GLUE_LEFT_SUBS_ITEM:
  case GLUE_RIGHT_SUBS_ITEM: {
    items_st << copy (item->glue[0]->p);
    items_en << copy (item->glue[0]->q);
    items_fl << item->glue[0]->f;
    items_st << copy (item->glue[1]->p);
    items_en << copy (item->glue[1]->q);
    items_fl << item->glue[1]->f;
    return path ();
  }
  default:
    items_st << copy (item->p);
    items_en << copy (item->q);
    items_fl << item->f;
    return path ();
  }
}

void
liner_rep::line_print (line_item item) {
  // cout << "Printing: " << item << "\n";
  if (item->type == CONTROL_ITEM) {
    if (is_func (item->t, HTAB)) tabs << N(items);
    if (is_func (item->t, VSPACE_BEFORE) ||
	is_func (item->t, VSPACE_AFTER))
    {
      SI vmin, vdef, vmax;
      if (N(item->t)==1) {
	vmin= env->decode_length (item->t[0]->label * "-");
	vdef= env->decode_length (item->t[0]->label);
	vmax= env->decode_length (item->t[0]->label * "+");
      }
      else {
	vmin= env->decode_length (item->t[0]->label);
	vdef= env->decode_length (item->t[1]->label);
	vmax= env->decode_length (item->t[2]->label);
      }
      if (is_func (item->t, VSPACE_BEFORE))
	par->vspace_before (space (vmin, vdef, vmax));
      else par->vspace_after (space (vmin, vdef, vmax));
    }
    if (is_func (item->t, "env_page") ||
	(item->t == tree (FORMAT, PAGE_BREAK)) ||
	(item->t == tree (FORMAT, NEW_PAGE)))
      par->print (item->t);
    if (item->t == tree (FORMAT, NO_PAGE_BREAK_BEFORE))
      par->no_page_break_before ();
    if (item->t == tree (FORMAT, NO_PAGE_BREAK_AFTER))
      par->no_page_break_after ();
  }

  if (N(spcs)>0) cur_w = cur_w + spcs[N(spcs)-1];
  items << item->b;
  spcs  << item->spc;
  path ctrl= treat_glue (item);
  if ((item->type==GLUE_LEFT_SUBS_ITEM) || (item->type==GLUE_RIGHT_SUBS_ITEM))
    scripts << path (N (items_st)- 2, path (item->type, item->data));
  if (!nil (ctrl))
    scripts << path (N (items_st)- script_gap (ctrl)- 1, ctrl);
  item->b->x0 =  cur_w->def;
  item->b->y0 =  0;
  cur_w =  cur_w + space (item->b->x2);
}

void
liner_rep::line_print (line_item item, path left, path right) {
  if (nil (left) && nil (right)) line_print (item);
  else {
    line_item item1, item2;
    hyphenate (item, nil (left)? right->item: left->item, item1, item2);
    line_print (nil (left) ? item1: item2,
		nil (left) ? left : left->next,
		nil (right)? right: right->next);
  }
}

void
liner_rep::line_print (path start, path end) {
  if (start->item == end->item)
    line_print (a[start->item], start->next, end->next);
  else {
    int i;
    line_print (a[start->item], start->next, path ());
    for (i=start->item+1; i<end->item; i++)
      line_print (a[i]);
    if (!atom (end))
      line_print (a[end->item], path (), end->next);
  }
}

/******************************************************************************
* Typesetting a line
******************************************************************************/

void
liner_rep::make_unit (string mode, SI the_width, bool break_flag) {
  int i;

  // format tabs
  //cout << "      " << N(tabs) << "] " << (cur_w->def/PIXEL)
  //     << " < " << (the_width/PIXEL) << "? (" << break_flag << ")\n";
  if (break_flag && (N(tabs)>0) && (cur_w->def<the_width)) {
    SI tab_spc= (the_width- cur_w->def)/N(tabs);
    for (i=cur_start; i<N(items)-1; i++) items_sp << spcs[i]->def;
    for (i=0; i<N(tabs); i++) items_sp[tabs[i]] += tab_spc;
    return;
  }

  // stretching case
  if (mode == "justify") {
    if ((cur_w->def < the_width) &&
	(cur_w->max > cur_w->def) &&
	(!break_flag)) {
      double f=
	((double) (the_width - cur_w->def)) /
	((double) (cur_w->max - cur_w->def));
      for (i=cur_start; i<N(items)-1; i++)
	items_sp <<
	  (spcs[i]->def+ ((SI) (f*((double) spcs[i]->max- spcs[i]->def))));
      return;
    }
  }
  
  // shrinking case
  if ((cur_w->def > the_width) &&
      (cur_w->def > cur_w->min)) {
    double f=
      ((double) (cur_w->def - the_width)) /
      ((double) (cur_w->def - cur_w->min));
    if (f>1.0) f=1.0;
    for (i=cur_start; i<N(items)-1; i++)
      items_sp <<
	(spcs[i]->def- ((SI) (f*((double) spcs[i]->def- spcs[i]->min))));
    return;
  }

  if (mode == "center")
    items_sp[cur_start] += (the_width- cur_w->def) >> 1;
  if (mode == "right")
    items_sp[cur_start] += the_width- cur_w->def;
  for (i=cur_start; i<N(items)-1; i++)
    items_sp << spcs[i]->def;
}

/******************************************************************************
* Making lines
******************************************************************************/

void
liner_rep::line_start () {
  items   = array<box> ();
  items_st= array<path> ();
  items_en= array<path> ();
  items_fl= array<int> ();
  items_sp= array<SI> ();
  scripts = array<path> ();
  spcs    = array<space> ();

  cur_r    = 0;
  cur_start= 0;
}

void
liner_rep::line_unit (path start, path end, bool break_flag,
		      string mode, SI the_left, SI the_right)
{
  cur_w= space (0);
  tabs = array<int> ();
  int n= N(items_sp);
  SI  m= max (the_left- cur_r, 0);
  items_sp << m;

  SI the_width= the_right- the_left;
  line_print (start, end);
  make_unit (mode, the_width, break_flag);

  int i;
  cur_r= the_left+ items_sp[n]- m;
  for (i= cur_start; i<N(items); i++)
    cur_r += items[i]->w()+ (i<N(items)-1? items_sp[i+1]: 0);
  cur_start= N(items);
}

void
liner_rep::line_end (space spc, int penalty) {
  if (N(items)==0) return;
  box b= new par_line_box_rep (items, items_sp,
			       items_st, items_en, items_fl, scripts);
  par->print (b, items_st[0], items_en[N(items_en)-1],
	      (items_fl[0]&1) | (items_fl[N(items_en)-1]&2));
  par->print (spc);
  par->penalty (penalty);
  par->flush ();
}

void
liner_rep::line_units (int start, int end,
		       bool is_start, bool is_end, string mode, string hyphen,
		       SI the_left, SI the_right, SI the_first, SI the_last)
{
  if (start == end) return;
  // cout << "  Line units " << start << ", " << end << "\n";
  // cout << "    is_end   : " << is_end << "\n";
  // cout << "    mode     : " << mode << "\n";
  // cout << "    hyphen   : " << hyphen << "\n";
  // cout << "    the_left : " << (the_left/PIXEL) << "\n";
  // cout << "    the_right: " << (the_right/PIXEL) << "\n";

  int i;
  bool ragged= (hyphen == "normal");
  array<path> hyphs= line_breaks (a, start, end, the_right-the_left,
				  the_first, the_last, ragged);
  for (i=0; i<N(hyphs)-1; i++) {
    if (i>0) line_start ();
    line_unit (hyphs[i], hyphs[i+1], i==N(hyphs)-2, mode,
	       the_left+ (is_start&&(i==0)? the_first: 0),
	       the_right- (is_end&&(i==N(hyphs)-2)? the_last: 0));
    if (i<N(hyphs)-2) line_end (line_spc, 1);
  }
  // cout << "    Done!\n";
}

/******************************************************************************
* Get environment parameters
******************************************************************************/

void
liner_rep::get_env_vars () {
  env->set (FONT_BASE_SIZE, style [FONT_BASE_SIZE]);
  env->set (FONT_SIZE, style [FONT_SIZE]);

  SI d1, d2, d3, d4, d5, d6, d7;
  env->get_page_pars (width, d1, d2, d3, d4, d5, d6, d7);

  mode      = style [PAR_MODE]->label;
  hyphen    = style [PAR_HYPHEN]->label;
  left      = env->decode_length (style [PAR_LEFT]->label);
  right     = env->decode_length (style [PAR_RIGHT]->label);
  first     = env->decode_length (style [PAR_FIRST]->label);
  last      = env->decode_length (style [PAR_LAST]->label);
  bot       = env->fn->ysub;
  top       = env->fn->ysup;
  sep       = env->decode_length (style [PAR_SEP]->label);
  height    = env->decode_length ("1fn")+ sep;
  hor_sep   = env->decode_length (style [PAR_HOR_SEP]->label);
  tab_sep   = hor_sep;

  SI l_min  = env->decode_length (style [PAR_LINE_SPC]->label * "-");
  SI l_def  = env->decode_length (style [PAR_LINE_SPC]->label);
  SI l_max  = env->decode_length (style [PAR_LINE_SPC]->label * "+");
  line_spc  = space (l_min, l_def, l_max);
  SI p_min  = env->decode_length (style [PAR_PAR_SPC]->label * "-");
  SI p_def  = env->decode_length (style [PAR_PAR_SPC]->label);
  SI p_max  = env->decode_length (style [PAR_PAR_SPC]->label * "+");
  par_spc   = space (p_min, p_def, p_max);

  if (mode == "center") { first= last= 0; }
  width -= right;

  env->reset (FONT_SIZE);
  env->reset (FONT_BASE_SIZE);
}

/******************************************************************************
* Typesetting a table
******************************************************************************/

array<SI>
liner_rep::compute_tabular_widths (int split, int start, int end) {
  // cout << "t    = " << a[start]->t << "\n";
  // cout << "start= " << start << "\n";
  // cout << "end  = " << end << "\n";
  
  int i, n= N (a[split]->t);
  array<SI> w (n);
  for (i=0; i<n; i++) w[i]= 0;

  int current= 0, last= start;
  for (i=start; i<=end; i++)
    if ((i==end) || ((a[i]->type == CONTROL_ITEM) &&
		     ((a[i]->t == tree (FORMAT, LINE_SEP)) ||
		      (a[i]->t == tree (FORMAT, NEXT_LINE)))))
      {
	int j;
	SI sum= 0;
	for (j=last; j<i; j++)
	  sum += a[j]->b->w()+ (j<i-1? a[j]->spc->def: 0);
	w [current]= max (w [current], sum);
	if ((i==end) || (a[i]->t != tree (FORMAT, LINE_SEP))) current=0;
	else current= min (current+1, n-1);
	last= i;
      }
  
  SI fit_w= (width- left- ((n-1)*tab_sep))/n;
  while (TRUE) {
    int fit= 0;
    SI  sum_w= 0;
    for (i=0; i<n; i++)
      if (w[i] <= fit_w) {
	fit++;
	sum_w += w[i];
      }
    if (fit==n) break;
    SI new_w= (width- left- (n-1)*tab_sep- sum_w)/(n-fit);
    if (fit_w == new_w) {
      for (i=0; i<n; i++)
	if (w[i] > fit_w) w[i]= fit_w;
      break;
    }
    fit_w= new_w;
  }

  // for (i=0; i<n; i++) cout << i << "] " << (w[i]/PIXEL) << "\n";
  return w;
}

void
liner_rep::typeset_table (int split, int start, int end,
			  bool is_start, bool is_end)
{
  // cout << "Table " << start << ", " << end << "\n";
  int i, j;
  array<SI> w= compute_tabular_widths (split, start, end);
  SI total_w= 0; for (j=0; j<N(w); j++) total_w += w[j];
  SI extra= width- left- right- total_w- ((N(w)-1) * tab_sep);
  if (mode == "right");
  else if (mode == "center") extra= extra>>1;
  else extra= 0;

  int current= 0, last= start;
  for (i=start; i<=end; i++)
    if ((i==end) || ((a[i]->type == CONTROL_ITEM) &&
		     ((a[i]->t == tree (FORMAT, LINE_SEP)) ||
		      (a[i]->t == tree (FORMAT, NEXT_LINE)))))
      {
	bool next_line= (i==end) || (a[i]->t != tree (FORMAT, LINE_SEP));
	SI l= left+ extra; for (j=0; j<current; j++) l += w[j]+ tab_sep;
	string unit_type= a[split]->t[current]->label;
	line_units (last, i, is_start, is_end,
		    unit_type, hyphen,
		    l, l+w[current], 0, 0);
	if (next_line) {
	  if (i<end) {
	    line_end (line_spc, 1);
	    line_start ();
	  }
	  current=0;
	}
	else current= min (current+1, N(w)-1);
	last= i;
      }
  // cout << "Table done\n";
}

/******************************************************************************
* Typesetting a paragraph
******************************************************************************/

void
liner_rep::typeset_paragraph_unit (int the_start, int the_end) {
  // cout << "Paragraph unit " << the_start << ", " << the_end << "\n";
  int i, start= the_start, end= the_start, split= the_start;
  for (i=the_start; i<=the_end; i++) {
    if ((i==the_end) ||
	((a[i]->type == CONTROL_ITEM) &&
	 (a[i]->t == tree (FORMAT, NEXT_LINE))))
    {
      start= end;
      end  = i;
      line_start ();
      line_units (start, end, start==the_start, end==the_end,
		  mode, hyphen,
		  left, width, first, last);
      if (end<the_end) line_end (line_spc, 1);
      else return;
    }

    if ((a[i]->type == CONTROL_ITEM) && (is_func (a[i]->t, SPLIT))) {
      start= end;
      split= i;
      for (i=split+1; i<the_end; i++)
	if ((a[i]->type == CONTROL_ITEM) && (is_func (a[i]->t, SPLIT))) break;
      end= i;
      line_start ();
      typeset_table (split, start, end, start==the_start, end==the_end);
      if (end<the_end) { line_end (line_spc, 1); i--; }
      else return;
    }
  }
  // cout << "Unit done\n";
}

void
liner_rep::pass3 () {
  int start= 0, i, j;
  // cout << "Typeset " << a << "\n";
  for (i=0; i<=N(a); i++) {
    // determine the next unit
    if (i<N(a)) {
      if (a[i]->type != CONTROL_ITEM) continue;
      if (a[i]->t == tree (FORMAT, NEW_LINE));
      else continue;
    }

    // determine the style parameters
    bool no_first= (style [PAR_NO_FIRST] == "true");
    bool no_last = (style [PAR_NO_LAST]  == "true");
    style (PAR_NO_FIRST)= "false";
    style (PAR_NO_LAST) = "false";
    if (no_first) style (PAR_FIRST)= "0cm";
    if (no_last)  style (PAR_LAST) = "0cm";
    for (j=start; j<i; j++)
      if (a[j]->type == CONTROL_ITEM)
	if (is_func (a[j]->t, "env_par"))
	  style (a[j]->t[0]->label)= a[j]->t[1];
    get_env_vars ();
    par->set_env_vars (height, sep, hor_sep, bot, top);

    // typeset paragraph unit
    typeset_paragraph_unit (start, i);
    line_end (space (sep)+ par_spc, 0);
    par->new_paragraph ();

    start= i;
  }
  // cout << "Paragraph done\n";
}

#endmodule // code_line_pass3
