
/******************************************************************************
* MODULE     : liner.gen.cc
* DESCRIPTION: Control routines for typesetting paragraphs
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <Paragraph/liner.gen.h>

#module code_liner
#import liner

/*****************************************************************************/
// Routines for the liner class
/*****************************************************************************/

liner_rep::liner_rep (edit_env env):
  typesetter_rep (env), style ("")
{
  style (FONT_BASE_SIZE) = env->get (FONT_BASE_SIZE);
  style (FONT_SIZE)      = env->get (FONT_SIZE);
  style (PAR_MODE)       = env->get (PAR_MODE);
  style (PAR_HYPHEN)     = env->get (PAR_HYPHEN);
  style (PAR_WIDTH)      = env->get (PAR_WIDTH); 
  style (PAR_LEFT)       = env->get (PAR_LEFT);
  style (PAR_RIGHT)      = env->get (PAR_RIGHT);
  style (PAR_FIRST)      = env->get (PAR_FIRST);
  style (PAR_LAST)       = env->get (PAR_LAST);
  style (PAR_NO_FIRST)   = env->get (PAR_NO_FIRST);
  style (PAR_NO_LAST)    = env->get (PAR_NO_LAST);
  style (PAR_SEP)        = env->get (PAR_SEP);
  style (PAR_HOR_SEP)    = env->get (PAR_HOR_SEP);
  style (PAR_LINE_SPC)   = env->get (PAR_LINE_SPC);
  style (PAR_PAR_SPC)    = env->get (PAR_PAR_SPC);
}

void
liner_rep::typeset_2_passes (tree t) {
  array<path> ps;
  // cout << "Typesetting " << t << "\n";
  
  int i;
  typeset (t, path ());
  for (i=N(a)-1; (i>0) && (a[i]->type == CONTROL_ITEM); i--)
    a[i-1]->spc= space (0);
  for (i=0; (i<N(a)) && (a[i]->type == CONTROL_ITEM); i++)
    a[i]->spc= space (0);

  /*
  cout << "pass 1: " << a << "\n";
  ps= array<path> (N(a));
  for (i=0; i<N(a); i++) ps[i]= a[i]->p;
  cout << "paths : " << ps << "\n";
  */

  pass2 ();

  /*
  cout << "pass 2: " << a << "\n";
  ps= array<path> (N(a));
  for (i=0; i<N(a); i++) ps[i]= a[i]->p;
  cout << "paths : " << ps << "\n";
  */
}

void
liner_rep::typeset_3_passes (tree t) {
  env->assign (PAR_NO_FIRST, "false");
  env->assign (PAR_NO_LAST , "false");
  typeset_2_passes (t);
  pass3 ();

  /*
  int i;
  array<path> ps;
  cout << "pass 3: " << a << "\n";
  ps= array<path> (N(a));
  for (i=0; i<N(a); i++) ps[i]= a[i]->p;
  cout << "paths : " << ps << "\n";
  */
}

/*****************************************************************************/
// Typesetting a concat
/*****************************************************************************/

box
liner_rep::make_concat () {
  int i;
  line_start ();
  line_print (path (0), path (N(a)));
  array<SI> spc (N(spcs));
  spc[0]=0;
  for (i=0; i<N(spcs)-1; i++) spc[i+1]=spcs[i]->def;
  return new par_line_box_rep (items, spc,
			       items_st, items_en, items_fl, scripts);
}

box
typesetter_rep::typeset_concat (tree t) {
  liner L= new liner_rep (env);
  L->typeset_2_passes (t);
  box b= L->make_concat ();
  delete L;
  return b;
}

#endmodule // code_liner
