
/******************************************************************************
* MODULE     : data.gen.h
* DESCRIPTION: the way data are organized for the math_editor
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <path.gen.h>
#include <tree.gen.h>
#include <hashmap.gen.h>

#module data
#import path
#import tree
#import hashmap (string,int)

/******************************************************************************
* Important hashmaps
******************************************************************************/

extern hashmap<string,int> math_primary;
extern hashmap<string,int> math_secondary;
extern hashmap<string,int> math_ternary;

/******************************************************************************
* The special operators from which our expressions are built up
******************************************************************************/

extern string DOCUMENT;
extern string PARAGRAPH;
extern string CONCAT;
extern string FORMAT;
  extern string LINE_BREAK;
  extern string NEW_LINE;
  extern string LINE_SEP;
  extern string NEXT_LINE;
  extern string YES_FIRST_INDENT;
  extern string YES_LAST_INDENT;
  extern string NO_FIRST_INDENT;
  extern string NO_LAST_INDENT;
  extern string PAGE_BREAK;
  extern string NEW_PAGE;
  extern string NO_PAGE_BREAK_BEFORE;
  extern string NO_PAGE_BREAK_AFTER;
extern string HSPACE;
extern string VSPACE_BEFORE;
extern string VSPACE_AFTER;
extern string SPACE;
extern string HTAB;
extern string SPLIT;
extern string MOVE;
extern string RESIZE;

extern string LEFT_SUB;
extern string LEFT_SUP;
extern string RIGHT_SUB;
extern string RIGHT_SUP;
extern string FRAC;
extern string SQRT;
extern string WIDE;
extern string NEG;
extern string TREE;
extern string MATRIX;
extern string TABLE;
extern string MOSAIC;
extern string MOSAIC_ITEM;

extern string INACTIVE;
extern string SYMBOL;
extern string LATEX;
extern string HYBRID;
extern string BACKUP;
extern string QUOTE;
extern string ASSIGN;
extern string SET;
extern string RESET;
extern string APPLY;
extern string BEGIN;
extern string END;
extern string FUNCTION;
extern string ENVIRONMENT;
extern string LABEL;
extern string REFERENCE;
extern string WRITE;

extern string PLUS;
extern string MINUS;
extern string TIMES;
extern string MERGE;
extern string NUMBER;
extern string TRANSLATE;

extern string GRAPHICS;
extern string POINT;
extern string LINE;
extern string CIRCLE;
extern string ARC;
extern string BEZIER;
extern string POLYGON;
extern string POSTSCRIPT;

string& SUB (bool right);
string& SUP (bool right);

#define Unknown            0
#define Document           1
#define Paragraph          2
#define Concat             3
#define Format             4
#define Hspace             5
#define Vspace_before      6
#define Vspace_after       7
#define Space              8
#define Htab               9
#define Split             10
#define Move              11
#define Resize            12

#define Left_sub          13
#define Left_sup          14
#define Right_sub         15
#define Right_sup         16
#define Frac              17
#define Sqrt              18
#define Wide              19
#define Neg               20
#define Tree              21
#define Matrix            22
#define Table             23
#define Mosaic            24
#define Mosaic_item       25

#define Inactive          26
#define Symbol            27
#define Latex             28
#define Hybrid            29
#define Backup            30
#define Quote             31
#define Assign            32
#define Set               33
#define Reset             34
#define Apply             35
#define Begin             36
#define End               37
#define Function          38
#define Environment       39
#define Label             40
#define Reference         41
#define Write             42

#define Plus              43
#define Minus             44
#define Times             45
#define Merge             46
#define Number            47
#define Translate         48

#define Graphics          49
#define Point             50
#define Line              51
#define Arc               52
#define Bezier            53
#define Postscript        54

#define CONSTRUCTOR_NR 55
extern string CONSTRUCTOR_NAME  [CONSTRUCTOR_NR];
extern int    CONSTRUCTOR_ARITY [CONSTRUCTOR_NR];

/******************************************************************************
* Determining properties of an expression
******************************************************************************/

bool is_document (tree t);
bool is_concat (tree t);
bool is_format (tree t);

bool is_tabular (tree t);
bool is_regular (tree t);
bool is_script (tree t, bool& right);
bool is_math (tree t);

bool is_modifier (tree t);
bool is_dynamic (tree t);

bool is_opening (tree t);
bool is_middle (tree t);
bool is_closing (tree t);
bool is_bracket (tree t);
bool is_bigop (tree t);
bool is_prime (tree t);

/******************************************************************************
* Miscellaneous
******************************************************************************/

int  right_index (tree t);
path correct_cursor (tree t, path p);
path start (tree t, path p);
path end (tree t, path p);
path start (tree t);
path end (tree t);

tree math_correct (tree t);

#endmodule // data
