
/******************************************************************************
* MODULE     : env.gen.h
* DESCRIPTION: edit environment for typesetting
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <vars.gen.h>
#include <font.gen.h>
#include <language.gen.h>
#include <rel_hashmap.gen.h>

#module env
#import vars
#import font
#import language
#import rel_hashmap(string,tree)
#import hashmap(string,int)

/******************************************************************************
* The edit environment
******************************************************************************/

class edit_env_rep: public concrete_struct {
public:
  display                  dis;
  rel_hashmap<string,tree> env;
  hashmap<string,int>      var_type;
  hashmap<string,tree>&    local_ref;
  hashmap<string,tree>&    global_ref;
  hashmap<string,tree>&    local_aux;
  hashmap<string,tree>&    global_aux;
  bool                     complete;    // typeset complete document ?

  int       dpi;
  double    magn;
  int       mode;
  language  lan;
  font      fn;
  int       fn_size;
  int       index_level;
  bool      display_style;
  bool      math_condensed;
  int       vert_pos;
  color     col;
  SI        lw;

public:
  edit_env_rep (hashmap<string,tree>& local_ref,
		hashmap<string,tree>& global_ref,
		hashmap<string,tree>& local_aux,
		hashmap<string,tree>& global_aux);
  void   set_display (display dis);
  void   style_init_env ();

  /* execution of trees and setting environment variables */
  tree   exec (tree t);
  void   set (string s, tree t);
  void   reset (string s);
  bool   assign (string s, tree t);
  void   save ();
  void   recover ();
  void   confirm ();

  /* retrieving environment variables */
  tree   get (string s);
  bool   get_bool (string env_var);
  int    get_int (string env_var);
  double get_double (string env_var);
  string get_string (string env_var);
  SI     get_length (string env_var);

  /* updating environment variables */
  void   update_font ();
  void   update_color ();
  void   update_mode ();
  void   update_language ();
  void   update ();
  void   update (string env_var);

  /* miscellaneous and utilities */
  void   script_size ();
  void   normal_size ();
  SI     decode_length (string length);
  string add_lengths (string l1, string l2);
  void   get_page_pars (SI& w, SI& h, SI& ww, SI& hh,
			SI& odd, SI& even, SI& top, SI& bottom);
};

class edit_env {
#import concrete (edit_env, edit_env_rep)
  edit_env (hashmap<string,tree>& local_ref,
	    hashmap<string,tree>& global_ref,
	    hashmap<string,tree>& local_aux,
	    hashmap<string,tree>& global_aux);
};
#import code_concrete (edit_env, edit_env_rep)

/******************************************************************************
* The different types of system environment variables
******************************************************************************/

#define Env_User               0
#define Env_Fixed              1
#define Env_Magnification      2
#define Env_Language           3
#define Env_Mode               4
#define Env_Font               5
#define Env_Font_Size          6
#define Env_Index_Level        7
#define Env_Display_Style      8
#define Env_Math_Condensed     9
#define Env_Vertical_Pos      10
#define Env_Color             11
#define Env_Paragraph         12
#define Env_Page              13

#endmodule // env
