
/******************************************************************************
* MODULE     : font.gen.h
* DESCRIPTION: fonts
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <space.gen.h>
#include <Boxes/boxes.gen.h>
#include <encoding.gen.h>
#include <display.gen.h>

#module font
#import space
#import boxes
#import encoding
#import display
#import resource (font_rep,font)

/******************************************************************************
* The font structure
******************************************************************************/

struct font_rep: rep<font> {
  display  dis;              // underlying display
  encoding enc;              // underlying encoding of the font
  SI       design_size;      // design size in points/256
  SI       display_size;     // display size in points/PIXEL
  double   slope;            // italic slope
  space    spc;              // usual space between words
  space    extra;            // extra space at end of words

  SI       y1;               // bottom y position
  SI       y2;               // top y position
  SI       yfrac;            // vertical position fraction bar
  SI       ysub;             // base line for subscripts
  SI       ysup;             // base line for superscripts

  SI       wpt;              // width of one point in font
  SI       wquad;            // wpt * design size in points
  SI       wunit;            // unit width for extendable fonts
  SI       wfrac;            // width fraction bar
  SI       wsqrt;            // width horzontal line in square root
  SI       wneg;             // width of negation line

  font_rep (display dis, string name);
  font_rep (display dis, string name, font fn);
  void copy_parameters (server_font fn);
  void copy_math_pars (font fn);
  virtual box convert (string s, color col, SI len1=0, SI len2=0) = 0;
};

font join (font fn1, font fn2);
font subfont (font fn, encoding enc);
font error_font (font fn1, font fn2);

font ps_font (display dis, string family, int size, int dpi);
font ps_rubber_font (font fn, int size, int dpi);
font tex_text_font (display dis, string fam, int size, int dpi, int dsize=10);
font tex_rubber_font (display dis, string trl_name,
		      string fam, int size, int dpi, int dsize=10);

void font_rule (tree which, tree by);
font find_font (display dis, string family, string fn_class,
		string series, string shape, int sz, int dpi);

int  script (int sz, int level);

#endmodule // font
