
/******************************************************************************
* MODULE     : page_type.gen.cc
* DESCRIPTION: Data base for page sizes and default settings
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <string.gen.h>
#include <hashmap.gen.h>

#module code_page_type
#import string
#import hashmap (string, string)

static bool page_data_base_initizalized= FALSE;
static hashmap<string,string> page_data_base ("");

static void
INIT (string type, string width, string height, string standard_format,
      string por_parw,
      string por_hmargin, string por_vmargin,
      string por_hred, string por_vred,
      string lan_parw,
      string lan_hmargin, string lan_vmargin,
      string lan_hred, string lan_vred)
{
  page_data_base (type * "-P-page width") = width;
  page_data_base (type * "-P-page height")= height;
  page_data_base (type * "-P-standard")= standard_format;
  page_data_base (type * "-P-paragraph width")= por_parw;
  page_data_base (type * "-P-odd page margin")= por_hmargin;
  page_data_base (type * "-P-even page margin")= por_hmargin;
  page_data_base (type * "-P-page right margin")= por_hmargin;
  page_data_base (type * "-P-page top margin")= por_vmargin;
  page_data_base (type * "-P-page bottom margin")= por_vmargin;
  page_data_base (type * "-P-reduction page left margin")= por_hred;
  page_data_base (type * "-P-reduction page right margin")= por_hred;
  page_data_base (type * "-P-reduction page top margin")= por_vred;
  page_data_base (type * "-P-reduction page bottom margin")= por_vred;

  page_data_base (type * "-L-page width") = height;
  page_data_base (type * "-L-page height")= width;
  page_data_base (type * "-L-standard")= standard_format;
  page_data_base (type * "-L-paragraph width")= lan_parw;
  page_data_base (type * "-L-odd page margin")= lan_hmargin;
  page_data_base (type * "-L-even page margin")= lan_hmargin;
  page_data_base (type * "-L-page right margin")= lan_hmargin;
  page_data_base (type * "-L-page top margin")= lan_vmargin;
  page_data_base (type * "-L-page bottom margin")= lan_vmargin;
  page_data_base (type * "-L-reduction page left margin")= lan_hred;
  page_data_base (type * "-L-reduction page right margin")= lan_hred;
  page_data_base (type * "-L-reduction page top margin")= lan_vred;
  page_data_base (type * "-L-reduction page bottom margin")= lan_vred;
}

static void
INIT_ALL () {
  INIT ("a2", "420mm", "594mm", "yes",
	"300mm",  "60mm",  "60mm",  "50mm",  "30mm",
	"474mm",  "60mm",  "60mm",  "50mm",  "30mm");
  INIT ("a3", "297mm", "420mm", "yes",
	"207mm",  "45mm",  "45mm",  "40mm",  "30mm",
	"330mm",  "45mm",  "45mm",  "40mm",  "30mm");
  INIT ("a4", "210mm", "297mm", "yes",
	"150mm",  "30mm",  "30mm",  "25mm",  "15mm",
	"237mm",  "30mm",  "30mm",  "25mm",  "15mm");
  INIT ("a5", "148mm", "210mm", "yes",
	"108mm",  "20mm",  "20mm",  "15mm",  "5mm",
	"170mm",  "20mm",  "20mm",  "15mm",  "5mm");
  INIT ("b3", "352mm", "500mm", "yes",
	"252mm",  "50mm",  "50mm",  "40mm",  "20mm",
	"400mm",  "50mm",  "50mm",  "40mm",  "20mm");
  INIT ("b4", "250mm", "352mm", "yes",
	"180mm",  "35mm",  "35mm",  "30mm",  "20mm",
	"282mm",  "35mm",  "35mm",  "30mm",  "20mm");
  INIT ("b5", "176mm", "250mm", "yes",
	"126mm",  "25mm",  "25mm",  "20mm",  "10mm",
	"200mm",  "25mm",  "25mm",  "20mm",  "10mm");
  INIT ("b6", "125mm", "176mm", "yes",
	"95mm",   "15mm",  "15mm",  "10mm",  "0mm",
	"146mm",  "15mm",  "15mm",  "10mm",  "0mm");
  INIT ("letter", "8.5in", "11in", "yes",
	"6.5in",  "1in",   "1in",   "0.7in", "0.3in",
	"9in",    "1in",   "1in",   "0.7in", "0.3in");
  INIT ("legal", "8.5in", "14in", "yes",
	"6.5in",  "1in",   "1in",   "0.7in", "0.3in",
	"12in",   "1in",   "1in",   "0.7in", "0.3in");
  INIT ("executive", "7.25in", "10.5in", "yes",
	"5.25in", "1in",   "1in",   "0.7in", "0.3in",
	"8.5in",  "1in",   "1in",   "0.7in",   "0.3in");
  INIT ("lecture note", "15.5cm", "23.5cm", "no",
	"121mm",  "17mm",  "17mm",  "0mm",   "0mm",
	"201mm",  "17mm",  "17mm",  "0mm",   "0mm");
  page_data_base_initizalized= TRUE;
}

string
page_get_feature (string type, string feature, bool landscape) {
  if (!page_data_base_initizalized) INIT_ALL ();
  if (landscape) return page_data_base [type * "-L-" * feature];
  else return page_data_base [type * "-P-" * feature];
}

#endmodule // page_type
