
/******************************************************************************
* MODULE     : specific_font.gen.cc
* DESCRIPTION: Initialization of window manager
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <PsDevice/specific_font.gen.h>

#module code_specific_font
#import specific_font

#define PLUS_INFINITY  ((SI) 0x3fffffff)
#define MINUS_INFINITY ((SI) 0xc0000000)

/******************************************************************************
* Getting extents of strings with spaces in general fonts
******************************************************************************/

void
server_font_rep::var_get_extents (string s, SI& x1, SI& y1, SI& x2, SI& y2,
				            SI& x3, SI& y3, SI& x4, SI& y4)
{
  bool flag=TRUE;
  int start=0, end;
  get_extents ("", x1, y1, x2, y2, x3, y3, x4, y4);
  while (start<N(s)) {
    for (end=start; (end<N(s)) && (s[end]!=' '); end++);
    if (start<end) {
      SI X1, Y1, X2, Y2, X3, Y3, X4, Y4;
      get_extents (s (start, end), X1, Y1, X2, Y2, X3, Y3, X4, Y4);
      if (flag) {
	x3= X3+ x2; y3= Y3+ x2; x4= X4; y4= Y4;
	x2 += X2;
	flag= FALSE;
      }
      else {
	x3= min (x3, x2+ X3); x4= max (x4, x2+ X4);
	y3= min (y3, Y3)    ; y4= max (y4, Y4);
	x2 += X2;
      }
    }
    for (; (end<N(s)) && (s[end]==' '); end++) x2 += spc->def;
    start= end;
  }
}

/******************************************************************************
* PS fonts
******************************************************************************/

server_ps_font_rep::server_ps_font_rep (string family2, int size2, int dpi2):
  server_font_rep (family2 * as_string (size2))
{
  family= family2;
  size  = size2;
  dpi   = dpi2;
}

void
server_ps_font_rep::initialize_parameters () {
  SI x1, x2, x3, y3, x4, y4, d1, d2;

  display_size = y2-y1;
  design_size  = size << 8;

  // get xheight
  get_extents ("x", x1, d1, x2, d2, x3, y3, x4, y4);
  ysub  = 0;
  ysup  = y4-y3;
  yfrac = (ysub + ysup) >> 1;
  wpt   = (dpi*PIXEL)/72;
  wquad = (wpt*design_size) >> 8;
  wunit = wquad/4;
  wfrac = (dpi<450)? (wquad/25): ((dpi<900)? (wquad/30): (wquad/35));
  wsqrt = (dpi<450)? (wquad/25): ((dpi<700)? (wquad/25): (wquad/30));
  wneg  = (dpi<450)? (wquad/25): ((dpi<700)? (wquad/25): (wquad/30));

  // get fraction bar parameters
  get_extents ("-", x1, d1, x2, d2, x3, y3, x4, y4);
  yfrac= (y3+y4)>>1;

  // get space length
  get_extents (" ", x1, d1, x2, d2, x3, y3, x4, y4);
  spc  = space ((3*(x2-x1))>>2, x2-x1, (x2-x1)<<1);
  extra= spc;

  // get_italic space
  get_extents ("f", x1, d1, x2, d2, x3, y3, x4, y4);
  SI italic_spc;
  if (x2-x1 >= x4-x3) italic_spc= 0;
  else italic_spc= (x4-x3)-(x2-x1);
  slope= ((double) italic_spc) / ((double) display_size);
}

void
server_ps_font_rep::get_extents (unsigned char cc, int n,
				 SI& x1, SI& y1, SI& x2, SI& y2,
				 SI& x3, SI& y3, SI& x4, SI& y4)
{
  (void) n;
  y1=server_ps_font_rep::y1; y2=server_ps_font_rep::y2;
  ps_char_prop* cp= & (c [cc]);
  x1= cp->x1; x2= cp->x2;
  x3= cp->x3; y3= cp->y3;
  x4= cp->x4; y4= cp->y4;
}

void
server_ps_font_rep::get_extents (string s,
				 SI& x1, SI& y1, SI& x2, SI& y2,
				 SI& x3, SI& y3, SI& x4, SI& y4)
{
  y1=server_ps_font_rep::y1; y2=server_ps_font_rep::y2;
  
  int n= N(s);
  if (n==0) {
    x1=x2=x3=x4=0;
    y3=server_ps_font_rep::y1; y4= server_ps_font_rep::y2;
  }
  else {
    ps_char_prop* cp= & (c [(unsigned char) s[0]]);
    x1= cp->x1; x2= cp->x2;
    x3= cp->x3; y3= cp->y3;
    x4= cp->x4; y4= cp->y4;

    int i;
    for (i=1; i<n; i++) {
      cp= & (c [(unsigned char) s[i]]);
      y3= min ((y3), cp->y3);
      x4= (x2)+ cp->x4; y4= max ((y4), cp->y4);
      (x2) += cp->x2;
    }
  }
}

SI
server_ps_font_rep::right_correction (QN cc, int n) { (void) n;
  ps_char_prop* cp= & (c [cc]);
  return (SI) (slope * cp->y4);
}

SI
server_ps_font_rep::right_correction (string s) {
  if (N(s) == 0) return 0;
  ps_char_prop* cp= & (c [(unsigned char) s[N(s)-1]]);
  return (SI) (slope * cp->y4);
}

/******************************************************************************
* TeX fonts
******************************************************************************/

#define conv(x) ((SI) (((double) (x))*unit))

server_tex_font_rep::server_tex_font_rep (
  string family, int size, int dpi, int dsize):
    server_font_rep (family * as_string (size) * "@" * as_string (dpi))
{
  type= 1;
  load_tex (family, size, dpi, dsize, tfm, pk);

  design_size = tfm->design_size () >> 12;
  display_size= (((design_size*dpi)/72)*PIXEL) >> 8;
  unit        = ((double) display_size) / ((double) (1<<20));
  slope       = tfm->slope ();
  spc->def    = conv (tfm->spc ());
  spc->min    = spc->def - conv (tfm->spc_shrink ());
  spc->max    = spc->def + conv (tfm->spc_stretch ());
  extra       = conv (tfm->spc_extra ());
  extra->min  = extra->min >> 1;
  extra->max  = extra->min << 1;

  y1    = conv (-262080);
  y2    = y1+ display_size;
  ysub  = 0;
  ysup  = conv (tfm->x_height ());
  yfrac = (ysub + ysup) >> 1;
  wpt   = (dpi*PIXEL)/72;
  wquad = (wpt*design_size) >> 8;
  wunit = wquad/4;
  wfrac = (dpi<450)? (wquad/25): ((dpi<900)? (wquad/30): (wquad/35));
  wsqrt = (dpi<450)? (wquad/25): ((dpi<700)? (wquad/25): (wquad/30));
  wneg  = (dpi<450)? (wquad/25): ((dpi<700)? (wquad/25): (wquad/30));

  if (dpi<350) yfrac -= wquad/40;
  else if (dpi<500);
  else if (dpi<700) yfrac += wquad/80;
  else yfrac += wquad/60;
}

/******************************************************************************
* TeX text fonts
******************************************************************************/

void
server_tex_font_rep::get_extents (string s,
				  SI& x1, SI& y1, SI& x2, SI& y2,
				  SI& x3, SI& y3, SI& x4, SI& y4)
{
  int n=N(s);
  y1= server_tex_font_rep::y1;
  y2= server_tex_font_rep::y2;
  if (n==0) {
    x1=x3= x2=x4=0;
    y3=y1; y4=y2;
  }
  else {
    register int i;
    int m=n+16;
    int s_copy[n], buf[m], ker[m];
    for (i=0; i<n; i++) s_copy[i]= ((QN) s[i]);
    tfm->execute (s_copy, n, buf, ker, m);

    x1= 0;
    x2= 0;
    x3= PLUS_INFINITY;
    x4= MINUS_INFINITY;
    y3= PLUS_INFINITY;
    y4= MINUS_INFINITY;

    for (i=0; i<m; i++) {
      register int      real_c= buf[i];
      if ((real_c<tfm->bc) || (real_c>tfm->ec)) continue;
      register int      c (real_c-tfm->bc);
      register pk_char& pkc (pk->pkf[c]);
      if (pkc.rep==NULL) continue;

      x3= min (x3, x2- pkc->dpy_xoff);
      x4= max (x4, x2+ (pkc->width * PIXEL)- pkc->dpy_xoff);
      y3= min (y3, pkc->dpy_yoff- (pkc->height * PIXEL));
      y4= max (y4, pkc->dpy_yoff);

      x2 += conv (tfm-> w(real_c)+ ker[i]);
    }
    if ((x3 == PLUS_INFINITY) || (x4 == MINUS_INFINITY) ||
	(y3 == PLUS_INFINITY) || (y4 == MINUS_INFINITY))
      {
	x1=x3= x2=x4=0;
	y3=y1; y4=y2;
	return;
      }
  }

  x3 -= 2*PIXEL; x4 += 2*PIXEL;
  y3 -= 2*PIXEL; y4 += 2*PIXEL;
}

SI
server_tex_font_rep::right_correction (QN c, int n) {
  QN real_c = tfm->nth_in_list (c, n);
  return conv (tfm->i (real_c));
}

SI
server_tex_font_rep::right_correction (string s) {
  if (N(s)==0) return 0;
  QN c= s [N(s)-1];
  return conv (tfm->i (c));
}

/******************************************************************************
* TeX rubber fonts
******************************************************************************/

void
server_tex_font_rep::get_extents (QN real_c,
				  SI& x1, SI& y1, SI& x2, SI& y2,
				  SI& x3, SI& y3, SI& x4, SI& y4) {
  x1=  0;
  y1= -conv (tfm->d(real_c));
  x2=  conv (tfm->w(real_c));
  y2=  conv (tfm->h(real_c));

  register int      c (real_c-tfm->bc);
  register pk_char& pkc (pk->pkf[c]);

  x3= -pkc->dpy_xoff;
  x4= pkc->width*PIXEL- pkc->dpy_xoff;
  y3= pkc->dpy_yoff- pkc->height*PIXEL;
  y4= pkc->dpy_yoff+ PIXEL;

  x3 -= 2*PIXEL; x4 += 2*PIXEL;
  y3 -= 2*PIXEL; y4 += 2*PIXEL;
}

void
server_tex_font_rep::get_extents_sub (QN pre_c,
				      SI& x1, SI& y1, SI& x2, SI& y2,
				      SI& x3, SI& y3, SI& x4, SI& y4) {
  SI X1, Y1, X2, Y2, X3, Y3, X4, Y4;
  get_extents (pre_c, X1, Y1, X2, Y2, X3, Y3, X4, Y4);
  x1= min (x1, X1); y1 -= (Y2-Y1);
  x2= max (x2, X2); (void) y2;
  x3= min (x3, X3); y3= min (y3, y1+ (Y3-Y1));
  x4= max (x4, X4); y4= max (y4, y1+ (Y4-Y1));
}

void
server_tex_font_rep::get_extents (QN pre_c, int n,
				  SI& x1, SI& y1, SI& x2, SI& y2,
				  SI& x3, SI& y3, SI& x4, SI& y4)
{
  int int_pre_c= pre_c;
  if ((int_pre_c<tfm->bc) || (int_pre_c>tfm->ec)) {
    x1= y1= x2= y2= x3= y3= x4= y4= 0;
    return;
  }

  QN c= tfm->nth_in_list (pre_c, n);
  if (tfm->tag (c) != 3) get_extents (c, x1, y1, x2, y2, x3, y3, x4, y4);
  else {
    int i, nr_rep= n- tfm->list_len (pre_c);

    x1= x3= y3= PLUS_INFINITY;
    x2= x4= y4= MINUS_INFINITY;
    y1= y2= 0;

    if (tfm->top (c)!=0)
      get_extents_sub (tfm->top (c), x1, y1, x2, y2, x3, y3, x4, y4);
    if ((tfm->rep (c)!=0) && (tfm->mid (c)!=0))
      for (i=0; i<nr_rep; i++)
	get_extents_sub (tfm->rep (c), x1, y1, x2, y2, x3, y3, x4, y4);
    if (tfm->mid (c)!=0)
      get_extents_sub (tfm->mid (c), x1, y1, x2, y2, x3, y3, x4, y4);
    if (tfm->rep (c)!=0)
      for (i=0; i<nr_rep; i++)
	get_extents_sub (tfm->rep (c), x1, y1, x2, y2, x3, y3, x4, y4);
    if (tfm->bot (c)!=0)
      get_extents_sub (tfm->bot (c), x1, y1, x2, y2, x3, y3, x4, y4);
  }
  SI delta= (-y1>>1) + yfrac;
  y1 += delta; y2 += delta;
  y3 += delta; y4 += delta;
}

#undef conv(x)

#endmodule // code_specific_font
