
/******************************************************************************
* MODULE     : x_display.gen.h
* DESCRIPTION: Abstract display class
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <display.gen.h>
#include <widget.gen.h>
#include <array.gen.h>
#include <hashmap.gen.h>

#module x_display
#import display
#import widget
#import list (widget)
#import array (int)
#import hashmap (string,pointer)
#import hashmap (int,string)
#define hashmap<string,server_font> hashmap<string,pointer>

class x_display_rep;
class x_drawable_rep;
class x_window_rep;
typedef x_display_rep* x_display;
typedef x_window_rep* x_window;

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>

/******************************************************************************
* For anti aliasing of TeX fonts
******************************************************************************/

struct font_mod_rep: concrete_struct {
  int   sf;
  color fg;
  color bg;
  font_mod_rep (int sf, color fg, color bg);
  friend class font_mod;
};

class font_mod {
#import concrete (font_mod, font_mod_rep)
  font_mod (int sf=1, color fg= 0, color bg= 1);
  operator tree ();
};
#import code_concrete (font_mod, font_mod_rep)

bool operator == (font_mod fm1, font_mod fm2);
bool operator != (font_mod fm1, font_mod fm2);
int hash (font_mod fm);
#define no_tree_converter<pointer>
#import hashmap (font_mod,pointer)
#import hashmap (int,pointer)
#import hashmap (int,int)
#define hashmap<int,Font> hashmap<int,int>

/******************************************************************************
* Delayed messages
******************************************************************************/

struct message_rep: concrete_struct {
  widget wid;
  string s;
  time_t t;
  message_rep (widget wid, string s, time_t t);
  friend class message;
};

class message {
#import concrete (message, message_rep)
  message (widget wid, string s, time_t t);
};
#import code_concrete (message, message_rep)

ostream& operator << (ostream& out, message m);
#define no_tree_converter<message>
#import list (message)

/******************************************************************************
* The x_display class
******************************************************************************/

class x_display_rep: public display_rep {
  Display*        dpy;
  GC              gc;
  GC              pixmap_gc;
  int             scr;
  Window          root;
  int             display_width;
  int             display_height;
  int             depth;
  Colormap        cols;
  color*          cmap;
  list<widget>    grab_ptr;
  list<widget>    grab_kbd;
  unsigned int    state;
  char*           selection;
  tree            selected_tree;
  list<message>   messages;
  x_drawable_rep* shadow;
  x_window_rep*   gswindow;
  int             argc;
  char**          argv;

  hashmap<font_mod,pointer>   color_scale;   // for anti-aliasing
  hashmap<int,string>         lower_key;
  hashmap<int,string>         upper_key;
  hashmap<string,server_font> server_ps_fn;
  hashmap<string,server_font> server_tex_fn;

public:
  x_display_rep (int argc, char** argv);
  ~x_display_rep ();

  /****************************** Color **************************************/
  int   alloc_color (int r, int g, int b);
  void  init_color_map ();
  void  initialize_colors ();
  void  prepare_color (int sfactor, color fg, color bg);
  color rgb (int r, int g, int b);
  void  get_rgb (color col, int& r, int& g, int& b);

  /****************************** Keyboard ***********************************/
  void insert_keysym (array<int>& a, int i, int j);
  void get_xmodmap ();
  void map (int key, string s);
  void Map (int key, string s);
  void initialize_keyboard_pointer ();
  string look_up_key (XKeyEvent* ev);
  string look_up_mouse (XButtonEvent* ev);
  unsigned int get_button_mask (XButtonEvent* ev);

  /**************************** Server fonts *********************************/
  void        set_shrinking_factor (int sfactor);
  server_font default_server_font ();
  server_font server_ps_font (string name);
  server_font server_ps_font (string family, int size, int dpi);
  server_font server_tex_font (string family, int size, int dpi, int dsize=10);
  void        get_extents (string s, server_font fn,
			   SI& x1, SI& y1, SI& x2, SI& y2,
			   SI& x3, SI& y3, SI& x4, SI& y4);

  /************************** Server languages *******************************/
  void   load_dictionary (string name, string from, string to);
  void   set_output_language (string lan);
  string translate (string s, string from, string to);

  /********************* extents, grabbing, selections ***********************/
  void   get_extents (SI& width, SI& height);
  void   set_button_state (unsigned int state);
  void   emulate_leave_enter (widget old_widget, widget new_widget);
  void   grab_pointer (widget wid);
  void   ungrab_pointer ();
  void   grab_keyboard (widget wid);
  void   ungrab_keyboard ();

  /*********************** interclient communication *************************/
  tree   get_selection (widget wid);
  bool   set_selection (widget wid, tree t, string s);
  void   clear_selection ();
  void   delayed_message (widget wid, string s, time_t delay);

  /************************** Event processing *******************************/
  void process_event (x_window win, XEvent* ev);
  void event_loop ();

  /*************************** And our friends *******************************/
  friend class x_drawable_rep;
  friend class x_window_rep;
  friend class x_ps_font_rep;
  friend class x_tex_font_rep;
};

#endmodule // x_display
