
/******************************************************************************
* MODULE     : x_font.gen.cc
* DESCRIPTION: server_ps_fonts and server_tex_fonts under X windows
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <X/x_window.gen.h>
#include <X/x_font.gen.h>

#module code_x_font
#import x_window
#import x_font

/******************************************************************************
* Post script fonts
******************************************************************************/

x_ps_font_rep::x_ps_font_rep (x_display dis2, string f2, int s2, int d2):
  server_ps_font_rep (f2, s2, d2), the_fn (0)
{
  type= 0;
  dis= dis2;
  prepare (1);

  Display* dpy= dis->dpy;
  XCharStruct xcs;
  int i, i1, i2, i3;
  for (i=0; i<256; i++) {
    ps_char_prop* cp= &(c[i]);
    char temp[1]; temp[0]= (char) i;

    XQueryTextExtents (dpy, (Font) the_fn[1], temp, 1, &i1, &i2, &i3, &xcs);
    cp->x1= 0;
    cp->x2= xcs.width * PIXEL;
    cp->x3= xcs.lbearing * PIXEL;
    cp->y3= (-1-xcs.descent) * PIXEL;
    cp->x4= xcs.rbearing * PIXEL;
    cp->y4= (-1+xcs.ascent) * PIXEL;
  }
  XQueryTextExtents (dpy, (Font) the_fn[1], "x", 1, &i1, &i2, &i3, &xcs);
  y1= (-1-i3) * PIXEL;
  y2= (-1+i2) * PIXEL;
  initialize_parameters ();
}

x_ps_font_rep::~x_ps_font_rep () {
  fatal_error ("Not yet implemented",
	       "x_ps_font_rep::~x_ps_font_rep", "x_font.gen.cc");
}

void
x_ps_font_rep::prepare (int sf) {
  if (the_fn->contains (sf)) return;
  string name= "-" * family;
  string sz1= as_string ((size*dpi)/(sf*72));
  string sz2= as_string (10*((size*dpi)/(sf*72)));
  name << "-*-" * sz1 * "-" * sz2 * "-*-*-*-*-*-*";
  if (size == 0) {
    name= family;
    if (sf!=1) fatal_error ("Can not shrink font", "x_ps_font_rep::prepare");
  }

  cout << "Preparing " << family << size << "@" << dpi << " / " << sf << "\n";
  if (debug (0)) cout << "TeXmacs] Loading ps font " << name << "\n";
  char* temp= as_charp (name);
  the_fn(sf) = XLoadFont (dis->dpy, temp);
  delete[] temp;
  if (XQueryFont (dis->dpy, (Font) the_fn[sf]) == NULL) {
    the_fn(1) = XLoadFont (dis->dpy, "*");
    if (XQueryFont (dis->dpy, (Font) the_fn[1]) == NULL)
      fatal_error ("Could not load default X font", "x_ps_font_rep::prepare");
  }
}

/******************************************************************************
* Displaying ps fonts
******************************************************************************/

void
x_drawable_rep::text (string s, x_ps_font fn, SI x, SI y) {
  fn->prepare (sfactor);
  XSetFont (dpy, gc, (Font) fn->the_fn[sfactor]);
  y -= PIXEL;
  decode (x, y);
  int i, n= N(s);
  char temp[n];
  for (i=0; i<n; i++) temp[i]=s[i];
  XDrawString (dpy, win, gc, x, y, temp, n);
}

void
x_drawable_rep::text (QN c, int n, x_ps_font fn, SI x, SI y) {
  (void) n;
  text (string (c), fn, x, y);
}

/******************************************************************************
* TeX fonts
******************************************************************************/

x_tex_font_rep::x_tex_font_rep (x_display dis2, string family,
				int size, int dpi, int dsize):
  server_tex_font_rep (family, size, dpi, dsize),
  dis (dis2), the_pms ((pointer) NULL), the_bms ((pointer) NULL)
{}

x_tex_font_rep::~x_tex_font_rep () {
  fatal_error ("Not yet implemented",
	       "x_tex_font_rep::~x_tex_font_rep", "specific_font.gen.cc");
}

void
x_tex_font_rep::prepare (int sf, color fg, color bg) {
  Bitmap* bms ((Bitmap*) (the_bms [sf]));
  if (bms == NULL) {
    int i, n= tfm->ec+ 1- tfm->bc;
    bms= new Bitmap [n];
    for (i=0; i<n; i++) bms[i]= NULL;
    the_bms (sf)= (pointer) bms;
  }
  Pixmap* pms ((Pixmap*) (the_pms [font_mod (sf, fg, bg)]));
  if (pms == NULL) {
    int i, n= tfm->ec+ 1- tfm->bc;
    pms= new Pixmap [n];
    for (i=0; i<n; i++) pms[i]= 0;
    the_pms (font_mod (sf, fg, bg))= (pointer) pms;
  }
  dis->prepare_color (sf, fg, bg);
}

Bitmap
x_tex_font_rep::get_bitmap (int real_c, int sf) {
  int c = real_c- tfm->bc;
  Bitmap* bms ((Bitmap*) the_bms [sf]);
  Bitmap& bm (bms [c]);

  if (bm == NULL) {
    pk_char& pre_pkc (pk->pkf[c]);
    pk_char pkc= shrink (pre_pkc, sf, sf);
    int i, j, b, on, w= pkc->width, h= pkc->height;
    int byte_width= ((w-1)>>3)+1;
    char data [byte_width * h];
    for (i=0; i<(byte_width * h); i++) data[i]=0;

    for (j=0; j<h; j++)
      for (i=0; i<w; i++) {
	b = j*byte_width + (i>>3);
	on= (pkc->Get(i,j)!=0 ? 1:0);
	if (on) data[b]= data[b] | (1<<(i&7));
      }
    bm= new Bitmap_rep;
    bm->bm    = XCreateBitmapFromData (dis->dpy, dis->root, data, w, h);
    bm->width = pkc->width;
    bm->height= pkc->height;
    bm->xoff  = pkc->dpy_xoff;
    bm->yoff  = pkc->dpy_yoff;
  }

  return bm;
}

Pixmap
x_tex_font_rep::get_pixmap (int real_c, int sf, color fg, color bg) {
  int c = real_c- tfm->bc;
  font_mod fm (sf, fg, bg);
  color* cols ((color*) dis->color_scale [fm]);
  Pixmap* pms ((Pixmap*) the_pms [fm]);
  Pixmap& pm (pms [c]);

  if (pm == 0) {
    pk_char& pre_pkc (pk->pkf[c]);
    pk_char pkc= shrink (pre_pkc, sf, sf);
    int i, j, w= pkc->width, h= pkc->height;
    pm= XCreatePixmap (dis->dpy, dis->root, w, h, dis->depth);
    for (j=0; j<h; j++)
      for (i=0; i<w; i++) {
	color col= cols [pkc->Get(i,j)];
	XSetForeground (dis->dpy, dis->pixmap_gc, dis->cmap[col]);
	XDrawPoint (dis->dpy, (Drawable) pm, dis->pixmap_gc, i, j);
      }
  }
  return pm;
}

/******************************************************************************
* Displaying TeX fonts
******************************************************************************/

bool char_clip= TRUE;

#define conv(x) ((SI) (((double) (x))*(fn->unit)))


void
x_drawable_rep::draw_clipped (Pixmap pm, Pixmap bm, int w, int h, SI x, SI y) {
  int x1=cx1-ox, y1=cy2-oy, x2= cx2-ox, y2= cy1-oy;
  decode (x, y);
  decode (x1, y1);
  decode (x2, y2);
  int X1= max (x1- x, 0); if (X1>=w) return;
  int Y1= max (y1- y, 0); if (Y1>=h) return;
  int X2= min (x2- x, w); if (X2<0) return;
  int Y2= min (y2- y, h); if (Y2<0) return;

  if (char_clip) {
    XSetClipMask (dpy, gc, bm);
    XSetClipOrigin (dpy, gc, x, y);
  }
  XCopyArea (dpy, (Drawable) pm, win, gc, X1, Y1, X2-X1, Y2-Y1, x+X1, y+Y1);
  if (char_clip) set_clipping (cx1- ox, cy1- oy, cx2- ox, cy2- oy);
}

void
x_drawable_rep::text (string str, x_tex_font fn, SI x, SI y) {
  color fg= cur_fg;
  color bg= cur_bg;
  fn->prepare (sfactor, fg, bg);

  register int i;
  int n= N(str);
  int m= n+16;
  SI s[n], buf[m], ker[m];
  for (i=0; i<n; i++) s[i]= ((QN) str[i]);
  fn->tfm->execute (s, n, buf, ker, m);

  for (i=0; i<m; i++) {
    register int real_c= buf[i];
    if ((real_c < fn->tfm->bc) || (real_c > fn->tfm->ec)) continue;
    register int c= real_c- fn->tfm->bc;
    pk_char& pkc (fn->pk->pkf[c]);
    if (pkc.rep==NULL) continue;

    Bitmap bm= fn->get_bitmap (real_c, sfactor);
    draw_clipped (fn->get_pixmap (real_c, sfactor, fg, bg),
		  bm->bm, bm->width, bm->height,
		  x- (bm->xoff*sfactor), y+ (bm->yoff*sfactor));

    x += conv (fn->tfm->w(real_c)+ ker[i]);
  }
}

void
x_drawable_rep::text (QN real_c, x_tex_font fn, SI x, SI& y, SI& real_y,
		      color fg, color bg)
{
  if ((real_c < fn->tfm->bc) || (real_c > fn->tfm->ec)) return;
  QN       c   (real_c- fn->tfm->bc);
  pk_char& pkc (fn->pk->pkf[c]);
  if (pkc.rep==NULL) return;

  Bitmap bm= fn->get_bitmap (real_c, sfactor);
  draw_clipped (fn->get_pixmap (real_c, sfactor, fg, bg),
		bm->bm, bm->width, bm->height,
		x- bm->xoff*sfactor, y+ bm->yoff*sfactor);

  y      -= bm->height * pixel;
  real_y -= conv (fn->tfm->h (real_c)+ fn->tfm->d (real_c));
  while (y <= real_y- pixel) y += pixel;
}

void
x_drawable_rep::text (QN pre_c, int n, x_tex_font fn, SI x, SI y) {
  int int_pre_c= pre_c;
  if ((int_pre_c<fn->tfm->bc) || (int_pre_c>fn->tfm->ec)) return;

  color fg= cur_fg;
  color bg= cur_bg;
  fn->prepare (sfactor, fg, bg);
  QN c = fn->tfm->nth_in_list (pre_c, n);

  if (fn->tfm->tag (c) != 3) text (c, fn, x, y, y, fg, bg);
  else {
    int i;
    int nr_rep= n- fn->tfm->list_len (pre_c);

    SI real_y= y, temp= x; decode (temp, y); encode (temp, y);
    if (fn->tfm->top (c)!=0) text (fn->tfm->top (c), fn, x, y, real_y, fg, bg);
    if (fn->tfm->rep (c)!=0)
      for (i=0; i<nr_rep; i++)
	text (fn->tfm->rep (c), fn, x, y, real_y, fg, bg);
    if (fn->tfm->mid (c)!=0) text (fn->tfm->mid (c), fn, x, y, real_y, fg, bg);
    if ((fn->tfm->rep (c)!=0) && (fn->tfm->mid (c)!=0))
      for (i=0; i<nr_rep; i++)
	text (fn->tfm->rep (c), fn, x, y, real_y, fg, bg);
    if (fn->tfm->bot (c)!=0) text (fn->tfm->bot (c), fn, x, y, real_y, fg, bg);
  }
}

#undef conv(x)

/******************************************************************************
* Server fonts
******************************************************************************/

server_font
x_display_rep::default_server_font () {
  // return server_ps_font ("b&h-lucidabright-medium-r-normal", 12, 300);
  // if (out_lan == "russian") return server_tex_font ("wncyr", 11, 300);
  // if (out_lan == "german") return server_tex_font ("ygoth", 14, 300, 0);
  return server_tex_font ("dcr", 11, 300);
}

server_font
x_display_rep::server_ps_font (string name) {
  if (!server_ps_fn->contains (name))
    server_ps_fn (name)=
      (pointer) new x_ps_font_rep (this, name, 0, 0);
  return (server_font) server_ps_fn [name];
}

server_font
x_display_rep::server_ps_font (string family, int size, int dpi) {
  string name= family * as_string (size);
  if (!server_ps_fn->contains (name))
    server_ps_fn (name)=
      (pointer) new x_ps_font_rep (this, family, size, dpi);
  return (server_font) server_ps_fn [name];
}

server_font
x_display_rep::server_tex_font (string family, int size, int dpi, int dsize) {
  string name= family * as_string (size) * "@" * as_string (dpi);
  if (!server_tex_fn->contains (name))
    server_tex_fn (name)=
      (pointer) new x_tex_font_rep (this, family, size, dpi, dsize);
  return (server_font) server_tex_fn [name];
}

void
x_display_rep::get_extents (string s, server_font fn,
			    SI& x1, SI& y1, SI& x2, SI& y2,
			    SI& x3, SI& y3, SI& x4, SI& y4) {
  fn->get_extents (s, x1, y1, x2, y2, x3, y3, x4, y4);
}

void
x_drawable_rep::text (string str, server_font fn, SI x, SI y) {
  if (fn->type==0) text (str, (x_ps_font) fn, x, y);
  else text (str, (x_tex_font) fn, x, y);
}

void
x_drawable_rep::text (QN c, int n, server_font fn, SI x, SI y) {
  if (fn->type==0) text (c, n, (x_ps_font) fn, x, y);
  else text (c, n, (x_tex_font) fn, x, y);
}

#endmodule // code_x_font
