
/******************************************************************************
* MODULE     : x_init.gen.cc
* DESCRIPTION: Initialization of window manager
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <X/x_window.gen.h>

#module code_x_init
#import x_window

#define CSCALES 4
#define CFACTOR 5
#define GREYS 16
#define CTOTAL (CFACTOR*CFACTOR*CFACTOR+GREYS+1)

/******************************************************************************
* Set up colors
******************************************************************************/

int
x_display_rep::alloc_color (int r, int g, int b) {
  XColor col;
  col.red  = r;
  col.green= g;
  col.blue = b;
  if (!XAllocColor (dpy, cols, &col))
    cerr << "Warning: can't allocate color\n";
  return col.pixel;
}

void
x_display_rep::init_color_map () {
  int i, r, g, b;

  for (i=0; i<=GREYS; i++)
    cmap[i]= alloc_color ((i*65535)/GREYS, (i*65535)/GREYS, (i*65535)/GREYS);

  for (r=0; r<=CSCALES; r++)
    for (g=0; g<=CSCALES; g++)
      for (b=0; b<=CSCALES; b++) {
	i= r*CFACTOR*CFACTOR+ g*CFACTOR+ b+ GREYS+ 1;
	cmap[i]= alloc_color ((r*65535)/CSCALES,
			      (g*65535)/CSCALES,
			      (b*65535)/CSCALES);
      }
}

color
x_display_rep::rgb (int r, int g, int b) {
  if ((r==g) && (g==b)) return (r*GREYS+ 128)/255;
  else {
    r= (r*CSCALES+ 128)/255;
    g= (g*CSCALES+ 128)/255;
    b= (b*CSCALES+ 128)/255;
    return r*CFACTOR*CFACTOR+ g*CFACTOR+ b+ GREYS+ 1;
  }
}

void
x_display_rep::get_rgb (color col, int& r, int& g, int& b) {
  if (col <= GREYS) {
    r= (col*255)/GREYS;
    g= (col*255)/GREYS;
    b= (col*255)/GREYS;
  }
  else {
    int rr, gg, bb;
    col-= (GREYS+1);
    bb  = col % CFACTOR;
    gg  = (col/CFACTOR) % CFACTOR;
    rr  = (col/(CFACTOR*CFACTOR)) % CFACTOR;
    r   = (rr*255)/CSCALES;
    g   = (gg*255)/CSCALES;
    b   = (bb*255)/CSCALES;
  }
}

void
x_display_rep::initialize_colors () {
  init_color_map ();

  black   = rgb (0, 0, 0);
  white   = rgb (255, 255, 255);
  red     = rgb (255, 0, 0);
  blue    = rgb (0, 0, 255);
  yellow  = rgb (255, 255, 0);
  green   = rgb (0, 255, 0);
  magenta = rgb (255, 0, 255);
  orange  = rgb (255, 128, 0);
  brown   = rgb (128, 32, 0);
  pink    = rgb (255, 128, 128);

  light_grey = rgb (208, 208, 208);
  grey       = rgb (184, 184, 184);
  dark_grey  = rgb (112, 112, 112);
}

/******************************************************************************
* Get xmodmap
******************************************************************************/

static int        mod_n;
static KeyCode*   mod_k;
static array<int> mod_shift;
static array<int> mod_ctrl;
static array<int> mod_alt;

void
x_display_rep::insert_keysym (array<int>& a, int i, int j) {
  int ks= XKeycodeToKeysym (dpy, mod_k[i*mod_n+j], 0);
  if (ks!=0) a << ks;
}

void
x_display_rep::get_xmodmap () {
  int i;
  XModifierKeymap* xmodmap= XGetModifierMapping (dpy);
  mod_n= xmodmap->max_keypermod;
  mod_k= xmodmap->modifiermap;
  for (i=0; i<mod_n; i++) {
    insert_keysym (mod_shift, 0, i);
    insert_keysym (mod_shift, 1, i);
    insert_keysym (mod_ctrl, 2, i);
    insert_keysym (mod_alt, 3, i);
  }
  XFreeModifiermap (xmodmap);
}

/******************************************************************************
* Set up keyboard
******************************************************************************/

void
x_display_rep::map (int key, string s) {
  lower_key (key)= s;
  upper_key (key)= "shift-" * s;
}

void
x_display_rep::Map (int key, string s) {
  lower_key (key)= s;
  upper_key (key)= s;
}

void
x_display_rep::initialize_keyboard_pointer () {
  Map (XK_a, "a");
  Map (XK_b, "b");
  Map (XK_c, "c");
  Map (XK_d, "d");
  Map (XK_e, "e");
  Map (XK_f, "f");
  Map (XK_g, "g");
  Map (XK_h, "h");
  Map (XK_i, "i");
  Map (XK_j, "j");
  Map (XK_k, "k");
  Map (XK_l, "l");
  Map (XK_m, "m");
  Map (XK_n, "n");
  Map (XK_o, "o");
  Map (XK_p, "p");
  Map (XK_q, "q");
  Map (XK_r, "r");
  Map (XK_s, "s");
  Map (XK_t, "t");
  Map (XK_u, "u");
  Map (XK_v, "v");
  Map (XK_w, "w");
  Map (XK_x, "x");
  Map (XK_y, "y");
  Map (XK_z, "z");
  Map (XK_A, "A");
  Map (XK_B, "B");
  Map (XK_C, "C");
  Map (XK_D, "D");
  Map (XK_E, "E");
  Map (XK_F, "F");
  Map (XK_G, "G");
  Map (XK_H, "H");
  Map (XK_I, "I");
  Map (XK_J, "J");
  Map (XK_K, "K");
  Map (XK_L, "L");
  Map (XK_M, "M");
  Map (XK_N, "N");
  Map (XK_O, "O");
  Map (XK_P, "P");
  Map (XK_Q, "Q");
  Map (XK_R, "R");
  Map (XK_S, "S");
  Map (XK_T, "T");
  Map (XK_U, "U");
  Map (XK_V, "V");
  Map (XK_W, "W");
  Map (XK_X, "X");
  Map (XK_Y, "Y");
  Map (XK_Z, "Z");
  Map (XK_0, "0");
  Map (XK_1, "1");
  Map (XK_2, "2");
  Map (XK_3, "3");
  Map (XK_4, "4");
  Map (XK_5, "5");
  Map (XK_6, "6");
  Map (XK_7, "7");
  Map (XK_8, "8");
  Map (XK_9, "9");

  Map (XK_exclam, "!");
  Map (XK_quotedbl, "\x22");
  Map (0xFE59, "\x22");
  Map (XK_numbersign, "#");
  Map (XK_dollar, "$");
  Map (XK_percent, "%");
  Map (XK_ampersand, "&");
  Map (XK_apostrophe, "'");
  Map (XK_quoteright, "'");
  Map (0xFE51, "'");
  Map (XK_parenleft, "(");
  Map (XK_parenright, ")");
  Map (XK_asterisk, "*");
  Map (XK_plus, "+");
  Map (XK_comma, ",");
  Map (XK_minus, "-");
  Map (XK_period, ".");
  Map (XK_slash, "/");
  Map (XK_colon, ":");
  Map (XK_semicolon, ";");
  Map (XK_less, "less");
  Map (XK_equal, "=");
  Map (XK_greater, "gtr");
  Map (XK_question, "?");
  Map (XK_at, "@");
  Map (XK_bracketleft, "[");
  Map (XK_backslash, "\\");
  Map (XK_bracketright, "]");
  Map (XK_asciicircum, "^");
  Map (0xFE52, "^");
  Map (XK_underscore, "_");
  Map (XK_grave, "`");
  Map (0xFE50, "`");
  Map (XK_quoteleft, "`");
  Map (XK_braceleft, "{");
  Map (XK_bar, "|");
  Map (XK_braceright, "}");
  Map (XK_asciitilde, "~");
  Map (0xFE53, "~");

  Map (XK_nobreakspace, "\xa0");
  Map (XK_exclamdown, "\xa1");
  Map (XK_cent, "\xa2");
  Map (XK_sterling, "\xa3");
  Map (XK_currency, "\xa4");
  Map (XK_yen, "\xa5");
  Map (XK_brokenbar, "\xa6");
  Map (XK_section, "\xa7");
  Map (XK_diaeresis, "\xa8");
  Map (0XFE57, "\xa8");
  Map (XK_copyright, "\xa9");
  Map (XK_ordfeminine, "\xaa");
  Map (XK_guillemotleft, "\xab");
  Map (XK_notsign, "\xac");
  Map (XK_hyphen, "\xad");
  Map (XK_registered, "\xae");
  Map (XK_macron, "\xaf");
  Map (XK_degree, "\xb0");
  Map (XK_plusminus, "\xb1");
  Map (XK_twosuperior, "\xb2");
  Map (XK_threesuperior, "\xb3");
  Map (XK_acute, "\xb4");
  Map (XK_mu, "\xb5");
  Map (XK_paragraph, "\xb6");
  Map (XK_periodcentered, "\xb7");
  Map (XK_cedilla, "\xb8");
  Map (XK_onesuperior, "\xb9");
  Map (XK_masculine, "\xba");
  Map (XK_guillemotright, "\xbb");
  Map (XK_onequarter, "\xbc");
  Map (XK_onehalf, "\xbd");
  Map (XK_threequarters, "\xbe");
  Map (XK_questiondown, "\xbf");
  Map (XK_Agrave, "\xc0");
  Map (XK_Aacute, "\xc1");
  Map (XK_Acircumflex, "\xc2");
  Map (XK_Atilde, "\xc3");
  Map (XK_Adiaeresis, "\xc4");
  Map (XK_Aring, "\xc5");
  Map (XK_AE, "\xc6");
  Map (XK_Ccedilla, "\xc7");
  Map (XK_Egrave, "\xc8");
  Map (XK_Eacute, "\xc9");
  Map (XK_Ecircumflex, "\xca");
  Map (XK_Ediaeresis, "\xcb");
  Map (XK_Igrave, "\xcc");
  Map (XK_Iacute, "\xcd");
  Map (XK_Icircumflex, "\xce");
  Map (XK_Idiaeresis, "\xcf");
  Map (XK_ETH, "\xd0");
  Map (XK_Eth, "\xd0");
  Map (XK_Ntilde, "\xd1");
  Map (XK_Ograve, "\xd2");
  Map (XK_Oacute, "\xd3");
  Map (XK_Ocircumflex, "\xd4");
  Map (XK_Otilde, "\xd5");
  Map (XK_Odiaeresis, "\xd6");
  Map (XK_multiply, "\xd7");
  Map (XK_Ooblique, "\xd8");
  Map (XK_Ugrave, "\xd9");
  Map (XK_Uacute, "\xda");
  Map (XK_Ucircumflex, "\xdb");
  Map (XK_Udiaeresis, "\xdc");
  Map (XK_Yacute, "\xdd");
  Map (XK_THORN, "\xde");
  Map (XK_Thorn, "\xde");
  Map (XK_ssharp, "\xdf");
  Map (XK_agrave, "\xe0");
  Map (XK_aacute, "\xe1");
  Map (XK_acircumflex, "\xe2");
  Map (XK_atilde, "\xe3");
  Map (XK_adiaeresis, "\xe4");
  Map (XK_aring, "\xe5");
  Map (XK_ae, "\xe6");
  Map (XK_ccedilla, "\xe7");
  Map (XK_egrave, "\xe8");
  Map (XK_eacute, "\xe9");
  Map (XK_ecircumflex, "\xea");
  Map (XK_ediaeresis, "\xeb");
  Map (XK_igrave, "\xec");
  Map (XK_iacute, "\xed");
  Map (XK_icircumflex, "\xee");
  Map (XK_idiaeresis, "\xef");
  Map (XK_eth, "\xf0");
  Map (XK_ntilde, "\xf1");
  Map (XK_ograve, "\xf2");
  Map (XK_oacute, "\xf3");
  Map (XK_ocircumflex, "\xf4");
  Map (XK_otilde, "\xf5");
  Map (XK_odiaeresis, "\xf6");
  Map (XK_division, "\xf7");
  Map (XK_oslash, "\xf8");
  Map (XK_ugrave, "\xf9");
  Map (XK_uacute, "\xfa");
  Map (XK_ucircumflex, "\xfb");
  Map (XK_udiaeresis, "\xfc");
  Map (XK_yacute, "\xfd");
  Map (XK_thorn, "\xfe");
  Map (XK_ydiaeresis, "\xff");

  map (XK_space, "spc");
  map (XK_Return, "return");
  map (XK_BackSpace, "del");
  map (XK_Delete, "del");
  map (XK_Insert, "ins");
  map (XK_Tab, "tab");
  map (XK_Escape, "esc");
  map (XK_Left, "left");
  map (XK_Right, "right");
  map (XK_Up, "up");
  map (XK_Down, "down");
  map (XK_Page_Up, "page-up");
  map (XK_Page_Down, "page-down");
  map (XK_Home, "home");
  map (XK_End, "end");
  map (XK_F1, "F1");
  map (XK_F2, "F2");
  map (XK_F3, "F3");
  map (XK_F4, "F4");
  map (XK_F5, "F5");
  map (XK_F6, "F6");
  map (XK_F7, "F7");
  map (XK_F8, "F8");
  map (XK_F9, "F9");
  map (XK_F10, "F10");
  map (XK_F11, "F11");
  map (XK_F12, "F12");

  map (XK_KP_Space, "spc");
  map (XK_KP_Enter, "return");
  map (XK_KP_Delete, "del");
  map (XK_KP_Insert, "ins");
  map (XK_KP_Tab, "tab");
  map (XK_KP_Left, "left");
  map (XK_KP_Right, "right");
  map (XK_KP_Up, "up");
  map (XK_KP_Down, "down");
  map (XK_KP_Page_Up, "page-up");
  map (XK_KP_Page_Down, "page-down");
  map (XK_KP_Home, "home");
  map (XK_KP_End, "end");
  map (XK_KP_F1, "F1");
  map (XK_KP_F2, "F2");
  map (XK_KP_F3, "F3");
  map (XK_KP_F4, "F4");
  Map (XK_KP_Equal, "=");
  Map (XK_KP_Multiply, "*");
  Map (XK_KP_Add, "+");
  Map (XK_KP_Subtract, "-");
  Map (XK_KP_Decimal, ".");
  Map (XK_KP_Divide, "/");
  Map (XK_KP_0, "0");
  Map (XK_KP_1, "1");
  Map (XK_KP_2, "2");
  Map (XK_KP_3, "3");
  Map (XK_KP_4, "4");
  Map (XK_KP_5, "5");
  Map (XK_KP_6, "6");
  Map (XK_KP_7, "7");
  Map (XK_KP_8, "8");
  Map (XK_KP_9, "9");
}

/******************************************************************************
* Miscellaneous
******************************************************************************/

void
x_display_rep::get_extents (SI& width, SI& height) {
  width = display_width *PIXEL;
  height= display_height*PIXEL;
}

/******************************************************************************
* Main initialization
******************************************************************************/

display_rep::display_rep () {}
display_rep::~display_rep () {}

x_display_rep::x_display_rep (int argc2, char** argv2):
  display_rep (), selection (NULL), selected_tree ("none"),
  color_scale ((void*) NULL), lower_key (""), upper_key (""),
  server_ps_fn (NULL), server_tex_fn (NULL)
{
  if ((dpy= XOpenDisplay (NULL)) == NULL)
    fatal_error ("I failed to connect to Xserver",
		 "x_display_rep::x_display_rep");

  XGCValues values;

  scr           = DefaultScreen (dpy);
  root          = RootWindow (dpy, scr);
  gc            = XCreateGC (dpy, root, 0, &values);
  pixmap_gc     = XCreateGC (dpy, root, 0, &values);
  depth         = DefaultDepth (dpy, scr);
  display_width = DisplayWidth  (dpy, scr);
  display_height= DisplayHeight (dpy, scr);
  cols          = DefaultColormap (dpy, DefaultScreen (dpy));
  cmap          = new color [CTOTAL];
  state         = 0;
  shadow        = NULL;
  gswindow      = NULL;
  argc          = argc2;
  argv          = argv2;

  XSetGraphicsExposures (dpy, gc, TRUE);

  get_xmodmap ();
  initialize_colors ();
  initialize_keyboard_pointer ();
  out_lan= "english";
}

x_display_rep::~x_display_rep () {
  clear_selection ();
  XFreeGC (dpy, gc);
  delete[] cmap;
  XCloseDisplay (dpy);
}

display
open_display (int argc2, char** argv2) {
  return new x_display_rep (argc2, argv2);
}

void
close_display (display dis) {
  delete dis;
}

#endmodule // code_x_init
