
/******************************************************************************
* MODULE     : x_loop.gen.cc
* DESCRIPTION: The main event loop
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public licence and comes WITHOUT
* ANY WARRENTY WHATSOEVER. See the file $TEXMACS_PATH/LICENCE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <timer.gen.h>
#include <X/x_window.gen.h>
#include <iterator.gen.cc>

#module code_x_loop
#import timer
#import x_window
#define no_tree_converter<Window>
#define no_tree_converter<pointer>
#import code_iterator (Window)
#import code_hashmap_iterator (Window,pointer)

int nr_windows=0;

/******************************************************************************
* Look up keys and mouse
******************************************************************************/

string
x_display_rep::look_up_key (XKeyEvent* ev) {
  KeySym key;
  if (ev->state<32) {
    if ((ev->state&3)==0) key= XLookupKeysym (ev, 0);
    else {
      key= XLookupKeysym (ev, 1);
      if (key==0) key= XLookupKeysym (ev, 0);
    }    
  }
  else {
    if ((ev->state&3)==0) key= XLookupKeysym (ev, 2);
    else {
      key= XLookupKeysym (ev, 3);
      if (key==0) key= XLookupKeysym (ev, 2);
    }
  }

  string s= ((ev->state&3)? upper_key [key]: lower_key [key]);
  if (N(s)==0) return s;
  if (ev->state&4) s= "ctrl-" * s;
  if (ev->state&8) s= "alt-" * s;
  if (N(s)>1) s= "<" * s * ">";
  if (s=="<spc>") s= " ";
  return s;
}

string
x_display_rep::look_up_mouse (XButtonEvent* ev) {
  switch (ev->button) {
  case Button1: return "left";
  case Button2: return "middle";
  case Button3: return "right";
  case Button4: return "extra1";
  case Button5: return "extra2";
  default: return "unknown";
  }
}

unsigned int
x_display_rep::get_button_mask (XButtonEvent* ev) {
  switch (ev->button) {
  case Button1: return Button1Mask;
  case Button2: return Button2Mask;
  case Button3: return Button3Mask;
  case Button4: return Button4Mask;
  case Button5: return Button5Mask;
  default: return 0;
  }
}

/******************************************************************************
* Process events
******************************************************************************/

char* event_name[]= {
  "?",
  "?",
  "Key press",
  "Key release",
  "Button press",
  "Button release",
  "Motion notify",
  "Enter notify",
  "Leave notify",
  "Focus in",
  "Focus out",
  "Keymap notify",
  "Expose",
  "Graphics expose",
  "No expose",
  "Visibility notify",
  "Create notify",
  "Destroy notify",
  "Unmap notify",
  "Map request",
  "Reparent notify",
  "Configure notify",
  "Configure request",
  "Gravity notify",
  "Resize request",
  "Circulate notify",
  "Circulate request",
  "Property notify",
  "Selection clear",
  "Selection request",
  "Selection notify",
  "Colormap notify",
  "Client message",
  "Mapping notify"
};

void
x_display_rep::process_event (x_window win, XEvent* ev) {
  // cout << "Event: " << event_name[ev->type] << "\n";
  switch (ev->type) {
  case Expose:
    {
      XExposeEvent& ee (ev->xexpose);
      /*
      cout << "Expose: " << ee.x << "," << ee.y << ","
	   << ee.x+ee.width << "," << ee.y+ee.height << "\n";
	   */
      win->invalidate_event (ee.x, ee.y, ee.x+ee.width, ee.y+ee.height);
      break;
    }
  case GraphicsExpose:
    {
      XGraphicsExposeEvent& ee (ev->xgraphicsexpose);
      /*
      cout << "Expose: " << ee.x << "," << ee.y << ","
	   << ee.x+ee.width << "," << ee.y+ee.height << "\n";
	   */
      win->invalidate_event (ee.x, ee.y, ee.x+ee.width, ee.y+ee.height);
      break;
    }
  case NoExpose:
    // cout << "No expose\n";
    break;
  case ConfigureNotify:
    /*
    cout << "Configure move  : " << ev->xconfigure.x << ","
	 << ev->xconfigure.y << "\n";
    cout << "Configure resize: " << ev->xconfigure.width << ","
	 << ev->xconfigure.height << "\n";
	 */
    if ((ev->xconfigure.x!=0) || (ev->xconfigure.y!=0) ||
	((ev->xconfigure.width == win->win_w) &&
	 (ev->xconfigure.height == win->win_h)))
      win->move_event (ev->xconfigure.x, ev->xconfigure.y);
    win->resize_event (ev->xconfigure.width, ev->xconfigure.height);
    break;
  case CreateNotify:
    break;
  case DestroyNotify:
    fatal_error ("destroy: this should not happen !", "event_loop");
    break;
  case UnmapNotify:
    break;
  case ButtonPress:
    set_button_state (ev->xbutton.state ^ get_button_mask (&ev->xbutton));
    win->mouse_event ("press-" * look_up_mouse (&ev->xbutton),
		      ev->xbutton.x, ev->xbutton.y);
    break;
  case ButtonRelease:
    set_button_state (ev->xbutton.state ^ get_button_mask (&ev->xbutton));
    win->mouse_event ("release-" * look_up_mouse (&ev->xbutton),
		      ev->xbutton.x, ev->xbutton.y);
    break;
  case EnterNotify:
    if (ev->xcrossing.mode == NotifyNormal) {
      // cout << "Enter at (" <<ev->xcrossing.x<<","<<ev->xcrossing.y << ")\n";
      set_button_state (ev->xcrossing.state);
      win->mouse_event ("enter", ev->xcrossing.x, ev->xcrossing.y);
    }
    break;
  case LeaveNotify:
    if (ev->xcrossing.mode == NotifyNormal) {
      // cout << "Leave at (" <<ev->xcrossing.x<<","<<ev->xcrossing.y << ")\n";
      set_button_state (ev->xcrossing.state);
      win->mouse_event ("leave", ev->xcrossing.x, ev->xcrossing.y);
    }
    break;
  case FocusIn:
    win->focus_in_event ();
    break;
  case FocusOut:
    win->focus_out_event ();
    break;
  case MotionNotify:
    // cout << "Move to (" << ev->xmotion.x << "," << ev->xmotion.y << ")\n";
    set_button_state (ev->xmotion.state);
    win->mouse_event ("move", ev->xmotion.x, ev->xmotion.y);
    break;
  case KeyPress:
    {
      string key= look_up_key (&ev->xkey);
      if (N(key)>0) win->key_event (key);
      break;
    }
  case SelectionRequest:
    {
      bool flag=TRUE;
      XSelectionRequestEvent& req (ev->xselectionrequest);
      if (selection==NULL) flag=FALSE;
      if ((req.target!=AnyPropertyType) && (req.target!=XA_STRING)) flag=FALSE;
      if (flag)
	XChangeProperty (dpy, req.requestor, req.property, XA_STRING,
			 8, PropModeReplace,
			 (unsigned char*) selection,
			 strlen (selection));
      XSelectionEvent sel;
      sel.type      = SelectionNotify;
      sel.serial    = req.serial;
      sel.send_event= TRUE;
      sel.display   = dpy;
      sel.requestor = req.requestor;
      sel.selection = req.selection;
      sel.target    = req.target;
      sel.property  = flag?req.property:None;
      sel.time      = req.time;
      XSendEvent (dpy, InputFocus, FALSE, 0, (XEvent*) &sel);
      break;
    }
  case SelectionClear:
    clear_selection ();
    break;
  case ClientMessage:
    {
      /*
      XClientMessageEvent& mess (ev->xclient);
      Atom page= XInternAtom (dpy, "PAGE", FALSE);
      Atom done= XInternAtom (dpy, "DONE", FALSE);
      Atom next= XInternAtom (dpy, "NEXT", FALSE);
      if (mess.message_type == page) {
	// Got page message from ghostscript
	XEvent back;
	Window back_win= mess.data.l[0];
	back.xclient.type = ClientMessage;
	back.xclient.display = dpy;
	back.xclient.window = back_win;
	back.xclient.message_type = next;
	back.xclient.format = 32;
	XSendEvent (dpy, back_win, FALSE, 0, &back);
	XFlush (dpy);
      }
      else if (mess.message_type == done) {
	// Got done message from ghostscript
      }
      else {
	// Unknown client message
      }
      */
      break;
    }
  }
}

/******************************************************************************
* Main event loop
******************************************************************************/

static void (*the_interpose_handler) (void) = NULL;
void set_interpose_handler (void (*r) (void)) { the_interpose_handler= r; }

void
x_display_rep::event_loop () {
  while (nr_windows>0) {
    XEvent report;
    XNextEvent (dpy, &report);
    x_window win= (x_window) Window_to_window[report.xany.window];
    if (win!=NULL) process_event (win, &report);
    if (XPending (dpy)>0) continue;
    if (the_interpose_handler != NULL) the_interpose_handler ();

    iterator<Window> it= iterate (Window_to_window);
    while (it->busy()) {
      x_window win= (x_window) Window_to_window[it->next()];
      win->repaint_invalid_regions();
    }

    while (!nil (messages)) {
      timeb tb; ftime (&tb); time_t ct= (tb.time*1000)+ tb.millitm;
      message m= messages->item;
      if ((m->t - ct) <= 0) {
	m->wid << emit_alarm (m->s, m->t);
	messages= messages->next;
      }
      else break;
    }
  }
}

#endmodule // code_x_loop
